@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,13 +14,8 @@
 * limitations under the License.
 *******************************************************************************/
 
-#ifndef COMPILER_WORKAROUNDS_HPP
-#define COMPILER_WORKAROUNDS_HPP
-
-#if (defined __GNUC__) && (!defined(__INTEL_COMPILER)) \
-        && (!defined(__INTEL_LLVM_COMPILER)) && (!defined(__clang__major__))
-#define NEED_GCC_WA_CHECK 1
-#endif
+#ifndef COMMON_COMPILER_WORKAROUNDS_HPP
+#define COMMON_COMPILER_WORKAROUNDS_HPP
 
 // Workaround 01: clang.
 //
@@ -32,7 +27,7 @@
 // vectorization for clang altogether for now.
 //
 // [1] https://bugs.llvm.org/show_bug.cgi?id=48104
-#if (defined __clang_major__) && (__clang_major__ >= 6)
+#if (defined __clang_major__) && (__clang_major__ < 13)
 #define CLANG_WA_01_SAFE_TO_USE_OMP_SIMD 0
 #else
 #define CLANG_WA_01_SAFE_TO_USE_OMP_SIMD 1
@@ -40,40 +35,15 @@
 
 // Workaround 02: clang.
 //
-// Clang 6+ generates incorrect code with OMP_SIMD in some particular cases.
+// Clang generates incorrect code with OMP_SIMD in some particular cases.
 // Unlike CLANG_WA_01_SAFE_TO_USE_OMP_SIMD, the issue happens even with -O3.
-#if (defined __clang_major__) && (__clang_major__ >= 6)
+#if (defined __clang_major__) && (__clang_major__ < 13)
 #define CLANG_WA_02_SAFE_TO_USE_OMP_SIMD 0
 #else
 #define CLANG_WA_02_SAFE_TO_USE_OMP_SIMD 1
 #endif
 
-// Workaround 03: GCC
-//
-// For very large functions with too much control flow (i.e. if, switch, goto
-// statements), GCC 7 may struggle to perform optimizations based on tree
-// dominator (i.e. -ftree-dominator-opts, which is enabled with O1), thereby
-// producing an internal compiler error (ICE). Specifically, it seems that the
-// jump threading optimization is the culprit, which cannot be disabled on its
-// own. There is no reliable way to reproduce the ICE, therefore it is not clear
-// which __GCC_MINOR__ version fixes issue.
-#if (defined NEED_GCC_WA_CHECK) && (__GNUC__ == 7)
-#define GCC_WA_NO_TREE_DOMINATOR_OPTS 1
-#else
-#define GCC_WA_NO_TREE_DOMINATOR_OPTS 0
-#endif
-
-// Workaround 05: GCC
-//
-// NOTE: inside lambda, type cast variables captured by reference using
-// either c-like "(type)var" or functional "type(var)" notation in order
-// to avoid gcc7 bug with c++14 standard
-// (https://gcc.gnu.org/bugzilla/show_bug.cgi?id=83204).
-#if (defined NEED_GCC_WA_CHECK) && (__GNUC__ <= 7)
-#define GCC_WA_LAMBDA_C_CAST
-#endif
-
-// Workaround 05: c++17 vs c++20
+// Workaround 03: MSVC c++17 vs c++20
 //
 // C++17/20 are contradictory wrt capturing this and using default '=' capture.
 // - C++17 and before have to return a warning for the [=, this] capture as
@@ -91,4 +61,4 @@
 #define WA_THIS_COPY_CAPTURE
 #endif
 
-#endif // COMPILER_WORKAROUNDS_HPP
+#endif // COMMON_COMPILER_WORKAROUNDS_HPP
