@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -51,6 +51,9 @@ status_t conv_desc_init(convolution_desc_t *conv_desc, prop_kind_t prop_kind,
     VCHECK_CONV(one_of(alg_kind, convolution_auto, convolution_direct,
                         convolution_winograd),
             VERBOSE_BAD_ALGORITHM);
+    VCHECK_CONV(!any_memory_desc_host_scalar(
+                        src_desc, weights_desc, bias_desc, dst_desc),
+            VERBOSE_UNSUPPORTED_FORMAT_KIND);
 
     if (padding_r == nullptr) padding_r = padding_l;
 
@@ -77,8 +80,8 @@ status_t conv_desc_init(convolution_desc_t *conv_desc, prop_kind_t prop_kind,
     if (with_bias)
         runtime_dims_or_strides = runtime_dims_or_strides
                 || memory_desc_wrapper(bias_desc).has_runtime_dims_or_strides();
-    VCONDCHECK(primitive, create, check, conv, !runtime_dims_or_strides,
-            status::unimplemented, VERBOSE_RUNTIMEDIM_UNSUPPORTED);
+    VCHECK_CONV_UNIMPL(
+            !runtime_dims_or_strides, VERBOSE_RUNTIMEDIM_UNSUPPORTED);
 
     (prop_kind == backward_data ? cd.diff_src_desc : cd.src_desc) = *src_desc;
     (is_fwd ? cd.dst_desc : cd.diff_dst_desc) = *dst_desc;
@@ -95,13 +98,15 @@ status_t conv_desc_init(convolution_desc_t *conv_desc, prop_kind_t prop_kind,
 
     VCHECK_CONV(memory_desc_wrapper(weights_desc).nelems(),
             VERBOSE_EMPTY_TENSOR, "weights");
-    VCHECK_CONV(src_desc->ndims == dst_desc->ndims, VERBOSE_INCONSISTENT_NDIMS,
-            "src", "dst");
+    VCHECK_CONV(src_desc->ndims == dst_desc->ndims,
+            VERBOSE_INCONSISTENT_NDIMS_WITH_VALS, "src", "dst", src_desc->ndims,
+            dst_desc->ndims);
     VCHECK_CONV(utils::one_of(src_desc->ndims, 3, 4, 5), VERBOSE_BAD_NDIMS,
             "src", src_desc->ndims);
     VCHECK_CONV(utils::one_of(weights_desc->ndims, src_desc->ndims,
                         src_desc->ndims + 1),
-            VERBOSE_INCONSISTENT_NDIMS, "src", "weights");
+            VERBOSE_INCONSISTENT_NDIMS_WITH_VALS, "src", "weights",
+            weights_desc->ndims, src_desc->ndims);
 
     const dim_t g = with_groups ? weights_desc->dims[0] : 1;
     const dim_t bias_dim = prop_kind == backward_data ? src_desc->dims[1]
@@ -116,6 +121,18 @@ status_t conv_desc_init(convolution_desc_t *conv_desc, prop_kind_t prop_kind,
             VERBOSE_INCONSISTENT_DIM, "src", 1, "weights", with_groups + 1);
     VCHECK_CONV(dst_desc->dims[1] == g * weights_desc->dims[with_groups + 0],
             VERBOSE_INCONSISTENT_DIM, "dst", 1, "weights", with_groups + 0);
+    // s4/u4/f4 weights requires channels to be multiple of 2 to be byte aligned
+    VCHECK_CONV(IMPLICATION(utils::one_of(weights_desc->data_type,
+                                    data_type::s4, data_type::u4,
+                                    data_type::f4_e2m1, data_type::f4_e3m0),
+                        weights_desc->dims[with_groups + 1] % 2 == 0),
+            VERBOSE_BAD_DIM, "weights", with_groups + 1);
+    // s4/u4/f4 src requires channels to be multiple of 2 to be byte aligned
+    VCHECK_CONV(IMPLICATION(utils::one_of(src_desc->data_type, data_type::s4,
+                                    data_type::u4, data_type::f4_e2m1,
+                                    data_type::f4_e3m0),
+                        src_desc->dims[1] % 2 == 0),
+            VERBOSE_BAD_DIM, "src", 1);
 
     int sp_dims = src_desc->ndims - 2;
     utils::array_copy(cd.strides, strides, sp_dims);
@@ -136,10 +153,23 @@ status_t conv_desc_init(convolution_desc_t *conv_desc, prop_kind_t prop_kind,
         dim_t dst = dst_desc->dims[i];
         dim_t ker_range = 1 + (ker - 1) * (dil + 1);
         VCHECK_CONV(str > 0, VERBOSE_BAD_DIM, "strides", i - 2);
-        VCHECK_CONV(dil >= 0 && pad_l >= 0 && pad_r + str > 0,
-                VERBOSE_INCONSISTENT_PRB);
+        VCHECK_CONV(dil >= 0, "%s: dilation (%d) must be non-negative",
+                VERBOSE_INCONSISTENT_PRB, static_cast<int>(dil));
+        VCHECK_CONV(pad_l >= 0,
+                "%s: left padding value (%d) must be non-negative",
+                VERBOSE_INCONSISTENT_PRB, static_cast<int>(pad_l));
+        VCHECK_CONV(pad_r + str > 0,
+                "%s: right padding (%d) and stride (%d) must sum up to a "
+                "positive value",
+                VERBOSE_INCONSISTENT_PRB, static_cast<int>(pad_r),
+                static_cast<int>(str));
         VCHECK_CONV((src - ker_range + pad_l + pad_r) / str + 1 == dst,
-                VERBOSE_INCONSISTENT_PRB);
+                "%s: mismatch between actual and computed dst dims, dst (%d) "
+                "!= (src(%d) - ker(%d) + pad_l(%d) + pad_r(%d))/ str(%d) + 1",
+                VERBOSE_INCONSISTENT_PRB, static_cast<int>(dst),
+                static_cast<int>(src), static_cast<int>(ker_range),
+                static_cast<int>(pad_l), static_cast<int>(pad_r),
+                static_cast<int>(str));
     }
 
     *conv_desc = cd;
@@ -159,18 +189,23 @@ status_t conv_attr_check(const convolution_desc_t &desc, const engine_t *engine,
         const data_type_t src_dt = desc.src_desc.data_type;
         const data_type_t dst_dt = desc.dst_desc.data_type;
 
-        auto fwd_attr_mask
-                = smask_t::post_ops | smask_t::sum_dt | smask_t::fpmath_mode;
+        auto fwd_attr_mask = smask_t::post_ops | smask_t::sum_dt
+                | smask_t::fpmath_mode | smask_t::rounding_mode;
+        const bool is_gpu = engine->kind() == engine_kind::gpu;
 
-        bool is_int8 = utils::one_of(src_dt, data_type::s8, data_type::u8);
-        if (engine->kind() == engine_kind::gpu)
-            is_int8 = is_int8
-                    || utils::one_of(dst_dt, data_type::s8, data_type::u8,
-                            data_type::s32);
-        if (is_int8)
-            fwd_attr_mask |= smask_t::scales_runtime
-                    | smask_t::zero_points_runtime
-                    | smask_t::zero_points_runtime_data_type;
+        const bool is_int8 = utils::one_of(src_dt, data_type::s8, data_type::u8)
+                || (is_gpu
+                        && utils::one_of(dst_dt, data_type::s8, data_type::u8,
+                                data_type::s32));
+        const bool is_fp8
+                = utils::one_of(src_dt, data_type::f8_e5m2, data_type::f8_e4m3)
+                || (is_gpu
+                        && utils::one_of(dst_dt, data_type::f8_e5m2,
+                                data_type::f8_e4m3));
+        const bool enable_quantization = is_int8 || is_fp8;
+        if (enable_quantization)
+            fwd_attr_mask |= smask_t::zero_points_data_type
+                    | smask_t::scales_data_type;
 
         VCHECK_CONV_UNIMPL(attr->has_default_values(fwd_attr_mask, dst_dt),
                 VERBOSE_UNSUPPORTED_ATTR);
@@ -178,27 +213,37 @@ status_t conv_attr_check(const convolution_desc_t &desc, const engine_t *engine,
         // Check scales
         if (!attr->scales_.has_default_values()) {
             const auto &sc = attr->scales_;
-            const int mask_src = sc.get(DNNL_ARG_SRC).mask_;
-            const int mask_wei = sc.get(DNNL_ARG_WEIGHTS).mask_;
-            const int mask_dst = sc.get(DNNL_ARG_DST).mask_;
             const bool with_groups
                     = desc.src_desc.ndims != desc.weights_desc.ndims;
-            VCHECK_CONV_UNIMPL(utils::everyone_is(0, mask_src, mask_dst)
-                            && utils::one_of(mask_wei, 0, with_groups ? 3 : 1),
+            VCHECK_CONV_UNIMPL(IMPLICATION(!sc.has_default_values(DNNL_ARG_SRC),
+                                       sc.get_mask(DNNL_ARG_SRC) == 0),
+                    VERBOSE_UNSUPPORTED_SCALES_CFG);
+            VCHECK_CONV_UNIMPL(
+                    IMPLICATION(!sc.has_default_values(DNNL_ARG_WEIGHTS),
+                            utils::one_of(sc.get_mask(DNNL_ARG_WEIGHTS), 0,
+                                    with_groups ? 3 : 1)),
+                    VERBOSE_UNSUPPORTED_SCALES_CFG);
+            VCHECK_CONV_UNIMPL(
+                    IMPLICATION(!sc.has_default_values(DNNL_ARG_DST),
+                            utils::one_of(sc.get_mask(DNNL_ARG_DST), 0, 2)),
                     VERBOSE_UNSUPPORTED_SCALES_CFG);
         }
 
         // Check zero points
         if (!attr->zero_points_.has_default_values()) {
             const auto &zp = attr->zero_points_;
-            int mask_src = 0, mask_wei = 0, mask_dst = 0;
-            zp.get(DNNL_ARG_SRC, &mask_src);
-            zp.get(DNNL_ARG_WEIGHTS, &mask_wei);
-            zp.get(DNNL_ARG_DST, &mask_dst);
 
-            VCHECK_CONV_UNIMPL((mask_src == 0 || mask_src == 1 << 1)
-                            && (mask_wei == 0)
-                            && (mask_dst == 0 || mask_dst == 1 << 1),
+            VCHECK_CONV_UNIMPL(IMPLICATION(!zp.has_default_values(DNNL_ARG_SRC),
+                                       utils::one_of(zp.get_mask(DNNL_ARG_SRC),
+                                               0, 1 << 1)),
+                    VERBOSE_UNSUPPORTED_ZP_CFG);
+            VCHECK_CONV_UNIMPL(
+                    IMPLICATION(!zp.has_default_values(DNNL_ARG_WEIGHTS),
+                            zp.get_mask(DNNL_ARG_WEIGHTS) == 0),
+                    VERBOSE_UNSUPPORTED_ZP_CFG);
+            VCHECK_CONV_UNIMPL(IMPLICATION(!zp.has_default_values(DNNL_ARG_DST),
+                                       utils::one_of(zp.get_mask(DNNL_ARG_DST),
+                                               0, 1 << 1)),
                     VERBOSE_UNSUPPORTED_ZP_CFG);
         }
 
@@ -213,9 +258,12 @@ status_t conv_attr_check(const convolution_desc_t &desc, const engine_t *engine,
             // Check sum
             VCHECK_CONV_UNIMPL(po.check_sum_consistency(dst_dt, is_int8, true),
                     VERBOSE_UNSUPPORTED_POSTOP);
+
+            // Note: verbose support is inside the call.
+            CHECK(po.validate_binary(engine->kind(), &desc.dst_desc));
         }
     } else {
-        auto bwd_attr_mask = smask_t::fpmath_mode;
+        auto bwd_attr_mask = smask_t::fpmath_mode | smask_t::accumulation_mode;
         VCHECK_CONV_UNIMPL(attr->has_default_values(bwd_attr_mask),
                 VERBOSE_UNSUPPORTED_ATTR);
     }
