@@ -18,8 +18,8 @@
 #include "utils.hpp"
 
 #if defined(DNNL_ENABLE_ITT_TASKS)
-#include "common/ittnotify/ittnotify.h"
 #include "dnnl_debug.h"
+#include "ittnotify/ittnotify.h"
 #endif
 
 namespace dnnl {
@@ -84,10 +84,12 @@ void primitive_task_start(primitive_kind_t kind) {
 #undef CASE
     int kind_idx = (int)kind;
     assert(kind_idx >= 0);
-    assert((size_t)kind_idx
-            < sizeof(prim_kind_itt_strings) / sizeof(prim_kind_itt_strings[0]));
-    __itt_task_begin(itt_domain(), __itt_null, __itt_null,
-            prim_kind_itt_strings[kind_idx]);
+    if (kind_idx < primitive_kind::internal_only_start) {
+        assert((size_t)kind_idx < sizeof(prim_kind_itt_strings)
+                        / sizeof(prim_kind_itt_strings[0]));
+        __itt_task_begin(itt_domain(), __itt_null, __itt_null,
+                prim_kind_itt_strings[kind_idx]);
+    }
     thread_primitive_kind = kind;
 }
 
