@@ -1,5 +1,7 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
+* Copyright 2024-2025 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -201,6 +203,9 @@ status_t memory_desc_wrapper::compute_blocking(
         C(BA16a32b, {1, 0}, {16, 32}, {0, 1});
         C(BA16a48b, {1, 0}, {16, 48}, {0, 1});
         C(BA16a64b, {1, 0}, {16, 64}, {0, 1});
+        C(BA24b8a, {1, 0}, {24, 8}, {1, 0});
+        C(aCB24c8b, {0, 2, 1}, {24, 8}, {2, 1});
+        C(abDC24d8c, {0, 1, 3, 2}, {24, 8}, {3, 2});
         C(BA16a16b2a, {1, 0}, {16, 16, 2}, {0, 1, 0});
         C(BA16a32b2a, {1, 0}, {16, 32, 2}, {0, 1, 0});
         C(BA16a48b2a, {1, 0}, {16, 48, 2}, {0, 1, 0});
@@ -516,7 +521,11 @@ status_t memory_desc_wrapper::compute_blocking(
         C(Acb8a, {0, 2, 1}, {8}, {0});
         C(AcB8a2b, {0, 2, 1}, {8, 2}, {0, 1});
         C(AcB8a4b, {0, 2, 1}, {8, 4}, {0, 1});
+        C(aCBd4b4c, {0, 2, 1, 3}, {4, 4}, {1, 2});
+        C(aCBd8b8c, {0, 2, 1, 3}, {8, 8}, {1, 2});
         C(aCBd16b16c, {0, 2, 1, 3}, {16, 16}, {1, 2});
+        C(aCBde4b4c, {0, 2, 1, 3, 4}, {4, 4}, {1, 2});
+        C(aCBde8b8c, {0, 2, 1, 3, 4}, {8, 8}, {1, 2});
         C(aCBde16b16c, {0, 2, 1, 3, 4}, {16, 16}, {1, 2});
         C(Acdb16a, {0, 2, 3, 1}, {16}, {0});
         C(AcdB16a2b, {0, 2, 3, 1}, {16, 2}, {0, 1});
@@ -532,7 +541,11 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AcdeB8a4b, {0, 2, 3, 4, 1}, {8, 4}, {0, 1});
         C(Acedb16a, {0, 2, 4, 3, 1}, {16}, {0});
         C(Adcb16a, {0, 3, 2, 1}, {16}, {0});
+        C(BAc4a4b, {1, 0, 2}, {4, 4}, {0, 1});
+        C(BAc8a8b, {1, 0, 2}, {8, 8}, {0, 1});
         C(BAc16a16b, {1, 0, 2}, {16, 16}, {0, 1});
+        C(BAcd4a4b, {1, 0, 2, 3}, {4, 4}, {0, 1});
+        C(BAcd8a8b, {1, 0, 2, 3}, {8, 8}, {0, 1});
         C(BAcd16a16b, {1, 0, 2, 3}, {16, 16}, {0, 1});
         C(ABc32a16b, {0, 1, 2}, {32, 16}, {0, 1});
         C(ABcd32a16b, {0, 1, 2, 3}, {32, 16}, {0, 1});
@@ -585,6 +598,8 @@ status_t memory_desc_wrapper::compute_blocking(
         C(aBCde2b8c8b2c, {0, 1, 2, 3, 4}, {2, 8, 8, 2}, {1, 2, 1, 2});
         C(aBdec32b, {0, 1, 3, 4, 2}, {32}, {1});
         C(aCBdef16c16b, {0, 2, 1, 3, 4, 5}, {16, 16}, {2, 1});
+        C(aCBdef4b4c, {0, 2, 1, 3, 4, 5}, {4, 4}, {1, 2});
+        C(aCBdef8b8c, {0, 2, 1, 3, 4, 5}, {8, 8}, {1, 2});
         C(aCBdef16b16c, {0, 2, 1, 3, 4, 5}, {16, 16}, {1, 2});
         C(Abcdef16a, {0, 1, 2, 3, 4, 5}, {16}, {0});
         C(Abcdef32a, {0, 1, 2, 3, 4, 5}, {32}, {0});
@@ -592,6 +607,8 @@ status_t memory_desc_wrapper::compute_blocking(
         C(aCBde16c16b, {0, 2, 1, 3, 4}, {16, 16}, {2, 1});
         C(Acdb32a, {0, 2, 3, 1}, {32}, {0});
         C(BAcd16b16a, {1, 0, 2, 3}, {16, 16}, {1, 0});
+        C(BAcde4a4b, {1, 0, 2, 3, 4}, {4, 4}, {0, 1});
+        C(BAcde8a8b, {1, 0, 2, 3, 4}, {8, 8}, {0, 1});
         C(BAcde16a16b, {1, 0, 2, 3, 4}, {16, 16}, {0, 1});
         C(BAc16b16a, {1, 0, 2}, {16, 16}, {1, 0});
         C(aBCd2b4c2b, {0, 1, 2, 3}, {2, 4, 2}, {1, 2, 1});
@@ -612,6 +629,7 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AB8a2b, {0, 1}, {8, 2}, {0, 1});
         C(abDc16d, {0, 1, 3, 2}, {16}, {3});
         C(abDc32d, {0, 1, 3, 2}, {32}, {3});
+        C(abDC16d4c, {0, 1, 3, 2}, {16, 4}, {3, 2});
         C(abDC32d4c, {0, 1, 3, 2}, {32, 4}, {3, 2});
         C(abCd4c, {0, 1, 2, 3}, {4}, {2});
         C(abCde4c, {0, 1, 2, 3, 4}, {4}, {2});
@@ -621,6 +639,7 @@ status_t memory_desc_wrapper::compute_blocking(
         C(abCdef32c, {0, 1, 2, 3, 4, 5}, {32}, {2});
         C(abdEc16e, {0, 1, 3, 4, 2}, {16}, {4});
         C(abdEc32e, {0, 1, 3, 4, 2}, {32}, {4});
+        C(abdEC16e4c, {0, 1, 3, 4, 2}, {16, 4}, {4, 2});
         C(abdEC32e2c, {0, 1, 3, 4, 2}, {32, 2}, {4, 2});
         C(abdEC32e4c, {0, 1, 3, 4, 2}, {32, 4}, {4, 2});
         C(abdEC64e2c, {0, 1, 3, 4, 2}, {64, 2}, {4, 2});
@@ -749,8 +768,10 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AcdeB16b64a4b, {0, 2, 3, 4, 1}, {16, 64, 4}, {1, 0, 1});
         C(decbA16a, {3, 4, 2, 1, 0}, {16}, {0});
         C(decbA8a, {3, 4, 2, 1, 0}, {8}, {0});
+        C(decbA4a, {3, 4, 2, 1, 0}, {4}, {0});
         C(defcbA16a, {3, 4, 5, 2, 1, 0}, {16}, {0});
         C(defcbA8a, {3, 4, 5, 2, 1, 0}, {8}, {0});
+        C(defcbA4a, {3, 4, 5, 2, 1, 0}, {4}, {0});
         C(aCB16c2b, {0, 2, 1}, {16, 2}, {2, 1});
         C(aCB16c4b, {0, 2, 1}, {16, 4}, {2, 1});
         C(BA16b2a, {1, 0}, {16, 2}, {1, 0});
@@ -984,6 +1005,13 @@ status_t memory_desc_wrapper::compute_blocking(
         C(aCB8b24c, {0, 2, 1}, {8, 24}, {1, 2});
         C(aCB8b16c, {0, 2, 1}, {8, 16}, {1, 2});
         C(aCB8b8c, {0, 2, 1}, {8, 8}, {1, 2});
+        C(BA16b8a, {1, 0}, {16, 8}, {1, 0});
+        C(BA8b8a, {1, 0}, {8, 8}, {1, 0});
+        C(aCB16c8b, {0, 2, 1}, {16, 8}, {2, 1});
+        C(aCB8c8b, {0, 2, 1}, {8, 8}, {2, 1});
+        C(abDC16d8c, {0, 1, 3, 2}, {16, 8}, {3, 2});
+        C(abDC8d8c, {0, 1, 3, 2}, {8, 8}, {3, 2});
+        C(AB2a4b, {0, 1}, {2, 4}, {0, 1});
         default: break;
     }
 
