@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@ namespace impl {
 
 /** thin wrapper class over \struct memory_desc_t which allows easy
  * manipulations with underlying C structure, which is taken by reference */
+// NOLINTNEXTLINE(readability-identifier-naming)
 struct memory_desc_wrapper : public c_compatible {
     const memory_desc_t *md_;
 
@@ -72,6 +73,68 @@ struct memory_desc_wrapper : public c_compatible {
     }
     bool is_sparse_desc() const { return format_kind() == format_kind::sparse; }
 
+    bool is_host_scalar_desc() const {
+        return format_kind() == format_kind::host_scalar;
+    }
+
+    /**
+     * Checks whether the memory descriptor represents a canonical (row-major)
+     * layout.
+     *
+     * A canonical layout is defined as one where the strides follow the
+     * standard row-major ordering of the remaining dimensions, ignoring any
+     * dimensions of size 1.
+     *
+     * @note Format tags (e.g., "abc", "acb") may correspond to different
+     *   stride arrays but the same physical layout when unit dimensions are
+     *   present. Therefore, checking for canonical layout is more reliable than
+     *   matching a specific tag.
+     *
+     * Example:
+     *   dims    = [10, 12, 1]
+     *   strides = [12, 1, 1]
+     *   tag     = "abc"
+     *   -> remove unit dims: dims = [10,12], strides = [12,1]
+     *   -> canonical (true)
+     *
+     *   dims    = [10, 1, 12]
+     *   strides = [12, 12, 1]
+     *   tag     = "acb"
+     *   -> remove unit dims: dims = [10,12], strides = [12,1]
+     *   -> canonical (true)
+     *
+     *   dims    = [10, 12, 3]
+     *   strides = [36, 1, 3]
+     *   tag     = "acb"
+     *   -> canonical (false)
+     *
+     * @return true if the descriptor has canonical layout, false otherwise.
+     */
+    bool is_canonical() const {
+        if (!is_plain()) return false;
+
+        dims_t normalized_dims = {};
+        dims_t normalized_strides = {};
+
+        int normalized_ndims = 0;
+        for (int i = 0; i < ndims(); ++i) {
+            if (dims()[i] != 1) {
+                normalized_dims[normalized_ndims] = dims()[i];
+                normalized_strides[normalized_ndims] = strides()[i];
+                normalized_ndims++;
+            }
+        }
+
+        if (normalized_ndims == 0) return true;
+
+        dim_t expected_stride = 1;
+        for (int i = normalized_ndims - 1; i >= 0; --i) {
+            if (normalized_strides[i] != expected_stride) return false;
+            expected_stride *= normalized_dims[i];
+        }
+        return true;
+    }
+
     const blocking_desc_t &blocking_desc() const {
         assert(is_blocking_desc() || is_sparse_packed_desc());
         if (!is_sparse_desc()) return md_->format_desc.blocking;
@@ -149,30 +212,28 @@ struct memory_desc_wrapper : public c_compatible {
     size_t additional_buffer_data_size(uint64_t flag_select) const {
         using namespace memory_extra_flags;
         if (flag_select & compensation_conv_s8s8) return sizeof(int32_t);
-        if ((flag_select & rnn_u8s8_compensation)
-                && !types::extra_flag_rnn_s8s8_compensation_is_set(flag_select))
-            return sizeof(float);
+        if (flag_select & rnn_u8s8_compensation) return sizeof(float);
         if (flag_select & compensation_conv_asymmetric_src)
             return sizeof(int32_t);
+        if (flag_select & compensation_gpu_conv_asymmetric_src)
+            return sizeof(int32_t);
         return 0;
     }
 
     /** return true if memory format has additional buffer */
     bool is_additional_buffer() const {
         using namespace memory_extra_flags;
-        // Currently compensation is not required for rnn_s8s8_compensation,
-        // but it has common bit with rnn_u8s8_compensation constant so we have
-        // to exclude rnn_s8s8_compensation case explicitly
-        return ((extra().flags
-                        & (compensation_conv_s8s8 | rnn_u8s8_compensation
-                                | compensation_conv_asymmetric_src))
-                && !types::extra_flag_rnn_s8s8_compensation_is_set(
-                        extra().flags));
+        return extra().flags
+                & (compensation_conv_s8s8 | rnn_u8s8_compensation
+                        | compensation_gpu_conv_asymmetric_src
+                        | compensation_conv_asymmetric_src);
     }
 
     /** returns the size required for a particular extra memory buffer */
     size_t additional_buffer_size(memory_extra_flags_t flag) const {
         using namespace memory_extra_flags;
+        const auto flags = extra().flags;
+        if (!(flags & flag)) return 0;
 
         const auto ndims = this->ndims();
         const auto &pdims = padded_dims();
@@ -186,26 +247,26 @@ struct memory_desc_wrapper : public c_compatible {
                       return (size_t)prod * buff_data_size;
                   };
 
-        if (extra().flags & compensation_conv_s8s8) {
+        if (flag == compensation_conv_s8s8) {
             return calculate_size(extra().compensation_mask,
                     additional_buffer_data_size(flag));
         }
-
-        if ((extra().flags & rnn_u8s8_compensation)
-                && !types::extra_flag_rnn_s8s8_compensation_is_set(
-                        extra().flags)) {
+        if (flag == rnn_u8s8_compensation) {
             return calculate_size(extra().compensation_mask,
                     additional_buffer_data_size(flag));
         }
-        if (extra().flags & compensation_conv_asymmetric_src) {
+        if (flag == compensation_conv_asymmetric_src) {
             return calculate_size(extra().asymm_compensation_mask,
                     additional_buffer_data_size(flag));
         }
+        if (flag == compensation_gpu_conv_asymmetric_src) {
+            return extra().dst_size;
+        }
 
         return 0;
     }
 
-    int blk_size() const {
+    dim_t blk_size() const {
         assert(is_blocking_desc() || is_sparse_packed_desc());
         const auto &bd = blocking_desc();
         return utils::array_product(bd.inner_blks, bd.inner_nblks);
@@ -220,12 +281,15 @@ struct memory_desc_wrapper : public c_compatible {
         buff_size += additional_buffer_size(compensation_conv_s8s8);
         buff_size += additional_buffer_size(rnn_u8s8_compensation);
         buff_size += additional_buffer_size(compensation_conv_asymmetric_src);
+        buff_size
+                += additional_buffer_size(compensation_gpu_conv_asymmetric_src);
         return buff_size;
     }
 
-    /** returns the size required to store described memory
-     * note: if offset0 != 0 returns 0 (need to specify the behavior) */
-    size_t size(int index = 0, bool include_additional_size = true) const {
+    /** returns the size required to store described memory note: does not
+        include offset0 by default */
+    size_t size(int index = 0, bool include_additional_size = true,
+            bool include_offset0 = false) const {
         if (utils::one_of(format_kind(), format_kind::undef, format_kind::any)
                 || is_zero() || has_zero_dim())
             return 0;
@@ -246,8 +310,6 @@ struct memory_desc_wrapper : public c_compatible {
         } else if (is_cublaslt_blocked_desc()) {
             return cublaslt_blocked_desc().size;
         } else if (is_blocking_desc()) {
-            if (offset0() != 0) return 0;
-
             dims_t blocks = {0};
             compute_blocks(blocks);
 
@@ -262,11 +324,13 @@ struct memory_desc_wrapper : public c_compatible {
             }
 
             if (max_size == 1 && bd.inner_nblks != 0) {
-                max_size = utils::array_product(bd.inner_blks, bd.inner_nblks);
+                max_size = static_cast<size_t>(blk_size());
             }
 
-            size_t data_size = max_size * data_type_size()
-                    / sub_byte_data_type_multiplier();
+            // `div_up` guarantees a spot in memory for odd number of half-byte
+            // elements. Crucial case is `1` when simple division returns 0.
+            size_t data_size = utils::div_up(max_size * data_type_size(),
+                    sub_byte_data_type_multiplier());
             if (is_additional_buffer()) {
                 // The additional buffers, typically of data type int32_t, float
                 // are stored at the end of data. Pad the data, so that the
@@ -275,7 +339,8 @@ struct memory_desc_wrapper : public c_compatible {
                 data_size = utils::rnd_up(data_size, alignment_in_bytes);
             }
             return data_size
-                    + (include_additional_size ? additional_buffer_size() : 0);
+                    + (include_additional_size ? additional_buffer_size() : 0)
+                    + (include_offset0 ? data_type_size() * offset0() : 0);
         } else if (is_sparse_desc()) {
             if (sparse_desc().encoding == sparse_encoding::csr) {
                 switch (index) {
@@ -327,6 +392,8 @@ struct memory_desc_wrapper : public c_compatible {
                 assert(!"unknown sparse encoding");
                 return 0;
             }
+        } else if (is_host_scalar_desc()) {
+            return data_type_size();
         } else {
             assert(!"unknown format kind");
             return 0;
@@ -447,8 +514,8 @@ struct memory_desc_wrapper : public c_compatible {
 
     /** returns true if the memory desc corresponds to the given format tag.
      * @sa memory_desc_matches_tag */
-    bool matches_tag(format_tag_t tag) const {
-        return memory_desc_matches_tag(*md_, tag);
+    bool matches_tag(format_tag_t tag, const dims_t strides = nullptr) const {
+        return memory_desc_matches_tag(*md_, tag, strides);
     }
 
     /** returns matching tag (or undef if match is not found)
@@ -461,12 +528,23 @@ struct memory_desc_wrapper : public c_compatible {
         return format_tag::undef;
     }
 
+    template <typename... Tags>
+    format_tag_t mb_stride_relaxed_match(Tags... tags) const {
+        dims_t skip_mb_stride {};
+        // See `memory_desc_matches_tag` comment.
+        skip_mb_stride[0] = -1;
+        for (const auto &tag : {tags...})
+            if (matches_tag(tag, skip_mb_stride)) return tag;
+        return format_tag::undef;
+    }
+
     /* offset section */
 
     /** returns physical offset by logical one. logical offset is represented by
      * an array \param pos. if \param is_pos_padded is true \param pos
      * represents the position in already padded area */
     dim_t off_v(const dims_t pos, bool is_pos_padded = false) const {
+        if (is_host_scalar_desc()) return 0;
         assert(is_blocking_desc() || is_sparse_packed_desc());
         const blocking_desc_t &blk = blocking_desc();
 
@@ -551,6 +629,16 @@ struct memory_desc_wrapper : public c_compatible {
                           : blk_off<T, Args...>(xn, args...);
     }
 
+    /** returns physical offset by logical one. Logical offset is represented by
+     * a tuple of block indices (\param bn, ..., \param b1, \param b0). It is a
+     * user responsibility to adjust the result to get offset within blocks.
+     * If @tparam sub_off0 is true, then offset0() will be subtracted
+     * from result.*/
+    template <bool skip_first, bool sub_off0, typename T, typename... Args>
+    dim_t blk_off(T xn, Args... args) const {
+        return blk_off<skip_first, Args...>(xn, args...) - sub_off0 * offset0();
+    }
+
     /* static functions section */
     /* TODO: replace with non-static, once md_ becomes non-const ref */
 
