@@ -1,6 +1,7 @@
 /*******************************************************************************
-* Copyright 2020-2023 Intel Corporation
-* Copyright 2023 FUJITSU LIMITED
+* Copyright 2020-2025 Intel Corporation
+* Copyright 2023-2025 FUJITSU LIMITED
+* Copyright 2024-2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -170,8 +171,8 @@ status_t brgemm_desc_init(brgemm_t *brg, cpu_isa_t isa,
     if (brg == nullptr) return status::invalid_arguments;
     if (transA || transB) return status::unimplemented;
 
-    brgemm_utils::init_brgemm_conf(brg, isa, type, dt_a, dt_b, layout, alpha,
-            beta, LDA, LDB, LDC, M, N, K, strides);
+    CHECK(brgemm_utils::init_brgemm_conf(brg, isa, type, dt_a, dt_b, layout,
+            alpha, beta, LDA, LDB, LDC, M, N, K, strides));
 
     if (M <= 0 || N <= 0 || K <= 0) return status::invalid_arguments;
     bool ldx_check = (brg->is_row_major()) ? (LDA < K)
@@ -197,8 +198,8 @@ status_t brdgmm_desc_init(brgemm_t *brg, cpu_isa_t isa,
     if (transA || layout != brgemm_row_major || alpha != 1.0f || beta != 0.f)
         return status::unimplemented;
 
-    brgemm_utils::init_brdgmm_conf(brg, isa, type, dt_a, dt_b, layout, alpha,
-            beta, LDA, LDC, M, N, strides);
+    CHECK(brgemm_utils::init_brdgmm_conf(brg, isa, type, dt_a, dt_b, layout,
+            alpha, beta, LDA, LDC, M, N, strides));
 
     const bool ldx_check = (LDA < N || LDC < N);
     if (ldx_check) return status::invalid_arguments;
@@ -228,27 +229,28 @@ status_t brgemm_desc_set_postops(brgemm_t *brg, const primitive_attr_t *attr,
     brg->LDD = LDD;
     const auto dt_d = dst_md->data_type;
 
-    if ((brg->dt_a == data_type::u8 && brg->dt_b == data_type::s8)
-            && (!one_of(dt_d, data_type::u8, data_type::s8, data_type::s32,
-                    data_type::f32))
-            && (!one_of(dt_bias, data_type::undef, data_type::u8, data_type::s8,
-                    data_type::s32, data_type::f32, data_type::bf16)))
-        return status::unimplemented;
+    if (brg->is_int8) {
+        if ((brg->dt_a == data_type::s8 && brg->dt_b == data_type::u8)
+                || (!one_of(dt_bias, data_type::undef, data_type::s8,
+                        data_type::u8, data_type::f32))
+                || (!one_of(dt_d, data_type::s8, data_type::u8, data_type::s32,
+                        data_type::f32)))
+            return status::unimplemented;
+    }
     if ((brg->dt_a == data_type::bf16 && brg->dt_b == data_type::bf16)
-            && (!one_of(dt_d, data_type::bf16, data_type::f32))
-            && (!one_of(dt_bias, data_type::undef, data_type::bf16,
-                    data_type::f32)))
+            && ((!one_of(dt_d, data_type::bf16, data_type::f32))
+                    || (!one_of(dt_bias, data_type::undef, data_type::bf16,
+                            data_type::f32))))
         return status::unimplemented;
     if ((brg->dt_a == data_type::f32 && brg->dt_b == data_type::f32)
             && (!one_of(dt_d, data_type::f32))
-            && (!one_of(dt_bias, data_type::undef, data_type::f32)))
+            && (!one_of(dt_bias, data_type::undef, data_type::s8, data_type::u8,
+                    data_type::f32)))
         return status::unimplemented;
 
     brg->dt_d = dt_d;
     brg->typesize_D = types::data_type_size(brg->dt_d);
 
-    if (brg->is_int8 || (brg->dt_d == bf16)) return status::unimplemented;
-
     if (!brg->attr) return status::success;
 
     using namespace injector;
@@ -290,41 +292,52 @@ status_t brgemm_desc_set_postops(brgemm_t *brg, const primitive_attr_t *attr,
 
     const auto &src_scales = attr->scales_.get(DNNL_ARG_SRC);
     const auto &wei_scales = attr->scales_.get(DNNL_ARG_WEIGHTS);
-    brg->with_scales = !src_scales.has_default_values()
-            || !wei_scales.has_default_values()
+    const bool has_src_scales = !src_scales.has_default_values();
+    const bool has_wei_scales = !wei_scales.has_default_values();
+    brg->with_scales = has_src_scales || has_wei_scales
             || brg->with_weights_scale_adjust;
     if (brg->with_scales) {
         // Note. the current version supports only two different output scale
         // types:
-        //     1) common (mask_ = 0)
+        //     1) common (mask = 0)
         //     2) per_n_dim_scale - broadcast across n dimension;
         //        for convolution and inner product promitives it corresponds
-        //        to "per_oc" mask_ = 1 << 1; for matmul - to
-        //        mask_ = (1 << (ndims - 1))), where ndims is number of
+        //        to "per_oc" mask = 1 << 1; for matmul - to
+        //        mask = (1 << (ndims - 1))), where ndims is number of
         //        dimensions for original matmul problem
-        // So if wei_scales.mask_ != 0 (not common) it's assumed here that scale
-        // type is per_n_dim_scale and driver which calls brgemm kernel checked
-        // that mask has correct value for this case
-        brg->is_oc_scale = wei_scales.mask_ != 0;
+        // So if wei_scales.get_mask() > 0 (not common) it's assumed here that
+        // scale type is per_n_dim_scale and driver which calls brgemm kernel
+        // checked that mask has correct value for this case
+        brg->is_oc_scale = wei_scales.get_mask() > 0;
     }
 
     const auto &dst_scales = attr->scales_.get(DNNL_ARG_DST);
-    brg->with_dst_scales = !dst_scales.has_default_values();
-    const bool scales_ok = src_scales.mask_ == 0 && dst_scales.mask_ == 0
+    const bool has_dst_scales = !dst_scales.has_default_values();
+    brg->with_dst_scales = has_dst_scales;
+    const bool scales_ok
+            = IMPLICATION(has_src_scales, src_scales.get_mask() == 0)
+            && IMPLICATION(has_dst_scales, dst_scales.get_mask() == 0)
             && attr->scales_.has_default_values(
                     {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST});
     if (!scales_ok) return status::unimplemented;
 
     auto init_zp_type
             = [&](brgemm_broadcast_t &zp_type, int mem_arg) -> status_t {
-        auto zero_points = attr->zero_points_;
+        const auto &zp = attr->zero_points_;
+        // Always init a default value;
+        zp_type = brgemm_broadcast_t::none;
 
-        // common zero point type is supported for now
-        if (!zero_points.common(mem_arg)) return status::unimplemented;
+        if (!zp.has_default_values(mem_arg)) {
+            int mask = zp.get_mask(mem_arg);
+            if (mask == 0) {
+                zp_type = brgemm_broadcast_t::per_tensor;
+            } else if (mask == (1 << 1)) {
+                zp_type = brgemm_broadcast_t::per_n;
+            } else {
+                return status::unimplemented;
+            }
+        }
 
-        zp_type = zero_points.has_default_values(mem_arg)
-                ? brgemm_broadcast_t::none
-                : brgemm_broadcast_t::per_tensor;
         return status::success;
     };
 
@@ -333,8 +346,10 @@ status_t brgemm_desc_set_postops(brgemm_t *brg, const primitive_attr_t *attr,
     init_zp_type(brg->zp_type_c, DNNL_ARG_DST);
 
     // src zero points require additional register in brgemm kernel
-    if (brg->zp_type_a != brgemm_broadcast_t::none
-            || (brg->is_bf16_emu && !brg->is_dgmm))
+    const bool is_zp_src = brg->zp_type_a != brgemm_broadcast_t::none;
+    if (brg->is_dgmm) {
+        if (is_zp_src) CHECK(brdgmm_blocking(brg));
+    } else if (is_zp_src || brg->is_bf16_emu)
         CHECK(brgemm_blocking(brg));
 
     return status::success;
@@ -398,9 +413,6 @@ status_t brgemm_desc_set_attr(brgemm_t *brg, const brgemm_attr_t &brgattr) {
     if (!IMPLICATION(brg->is_blocked, brg->layout = brgemm_row_major))
         return status::invalid_arguments;
 
-    if (brgattr.max_top_vpad > 0 || brgattr.max_bottom_vpad > 0)
-        return status::unimplemented;
-
     brg->prfA = brgattr.hint_prfA;
     brg->prfB = brgattr.hint_prfB;
     brg->prfC = brgattr.hint_prfC;
@@ -417,6 +429,11 @@ status_t brgemm_desc_set_attr(brgemm_t *brg, const brgemm_attr_t &brgattr) {
     return status::success;
 }
 
+status_t brgemm_desc_finalize(brgemm_t *brg) {
+    // TODO: implement functionality here similar to corresponding one in x64
+    return status::success;
+}
+
 status_t brgemm_kernel_create(
         brgemm_kernel_t **brg_kernel, const brgemm_t &brg) {
     if (!brg_kernel) return status::invalid_arguments;
@@ -444,7 +461,7 @@ status_t brgemm_init_tiles(const brgemm_t &brg, char palette[64]) {
 
 namespace {
 template <typename T>
-static inline int sign(T v) {
+inline int sign(T v) {
     return (v > 0) ? 1 : ((v < 0) ? -1 : 0);
 }
 
@@ -513,11 +530,13 @@ int brgemm_cmp(const brgemm_t &lhs, const brgemm_t &rhs) {
     CMP_BRGEMM_FIELD(brgattr.hint_prfB.dist2);
     CMP_BRGEMM_FIELD(brgattr.hint_prfC.dist1);
     CMP_BRGEMM_FIELD(brgattr.hint_prfC.dist2);
-    CMP_BRGEMM_FIELD(brgattr.wary_tail_read);
+    CMP_BRGEMM_FIELD(brgattr.wary_A_k_tail_read);
+    CMP_BRGEMM_FIELD(brgattr.extendable_k);
     CMP_BRGEMM_FIELD(brgattr.generate_skip_accumulation);
     CMP_BRGEMM_FIELD(brgattr.bd_mask_level);
     CMP_BRGEMM_FIELD(brgattr.use_uker);
     CMP_BRGEMM_FIELD(brgattr.use_interleave_stores);
+    CMP_BRGEMM_FIELD(brgattr.b_is_vnni);
     CMP_BRGEMM_FIELD(brgattr.fpmath_mode);
     CMP_BRGEMM_FIELD(brgattr.LDA2);
     CMP_BRGEMM_FIELD(brgattr.LDB2);
