@@ -1,6 +1,7 @@
 /*******************************************************************************
 * Copyright 2021-2023 Intel Corporation
-* Copyright 2024 FUJITSU LIMITED
+* Copyright 2024-2025 FUJITSU LIMITED
+* Copyright 2024-2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,47 +31,27 @@
 #define GET_OFF(field) (uint32_t) offsetof(brgemm_kernel_params_t, field)
 #define GET_OFF_BATCH_ELEMENT(field) \
     (uint32_t) offsetof(brgemm_batch_element_t, field)
-#define LD_MUL_VL(mn, op, mask, addr, off, size) \
-    { \
-        const int mul_vl_len = (cpu_sveLen / 4) * size; \
-        const int off_mod = (off) % mul_vl_len; \
-        const int off_mul_vl = (off) / mul_vl_len; \
-        if (off_mod == 0 && -8 <= off_mul_vl && off_mul_vl <= 7) \
-            mn(op, mask / T_z, ptr(addr, off_mul_vl, MUL_VL)); \
-        else \
-            mn(op, mask / T_z, \
-                    ptr(addr_off(addr, off, X_DEFAULT_ADDR, X_TMP_0))); \
-    }
-#define ST_MUL_VL(mn, op, mask, addr, off, size) \
-    { \
-        const int mul_vl_len = (cpu_sveLen / 4) * size; \
-        const int off_mod = (off) % mul_vl_len; \
-        const int off_mul_vl = (off) / mul_vl_len; \
-        if (off_mod == 0 && -8 <= off_mul_vl && off_mul_vl <= 7) \
-            mn(op, mask, ptr(addr, off_mul_vl, MUL_VL)); \
-        else \
-            mn(op, mask, ptr(addr_off(addr, off, X_DEFAULT_ADDR, X_TMP_0))); \
-    }
+
 #define LDR_IMM(reg, addr, off) \
-    { \
+    do { \
         const uint64_t IMM12_MASK = ~uint64_t(0xfff); \
-        if ((off & IMM12_MASK) == 0) { \
+        if (((off)&IMM12_MASK) == 0) { \
             ldr(reg, ptr(addr, off)); \
         } else { \
             add_imm(X_DEFAULT_ADDR, addr, off, X_TMP_0); \
             ldr(reg, ptr(X_DEFAULT_ADDR)); \
         } \
-    }
+    } while (0)
 #define STR_IMM(reg, addr, off) \
-    { \
+    do { \
         const uint64_t IMM12_MASK = ~uint64_t(0xfff); \
-        if ((off & IMM12_MASK) == 0) { \
+        if (((off)&IMM12_MASK) == 0) { \
             str(reg, ptr(addr, off)); \
         } else { \
             add_imm(X_DEFAULT_ADDR, addr, off, X_TMP_0); \
             str(reg, ptr(X_DEFAULT_ADDR)); \
         } \
-    }
+    } while (0)
 
 using namespace Xbyak_aarch64;
 
@@ -81,9 +62,9 @@ namespace aarch64 {
 
 using namespace dnnl::impl::utils;
 
-struct jit_brgemm_kernel_t : public jit_generator {
+struct jit_brgemm_kernel_t : public jit_generator_t {
     jit_brgemm_kernel_t(const brgemm_t &abrg)
-        : jit_generator(nullptr, MAX_CODE_SIZE, true, sve_512)
+        : jit_generator_t(nullptr, MAX_CODE_SIZE, true, sve_512)
         , brg(abrg)
         , postops_injector_(nullptr)
         , max_effective_vregs(
@@ -241,7 +222,7 @@ private:
     PReg ld_full_mask = PReg(2);
     PReg ld_tail_mask = PReg(3);
 
-    ZReg accm(int ld_block, int bd, int ld) {
+    ZReg accm(int ld_block, int bd, int ld) const {
         return ZReg(max_effective_vregs - 1 - (bd * ld_block + ld));
     }
 
@@ -312,7 +293,7 @@ private:
             int bd_block, int ld_block2, bool is_ld_tail, int vpad);
 
     void dot_product(ZReg z1, ZReg z2, ZReg z3);
-    void gemm_microkernel_sve512(int bd_block2, bool is_bdb_tail, int ld_block,
+    void gemm_microkernel(int bd_block2, bool is_bdb_tail, int ld_block,
             bool is_rd_tail, bool is_ld_tail, int vpad, int rows_for_rd_tail);
 
     void ldb_loop(int bd_block2, bool is_bdb_tail, int ld_block,
@@ -502,18 +483,36 @@ void jit_brgemm_kernel_t::cvt2ps(data_type_t type_in, const ZReg zmm_in,
     const auto mask = mask_flag ? ktail_mask : P_ALL_ONE;
     switch (type_in) {
         case data_type::f32:
-        case data_type::s32:
+        case data_type::s32: {
             LD_MUL_VL(ld1w, z_tmp_1().s, mask, addr, offset - base_offset, 4);
             if (store) //Merging
                 mov(zmm_in.s, ktail_mask / T_m, z_tmp_1().s);
             break;
-        case data_type::bf16: assert(!"unsupported data type\n"); break;
-        case data_type::s8: assert(!"unsupported data type\n"); break;
-        case data_type::u8: assert(!"unsupported data type\n"); break;
+        }
+        case data_type::bf16: {
+            add_imm(X_TMP_1, addr, offset - base_offset, X_TMP_0);
+            ld1h(z_tmp_1().s, mask, ptr(X_TMP_1));
+            lsl(z_tmp_1().s, z_tmp_1().s, 16);
+            if (store) //Merging
+                mov(zmm_in.s, ktail_mask / T_m, z_tmp_1().s);
+            break;
+        }
+        case data_type::s8:
+            LD_MUL_VL(ld1sb, z_tmp_1().s, mask, addr, offset - base_offset, 1);
+            if (store) // Merging
+                mov(zmm_in.s, ktail_mask / T_m, z_tmp_1().s);
+            break;
+        case data_type::u8: {
+            LD_MUL_VL(ld1b, z_tmp_1().s, mask, addr, offset - base_offset, 1);
+            if (store) // Merging
+                mov(zmm_in.s, ktail_mask / T_m, z_tmp_1().s);
+            break;
+        }
         default: assert(!"unsupported data type");
     }
-    if (!one_of(type_in, data_type::f32, data_type::bf16))
-        assert(!"unsupported data type\n");
+    if (types::is_integral_dt(type_in)) {
+        scvtf(zmm_in.s, P_ALL_ONE / T_m, zmm_in.s);
+    }
 }
 
 void jit_brgemm_kernel_t::advance_ldb_post_op_regs() {
@@ -766,10 +765,38 @@ void jit_brgemm_kernel_t::read_params() {
 void jit_brgemm_kernel_t::zero_accumulators(int bd_block2, bool is_bdb_tail,
         int ld_block2, bool is_ld_tail, bool skip_accumulation) {
     int bd_block = (is_bdb_tail) ? brg.bdb_tail : brg.bd_block;
+    const bool need_to_apply_beta = brg.beta != 0.f;
     for_(int bd = 0; bd < bd_block; bd++)
     for (int ld = 0; ld < ld_block2; ld++) {
         auto zmm = accm(ld_block2, bd, ld);
-        eor(zmm.d, zmm.d, zmm.d);
+        // This part is moved here from apply_alpha_beta function so that fadd instruction can be avoided.
+        // This is also required only when K is blocked.
+        if (need_to_apply_beta) {
+            const bool is_tail = is_ld_tail && ld + 1 == ld_block2;
+            const auto k_mask = is_tail ? ld_tail_mask : ld_full_mask;
+
+            const int offset = C_offset(bd, ld);
+
+            int base_offset = 0;
+            auto x_addr = reg_aux_C;
+
+            if ((unsigned)(offset - base_offset) > cpu_sveLen * 7) {
+                add_imm(reg_tmp_, reg_aux_C, offset, X_TMP_0);
+                base_offset = offset;
+                x_addr = reg_tmp_;
+            }
+            LD_MUL_VL(ld1w, zmm.s, k_mask, x_addr, offset - base_offset, 4);
+
+            const bool need_init_beta_vmm = brg.beta != 1.f;
+            auto vmm_beta = z_tail_mask();
+            if (need_init_beta_vmm) {
+                auto wreg_tmp = WReg(reg_tmp_gpr.getIdx());
+                mov_imm(wreg_tmp, float2int(static_cast<float>(brg.beta)));
+                dup(vmm_beta.s, wreg_tmp);
+                fmul(zmm.s, zmm.s, vmm_beta.s);
+            }
+        } else
+            eor(zmm.d, zmm.d, zmm.d);
     }
 }
 
@@ -790,58 +817,6 @@ void jit_brgemm_kernel_t::apply_alpha_beta(
         if (dq2ps_required) { scvtf(vmm.s, P_ALL_ONE / T_m, vmm.s); }
         if (apply_alpha) { fmul(vmm.s, vmm.s, vmm_alpha.s); }
     }
-
-    if (brg.beta == 0.f) return;
-    const bool use_vadd_for_beta = brg.beta == 1.f && !dq2ps_required;
-    const bool need_init_beta_vmm = brg.beta != 1.f;
-    auto vmm_prev_dst = z_tmp_1();
-    auto vmm_beta = z_tail_mask();
-    if (need_init_beta_vmm) {
-        auto wreg_tmp = WReg(reg_tmp_gpr.getIdx());
-        mov_imm(wreg_tmp, float2int(static_cast<float>(brg.beta)));
-        dup(vmm_beta.s, wreg_tmp);
-    }
-
-    int base_offset = 0;
-    auto x_addr = reg_aux_C;
-    for_(int bd = 0; bd < bd_block; bd++)
-    for (int ld = 0; ld < ld_block2; ld++) {
-        const bool is_tail = is_ld_tail && ld + 1 == ld_block2;
-        const auto k_mask = is_tail ? ld_tail_mask : ld_full_mask;
-        auto vmm = accm(ld_block2, bd, ld);
-        if (use_vadd_for_beta) {
-            if (brg.is_int8) {
-                assert(!"unsupported\n");
-            } else {
-                ZRegS z_masked = vmm.s;
-                ZRegS z(vmm.getIdx());
-
-                const int offset = C_offset(bd, ld);
-
-                if ((unsigned)(offset - base_offset) > cpu_sveLen * 7) {
-                    add_imm(reg_tmp_, reg_aux_C, offset, X_TMP_0);
-                    base_offset = offset;
-                    x_addr = reg_tmp_;
-                }
-                LD_MUL_VL(ld1w, vmm_prev_dst.s, k_mask, x_addr,
-                        offset - base_offset, 4);
-                if (is_ld_tail) {
-                    movprfx(z_masked, k_mask / T_z, z);
-                    fadd(z_masked, k_mask / T_m, vmm_prev_dst.s);
-                } else {
-                    fadd(z_masked, z_masked, vmm_prev_dst.s);
-                }
-            }
-        } else {
-            add_imm(X_DEFAULT_ADDR, reg_aux_C, C_offset(bd, ld), X_TMP_0);
-            ld1w(vmm_prev_dst.s, k_mask / T_z, ptr(X_DEFAULT_ADDR));
-            if (brg.beta == 1.f) {
-                fadd(vmm.s, vmm.s, vmm_prev_dst.s);
-            } else {
-                fmla(vmm.s, P_ALL_ONE / T_m, vmm_prev_dst.s, vmm_beta.s);
-            }
-        }
-    }
 }
 
 void jit_brgemm_kernel_t::apply_post_ops(
@@ -883,18 +858,20 @@ void jit_brgemm_kernel_t::apply_post_ops(
                     register_guard_sum_zp(
                             p_sum_zp_reg_set, this, {reg_ptr_sum_zp});
 
-            const auto &vmm_sum_zp = z_tmp_2();
+            const auto &vmm_sum_scale = z_tmp_2();
+            const auto &vmm_sum_zp = z_tmp_3();
 
             if (p_sum_zp_reg_set) {
                 mov_imm(reg_ptr_sum_zp, reinterpret_cast<size_t>(p_sum_zp));
-                ld1rw(z_tmp_3().s, P_ALL_ONE / T_z, ptr(reg_ptr_sum_zp));
-                scvtf(vmm_sum_zp.s, P_ALL_ONE / T_m, z_tmp_3().s);
+                ld1rw(vmm_sum_zp.s, P_ALL_ONE / T_z, ptr(reg_ptr_sum_zp));
+                scvtf(vmm_sum_zp.s, P_ALL_ONE / T_m, vmm_sum_zp.s);
             }
 
             if (p_sum_scale_reg_set) {
                 // embd bcast fma
                 mov_imm(reg_ptr_sum_scale,
                         reinterpret_cast<size_t>(p_sum_scale));
+                ld1rw(vmm_sum_scale.s, P_ALL_ONE / T_z, ptr(reg_ptr_sum_scale));
             }
 
             for_(int bd = 0; bd < bd_block; bd++)
@@ -904,13 +881,13 @@ void jit_brgemm_kernel_t::apply_post_ops(
                 const bool is_tail = is_ld_tail && ld + 1 == ld_block2;
                 const auto k_mask = is_tail ? ld_tail_mask : ld_full_mask;
                 add_imm(X_DEFAULT_ADDR, reg_aux_D, D_offset(bd, ld), X_TMP_0);
-                ld1w(vmm_prev_dst.s, k_mask / T_z, ptr(X_DEFAULT_ADDR));
+                cvt2ps(brg.sum_dt, vmm_prev_dst, X_DEFAULT_ADDR, is_tail, false,
+                        k_mask, 0, 0);
                 if (p_sum_zp_reg_set)
                     fsub(vmm_prev_dst.s, vmm_prev_dst.s, vmm_sum_zp.s);
                 if (p_sum_scale_reg_set) {
-                    const auto ztmp2 = z_tmp_2();
-                    ld1rw(ztmp2.s, P_ALL_ONE / T_z, ptr(reg_ptr_sum_scale));
-                    fmla(vmm.s, P_ALL_ONE / T_m, vmm_prev_dst.s, ztmp2.s);
+                    fmla(vmm.s, P_ALL_ONE / T_m, vmm_prev_dst.s,
+                            vmm_sum_scale.s);
                 } else
                     fadd(vmm.s, vmm.s, vmm_prev_dst.s);
             }
@@ -928,7 +905,7 @@ void jit_brgemm_kernel_t::apply_post_ops(
 }
 
 static inline bool isa_has_masks(cpu_isa_t isa) {
-    return is_superset(isa, sve_512);
+    return is_superset(isa, sve_128);
 }
 
 void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
@@ -943,6 +920,9 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
             = brg.beta == 1.f && IMPLICATION(brg.is_int8, brg.alpha == 1.0f);
     const bool dq2ps_required = brg.is_int8
             && IMPLICATION(alpha_or_beta_applicable, beta_uses_vadd);
+    // This flag tracks whether the conversion has happened, since it must be
+    // done only once despite all scales and bias applications requiring it.
+    bool dq2ps_cvt_done = false;
 
     if (brg.with_scales) {
         add_imm(X_DEFAULT_ADDR, X_SP, reg_aux_scales_offs_, X_TMP_0);
@@ -959,10 +939,13 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
             }
             for (int bd = 0; bd < bd_block; bd++) {
                 auto vmm = accm(ld_block2, bd, ld);
-                if (dq2ps_required) { scvtf(vmm.s, P_ALL_ONE / T_m, vmm.s); }
+                if (dq2ps_required && !dq2ps_cvt_done) {
+                    scvtf(vmm.s, P_ALL_ONE / T_m, vmm.s);
+                }
                 fmul(vmm.s, vmm.s, vmm_scales.s);
             }
         }
+        dq2ps_cvt_done = true;
     }
 
     if (brg.with_bias) { LDR_IMM(reg_aux_bias, X_SP, reg_aux_bias_offs_); }
@@ -979,12 +962,15 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
                 base_offset = offset;
                 x_addr = reg_tmp_;
             }
-            cvt2ps(brg.dt_bias, zmm_bias, x_addr, true, false, k_mask, offset,
-                    base_offset);
+            cvt2ps(brg.dt_bias, zmm_bias, x_addr, is_ld_tail, false, k_mask,
+                    offset, base_offset);
         }
+
         for (int bd = 0; bd < bd_block; bd++) {
             auto zmm = accm(ld_block2, bd, ld);
-            if (dq2ps_required) { scvtf(zmm.s, P_ALL_ONE / T_m, zmm.s); }
+            if (dq2ps_required && !dq2ps_cvt_done) {
+                scvtf(zmm.s, P_ALL_ONE / T_m, zmm.s);
+            }
             if (brg.with_bias) { fadd(zmm.s, zmm.s, zmm_bias.s); }
         }
     }
@@ -1012,7 +998,7 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
         auto vmm_zp_c = z_tmp_1();
         if (brg.zp_type_c == brgemm_broadcast_t::per_tensor) {
             add_imm(X_DEFAULT_ADDR, reg_aux_zp_c_values, 0, X_TMP_0);
-            ldr(z_tmp_2(), ptr(X_DEFAULT_ADDR));
+            ld1rw(z_tmp_2().s, P_ALL_ONE, ptr(X_DEFAULT_ADDR));
             scvtf(vmm_zp_c.s, k_mask / T_m, z_tmp_2().s);
         }
         for (int ld = 0; ld < ld_block2; ld++) {
@@ -1033,7 +1019,20 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
 
     const bool dt_requires_saturation
             = one_of(brg.dt_d, data_type::u8, data_type::s8, data_type::s32);
-    if (dt_requires_saturation) { assert(!"unsupported\n"); }
+    auto zmm_lbound = z_tmp_1();
+    auto zmm_ubound = z_tmp_2();
+    if (dt_requires_saturation) {
+        init_saturate_f32(
+                zmm_lbound, zmm_ubound, reg_tmp_gpr, data_type::f32, brg.dt_d);
+        for (int bd = 0; bd < bd_block; bd++) {
+            for (int ld = 0; ld < ld_block2; ld++) {
+                auto zmm = accm(ld_block2, bd, ld);
+                saturate_f32(zmm, zmm_lbound, zmm_ubound, brg.dt_d, k_mask);
+                frinti(zmm.s, k_mask, zmm.s);
+                fcvtzs(zmm.s, k_mask, zmm.s);
+            }
+        }
+    }
 
     x_addr = reg_aux_D;
     base_offset = 0;
@@ -1053,9 +1052,24 @@ void jit_brgemm_kernel_t::store_accumulators_apply_post_ops(
                     ST_MUL_VL(st1w, zmm.s, k_mask, x_addr, offset - base_offset,
                             4);
                     break;
-                case data_type::bf16: assert(!"unsupported\n"); break;
-                case data_type::s8: assert(!"unsupported\n"); break;
-                case data_type::u8: assert(!"unsupported\n"); break;
+                case data_type::bf16: {
+                    bfcvt(zmm.h, k_mask / T_m, zmm.s);
+                    st1h(zmm.s, k_mask,
+                            ptr(addr_off(x_addr, offset - base_offset,
+                                    X_DEFAULT_ADDR, X_TMP_0)));
+                    break;
+                }
+                case data_type::s8:
+                    smin(zmm.s, std::numeric_limits<int8_t>::max());
+                    smax(zmm.s, std::numeric_limits<int8_t>::min());
+                    ST_MUL_VL(st1b, zmm.s, k_mask, x_addr, offset - base_offset,
+                            1);
+                    break;
+                case data_type::u8:
+                    umin(zmm.s, std::numeric_limits<uint8_t>::max());
+                    ST_MUL_VL(st1b, zmm.s, k_mask, x_addr, offset - base_offset,
+                            1);
+                    break;
                 default: assert(!"unknown dst_dt");
             }
         }
@@ -1071,7 +1085,7 @@ void jit_brgemm_kernel_t::apply_compensation(
     if (!brg.req_cal_comp_pads && brg.zp_type_a != brgemm_broadcast_t::none) {
         auto vmm_zp_a_val = z_tmp_2();
         add_imm(X_DEFAULT_ADDR, X_SP, reg_zp_a_val_offs_, X_TMP_0);
-        ldr(reg_zp_a_val, ptr(X_DEFAULT_ADDR));
+        add_imm(reg_zp_a_val, X_SP, reg_zp_a_val_offs_, X_TMP_0);
         ldr(W_TMP_0, ptr(reg_zp_a_val));
         dup(vmm_zp_a_val.s, W_TMP_0);
 
@@ -1119,13 +1133,15 @@ void jit_brgemm_kernel_t::apply_compensation(
             const bool is_tail = is_ld_tail && ld + 1 == ld_block2;
             if (IMPLICATION(is_tail, is_superset(brg.isa_impl, sve_512))) {
                 const auto mask = is_tail ? k_mask : P_ALL_ONE;
-                ld1w(vmm_comp.s, mask / T_z,
-                        ptr(reg_aux_compensation, comp_offset));
+                add_imm(X_DEFAULT_ADDR, reg_aux_compensation, comp_offset,
+                        X_TMP_1);
+                ld1w(vmm_comp.s, mask / T_z, ptr(X_DEFAULT_ADDR));
             } else {
                 not_(P_TMP.b, P_ALL_ONE, P_NOT_256.b);
                 cmplt(P_TMP.s, P_TMP / T_z, z_tail_mask().s, 0);
-                ld1w(vmm_comp.s, P_TMP / T_z,
-                        ptr(reg_aux_compensation, comp_offset));
+                add_imm(X_DEFAULT_ADDR, reg_aux_compensation, comp_offset,
+                        X_TMP_1);
+                ld1w(vmm_comp.s, P_TMP / T_z, ptr(X_DEFAULT_ADDR));
             }
 
             for (int bd = 0; bd < bd_block; bd++) {
@@ -1146,7 +1162,22 @@ void jit_brgemm_kernel_t::store_accumulators_without_post_ops(
             = brg.beta == 1.f && IMPLICATION(brg.is_int8, brg.alpha == 1.0f);
     const bool dt_requires_saturation = brg.is_int8
             && !IMPLICATION(alpha_or_beta_applicable, beta_uses_vadd);
-    if (dt_requires_saturation) { assert(!"unsupported\n"); }
+    auto zmm_lbound = z_tmp_1();
+    auto zmm_ubound = z_tmp_2();
+    assert(zmm_lbound.getIdx() != zmm_ubound.getIdx());
+    if (dt_requires_saturation) {
+        init_saturate_f32(
+                zmm_lbound, zmm_ubound, reg_tmp_gpr, data_type::f32, brg.dt_d);
+        for (int bd = 0; bd < bd_block; bd++) {
+            for (int ld = 0; ld < ld_block2; ld++) {
+                auto zmm = accm(ld_block2, bd, ld);
+                saturate_f32(
+                        zmm, zmm_lbound, zmm_ubound, brg.dt_d, ld_full_mask);
+                frinti(zmm.s, ld_full_mask, zmm.s);
+                fcvtzs(zmm.s, ld_full_mask, zmm.s);
+            }
+        }
+    }
     auto x_addr = reg_aux_C;
     int base_offset = 0;
 
@@ -1172,12 +1203,18 @@ void jit_brgemm_kernel_t::store_accumulators(int bd_block2, bool is_bdb_tail,
             brg.zp_type_a, brg.zp_type_b, brg.zp_type_c);
     const bool are_post_ops_applicable = one_of(true, brg.with_eltwise,
             brg.with_binary, brg.with_scales, brg.with_bias, brg.with_sum,
-            brg.dt_d != brg.dt_c, brg.req_s8s8_compensation, has_zero_points);
+            brg.dt_d != brg.dt_c, brg.with_dst_scales,
+            brg.req_s8s8_compensation, has_zero_points);
     const bool need_to_apply_alpha_beta = brg.beta != 0.f || brg.alpha != 1.f;
     int bd_block = (is_bdb_tail) ? brg.bdb_tail : brg.bd_block;
 
     if (brg.is_int8 && (brg.req_s8s8_compensation || has_zero_points)) {
-        assert(!"unsupported\n");
+        Label label_store_without_comp;
+        ldr(reg_do_comp, ptr(X_SP, reg_do_comp_offs_));
+        cmp_imm(reg_do_comp, 0, X_TMP_0);
+        b(EQ, label_store_without_comp);
+        apply_compensation(bd_block, ld_block2, is_ld_tail);
+        L_aligned(label_store_without_comp);
     }
 
     if (need_to_apply_alpha_beta)
@@ -1277,14 +1314,25 @@ void jit_brgemm_kernel_t::set_A_B_matrices() {
     add(reg_aux_B, reg_aux_B, reg_b_offset);
 }
 
-void jit_brgemm_kernel_t::dot_product(ZReg v1, ZReg v2, ZReg v3) {
+void jit_brgemm_kernel_t::dot_product(ZReg v_acc, ZReg v_b, ZReg v_a) {
     if (brg.is_f32) {
-        fmla(v1.s, P_ALL_ONE / T_m, v2.s, v3.s);
+        fmla(v_acc.s, P_ALL_ONE / T_m, v_a.s, v_b.s);
     } else if (brg.is_bf16)
-        assert(!"unsupported\n");
-    else if (brg.is_int8)
-        assert(!"unsupported\n");
-    else
+        bfdot(v_acc.s, v_b.h, v_a.h);
+    else if (brg.is_int8 && isa_has_s8s8(brg.isa_impl)) {
+        // SDOT/USDOT/UDOT implicitly produce int32 output.
+        // we reorder RHS to align for SDOT lane-wise ops.
+        if (brg.dt_a == data_type::u8 && brg.dt_b == data_type::u8)
+            udot(v_acc.s, v_a.b, v_b.b);
+        else if (brg.dt_a == data_type::s8 && brg.dt_b == data_type::s8)
+            sdot(v_acc.s, v_a.b, v_b.b);
+        else if (brg.dt_a == data_type::u8 && brg.dt_b == data_type::s8)
+            usdot(v_acc.s, v_a.b, v_b.b);
+        // TODO: Add support for s8u8 once zero point handling can be properly tested.
+        // Currently excluded as we are unable to test zero point compensation.
+        else if (brg.dt_a == data_type::s8 && brg.dt_b == data_type::u8)
+            assert(!"unsupported\n");
+    } else
         assert(!"unsupported\n");
 }
 
@@ -1317,7 +1365,7 @@ void jit_brgemm_kernel_t::compute_int8_compensation(int rd_loop, int bd_b,
         if (brg.zp_type_a != brgemm_broadcast_t::none) {
             eor(vmm_tmp.d, vmm_tmp.d, vmm_tmp.d);
             dot_product(vmm_tmp, vmm_load, z_one_bytes());
-            mul(vmm_tmp.s, P_ALL_ONE / T_m, z_inp_shift().s);
+            mul(vmm_tmp.s, P_ALL_ONE / T_m, z_zp_a_shift().s);
 
             for (int bd = bd_b; bd < bd_e; bd++) {
                 auto vmm = accm(ld_block2, bd, ld);
@@ -1344,7 +1392,8 @@ void jit_brgemm_kernel_t::compute_int8_compensation(int rd_loop, int bd_b,
     for (int ld = 0; ld < ld_block2; ++ld) {
         const bool is_tail = is_ld_tail && ld + 1 == ld_block2;
         const auto mask = is_tail ? ld_tail_mask : P_ALL_ONE;
-        ld1w(load().s, mask / T_z, ptr(reg_aux_B, B_offset(ld, rd)));
+        add_imm(X_DEFAULT_ADDR, reg_aux_B, B_offset(ld, rd), X_TMP_0);
+        ld1w(load().s, mask / T_z, ptr(X_DEFAULT_ADDR));
 
         if (brg.req_cal_comp_pads) {
             compensation_padding(load(), bcst(), ld, bd_b, bd_e);
@@ -1356,9 +1405,9 @@ void jit_brgemm_kernel_t::compute_int8_compensation(int rd_loop, int bd_b,
     }
 }
 
-void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
-        bool is_bdb_tail, int ld_block2, bool is_rd_tail, bool is_ld_tail,
-        int vpad, int rows_for_rd_tail) {
+void jit_brgemm_kernel_t::gemm_microkernel(int bd_block2, bool is_bdb_tail,
+        int ld_block2, bool is_rd_tail, bool is_ld_tail, int vpad,
+        int rows_for_rd_tail) {
     MAYBE_UNUSED(bd_block2);
     int bd_block = (is_bdb_tail) ? brg.bdb_tail : brg.bd_block;
     const auto bd_b = nstl::max(0, vpad);
@@ -1371,8 +1420,11 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
 
     int rd_loop = 0, rd_tail_size = 0;
     if (is_rd_tail) {
+        rd_tail_size = brg.rdb_tail % brg.rd_step;
         if (brg.is_bf16 || brg.is_int8) {
-            assert(!"unsupported\n");
+            rd_loop = (rd_tail_size != 0)
+                    ? ((brg.rdb_tail / brg.rd_step) + 1) * brg.rd_step
+                    : brg.rdb_tail;
         } else
             rd_loop = brg.rdb_tail;
     } else
@@ -1383,10 +1435,16 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
         if (is_tail) {
             eor(z1.d, z1.d, z1.d);
             auto xmm_tmp = z_tmp_1();
-            add_imm(X_DEFAULT_ADDR, reg_aux_A, rd_tail_size * brg.typesize_A,
-                    X_TMP_0);
-            set_preg(P_TMP.b, offset);
-            ld1b(xmm_tmp.b, P_TMP / T_z, ptr(X_DEFAULT_ADDR));
+            add_imm(X_DEFAULT_ADDR, reg_aux_A, offset, X_TMP_0);
+            if (brg.is_int8) {
+                set_preg(P_TMP.b, rd_tail_size, X_TMP_0, X_TMP_1);
+                ld1b(xmm_tmp.b, P_TMP / T_z, ptr(X_DEFAULT_ADDR));
+            } else if (brg.is_bf16) {
+                set_preg(P_TMP.h, rd_tail_size, X_DEFAULT_ADDR, X_TMP_1);
+                ld1h(xmm_tmp.h, P_TMP / T_z, ptr(X_DEFAULT_ADDR));
+            } else if (brg.is_f16) {
+                assert(!"unsupported\n");
+            }
             dup(z1.s, xmm_tmp.s[0]);
         } else {
             if (dt == data_type::f32) {
@@ -1397,10 +1455,10 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
                     add_imm(X_DEFAULT_ADDR, reg_aux_A, offset, X_TMP_0);
                     ld1rw(z1.s, P_ALL_ONE / T_z, ptr(X_DEFAULT_ADDR));
                 }
-            } else if (dt == data_type::bf16) {
-                assert(!"unsupported\n");
-            } else if (one_of(dt, data_type::s8, data_type::u8)) {
-                assert(!"unsupported\n");
+            } else if (one_of(dt, data_type::s8, data_type::u8,
+                               data_type::bf16)) {
+                add_imm(X_DEFAULT_ADDR, reg_aux_A, offset, X_TMP_0);
+                ld1rw(z1.s, P_ALL_ONE / T_z, ptr(X_DEFAULT_ADDR));
             } else if (dt == data_type::f16) {
                 assert(!"unsupported\n");
             }
@@ -1412,9 +1470,12 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
     const bool comp_vpad = vpad != 0
             && (brg.req_s8s8_compensation
                     || brg.zp_type_a != brgemm_broadcast_t::none);
-    if (brg.req_cal_comp_pads || comp_vpad) assert(!"unsupported\n");
+    if (brg.req_cal_comp_pads || comp_vpad)
+        compute_int8_compensation(
+                rd_loop, bd_b, bd_e, bd_block, ld_block2, is_ld_tail, vpad);
 
-    bool maybe_load_bytes = (rows_for_rd_tail > 0 || brg.brgattr.wary_tail_read)
+    bool maybe_load_bytes
+            = (rows_for_rd_tail > 0 || brg.brgattr.wary_A_k_tail_read)
             && is_rd_tail && rd_tail_size != 0 && (brg.is_bf16 || brg.is_int8);
     if (n_bcast_1_load) {
         for (int rd = 0; rd < rd_loop; rd += brg.rd_step) {
@@ -1424,22 +1485,19 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
             auto rows_by_load_bytes = have_to_load_bytes ? rows_for_rd_tail : 0;
             for (int bd = bd_b; bd < bd_e && !is_emdbd; bd++) {
                 const auto bd_by_load_bytes = (bd >= bd_e - rows_by_load_bytes
-                        || brg.brgattr.wary_tail_read);
+                        || brg.brgattr.wary_A_k_tail_read);
                 broadcast(bcst(bd), A_offset(bd, rd),
                         have_to_load_bytes && bd_by_load_bytes, brg.dt_a);
             }
             for (int ld = 0; ld < ld_block2; ld++) {
-                const auto addr = ptr(reg_aux_B, B_offset(ld, rd));
                 const auto mask = is_ld_tail ? ld_tail_mask : P_ALL_ONE;
+                add_imm(X_DEFAULT_ADDR, reg_aux_B, B_offset(ld, rd), X_TMP_0);
                 if (brg.dt_b == data_type::f16) {
                     assert(!"unsupported\n");
-                } else if (brg.dt_b == data_type::bf16
-                        && brg.isa_impl == sve_256) {
-                    assert(!"unsupported\n");
                 } else if (is_ld_tail) {
-                    ld1w(load().s, ld_tail_mask / T_z, addr);
+                    ld1w(load().s, ld_tail_mask / T_z, ptr(X_DEFAULT_ADDR));
                 } else {
-                    ld1w(load().s, P_ALL_ONE / T_z, addr);
+                    ld1w(load().s, P_ALL_ONE / T_z, ptr(X_DEFAULT_ADDR));
                 }
                 for (int bd = bd_b; bd < bd_e; bd++) {
                     auto vmm = accm(ld_block2, bd, ld);
@@ -1464,14 +1522,10 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
         int base_offset = 0;
 
         for (int rd = 0; rd < rd_loop; rd += brg.rd_step) {
-            int prefetch_count_B = 0;
             for (int ld = 0; ld < ld_block2; ld++) {
                 const auto mask = is_ld_tail ? ld_tail_mask : P_ALL_ONE;
                 if (brg.dt_b == data_type::f16) {
                     assert(!"unsupported\n");
-                } else if (brg.dt_b == data_type::bf16
-                        && brg.isa_impl == sve_256) {
-                    assert(!"unsupported\n");
                 } else {
                     const int offset = B_offset(ld, rd);
                     if ((unsigned)(offset - base_offset) > cpu_sveLen * 7) {
@@ -1492,17 +1546,11 @@ void jit_brgemm_kernel_t::gemm_microkernel_sve512(int bd_block2,
                 if (!is_emdbd) {
                     const auto bd_by_load_bytes
                             = (bd >= bd_e - rows_by_load_bytes
-                                    || brg.brgattr.wary_tail_read);
+                                    || brg.brgattr.wary_A_k_tail_read);
                     broadcast(bcst(), A_offset(bd, rd),
-                            have_to_load_bytes && bd_by_load_bytes, brg.dt_a);
-                }
-                if (prefetch_count_B < ld_block2) {
-                    add_imm(X_DEFAULT_ADDR, reg_aux_B,
-                            B_offset(prefetch_count_B++, rd)
-                                    + brg.LDB * brg.rd_block * brg.typesize_B,
-                            X_TMP_0);
-                    prfm(PLDL1KEEP, ptr(X_DEFAULT_ADDR));
+                            (have_to_load_bytes && bd_by_load_bytes), brg.dt_a);
                 }
+                //The current implementaion of prefetch is not giving any gain in performance but is rather introducing some latency. Therefore it is removed util a new useful implementation is deviced.
                 for (int ld = 0; ld < ld_block2; ld++) {
                     auto zmm = accm(ld_block2, bd, ld);
                     if (is_emdbd) {
@@ -1551,8 +1599,8 @@ void jit_brgemm_kernel_t::ldb_loop(int bd_block2, bool is_bdb_tail,
             L_aligned(rdb_loop_label, 64);
             {
                 const bool is_rd_tail = false;
-                gemm_microkernel_sve512(bd_block2, is_bdb_tail, ld_block2,
-                        is_rd_tail, is_ld_tail, vpad, rows_for_rd_tail);
+                gemm_microkernel(bd_block2, is_bdb_tail, ld_block2, is_rd_tail,
+                        is_ld_tail, vpad, rows_for_rd_tail);
 
                 add_imm(reg_aux_A, reg_aux_A, rdb_A_offset(), X_TMP_0);
                 add_imm(reg_aux_B, reg_aux_B, rdb_B_offset(), X_TMP_0);
@@ -1565,8 +1613,8 @@ void jit_brgemm_kernel_t::ldb_loop(int bd_block2, bool is_bdb_tail,
         if (brg.rdb_tail != 0) {
             const bool is_rd_tail = true;
 
-            gemm_microkernel_sve512(bd_block2, is_bdb_tail, ld_block2,
-                    is_rd_tail, is_ld_tail, vpad, rows_for_rd_tail);
+            gemm_microkernel(bd_block2, is_bdb_tail, ld_block2, is_rd_tail,
+                    is_ld_tail, vpad, rows_for_rd_tail);
         }
     };
     if (is_ldb_loop_) { mov_imm(reg_ldb_loop, ldb_loop_length); }
@@ -1590,7 +1638,12 @@ void jit_brgemm_kernel_t::ldb_loop(int bd_block2, bool is_bdb_tail,
 
             if (brg.req_s8s8_compensation) { assert(!"unsupported\n"); }
             if (need_comp_pads && brg.zp_type_a != brgemm_broadcast_t::none) {
-                assert(!"unsupported\n");
+                str(reg_bdb_loop, ptr(X_SP, reg_bdb_loop_offs_));
+                const auto reg32_scratch = WReg(reg_zp_a_input_shift.getIdx());
+                mov(z_one_bytes().b, 1);
+                ldr(reg32_scratch, ptr(X_SP, reg_zp_a_val_offs_));
+                dup(z_zp_a_shift().s, reg32_scratch);
+                ldr(reg_bdb_loop, ptr(X_SP, reg_bdb_loop_offs_));
             }
 
             if (brg.brgattr.max_bs > 1) { mov(reg_BS_loop, reg_BS); }
@@ -1876,7 +1929,21 @@ void jit_brgemm_kernel_t::bdb_loop() {
 }
 
 void jit_brgemm_kernel_t::generate() {
+
+    if (!one_of(brg.isa_impl, sve_512, sve_256, sve_128)) {
+        assert(!"unsupported isa: jit_brgemm_kernel_t only supports SVE 512, "
+                "256 and 128, this should have been checked earlier in the "
+                "implementation");
+    }
+
+    size_t simd_w_ = simd_elems(data_type::f32, brg.isa_impl);
+
     preamble();
+    if (simd_w_ != cpu_sveLen / sizeof(float)) {
+        set_preg(P_ALL_ONE.b, simd_w_ * 4, X_TMP_0, X_TMP_1);
+        set_preg(ld_full_mask.b, simd_w_ * 4, X_TMP_0, X_TMP_1);
+    } else
+        ptrue(ld_full_mask.b);
 
     mov(x7, x0);
     mov(x6, x1);
@@ -1896,7 +1963,6 @@ void jit_brgemm_kernel_t::generate() {
                              brg.req_s8s8_compensation)
             && IMPLICATION(!vpad_exist, brg.req_cal_comp_pads);
 
-    ptrue(ld_full_mask.b);
     set_preg(ld_tail_mask.s, brg.ldb_tail, X_TMP_0, X_TMP_1);
     if (brg.is_int8 && !brg.has_int8_vnni) { assert(!"unsupported\n"); }
 
@@ -1921,7 +1987,8 @@ brgemm_attr_t::brgemm_attr_t()
     , hint_innermost_loop(brgemm_ld_loop_innermost)
     , hint_loop_order(brgemm_kernel_loop_order_t::brgemm_lo_default)
     , hint_prefetching(brgemm_kernel_prefetching_t::brgemm_prf_default)
-    , wary_tail_read(true)
+    , wary_A_k_tail_read(true)
+    , extendable_k(false)
     , generate_skip_accumulation(false)
     , bd_mask_level(0)
     , use_uker(false)
@@ -1933,9 +2000,8 @@ brgemm_attr_t::brgemm_attr_t()
     , bd_mask(nullptr)
     , static_offsets(nullptr) {}
 
-brgemm_kernel_common_t::brgemm_kernel_common_t(const brgemm_t abrd) {
-    brgemm_kernel_ = new jit_brgemm_kernel_t(abrd);
-}
+brgemm_kernel_common_t::brgemm_kernel_common_t(const brgemm_t abrd)
+    : brgemm_kernel_(new jit_brgemm_kernel_t(abrd)) {}
 
 status_t brgemm_kernel_common_t::create_kernel() {
     return brgemm_kernel_->create_kernel();
@@ -1945,7 +2011,7 @@ void brgemm_kernel_common_t::operator()(brgemm_kernel_params_t *params) const {
     (*brgemm_kernel_)(params);
 }
 
-const jit_generator *brgemm_kernel_common_t::get_jit_generator() const {
+const jit_generator_t *brgemm_kernel_common_t::get_jit_generator() const {
     return brgemm_kernel_;
 }
 
