@@ -1,6 +1,7 @@
 /*******************************************************************************
 * Copyright 2020-2021 Intel Corporation
 * Copyright 2020-2024 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -98,8 +99,8 @@ void reduce_balancer_t::balance() {
 using namespace Xbyak_aarch64;
 
 template <impl::data_type_t data_type, cpu_isa_t isa>
-struct reducer_2d_driver_t : public jit_generator {
-    using data_t = typename prec_traits<data_type>::type;
+struct reducer_2d_driver_t : public jit_generator_t {
+    using data_t = typename prec_traits_t<data_type>::type;
 
     reducer_2d_driver_t(int n_src, size_t src_ld, size_t src_step,
             size_t dst_step, bool nullify_dst)
@@ -122,11 +123,11 @@ template <impl::data_type_t data_type, cpu_isa_t isa>
 struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type, isa> {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(reducer_2d_driver_f_s_32_t)
 
-    using data_t = typename prec_traits<data_type>::type;
+    using data_t = typename prec_traits_t<data_type>::type;
 
     void operator()(
             data_t *dst, const data_t *srcs, size_t ny, size_t nx) override {
-        jit_generator::operator()(dst, srcs, ny, nx);
+        jit_generator_t::operator()(dst, srcs, ny, nx);
     }
 
     /* cpu specific part */
@@ -134,7 +135,7 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type, isa> {
 
     const int vlen = cpu_isa_traits<isa>::vlen;
     const int typesize
-            = sizeof(typename dnnl::impl::prec_traits<data_type>::type);
+            = sizeof(typename dnnl::impl::prec_traits_t<data_type>::type);
     XReg reg_dst = abi_param1;
     XReg reg_src = abi_param2;
     XReg reg_ny = abi_param3;
@@ -217,7 +218,11 @@ struct reducer_2d_driver_f_s_32_t : public reducer_2d_driver_t<data_type, isa> {
         const int load_len[nbranches] = {vlen, vlen, typesize};
         Label loop_x_label[nbranches + 1];
 
-        this->ptrue(preg_all.b);
+        switch (isa) {
+            case sve_256: this->ptrue(preg_all.b, VL32); break;
+            case sve_512: this->ptrue(preg_all.b, VL64); break;
+            default: assert(!"Unsupported ISA"); break;
+        }
         if (typesize == 4)
             this->ptrue(preg_one.s, VL1);
         else
@@ -570,9 +575,8 @@ template struct cpu_reducer_2d_t<data_type::s32, sve_256>;
 /* accumulator section */
 
 template <impl::data_type_t data_type, cpu_isa_t isa>
-cpu_accumulator_1d_t<data_type, isa>::cpu_accumulator_1d_t() : drv_(nullptr) {
-    drv_ = create_reduce_2d_drv<data_type, isa>(1, 0, 0, 0, false);
-}
+cpu_accumulator_1d_t<data_type, isa>::cpu_accumulator_1d_t()
+    : drv_(create_reduce_2d_drv<data_type, isa>(1, 0, 0, 0, false)) {}
 
 template <impl::data_type_t data_type, cpu_isa_t isa>
 cpu_accumulator_1d_t<data_type, isa>::~cpu_accumulator_1d_t() {
