@@ -1,6 +1,7 @@
 /*******************************************************************************
 * Copyright 2021-2023 Intel Corporation
-* Copyright 2024 FUJITSU LIMITED
+* Copyright 2024-2025 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -54,32 +55,65 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
 
     using skip_mask_t = primitive_attr_t::skip_mask_t;
     auto skip_mask = skip_mask_t::post_ops | skip_mask_t::sum_dt
-            | skip_mask_t::zero_points_runtime;
-    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::scales_runtime;
+            | skip_mask_t::zero_points;
+    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::scales;
 
     bool ok = is_fwd() && set_default_alg_kind(alg_kind::convolution_direct)
             && expect_data_types(src_type, wei_type, data_type::undef, dst_type,
                     data_type::undef)
             && IMPLICATION(is_int8,
-                    one_of(bias_md_.data_type, data_type::undef, f32, s32, s8,
-                            u8))
+                    one_of(dst_type, u8, f32)
+                            && one_of(bias_md_.data_type, data_type::undef, f32,
+                                    s32, s8, u8))
             && IMPLICATION(!is_int8,
                     one_of(bias_md_.data_type, data_type::undef, f32, src_type))
             && attr()->has_default_values(skip_mask, dst_type)
             && attr()->post_ops_.check_sum_consistency(dst_type, is_int8)
-            && !has_zero_dim_memory() && zero_points_ok() && arg_scales_ok();
+            && !has_zero_dim_memory() && zero_points_ok()
+            && impl::is_dense_format_kind({src_md(), weights_md(), dst_md()});
     if (!ok) return status::unimplemented;
 
+    CHECK(attr_scales_ok());
+
     CHECK(brgemm_convolution_utils::init_1x1_conf(jcp_, isa, *desc(), src_md_,
             weights_md_, dst_md_, bias_md_, attr_, dnnl_get_max_threads()));
 
+    // brgemm is slower than jit_sve when combined with reorders for shapes where strides < 2
+    const convolution_desc_t &cd = *desc();
+    if (!cd.use_inversion && one_of(data_type::f32, src_type, wei_type)
+            && (jcp_.stride_w < 2 || jcp_.stride_h < 2)) {
+        return status::unimplemented;
+    }
+
     brgs_ = std::make_shared<brgemm_containers::brgemm_desc_container_t>(16);
 
     const float alpha = 1.0;
     const float beta = 1.0;
     const auto &p = attr()->post_ops_;
+
+    // TODO: fix failing post ops for bf16 on sve 128
+    const bool is_bf16
+            = src_type == data_type::bf16 && wei_type == data_type::bf16;
+    if (is_bf16 && get_max_cpu_isa() == sve_128) {
+        for (auto const &entry : p.entry_) {
+            const bool is_failing_po = entry.is_eltwise()
+                    && one_of(entry.eltwise.alg,
+                            // these fail due to label offset being too large
+                            alg_kind::eltwise_tanh, alg_kind::eltwise_gelu_tanh,
+                            alg_kind::eltwise_gelu_erf);
+            VDISPATCH_CONV(!is_failing_po, VERBOSE_BAD_ALGORITHM);
+        }
+    }
+
     const int sum_idx = p.find(primitive_kind::sum);
     with_sum = (sum_idx != -1);
+    // Check if postop sum datatype is supported
+    if (with_sum) {
+        const auto &sum_po = p.entry_[sum_idx];
+        if (!one_of(sum_po.sum.dt, data_type::undef, data_type::f32,
+                    data_type::s32, data_type::u8, data_type::s8))
+            return status::unimplemented;
+    }
     sum_scale = with_sum ? p.entry_[sum_idx].sum.scale : 0.0;
 
     ic_chunks = div_up(jcp_.nb_ic, jcp_.nb_ic_blocking);
@@ -115,6 +149,7 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
         brg.with_weights_scale_adjust = jcp_.scale_adjust_factor != 1.0f;
         CHECK(brgemm_desc_set_postops(
                 &brg, attr(), &dst_md_, LDD, jcp_.bia_dt));
+        CHECK(brgemm_desc_finalize(&brg));
         brgs_->insert(brg_idx, brg);
     }
 
@@ -223,7 +258,7 @@ void brgemm_1x1_convolution_fwd_t<isa>::maybe_rtus(int ithr,
         const auto inp_offset = n * src_d_sz + id * src_h_sz + ih * src_w_sz
                 + iw * jcp.ngroups * jcp.ic_without_padding + g_ic;
         auto p = jit_sve_core_brgemm_conv_trans_kernel::
-                jit_brgemm_conv_trans_kernel_call_s();
+                jit_brgemm_conv_trans_kernel_args_t();
         p.h_count = nh;
         p.owb = nw;
         p.src = src + src_dt_size * inp_offset;
@@ -270,7 +305,7 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
         char *const c_buffer, const char *inp_buffer, int g, int n, int ocb,
         int od, int oh, int ow, int icc, int *last_brg_idx,
         const float *oscales, int32_t src_zp_vals, int32_t *src_zp_comp,
-        int32_t *dst_zp_vals, int32_t *s8s8_compensation,
+        const int32_t *dst_zero_points, int32_t *s8s8_compensation,
         const float *dst_scales) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
@@ -360,8 +395,8 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
                     post_ops_binary_rhs_arg_vec.data(),
                     static_cast<size_t>(g_oc), 0, dst, 0,
                     static_cast<void *>(src_zp_comp_ptr), nullptr,
-                    static_cast<void *>(dst_zp_vals), false, src_zp_vals, false,
-                    false, dst_scales};
+                    dst_zero_points, false, src_zp_vals, false, false,
+                    dst_scales};
 
             void *scratch = static_cast<void *>(s8s8_comp_ptr);
             brgemm_kernel_execute_postops(brg_ker, n_ic_blocks, brg_batch,
@@ -409,8 +444,11 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
             src_scales, wei_scales, pd()->OC(), pd()->attr(),
             jcp.scale_adjust_factor);
 
-    DEFINE_ZERO_POINT_VALUE(src_zero_point, DNNL_ARG_SRC);
-    DEFINE_ZERO_POINT_VALUE(dst_zero_point, DNNL_ARG_DST);
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+    const int src_zero_point = src_zero_points ? src_zero_points[0] : 0;
 
     const auto extra_data_offset
             = weights_d.size() - weights_d.additional_buffer_size();
@@ -424,7 +462,6 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
                                     ? jcp.s8s8_comp_buffer_size
                                     : 0)
             : nullptr;
-    int32_t *dst_zp_vals = jcp.dst_zero_point ? &dst_zero_point : nullptr;
 
     brgemm_batch_element_t *const brg_batch_global
             = (jcp.brg_type != brgemm_strd)
@@ -487,8 +524,8 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
                     exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer, \
                             inp_buffer_sp, g, n, ocb, od, oh, ow, icc, \
                             &last_brg_idx, oscales, src_zero_point, \
-                            zp_compensation, dst_zp_vals, s8s8_compensation, \
-                            dst_scales); \
+                            zp_compensation, dst_zero_points, \
+                            s8s8_compensation, dst_scales); \
                 } \
             } \
             last_n = n; \
@@ -528,7 +565,7 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
                 const int ow = owb * jcp.ow_block; \
                 exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer, nullptr, g, n, \
                         ocb, od, oh, ow, icc, &last_brg_idx, oscales, \
-                        src_zero_point, zp_compensation, dst_zp_vals, \
+                        src_zero_point, zp_compensation, dst_zero_points, \
                         s8s8_compensation, dst_scales); \
             } \
             nd_iterator_step(__VA_ARGS__); \
@@ -552,6 +589,7 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
 
 template struct brgemm_1x1_convolution_fwd_t<sve_512>;
 template struct brgemm_1x1_convolution_fwd_t<sve_256>;
+template struct brgemm_1x1_convolution_fwd_t<sve_128>;
 
 } // namespace aarch64
 } // namespace cpu
