@@ -1,6 +1,7 @@
 /*******************************************************************************
 * Copyright 2020-2021 Intel Corporation
 * Copyright 2020-2024 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,14 +40,20 @@ template <impl::data_type_t src_type, impl::data_type_t wei_type = src_type,
         impl::data_type_t dst_type = src_type, cpu_isa_t isa = isa_undef>
 struct jit_sve_convolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_convolution_fwd_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const typename pd_t::base_class *hint_fwd_pd)
-            : cpu_convolution_fwd_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_fwd_pd_t::cpu_convolution_fwd_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit:", isa, ""),
                 jit_sve_convolution_fwd_t);
 
         status_t init(engine_t *engine) {
+#if defined(DNNL_AARCH64_USE_ACL)
+            if (attr()->fpmath_.mode_ == fpmath_mode::bf16) {
+                // prefer ACL to jit for fpmath_mode::bf16 if available
+                // since it supports lower precision calculation
+                return status::unimplemented;
+            }
+#endif
+
             bool ok = true && mayiuse(isa) && is_fwd()
                     && set_default_alg_kind(alg_kind::convolution_direct)
                     && expect_data_types(src_type, wei_type, dst_type, dst_type,
@@ -56,29 +63,29 @@ struct jit_sve_convolution_fwd_t : public primitive_t {
                     && !has_zero_dim_memory();
             if (!ok) return status::unimplemented;
 
-            status_t status = jit_sve_conv_fwd_kernel<isa>::init_conf(jcp_,
+            status_t status = jit_sve_conv_fwd_kernel_t<isa>::init_conf(jcp_,
                     *desc(), src_md_, weights_md_, dst_md_, bias_md_, *attr(),
                     dnnl_get_max_threads());
             if (status != status::success) return status;
 
             auto scratchpad = scratchpad_registry().registrar();
-            jit_sve_conv_fwd_kernel<isa>::init_scratchpad(scratchpad, jcp_);
+            jit_sve_conv_fwd_kernel_t<isa>::init_scratchpad(scratchpad, jcp_);
 
             return status;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
     };
 
     jit_sve_convolution_fwd_t(const pd_t *apd) : primitive_t(apd) {}
 
-    typedef typename prec_traits<src_type>::type src_data_t;
-    typedef typename prec_traits<wei_type>::type wei_data_t;
-    typedef typename prec_traits<dst_type>::type dst_data_t;
+    using src_data_t = typename prec_traits_t<src_type>::type;
+    using wei_data_t = typename prec_traits_t<wei_type>::type;
+    using dst_data_t = typename prec_traits_t<dst_type>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
-                new jit_sve_conv_fwd_kernel<isa>(pd()->jcp_, *pd()->attr())));
+                new jit_sve_conv_fwd_kernel_t<isa>(pd()->jcp_, *pd()->attr())));
         return kernel_->create_kernel();
     }
 
@@ -105,7 +112,7 @@ private:
     void execute_forward_3d(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
 
-    std::unique_ptr<jit_sve_conv_fwd_kernel<isa>> kernel_;
+    std::unique_ptr<jit_sve_conv_fwd_kernel_t<isa>> kernel_;
 };
 
 template <impl::data_type_t diff_dst_type,
@@ -114,9 +121,7 @@ template <impl::data_type_t diff_dst_type,
         cpu_isa_t isa = isa_undef>
 struct jit_sve_convolution_bwd_data_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_data_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_data_pd_t(adesc, attr, hint_fwd_pd), jcp_() {}
+        using cpu_convolution_bwd_data_pd_t::cpu_convolution_bwd_data_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit:", isa, ""),
                 jit_sve_convolution_bwd_data_t);
@@ -129,30 +134,31 @@ struct jit_sve_convolution_bwd_data_t : public primitive_t {
                     && attr()->has_default_values() && !has_zero_dim_memory();
             if (!ok) return status::unimplemented;
 
-            status_t status = jit_sve_conv_bwd_data_kernel_f32<isa>::init_conf(
-                    jcp_, *desc(), diff_src_md_, weights_md_, diff_dst_md_,
-                    dnnl_get_max_threads());
+            status_t status
+                    = jit_sve_conv_bwd_data_kernel_f32_t<isa>::init_conf(jcp_,
+                            *desc(), diff_src_md_, weights_md_, diff_dst_md_,
+                            dnnl_get_max_threads());
             if (status != status::success) return status;
 
             auto scratchpad = scratchpad_registry().registrar();
-            jit_sve_conv_bwd_data_kernel_f32<isa>::init_scratchpad(
+            jit_sve_conv_bwd_data_kernel_f32_t<isa>::init_scratchpad(
                     scratchpad, jcp_);
 
             return status::success;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
     };
 
     jit_sve_convolution_bwd_data_t(const pd_t *apd) : primitive_t(apd) {}
 
-    typedef typename prec_traits<diff_dst_type>::type diff_dst_data_t;
-    typedef typename prec_traits<wei_type>::type wei_data_t;
-    typedef typename prec_traits<diff_src_type>::type diff_src_data_t;
+    using diff_dst_data_t = typename prec_traits_t<diff_dst_type>::type;
+    using wei_data_t = typename prec_traits_t<wei_type>::type;
+    using diff_src_data_t = typename prec_traits_t<diff_src_type>::type;
 
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
-                new jit_sve_conv_bwd_data_kernel_f32<isa>(pd()->jcp_)));
+                new jit_sve_conv_bwd_data_kernel_f32_t<isa>(pd()->jcp_)));
         return kernel_->create_kernel();
     }
 
@@ -174,7 +180,7 @@ private:
     void execute_backward_data_3d(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
 
-    std::unique_ptr<jit_sve_conv_bwd_data_kernel_f32<isa>> kernel_;
+    std::unique_ptr<jit_sve_conv_bwd_data_kernel_f32_t<isa>> kernel_;
 };
 
 template <impl::data_type_t src_type,
@@ -183,10 +189,8 @@ template <impl::data_type_t src_type,
         cpu_isa_t isa = isa_undef>
 struct jit_sve_convolution_bwd_weights_t : public primitive_t {
     struct pd_t : public cpu_convolution_bwd_weights_pd_t {
-        pd_t(const convolution_desc_t *adesc, const primitive_attr_t *attr,
-                const convolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_convolution_bwd_weights_pd_t(adesc, attr, hint_fwd_pd)
-            , jcp_() {}
+        using cpu_convolution_bwd_weights_pd_t::
+                cpu_convolution_bwd_weights_pd_t;
 
         DECLARE_COMMON_PD_T(JIT_IMPL_NAME_HELPER("jit:", isa, ""),
                 jit_sve_convolution_bwd_weights_t);
@@ -200,15 +204,16 @@ struct jit_sve_convolution_bwd_weights_t : public primitive_t {
             if (!ok) return status::unimplemented;
 
             status_t status
-                    = jit_sve_conv_bwd_weights_kernel_f32<isa>::init_conf(jcp_,
-                            *desc(), src_md_, diff_weights_md_, diff_bias_md_,
-                            diff_dst_md_, dnnl_get_max_threads());
+                    = jit_sve_conv_bwd_weights_kernel_f32_t<isa>::init_conf(
+                            jcp_, *desc(), src_md_, diff_weights_md_,
+                            diff_bias_md_, diff_dst_md_,
+                            dnnl_get_max_threads());
             if (status != status::success) return status;
 
             init_balancers();
 
             auto scratchpad = scratchpad_registry().registrar();
-            jit_sve_conv_bwd_weights_kernel_f32<isa>::init_scratchpad(
+            jit_sve_conv_bwd_weights_kernel_f32_t<isa>::init_scratchpad(
                     scratchpad, jcp_);
 
             auto reducer_bia_scratchpad = memory_tracking::registrar_t(
@@ -218,7 +223,7 @@ struct jit_sve_convolution_bwd_weights_t : public primitive_t {
             return status;
         }
 
-        jit_conv_conf_t jcp_;
+        jit_conv_conf_t jcp_ = utils::zero<decltype(jcp_)>();
         typename cpu_reducer_t<diff_weights_type, isa>::conf_t
                 reducer_bia_conf_;
 
@@ -235,9 +240,9 @@ struct jit_sve_convolution_bwd_weights_t : public primitive_t {
 
     jit_sve_convolution_bwd_weights_t(const pd_t *apd) : primitive_t(apd) {}
 
-    typedef typename prec_traits<src_type>::type src_data_t;
-    typedef typename prec_traits<diff_dst_type>::type diff_dst_data_t;
-    typedef typename prec_traits<diff_weights_type>::type diff_weights_data_t;
+    using src_data_t = typename prec_traits_t<src_type>::type;
+    using diff_dst_data_t = typename prec_traits_t<diff_dst_type>::type;
+    using diff_weights_data_t = typename prec_traits_t<diff_weights_type>::type;
 
     status_t init(engine_t *engine) override;
 
@@ -262,7 +267,7 @@ private:
 
     int nthr_, nthr_mb_, nthr_g_, nthr_oc_b_, nthr_ic_b_;
 
-    jit_sve_conv_bwd_weights_kernel_f32<isa> *kernel_;
+    jit_sve_conv_bwd_weights_kernel_f32_t<isa> *kernel_;
     cpu_accumulator_1d_t<diff_weights_type, isa> *acc_ker_;
     cpu_reducer_t<diff_weights_type, isa> *reducer_bias_;
 };
@@ -274,4 +279,4 @@ private:
 
 #endif
 
-// vim: et ts=4 sw=4 cindent cino+=l0,\:4,N-s
\ No newline at end of file
+// vim: et ts=4 sw=4 cindent cino+=l0,\:4,N-s
