@@ -2,6 +2,7 @@
 * Copyright 2017-2023 Intel Corporation
 * Copyright 2018 YANDEX LLC
 * Copyright 2020-2024 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,7 +33,8 @@ namespace aarch64 {
 using namespace Xbyak_aarch64;
 using namespace alg_kind;
 
-#define GET_OFF(field) static_cast<uint32_t>(offsetof(jit_pool_call_s, field))
+#define GET_OFF(field) \
+    static_cast<uint32_t>(offsetof(jit_uni_pooling_args_t, field))
 
 static bcast_set_t get_supported_bcast_strategies() {
     return {broadcasting_strategy_t::scalar, broadcasting_strategy_t::per_oc,
@@ -40,12 +42,12 @@ static bcast_set_t get_supported_bcast_strategies() {
 }
 
 template <cpu_isa_t isa>
-jit_uni_pool_kernel<isa>::~jit_uni_pool_kernel() = default;
+jit_uni_pool_kernel_t<isa>::~jit_uni_pool_kernel_t() = default;
 
 template <cpu_isa_t isa>
-jit_uni_pool_kernel<isa>::jit_uni_pool_kernel(
+jit_uni_pool_kernel_t<isa>::jit_uni_pool_kernel_t(
         const jit_pool_conf_t &ajpp, const memory_desc_t *dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true), jpp(ajpp) {
+    : jit_generator_t(nullptr, MAX_CODE_SIZE, true), jpp(ajpp) {
     if (jpp.with_postops) {
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = true;
@@ -95,7 +97,7 @@ static status_t set_binary_postops_formats(
 }
 
 template <cpu_isa_t isa>
-status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
+status_t jit_uni_pool_kernel_t<isa>::init_conf(jit_pool_conf_t &jpp,
         memory_tracking::registrar_t &scratchpad, primitive_attr_t &attr,
         const pooling_pd_t *ppd) {
 
@@ -332,7 +334,7 @@ static int reg_ind(int shift, int bc, int j, int ur_bc, int ur_w) noexcept {
 };
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::prepare_tail_mask() {
+inline void jit_uni_pool_kernel_t<isa>::prepare_tail_mask() {
     /* PRegS(k_c_tail_mask) keeps flags in the context
            of 8-bit elements. */
     set_preg(k_c_tail_mask_b.b, jpp.c_tail, X_TMP_0, X_TMP_1);
@@ -341,8 +343,9 @@ inline void jit_uni_pool_kernel<isa>::prepare_tail_mask() {
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::load(const int idx, const xreg_t &reg_ptr,
-        const int offset, const bool is_c_tail_proccessing) {
+inline void jit_uni_pool_kernel_t<isa>::load(const int idx,
+        const xreg_t &reg_ptr, const int offset,
+        const bool is_c_tail_proccessing) {
     if (is_c_tail_proccessing && !jpp.is_c_padded) {
         add_imm(X_DEFAULT_ADDR, reg_ptr, offset, X_TMP_0);
         ld1w(ZRegS(idx), k_c_tail_mask_s / T_z, ptr(X_DEFAULT_ADDR));
@@ -353,7 +356,7 @@ inline void jit_uni_pool_kernel<isa>::load(const int idx, const xreg_t &reg_ptr,
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::store(const int idx,
+inline void jit_uni_pool_kernel_t<isa>::store(const int idx,
         const xreg_t &reg_ptr, const int offset,
         const bool is_c_tail_proccessing) {
     add_imm(X_DEFAULT_ADDR, reg_ptr, offset, X_TMP_0);
@@ -371,7 +374,7 @@ inline void jit_uni_pool_kernel<isa>::store(const int idx,
 }
 
 template <cpu_isa_t isa>
-bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
+bool jit_uni_pool_kernel_t<isa>::post_ops_ok(jit_pool_conf_t &jpp,
         const primitive_attr_t &attr, const memory_desc_wrapper &dst_d) {
     const auto &post_ops = attr.post_ops_;
     const auto &entries = post_ops.entry_;
@@ -383,10 +386,11 @@ bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
         for (const auto &entry : entries) {
             if (entry.is_eltwise()) {
                 const auto alg = entry.eltwise.alg;
-                jpp.with_eltwise = eltwise_injector::is_supported(isa, alg);
+                jpp.with_eltwise
+                        = eltwise_injector::is_supported(to_vla_sve(isa), alg);
             } else if (entry.is_binary()) {
                 if (entry.binary.src1_desc.data_type == data_type::bf16
-                        && entry.binary.src1_desc.data_type == data_type::f16)
+                        || entry.binary.src1_desc.data_type == data_type::f16)
                     return false;
 
                 jpp.with_binary = true;
@@ -402,7 +406,7 @@ bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
 }
 
 template <cpu_isa_t isa>
-void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block,
+void jit_uni_pool_kernel_t<isa>::apply_postops(int ur_bc, int ur_w, int c_block,
         const std::function<bool(int)> &is_tail_predicate) {
     binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
     const int end_idx = vmm_idx_upper_bound() + 1;
@@ -445,7 +449,7 @@ void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block,
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::maybe_recalculate_divisor(
+inline void jit_uni_pool_kernel_t<isa>::maybe_recalculate_divisor(
         int jj, int ur_w, int pad_l, int pad_r, bool with_c_tail_proccessing) {
     if (jpp.alg == pooling_avg_exclude_padding) {
         int kw = jpp.kw;
@@ -466,7 +470,7 @@ inline void jit_uni_pool_kernel<isa>::maybe_recalculate_divisor(
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
+inline void jit_uni_pool_kernel_t<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
         int pad_r, bool with_c_tail_proccessing) {
 
     auto iw = jpp.iw;
@@ -595,7 +599,7 @@ inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
+inline void jit_uni_pool_kernel_t<isa>::max_step_fwd(int ur_w, int ur_bc,
         int pad_l, int pad_r, bool with_c_tail_proccessing) {
     int iw = jpp.iw;
     int kw = jpp.kw;
@@ -728,7 +732,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
 }
 
 template <cpu_isa_t isa>
-inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
+inline void jit_uni_pool_kernel_t<isa>::max_step_bwd(int ur_w, int ur_bc,
         int pad_l, int pad_r, bool with_c_tail_proccessing) {
 
     int iw = jpp.iw;
@@ -831,7 +835,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
 }
 
 template <cpu_isa_t isa>
-void jit_uni_pool_kernel<isa>::zero_diff_src(
+void jit_uni_pool_kernel_t<isa>::zero_diff_src(
         int ur_bc, bool with_c_tail_proccessing) {
     const int c_off = (jpp.tag_kind == jit_memory_tag_kind_t::nspc)
             ? jpp.c
@@ -889,7 +893,7 @@ void jit_uni_pool_kernel<isa>::zero_diff_src(
 }
 
 template <cpu_isa_t isa>
-void jit_uni_pool_kernel<isa>::generate() {
+void jit_uni_pool_kernel_t<isa>::generate() {
 
     this->preamble();
 
@@ -1059,8 +1063,8 @@ void jit_uni_pool_kernel<isa>::generate() {
         postops_injector_->prepare_table();
 }
 
-template struct jit_uni_pool_kernel<sve_512>;
-template struct jit_uni_pool_kernel<sve_256>;
+template struct jit_uni_pool_kernel_t<sve_512>;
+template struct jit_uni_pool_kernel_t<sve_256>;
 
 } // namespace aarch64
 } // namespace cpu
