@@ -1,6 +1,7 @@
 /*******************************************************************************
 * Copyright 2019-2023 Intel Corporation
 * Copyright 2020-2024 FUJITSU LIMITED
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,6 +17,7 @@
 *******************************************************************************/
 
 #include <assert.h>
+#include <memory>
 
 #include "common/c_types_map.hpp"
 #include "common/dnnl_thread.hpp"
@@ -40,7 +42,7 @@ namespace aarch64 {
 using namespace Xbyak_aarch64;
 
 template <cpu_isa_t isa>
-struct jit_softmax_base_t : public jit_generator {
+struct jit_softmax_base_t : public jit_generator_t {
     struct call_params_t {
         // keep all sizes at 8 bytes -- jit code expects this
         const void *src, *dst, *diff_dst; // src dubs as diff_src
@@ -60,8 +62,10 @@ struct jit_softmax_base_t : public jit_generator {
     const memory_desc_wrapper src_d_, dst_d_, diff_dst_d_;
 
     virtual void operator()(const call_params_t *p) = 0;
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> exp_injector_;
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> log_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_f32_t<to_vla_sve(isa)>>
+            exp_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_f32_t<to_vla_sve(isa)>>
+            log_injector_;
 
     XReg reg_param = abi_param1;
 
@@ -585,13 +589,15 @@ struct jit_softmax_base_t : public jit_generator {
     // initialization.
     void generate() override {
         if (pd_->is_fwd() || is_logsoftmax_)
-            exp_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, true,
+            exp_injector_.reset(new jit_uni_eltwise_injector_f32_t<sve_128>(
+                    this, alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, true,
                     reg_exp_injector_table, injector_mask, injector_tmp));
         if (pd_->is_fwd() && is_logsoftmax_) {
-            log_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, true,
-                    reg_log_injector_table, injector_mask, injector_tmp));
+            log_injector_.reset(
+                    new jit_uni_eltwise_injector_f32_t<to_vla_sve(isa)>(this,
+                            alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, true,
+                            reg_log_injector_table, injector_mask,
+                            injector_tmp));
         }
 
         compute_predefined_variables();
@@ -616,15 +622,15 @@ struct jit_softmax_base_t : public jit_generator {
     }
 
     jit_softmax_base_t(const softmax_pd_t *pd)
-        : jit_generator(nullptr, MAX_CODE_SIZE, true)
+        : jit_generator_t(nullptr, MAX_CODE_SIZE, true)
         , pd_(pd)
         , src_d_(pd_->is_fwd() ? pd_->src_md() : pd_->diff_src_md())
         , dst_d_(pd_->dst_md())
-        , diff_dst_d_(pd_->diff_dst_md()) {
-        simd_w_ = vlen / sizeof(float); // bf16 works on ymms
-        need_scratchpad_ = utils::one_of(
-                dst_d_.data_type(), data_type::u8, data_type::s8);
-    }
+        , diff_dst_d_(pd_->diff_dst_md())
+        , need_scratchpad_(utils::one_of(
+                  dst_d_.data_type(), data_type::u8, data_type::s8))
+        , simd_w_(vlen / sizeof(float)) // bf16 works on ymms
+    {}
 };
 
 template <cpu_isa_t isa>
@@ -637,7 +643,7 @@ struct jit_softmax_t<sve_512> : public jit_softmax_base_t<sve_512> {
     jit_softmax_t(const softmax_pd_t *pd) : jit_softmax_base_t(pd) {}
 
     void operator()(const call_params_t *p) override {
-        return jit_generator::operator()(p);
+        return jit_generator_t::operator()(p);
     }
 }; // namespace aarch64
 
@@ -649,7 +655,7 @@ struct jit_softmax_t<sve_256> : public jit_softmax_base_t<sve_256> {
     jit_softmax_t(const softmax_pd_t *pd) : jit_softmax_base_t(pd) {}
 
     void operator()(const call_params_t *p) override {
-        return jit_generator::operator()(p);
+        return jit_generator_t::operator()(p);
     }
 }; // namespace aarch64
 
@@ -661,19 +667,17 @@ struct jit_softmax_t<sve_128> : public jit_softmax_base_t<sve_128> {
     jit_softmax_t(const softmax_pd_t *pd) : jit_softmax_base_t(pd) {}
 
     void operator()(const call_params_t *p) override {
-        return jit_generator::operator()(p);
+        return jit_generator_t::operator()(p);
     }
 }; // namespace aarch64
 
 template <cpu_isa_t isa>
 jit_uni_softmax_fwd_t<isa>::jit_uni_softmax_fwd_t(const pd_t *apd)
     : primitive_t(apd)
-    , softmax_driver_(new softmax_impl::driver_t<isa>(pd())) {}
+    , softmax_driver_(utils::make_unique<softmax_impl::driver_t<isa>>(pd())) {}
 
 template <cpu_isa_t isa>
-jit_uni_softmax_fwd_t<isa>::~jit_uni_softmax_fwd_t() {
-    delete softmax_driver_;
-}
+jit_uni_softmax_fwd_t<isa>::~jit_uni_softmax_fwd_t() = default;
 
 template <cpu_isa_t isa>
 status_t jit_uni_softmax_fwd_t<isa>::init(engine_t *engine) {
@@ -725,12 +729,10 @@ status_t jit_uni_softmax_fwd_t<isa>::execute(const exec_ctx_t &ctx) const {
 template <cpu_isa_t isa>
 jit_uni_softmax_bwd_t<isa>::jit_uni_softmax_bwd_t(const pd_t *apd)
     : primitive_t(apd)
-    , softmax_driver_(new softmax_impl::driver_t<isa>(pd())) {}
+    , softmax_driver_(utils::make_unique<softmax_impl::driver_t<isa>>(pd())) {}
 
 template <cpu_isa_t isa>
-jit_uni_softmax_bwd_t<isa>::~jit_uni_softmax_bwd_t() {
-    delete softmax_driver_;
-}
+jit_uni_softmax_bwd_t<isa>::~jit_uni_softmax_bwd_t() = default;
 
 template <cpu_isa_t isa>
 status_t jit_uni_softmax_bwd_t<isa>::init(engine_t *engine) {
