@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
+* Copyright 2025 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -40,7 +41,8 @@ using namespace dnnl::impl::memory_tracking::names;
 
 const int32_t *mul_zp_src_comp_from_wei_by_zp_src(const int zp_comp_size,
         int32_t *zp_src_comp_scratch_dst,
-        const int32_t *const zp_src_comp_from_wei, const int32_t zp_src) {
+        const int32_t *const zp_src_comp_from_wei,
+        const int32_t src_zero_point) {
     static constexpr int cache_line_size
             = platform::get_cache_line_size() / sizeof(int);
     const auto res = std::div(zp_comp_size, cache_line_size);
@@ -53,7 +55,7 @@ const int32_t *mul_zp_src_comp_from_wei_by_zp_src(const int zp_comp_size,
 
             PRAGMA_OMP_SIMD()
             for (int i = 0; i < cache_line_size; ++i) {
-                dst[i] = src[i] * zp_src;
+                dst[i] = src[i] * src_zero_point;
             }
         });
     }
@@ -65,7 +67,7 @@ const int32_t *mul_zp_src_comp_from_wei_by_zp_src(const int zp_comp_size,
 
         PRAGMA_OMP_SIMD()
         for (int i = 0; i < res.rem; ++i) {
-            dst[i] = src[i] * zp_src;
+            dst[i] = src[i] * src_zero_point;
         }
     }
 
@@ -75,7 +77,7 @@ const int32_t *mul_zp_src_comp_from_wei_by_zp_src(const int zp_comp_size,
 static zero_point_call_params_t prepare_zp_params(const conv_gemm_conf_t &jcp,
         const memory_tracking::grantor_t &scratchpad, const int8_t *weights,
         const memory_desc_wrapper &weights_md, bool with_groups,
-        const int32_t *zp_src, const int32_t *zp_dst) {
+        const int32_t *src_zero_points, const int32_t *dst_zero_points) {
 
     int32_t *zp_src_comp_pad = nullptr;
     const int32_t *zp_src_comp = nullptr;
@@ -91,7 +93,8 @@ static zero_point_call_params_t prepare_zp_params(const conv_gemm_conf_t &jcp,
 
         if (jcp.zp.src_is_common) {
             zp_src_comp = mul_zp_src_comp_from_wei_by_zp_src(zp_comp_size,
-                    zp_src_comp_scratch, zp_src_comp_from_wei, *zp_src);
+                    zp_src_comp_scratch, zp_src_comp_from_wei,
+                    *src_zero_points);
         } else
             zp_src_comp = zp_src_comp_from_wei;
 
@@ -100,12 +103,12 @@ static zero_point_call_params_t prepare_zp_params(const conv_gemm_conf_t &jcp,
                     ? utils::rnd_up(zp_comp_size, cache_line_size)
                     : 0;
             zp_src_comp_pad = zp_src_comp_scratch + shift;
-            compute_zp_src_comp_pad(jcp, zp_src_comp_pad, zp_src, weights,
-                    weights_md, with_groups);
+            compute_zp_src_comp_pad(jcp, zp_src_comp_pad, src_zero_points,
+                    weights, weights_md, with_groups);
         }
     }
 
-    return {zp_src, zp_dst, zp_src_comp, zp_src_comp_pad};
+    return {src_zero_points, dst_zero_points, zp_src_comp, zp_src_comp_pad};
 }
 
 status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward(
@@ -115,8 +118,12 @@ status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward(
     auto wei_base = CTX_IN_MEM(const int8_t *, DNNL_ARG_WEIGHTS);
     auto bia_base = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
     auto dst_base = CTX_OUT_MEM(void *, DNNL_ARG_DST);
-    DEFINE_ZERO_POINTS_BUFFER(zp_src, DNNL_ARG_SRC);
-    DEFINE_ZERO_POINTS_BUFFER(zp_dst, DNNL_ARG_DST);
+
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector_utils::prepare_binary_args(
                     this->pd()->attr()->post_ops_, ctx);
@@ -127,7 +134,7 @@ status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward(
 
     const zero_point_call_params_t zp = prepare_zp_params(jcp, scratchpad,
             wei_base, memory_desc_wrapper(pd()->weights_md(0)),
-            this->pd()->with_groups(), zp_src, zp_dst);
+            this->pd()->with_groups(), src_zero_points, dst_zero_points);
 
     std::atomic<status_t> st(status::success);
 
@@ -135,10 +142,9 @@ status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward(
     DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
     DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
 
-    const int wei_scale_mask
-            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const int wei_scale_mask = pd()->attr()->scales_.get_mask(DNNL_ARG_WEIGHTS);
     const float *scales = precompute_scales(scratchpad, src_scales, wei_scales,
-            pd()->IC(), pd()->OC(), false, wei_scale_mask != 0, pd()->attr());
+            pd()->IC(), pd()->OC(), false, wei_scale_mask > 0, pd()->attr());
 
     parallel(jcp.nthr, [&](const int ithr, const int nthr) {
         status_t st_thr = execute_forward_thr(ithr, nthr, src_base, wei_base,
@@ -275,7 +281,7 @@ status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward_thr(const int ithr,
             const float onef = 1.f, zerof = 0.f;
             const char *__restrict src_od
                     = src + od * jcp.oh * jcp.ow * jcp.ngroups * jcp.ic;
-            st = gemm_s8x8s32("N", BT, jcp.signed_input ? "C" : "F", &M, &N, &K,
+            st = gemm_s8u8s32("N", BT, jcp.signed_input ? "C" : "F", &M, &N, &K,
                     &onef, wei, &LDA, &off_a,
                     jcp.im2col_sz ? col : (uint8_t *)src_od, &LDB, &off_b,
                     &zerof, acc, &M, jcp.signed_input ? wei_comp : &off_c);
@@ -302,7 +308,7 @@ status_t gemm_x8s8s32x_convolution_fwd_t::execute_forward_thr(const int ithr,
                 balance211(N * jcp.oc, nthr, ithr, _start, _end);
 
                 (*pp_ker_)(dst, acc, bia_base, scales, dst_scales[0], sum_scale,
-                        1.f / wei_adj_scale, g, _start, _end, zp,
+                        1.f / wei_adj_scale, g, n, _start, _end, zp,
                         post_ops_binary_rhs_arg_vec, dst_base, ctx,
                         *pd()->dst_md(), chunk_desc);
             });
@@ -358,16 +364,15 @@ status_t gemm_x8s8s32x_convolution_bwd_data_t::execute_backward_data_thr(
     const auto diff_src_dt_size
             = types::data_type_size(diff_src_md.data_type());
 
-    const int scale_idx_mult = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_
+    const int scale_idx_mult = pd()->attr()->scales_.get_mask(DNNL_ARG_WEIGHTS)
             == (1 << static_cast<int>(pd()->with_groups()));
     DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
     DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
     DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
 
-    const int wei_scale_mask
-            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const int wei_scale_mask = pd()->attr()->scales_.get_mask(DNNL_ARG_WEIGHTS);
     const float *scales = precompute_scales(scratchpad, src_scales, wei_scales,
-            pd()->IC(), pd()->OC(), false, wei_scale_mask != 0, pd()->attr());
+            pd()->IC(), pd()->OC(), false, wei_scale_mask > 0, pd()->attr());
 
     const dim_t work_amount = jcp.ngroups * jcp.mb;
 
@@ -403,7 +408,7 @@ status_t gemm_x8s8s32x_convolution_bwd_data_t::execute_backward_data_thr(
                         = reinterpret_cast<const int8_t *>(diff_dst_base)
                         + n * diff_dst_mb_stride + g * diff_dst_g_stride;
                 const int8_t off_b = 0;
-                st = gemm_s8x8s32("T", "N", "F", &M, &N, &K, &onef, wei, &LD,
+                st = gemm_s8s8s32("T", "N", "F", &M, &N, &K, &onef, wei, &LD,
                         &off_a, diff_dst, &LD, &off_b, &zerof,
                         jcp.im2col_sz ? col : acc, &M, &off_c);
             } break;
@@ -412,7 +417,7 @@ status_t gemm_x8s8s32x_convolution_bwd_data_t::execute_backward_data_thr(
                         = reinterpret_cast<const uint8_t *>(diff_dst_base)
                         + n * diff_dst_mb_stride + g * diff_dst_g_stride;
                 const uint8_t off_b = 0;
-                st = gemm_s8x8s32("T", "N", "F", &M, &N, &K, &onef, wei, &LD,
+                st = gemm_s8u8s32("T", "N", "F", &M, &N, &K, &onef, wei, &LD,
                         &off_a, diff_dst, &LD, &off_b, &zerof,
                         jcp.im2col_sz ? col : acc, &M, &off_c);
             } break;
