@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2023 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -174,8 +174,7 @@ status_t ref_deconvolution_fwd_t::compute_oscale(
 
     DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
     DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
-    const int wei_scale_mask
-            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const int wei_scale_mask = pd()->attr()->scales_.get_mask(DNNL_ARG_WEIGHTS);
 
     const memory_desc_wrapper dst_d(pd()->dst_md());
 
@@ -190,7 +189,7 @@ status_t ref_deconvolution_fwd_t::compute_oscale(
     const auto maybe_oscale = [](float &d, dim_t oc, const float *src_scales,
                                       const float *wei_scales, int wei_mask) {
         // scale_idx_mult = 1 for per_oc scales and 0, otherwise
-        const int wei_scale_idx_mult = wei_mask != 0;
+        const int wei_scale_idx_mult = wei_mask > 0;
         d *= src_scales[0] * wei_scales[oc * wei_scale_idx_mult];
     };
 
@@ -216,11 +215,16 @@ status_t ref_deconvolution_fwd_t::compute_ref_attrs(const exec_ctx_t &ctx,
     auto dst = CTX_OUT_MEM(void *, DNNL_ARG_DST);
 
     DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
-    const int dst_scale_mask = pd()->attr()->scales_.get(DNNL_ARG_DST).mask_;
+    const bool has_dst_scales
+            = !pd()->attr()->scales_.has_default_values(DNNL_ARG_DST);
+    const int dst_scale_mask = pd()->attr()->scales_.get_mask(DNNL_ARG_DST);
 
-    DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
-    const bool is_dst_zp_common
-            = pd()->attr()->zero_points_.common(DNNL_ARG_DST);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+
+    const bool has_dst_zp
+            = !pd()->attr()->zero_points_.has_default_values(DNNL_ARG_DST);
+    const int dst_zp_mask = pd()->attr()->zero_points_.get_mask(DNNL_ARG_DST);
 
     const memory_desc_wrapper dst_d(pd()->dst_md());
 
@@ -232,20 +236,6 @@ status_t ref_deconvolution_fwd_t::compute_ref_attrs(const exec_ctx_t &ctx,
     const auto OCP = dst_d.padded_dims()[1];
     const auto ndims = pd()->desc()->src_desc.ndims;
 
-    const auto maybe_dst_zero_point = [=](float &result, dim_t oc) {
-        if (is_dst_zp_common)
-            result += dst_zero_point[0];
-        else
-            result += dst_zero_point[oc];
-    };
-
-    const auto maybe_scale
-            = [](float &d, dim_t oc, const float *scales, int mask) {
-                  // scale_idx_mult = 1 for per_oc scales and 0, otherwise
-                  const int scale_idx_mult = mask != 0;
-                  d *= scales[oc * scale_idx_mult];
-              };
-
     const auto sum_dt = pd()->attr()->post_ops_.get_sum_dt(dst_d.data_type());
 
     parallel_nd(MB, OCP, OD, OH, OW,
@@ -269,8 +259,13 @@ status_t ref_deconvolution_fwd_t::compute_ref_attrs(const exec_ctx_t &ctx,
                     args.l_offset = dst_l_off;
                     args.dst_md = pd()->dst_md();
                     ref_post_ops->execute(tmp_result, args);
-                    maybe_scale(tmp_result, ocp, dst_scales, dst_scale_mask);
-                    maybe_dst_zero_point(tmp_result, ocp);
+                    if (has_dst_scales) {
+                        // scale_idx_mult = 1 for per_oc scales and 0, otherwise
+                        tmp_result *= dst_scales[ocp * (dst_scale_mask > 0)];
+                    }
+                    if (has_dst_zp) {
+                        tmp_result += dst_zero_points[ocp * (dst_zp_mask > 0)];
+                    }
                 }
                 io::store_float_value(
                         dst_d.data_type(), tmp_result, dst, dst_off);
@@ -299,8 +294,8 @@ dim_t get_weights_off(const memory_desc_wrapper &wei_d, bool with_groups,
 
 template <data_type_t wei_type>
 static void compute_src_zp_compensation(const exec_ctx_t &ctx,
-        const int32_t *src_zero_point, const bool is_src_zp_common,
-        typename prec_traits<wei_type>::type *wei,
+        const int32_t *src_zero_points, const bool is_src_zp_common,
+        typename prec_traits_t<wei_type>::type *wei,
         const cpu_deconvolution_fwd_pd_t *pd) {
     using namespace memory_tracking::names;
 
@@ -333,9 +328,9 @@ static void compute_src_zp_compensation(const exec_ctx_t &ctx,
                 const int32_t wei32 = static_cast<int32_t>(wei[weights_offset]);
 
                 if (is_src_zp_common)
-                    acc += wei32 * src_zero_point[0];
+                    acc += wei32 * src_zero_points[0];
                 else
-                    acc += wei32 * src_zero_point[g * IC + ic];
+                    acc += wei32 * src_zero_points[g * IC + ic];
             }
         }
 
@@ -346,8 +341,9 @@ static void compute_src_zp_compensation(const exec_ctx_t &ctx,
 template <data_type_t wei_type>
 static std::function<int32_t(
         const dim_t, const dim_t, const dim_t, const dim_t, const dim_t)>
-prepare_zp_pad_comp_ker(const dim_t ndims, const int32_t *src_zero_point,
-        const bool is_src_zp_common, typename prec_traits<wei_type>::type *wei,
+prepare_zp_pad_comp_ker(const dim_t ndims, const int32_t *src_zero_points,
+        const bool is_src_zp_common,
+        typename prec_traits_t<wei_type>::type *wei,
         const cpu_deconvolution_fwd_pd_t *deconv_pd) {
 
     const auto KH = deconv_pd->KH();
@@ -406,7 +402,7 @@ prepare_zp_pad_comp_ker(const dim_t ndims, const int32_t *src_zero_point,
                                 zp_pad_compensation += wei32;
                             else
                                 zp_pad_compensation
-                                        += wei32 * src_zero_point[g * IC + ic];
+                                        += wei32 * src_zero_points[g * IC + ic];
                         }
                     }
                 }
@@ -414,7 +410,7 @@ prepare_zp_pad_comp_ker(const dim_t ndims, const int32_t *src_zero_point,
         }
 
         if (is_src_zp_common && zp_pad_compensation)
-            zp_pad_compensation *= src_zero_point[0];
+            zp_pad_compensation *= src_zero_points[0];
 
         return zp_pad_compensation;
     };
@@ -422,35 +418,34 @@ prepare_zp_pad_comp_ker(const dim_t ndims, const int32_t *src_zero_point,
 
 template <data_type_t wei_type>
 static status_t apply_src_zero_point(const exec_ctx_t &ctx,
-        const cpu_deconvolution_fwd_pd_t *deconv_pd, float *conv_output) {
-    using wei_data_t = typename prec_traits<wei_type>::type;
+        const cpu_deconvolution_fwd_pd_t *pd, float *conv_output) {
+    using wei_data_t = typename prec_traits_t<wei_type>::type;
     using namespace memory_tracking::names;
     using namespace data_type;
 
-    // required by DEFINE_ZERO_POINTS_BUFFER macro
-    const auto pd = [&]() { return deconv_pd; };
     const auto wei = CTX_OUT_MEM(wei_data_t *, DNNL_ARG_WEIGHTS);
-    DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
     const bool is_src_zp_common
-            = deconv_pd->attr()->zero_points_.common(DNNL_ARG_SRC);
+            = pd->attr()->zero_points_.get_mask(DNNL_ARG_SRC) == 0;
 
     const auto scratchpad = ctx.get_scratchpad_grantor();
     const int32_t *const zp_src_compensation
             = scratchpad.get<int32_t>(key_deconv_zp);
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd->dst_md());
     const auto ndims = dst_d.ndims();
 
-    const auto G = pd()->G();
-    const auto MB = pd()->MB();
-    const auto OH = pd()->OH();
-    const auto OW = pd()->OW();
-    const auto OD = pd()->OD();
-    const auto OC = pd()->OC() / G;
+    const auto G = pd->G();
+    const auto MB = pd->MB();
+    const auto OH = pd->OH();
+    const auto OW = pd->OW();
+    const auto OD = pd->OD();
+    const auto OC = pd->OC() / G;
 
     compute_src_zp_compensation<wei_type>(
-            ctx, src_zero_point, is_src_zp_common, wei, deconv_pd);
+            ctx, src_zero_points, is_src_zp_common, wei, pd);
     const auto zp_pad_comp_ker = prepare_zp_pad_comp_ker<wei_type>(
-            ndims, src_zero_point, is_src_zp_common, wei, deconv_pd);
+            ndims, src_zero_points, is_src_zp_common, wei, pd);
 
     parallel_nd(MB, G, OC, OD, OH, OW,
             [&](const dim_t mb, const dim_t g, const dim_t oc, const dim_t od,
@@ -487,9 +482,11 @@ status_t ref_deconvolution_fwd_t::execute(const exec_ctx_t &ctx) const {
 
     // Create intermediate memory for f32 output if needed.
     auto dst = args.at(DNNL_ARG_DST);
-    memory_t tmp_memory(dst.mem->engine(), pd()->conv_pd_->diff_src_md(),
-            scratchpad.get_memory_storage(key_deconv_bias));
-    memory_arg_t tmp_conv_output = {&tmp_memory, false};
+    std::unique_ptr<memory_t, memory_deleter_t> tmp_memory;
+    CHECK(safe_ptr_assign(tmp_memory,
+            new memory_t(dst.mem->engine(), pd()->conv_pd_->diff_src_md(),
+                    scratchpad.get_memory_storage(key_deconv_bias))));
+    memory_arg_t tmp_conv_output = {tmp_memory.get(), false};
 
     conv_args[DNNL_ARG_DIFF_SRC]
             = ref_bias || non_default_attr ? tmp_conv_output : dst;
@@ -534,11 +531,10 @@ status_t ref_deconvolution_fwd_t::execute(const exec_ctx_t &ctx) const {
 
     float *conv_output = scratchpad.get<float>(key_deconv_bias);
 
-    const auto &arg_scales = pd()->attr()->scales_;
-    const auto &src_scales = arg_scales.get(DNNL_ARG_SRC);
-    const auto &wei_scales = arg_scales.get(DNNL_ARG_WEIGHTS);
+    const auto &scales = pd()->attr()->scales_;
 
-    if (!src_scales.has_default_values() || !wei_scales.has_default_values()) {
+    if (!scales.has_default_values(DNNL_ARG_SRC)
+            || !scales.has_default_values(DNNL_ARG_WEIGHTS)) {
         compute_oscale(ctx, conv_output);
     }
 
@@ -599,8 +595,8 @@ void ref_deconvolution_bwd_weights_t::compute_bwd_bias(
 
 template <data_type_t dbia_type, data_type_t ddst_type>
 void ref_deconvolution_bwd_weights_t::compute_bwd_bias_ncdhw(
-        typename prec_traits<dbia_type>::type *diff_bias,
-        const typename prec_traits<ddst_type>::type *diff_dst) const {
+        typename prec_traits_t<dbia_type>::type *diff_bias,
+        const typename prec_traits_t<ddst_type>::type *diff_dst) const {
     const memory_desc_wrapper diff_dst_d(pd()->diff_dst_md());
 
     const auto OC = pd()->OC();
@@ -622,8 +618,8 @@ void ref_deconvolution_bwd_weights_t::compute_bwd_bias_ncdhw(
 
 template <data_type_t dbia_type, data_type_t ddst_type>
 void ref_deconvolution_bwd_weights_t::compute_bwd_bias_ndhwc(
-        typename prec_traits<dbia_type>::type *diff_bias,
-        const typename prec_traits<ddst_type>::type *diff_dst) const {
+        typename prec_traits_t<dbia_type>::type *diff_bias,
+        const typename prec_traits_t<ddst_type>::type *diff_dst) const {
     const auto MB = pd()->MB();
     const auto SP = pd()->OW() * pd()->OH() * pd()->OD();
     const auto OC = pd()->OC();
@@ -637,14 +633,15 @@ void ref_deconvolution_bwd_weights_t::compute_bwd_bias_ndhwc(
                 db += diff_dst[offset];
             }
         }
-        diff_bias[oc] = static_cast<typename prec_traits<dbia_type>::type>(db);
+        diff_bias[oc]
+                = static_cast<typename prec_traits_t<dbia_type>::type>(db);
     });
 }
 
 template <data_type_t dbia_type, data_type_t ddst_type, dim_t blksize>
 void ref_deconvolution_bwd_weights_t::compute_bwd_bias_nCdhwXc(
-        typename prec_traits<dbia_type>::type *diff_bias,
-        const typename prec_traits<ddst_type>::type *diff_dst) const {
+        typename prec_traits_t<dbia_type>::type *diff_bias,
+        const typename prec_traits_t<ddst_type>::type *diff_dst) const {
     const memory_desc_wrapper diff_dst_d(pd()->diff_dst_md());
 
     const auto OC = pd()->OC();
@@ -677,8 +674,8 @@ void ref_deconvolution_bwd_weights_t::compute_bwd_bias_nCdhwXc(
 template <data_type_t dbia_type, data_type_t ddst_type>
 void ref_deconvolution_bwd_weights_t::compute_bias(
         const exec_ctx_t &ctx) const {
-    using dbia_data_t = typename prec_traits<dbia_type>::type;
-    using ddst_data_t = typename prec_traits<ddst_type>::type;
+    using dbia_data_t = typename prec_traits_t<dbia_type>::type;
+    using ddst_data_t = typename prec_traits_t<ddst_type>::type;
 
     auto diff_bias = CTX_OUT_MEM(dbia_data_t *, DNNL_ARG_DIFF_BIAS);
     auto diff_dst = CTX_IN_MEM(const ddst_data_t *, DNNL_ARG_DIFF_DST);
