@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,16 +27,16 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
-int jit_avx2_kernel_sgemm_kern::next_acc(int idx, int um, int un) const {
+int jit_avx2_kernel_sgemm_kern_t::next_acc(int idx, int um, int un) const {
     while (!(((idx / unroll_n_) < std::max(1, um / nelt_per_vecreg_))
             || ((idx % unroll_n_) < un)))
         idx++;
     return idx;
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchB_beforeBload(
+void jit_avx2_kernel_sgemm_kern_t::prefetchB_beforeBload(
         int um, int un, int k_idx, int n_idx) {
-    if (!mayiuse(avx512_core)) {
+    if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512)) {
         if ((n_idx == 0) && (k_idx == 0) && (un == unroll_n_) && (um != 16)) {
             prefetcht0(ptr[BO_ + elt_size_ * (PREFETCHSIZEB_ + offb_)]);
             offb_ += 16;
@@ -44,9 +44,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchB_beforeBload(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchB_beforeFMA(
+void jit_avx2_kernel_sgemm_kern_t::prefetchB_beforeFMA(
         int um, int un, int k_idx, int n_idx, int m_idx) {
-    if (!mayiuse(avx512_core)) {
+    if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512)) {
         if ((um == 16) || (un < unroll_n_)) {
             if ((k_idx + m_idx + n_idx) == 0) {
                 prefetcht0(ptr[BO_ + elt_size_ * (PREFETCHSIZEB_ + offb_)]);
@@ -61,9 +61,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchB_beforeFMA(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchA_afterFMA(
+void jit_avx2_kernel_sgemm_kern_t::prefetchA_afterFMA(
         int um, int un, int k_idx, int n_idx, int m_idx) {
-    if (mayiuse(avx512_core)) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         if ((um < unroll_m_) && (m_idx == 0)) {
             if (((k_idx % (nb_zmm_a_ / unroll_m_reg_) == 0) && (n_idx % 6 == 0))
                     || ((k_idx % (nb_zmm_a_ / unroll_m_reg_) == 1)
@@ -85,9 +85,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchA_afterFMA(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchA_afterBload(
+void jit_avx2_kernel_sgemm_kern_t::prefetchA_afterBload(
         int um, int un, int k_idx, int n_idx) {
-    if (!mayiuse(avx512_core)) {
+    if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512)) {
         if ((um == unroll_m_) && (un == 2)) {
             if (k_idx % 3 == 0) {
                 if (n_idx == 1) {
@@ -109,9 +109,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchA_afterBload(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchB_afterFMA(
+void jit_avx2_kernel_sgemm_kern_t::prefetchB_afterFMA(
         int k_idx, int n_idx, int m_idx) {
-    if (mayiuse(avx512_core)) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         if (((m_idx + (k_idx % (nb_zmm_a_ / unroll_m_reg_)) * unroll_m_reg_)
                     == 0)
                 && (n_idx == 1)) {
@@ -124,9 +124,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchB_afterFMA(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchA_beforeFMA(
+void jit_avx2_kernel_sgemm_kern_t::prefetchA_beforeFMA(
         int um, int un, int k_idx, int n_idx, int m_idx) {
-    if (!mayiuse(avx512_core)) {
+    if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512)) {
         if ((um == unroll_m_) && (un == unroll_n_)) {
             if (((k_idx == 0) && (n_idx % 2 == 1) && (m_idx == 0))
                     || ((k_idx == 1) && (n_idx == 2) && (m_idx == 0))
@@ -158,9 +158,9 @@ void jit_avx2_kernel_sgemm_kern::prefetchA_beforeFMA(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchC_afterBload(
+void jit_avx2_kernel_sgemm_kern_t::prefetchC_afterBload(
         int um, int un, int k_idx, int n_idx) {
-    if (mayiuse(avx512_core)) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         if (um == unroll_m_) {
             if (n_idx == std::min(1, un - 1)) {
                 if (k_idx == unroll_k_ - 1)
@@ -172,8 +172,8 @@ void jit_avx2_kernel_sgemm_kern::prefetchC_afterBload(
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::prefetchC_beforeKloop(int um) {
-    if (mayiuse(avx512_core)) {
+void jit_avx2_kernel_sgemm_kern_t::prefetchC_beforeKloop(int um) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         if (um < unroll_m_) {
             prefetchw(ptr[CO2_ + elt_size_ * 0]);
             prefetchw(ptr[CO2_ + elt_size_ * 8]);
@@ -199,7 +199,7 @@ void jit_avx2_kernel_sgemm_kern::prefetchC_beforeKloop(int um) {
     }
 }
 
-void jit_avx2_kernel_sgemm_kern::generate() {
+void jit_avx2_kernel_sgemm_kern_t::generate() {
 
     int i, unroll_x, unroll_y, uy_bin, ux_bin;
     int C_off = is_windows ? 56 : 8;
@@ -228,7 +228,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
     mov(C_, ptr[rsp + get_size_of_abi_save_regs() + C_off]);
     mov(LDC_, ptr[rsp + get_size_of_abi_save_regs() + LDC_off]);
 
-    if (mayiuse(avx512_core)) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         for (i = zmm_acc_idx_; i < unroll_m_reg_ * unroll_n_ + zmm_acc_idx_;
                 i++)
             vpxorq(Xbyak::Zmm(i), Xbyak::Zmm(i), Xbyak::Zmm(i));
@@ -267,7 +267,8 @@ void jit_avx2_kernel_sgemm_kern::generate() {
         add(AA_, A_);
         mov(CO1_, C_);
 
-        if ((unroll_x == unroll_m_) || (!mayiuse(avx512_core)))
+        if ((unroll_x == unroll_m_)
+                || !(mayiuse(avx512_core) && __BUILD_GEMM_AVX512))
             lea(CO2_, ptr[C_ + LDC_ * 2]);
 
         add(C_, unroll_x * elt_size_);
@@ -292,12 +293,12 @@ void jit_avx2_kernel_sgemm_kern::generate() {
                             T_NEAR);
             }
 
-            if (!mayiuse(avx512_core))
+            if (!(mayiuse(avx512_core) && __BUILD_GEMM_AVX512))
                 prefetcht2(ptr[AA_ - addr_off_ * elt_size_]);
 
             switch (unroll_x) {
                 case 8:
-                    if (mayiuse(avx512_core)) {
+                    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                         loop<Xbyak::Zmm, Xbyak::Zmm, Xbyak::Address, Xbyak::Xmm,
                                 Xbyak::Operand>(unroll_x, unroll_y,
                                 &Xbyak::CodeGenerator::vbroadcastf64x4,
@@ -319,7 +320,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
 
                     break;
                 case 4:
-                    if (mayiuse(avx512_core)) {
+                    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                         loop<Xbyak::Zmm, Xbyak::Ymm, Xbyak::Address, Xbyak::Xmm,
                                 Xbyak::Operand>(unroll_x, unroll_y,
                                 &Xbyak::CodeGenerator::vbroadcastf32x4,
@@ -340,7 +341,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
 
                     break;
                 case 2:
-                    if (mayiuse(avx512_core)) {
+                    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                         loop<Xbyak::Zmm, Xbyak::Ymm, Xbyak::Operand, Xbyak::Xmm,
                                 Xbyak::Operand>(unroll_x, unroll_y,
                                 &Xbyak::CodeGenerator::vbroadcastsd,
@@ -357,7 +358,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
                             &Xbyak::CodeGenerator::vmovsd);
                     break;
                 case 1:
-                    if (mayiuse(avx512_core)) {
+                    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                         loop<Xbyak::Zmm, Xbyak::Xmm, Xbyak::Operand, Xbyak::Xmm,
                                 Xbyak::Operand>(unroll_x, unroll_y,
                                 &Xbyak::CodeGenerator::vbroadcastss,
@@ -377,7 +378,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
 
                     break;
                 default:
-                    if (mayiuse(avx512_core)) {
+                    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                         loop<Xbyak::Zmm, Xbyak::Xmm, Xbyak::Operand, Xbyak::Xmm,
                                 Xbyak::Operand>(unroll_x, unroll_y,
                                 &Xbyak::CodeGenerator::vmovups,
@@ -400,7 +401,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
                     break;
             }
 
-            if (mayiuse(avx512_core)) {
+            if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
                 sub(AA_, -16 * elt_size_);
             } else {
                 if ((unroll_y != unroll_n_) || (unroll_x <= 4)) {
@@ -434,8 +435,8 @@ void jit_avx2_kernel_sgemm_kern::generate() {
     postamble();
 }
 
-jit_avx2_kernel_sgemm_kern::jit_avx2_kernel_sgemm_kern(bool beta_zero)
-    : jit_generator(jit_name()), beta_zero_(beta_zero) {}
+jit_avx2_kernel_sgemm_kern_t::jit_avx2_kernel_sgemm_kern_t(bool beta_zero)
+    : jit_generator_t(jit_name()), beta_zero_(beta_zero) {}
 
 } // namespace x64
 } // namespace cpu
