@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,7 +34,7 @@
 #include "cpu/x64/jit_avx512_common_1x1_conv_kernel.hpp"
 #include "cpu/x64/jit_uni_1x1_conv_utils.hpp"
 
-#define GET_OFF(field) offsetof(jit_1x1_conv_call_s, field)
+#define GET_OFF(field) offsetof(jit_1x1_conv_args_t, field)
 
 namespace dnnl {
 namespace impl {
@@ -47,10 +47,10 @@ using namespace dnnl::impl::utils;
 
 using namespace Xbyak;
 
-jit_avx512_common_1x1_conv_kernel::jit_avx512_common_1x1_conv_kernel(
+jit_avx512_common_1x1_conv_kernel_t::jit_avx512_common_1x1_conv_kernel_t(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(jit_name()), jcp(ajcp), attr_(attr) {
+    : jit_generator_t(jit_name()), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -73,7 +73,7 @@ jit_avx512_common_1x1_conv_kernel::jit_avx512_common_1x1_conv_kernel(
     }
 }
 
-void jit_avx512_common_1x1_conv_kernel::bcast_loop(int load_loop_blk) {
+void jit_avx512_common_1x1_conv_kernel_t::bcast_loop(int load_loop_blk) {
     mov(aux1_reg_bcast_data, EVEX_compress_addr(rsp, reg_bcast_data_off));
     mov(aux_reg_bcast_data, EVEX_compress_addr(rsp, reg_bcast_data_off));
 
@@ -130,7 +130,7 @@ void jit_avx512_common_1x1_conv_kernel::bcast_loop(int load_loop_blk) {
     }
 }
 
-Address jit_avx512_common_1x1_conv_kernel::output_ptr(
+Address jit_avx512_common_1x1_conv_kernel_t::output_ptr(
         const bool is_out_layout_nxc, const int i_load, const int i_ur) {
     if (one_of(jcp.prop_kind, forward_training, forward_inference,
                 backward_data)) {
@@ -167,7 +167,7 @@ static void iterate(const int load_loop_blk, const int ur, const F &fun) {
     iterate(load_loop_blk, ur, false, fun);
 }
 
-void jit_avx512_common_1x1_conv_kernel::apply_postops(
+void jit_avx512_common_1x1_conv_kernel_t::apply_postops(
         const bool is_out_layout_nxc, const int load_loop_blk, const int ur) {
     injector_utils::vmm_index_set_t vmm_idxs;
     if (jcp.with_binary) {
@@ -209,7 +209,7 @@ void jit_avx512_common_1x1_conv_kernel::apply_postops(
     }
 }
 
-void jit_avx512_common_1x1_conv_kernel::reduce_loop(
+void jit_avx512_common_1x1_conv_kernel_t::reduce_loop(
         int load_loop_blk, int ur, int substep, bool wraparound) {
     const bool out_layout_nxc = is_out_layout_nxc(jcp);
     const bool load_layout_nxc = is_load_layout_nxc(jcp);
@@ -339,7 +339,7 @@ void jit_avx512_common_1x1_conv_kernel::reduce_loop(
         };
 
         Label unaligned_store, end_store;
-        test(aux_reg_output_data, cpu_isa_traits<avx512_core>::vlen - 1);
+        test(aux_reg_output_data, cpu_isa_traits_t<avx512_core>::vlen - 1);
         jnz(unaligned_store, T_NEAR);
         store_output(true);
         jmp(end_store, T_NEAR);
@@ -406,7 +406,7 @@ void jit_avx512_common_1x1_conv_kernel::reduce_loop(
     store();
 }
 
-void jit_avx512_common_1x1_conv_kernel::generate() {
+void jit_avx512_common_1x1_conv_kernel_t::generate() {
     preamble();
 
     sub(rsp, stack_space_needed);
@@ -556,20 +556,26 @@ void jit_avx512_common_1x1_conv_kernel::generate() {
         postops_injector_->prepare_table(/* generate = */ true);
 }
 
-status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
-        const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
-        const memory_desc_wrapper &weights_d, const memory_desc_wrapper &dst_d,
-        const primitive_attr_t &attr, int nthreads, bool reduce_src) {
+status_t jit_avx512_common_1x1_conv_kernel_t::init_conf(
+        jit_1x1_conv_conf_t &jcp, const convolution_desc_t &cd,
+        const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
+        const memory_desc_wrapper &dst_d, const primitive_attr_t &attr,
+        int nthreads, bool reduce_src) {
     if (!mayiuse(avx512_core)) return status::unimplemented;
 
-    if (!everyone_is(data_type::f32, src_d.data_type(), weights_d.data_type(),
-                dst_d.data_type()))
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(everyone_is(data_type::f32, src_d.data_type(),
+                              weights_d.data_type(), dst_d.data_type()),
+            VERBOSE_UNSUPPORTED_DT_CFG);
+
+    // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
+    // TODO: change data type of jcp fields to size_t
+    VDISPATCH_CONV_IC(!has_large_size(cd, src_d, weights_d, dst_d),
+            VERBOSE_BAD_PARAM, "large size is not supported");
 
     jcp.nthr = nthreads;
 
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
-    const int simd_w = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
+    const int simd_w = cpu_isa_traits_t<avx512_core>::vlen / sizeof(float);
     const int ndims = src_d.ndims();
 
     jcp.prop_kind = cd.prop_kind;
@@ -616,9 +622,9 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
     const int eltwise_ind
             = post_ops.find(primitive_kind::eltwise, 0, dw_conv_ind);
     jcp.with_eltwise = eltwise_ind != -1;
-    if (jcp.with_eltwise) {
-        if (dst_d.data_type() == data_type::s32) return status::unimplemented;
-    }
+    if (jcp.with_eltwise)
+        VDISPATCH_CONV_IC(
+                dst_d.data_type() != data_type::s32, VERBOSE_UNSUPPORTED_DT);
 
     const int sum_ind = post_ops.find(primitive_kind::sum, 0, dw_conv_ind);
     jcp.with_sum = sum_ind != -1;
@@ -638,8 +644,8 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
 
     const auto dat_tag_nxc = pick(ndims - 3, nwc, nhwc, ndhwc);
     const auto dat_tag_nCx16c = pick(ndims - 3, nCw16c, nChw16c, nCdhw16c);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx16c);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx16c);
     bool is_data_layout_nxc
             = utils::everyone_is(dat_tag_nxc, jcp.src_tag, jcp.dst_tag);
     auto required_dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
@@ -656,20 +662,35 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = true;
     static constexpr bool sum_requires_zp_zero = true;
-    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
+    bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
             {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
             sum_requires_scale_one, sum_requires_zp_zero));
-    if (!post_ops_ok_) return status::unimplemented;
+    // temporary workaround that skips avx512 implementation for ternary
+    // post-ops with scalar broadcasting to avoid register collisions.
+    post_ops_ok_ = post_ops_ok_
+            && IMPLICATION(jcp.with_binary,
+                    !binary_injector::
+                            any_binary_postop_rhs_with_ternary_scalar_bcast(
+                                    post_ops, dst_d));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
-    bool args_ok = true && jcp.ngroups == 1 && jcp.src_tag == required_dat_tag
-            && jcp.dst_tag == required_dat_tag
-            && IMPLICATION(!is_data_layout_nxc,
-                    jcp.oc % simd_w == 0 && jcp.ic % simd_w == 0)
-            && jcp.f_pad == 0 && jcp.t_pad == 0 && jcp.l_pad == 0
-            && jcp.stride_w == 1 && jcp.stride_h == 1 && jcp.stride_d == 1
-            && jcp.kd == 1 && jcp.kh == 1 && jcp.kw == 1 && jcp.ow == jcp.iw
-            && jcp.oh == jcp.ih && jcp.od == jcp.id; // enforce rpad=0
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.ngroups == 1, VERBOSE_BAD_PARAM, "number of groups");
+    VDISPATCH_CONV_IC(
+            jcp.src_tag == required_dat_tag && jcp.dst_tag == required_dat_tag,
+            VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_CONV_IC(IMPLICATION(!is_data_layout_nxc,
+                              jcp.oc % simd_w == 0 && jcp.ic % simd_w == 0),
+            VERBOSE_BLOCKING_FAIL, "bad blocking dimensions");
+    VDISPATCH_CONV_IC(jcp.f_pad == 0 && jcp.t_pad == 0 && jcp.l_pad == 0,
+            VERBOSE_UNSUPPORTED_PAD_FEATURE, "f,t,l padding not supported");
+    VDISPATCH_CONV_IC(
+            jcp.stride_w == 1 && jcp.stride_h == 1 && jcp.stride_d == 1,
+            VERBOSE_UNSUPPORTED_FEATURE, "non-unit strides");
+    VDISPATCH_CONV_IC(jcp.kd == 1 && jcp.kh == 1 && jcp.kw == 1,
+            VERBOSE_BAD_PARAM, "non-unit kernel dimensions");
+    VDISPATCH_CONV_IC(jcp.ow == jcp.iw && jcp.oh == jcp.ih && jcp.od == jcp.id,
+            VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "inconsistent i/o dimensions"); // enforce rpad=0
 
     jcp.ic_block = jcp.oc_block = simd_w;
 
@@ -681,18 +702,19 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
                     IOhw16o16i, OIdhw16i16o, IOdhw16o16i);
 
     jcp.wei_tag = weights_d.matches_one_of_tag(wei_tag);
-    if (jcp.wei_tag != wei_tag) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.wei_tag == wei_tag, VERBOSE_UNSUPPORTED_TAG_S, "wei");
 
-    jcp.typesize_in = sizeof(prec_traits<data_type::f32>::type);
-    jcp.typesize_out = sizeof(prec_traits<data_type::f32>::type);
+    jcp.typesize_in = sizeof(prec_traits_t<data_type::f32>::type);
+    jcp.typesize_out = sizeof(prec_traits_t<data_type::f32>::type);
 
     /* once all the formats are set, check the padding consistency */
     if (!is_data_layout_nxc) {
-        args_ok = true && jcp.ic <= src_d.padded_dims()[1]
+        const bool args_ok = true && jcp.ic <= src_d.padded_dims()[1]
                 && jcp.oc <= dst_d.padded_dims()[1]
                 && jcp.ic <= weights_d.padded_dims()[with_groups + 1]
                 && jcp.oc <= weights_d.padded_dims()[with_groups + 0];
-        if (!args_ok) return status::unimplemented;
+        VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+                "inconsistent padded dimensions");
     }
 
     const int SMALL_SPATIAL = 10;
@@ -1100,7 +1122,7 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
 
         reduce_blocking_max = rnd_dn(reduce_blocking * 3 / 2, jcp.reduce_block);
     } else
-        return status::unimplemented;
+        VDISPATCH_CONV_IC(false, VERBOSE_BAD_PROPKIND);
 
     assert(load_blocking);
     assert(load_blocking_max);
@@ -1134,7 +1156,7 @@ status_t jit_avx512_common_1x1_conv_kernel::init_conf(jit_1x1_conv_conf_t &jcp,
     return status::success;
 }
 
-void jit_avx512_common_1x1_conv_kernel::init_scratchpad(
+void jit_avx512_common_1x1_conv_kernel_t::init_scratchpad(
         memory_tracking::registrar_t &scratchpad,
         const jit_1x1_conv_conf_t &jcp) {
     using namespace dnnl::impl::memory_tracking::names;
@@ -1164,7 +1186,7 @@ void jit_avx512_common_1x1_conv_kernel::init_scratchpad(
     }
 }
 
-void jit_avx512_common_1x1_conv_kernel::balance(jit_1x1_conv_conf_t &jcp) {
+void jit_avx512_common_1x1_conv_kernel_t::balance(jit_1x1_conv_conf_t &jcp) {
     int nthreads = jcp.nthr;
     // initialize jcp reduction threading properties
     jcp.nthr = jcp.nthr_mb = jcp.nthr_g = jcp.nthr_oc_b = jcp.nthr_ic_b = 1;
