@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -167,6 +167,7 @@ struct jit_conv_conf_t {
     data_type_t dsrc_dt;
     data_type_t dwei_dt;
     bool is_bf32 {false};
+    bool is_tf32 {false};
     bool expl_bcast;
     bool large_spatial, large_w_filter;
     int is_ic_scale, is_oc_scale;
@@ -189,7 +190,11 @@ struct jit_conv_conf_t {
     bool req_zero_point_buffer; // used for calculating padding compensation
     bool zp_pbuff_outer_compute; // indicates if zp_bbuff is computed in
 
-    bool dst_scale;
+    bool dst_scale = false; // TODO: delete me
+
+    bool with_src_scales = false;
+    bool with_wei_scales = false;
+    bool with_dst_scales = false;
 
     // a separate parallel region
     int ow_pad, oh_pad, od_pad; // output elements with padding & filter overlap
@@ -276,117 +281,147 @@ inline status_t init_tag(format_tag_t &tag, const memory_desc_wrapper &mdw,
     return tag == tag_value ? status::success : status::unimplemented;
 }
 
-struct jit_conv_call_s {
-    const void *src; /* hack, non-const for backward_data */
-    const void *dst; /* hack, non-const for forward */
-    const void *filt; /* hack, non-const for backward_weights */
-    const void *bias; /* hack, non-const for backward_bias */
-    const void *src_prf;
-    const void *dst_prf;
-    const void *filt_prf;
-    const void *bias_prf;
-    const void *scales;
-    const void *acc_s32;
-    const void *compensation;
-    const int32_t *zp_compensation;
-    const int32_t *src_zero_point;
-    const int32_t *zero_point_pbuff;
-    const int32_t *dst_zero_point;
-    const void *tile_cfg;
-    const void *tile_cfg_tail;
-    const void *dst_scale;
+inline bool has_large_size(const convolution_desc_t &cd,
+        const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
+        const memory_desc_wrapper &dst_d) {
+    auto is_large = [](const dim_t val) { return val > INT_MAX; };
+    auto img_size = [](const memory_desc_wrapper &mem_d) {
+        // assuming that first dimension for src and dst is minibatch
+        const auto mb = mem_d.dims()[0] != 0 ? mem_d.dims()[0] : 1;
+        return mem_d.nelems() / mb;
+    };
+
+    // Check if numbers of elements (excepting minibatch) for any memory
+    // descriptor is greater than INT_MAX
+    if (is_large(img_size(src_d)) || is_large(weights_d.nelems())
+            || is_large(img_size(dst_d)))
+        return true;
+
+    const int ndims = src_d.ndims();
+    for (int d = 3; d <= ndims; d++) {
+        if (utils::one_of(true, is_large(cd.strides[ndims - d]),
+                    is_large(cd.padding[0][ndims - d]),
+                    is_large(cd.dilates[ndims - d])))
+            return true;
+    }
+    return false;
+}
+
+struct jit_conv_args_t {
+    const void *src = nullptr; /* hack, non-const for backward_data */
+    const void *dst = nullptr; /* hack, non-const for forward */
+    const void *filt = nullptr; /* hack, non-const for backward_weights */
+    const void *bias = nullptr; /* hack, non-const for backward_bias */
+    const void *src_prf = nullptr;
+    const void *dst_prf = nullptr;
+    const void *filt_prf = nullptr;
+    const void *bias_prf = nullptr;
+    const void *acc_s32 = nullptr;
+    const void *compensation = nullptr;
+    const int32_t *zp_compensation = nullptr;
+    const int32_t *src_zero_point = nullptr;
+    const int32_t *zero_point_pbuff = nullptr;
+    const int32_t *dst_zero_point = nullptr;
+    const void *tile_cfg = nullptr;
+    const void *tile_cfg_tail = nullptr;
+    const void *scales = nullptr; // TODO: delete me
+    const void *dst_scale = nullptr; // TODO: delete me
+
+    const void *src_scales = nullptr;
+    const void *wei_scales = nullptr;
+    const void *dst_scales = nullptr;
 
     // ptr to table of void * elements that are pointers to
     // post_op binary src1 tensors
-    const void *post_ops_binary_rhs_arg_vec;
-    const void *dst_orig; // pointer to dst memory (no offset)
+    const void *post_ops_binary_rhs_arg_vec = nullptr;
+    const void *dst_orig = nullptr; // pointer to dst memory (no offset)
 
-    size_t kd_offset;
-    size_t kd_offset_prf;
-    size_t kh_offset;
-    size_t kh_offset_prf;
-    size_t os_index_begin;
-    size_t os_index_begin_prf;
-    size_t os_index_end;
-    size_t os_index_end_prf;
-    size_t kd_padding;
-    size_t kd_padding_prf;
-    size_t kh_padding;
-    size_t kh_padding_prf;
-    size_t iwb;
-    size_t iwb_prf;
-    size_t owb;
-    size_t owb_prf;
-    size_t ohb;
-    size_t kw_padding;
-    size_t channel;
-    size_t channel_prf;
-    size_t ic_blocks;
-    size_t oc_blocks;
-    size_t ur_w;
-    size_t ur_str_w;
-    size_t ch_blocks;
-    size_t ch_blocks_prf;
-    size_t reduce_work;
-    size_t reduce_work_prf;
-    size_t load_work;
-    size_t load_work_prf;
-    size_t l_overflow;
-    size_t r_overflow;
-    size_t t_overflow;
-    size_t b_overflow;
-    size_t f_overflow;
-    size_t back_overflow;
-    size_t last_h;
-    size_t tail;
-    size_t current_iw;
-    size_t is_osb;
-    int flags;
-    int flags_prf;
-    int oc_flag;
-    size_t last_ic_block;
-    size_t last_oc_block;
+    size_t kd_offset = 0;
+    size_t kd_offset_prf = 0;
+    size_t kh_offset = 0;
+    size_t kh_offset_prf = 0;
+    size_t os_index_begin = 0;
+    size_t os_index_begin_prf = 0;
+    size_t os_index_end = 0;
+    size_t os_index_end_prf = 0;
+    size_t kd_padding = 0;
+    size_t kd_padding_prf = 0;
+    size_t kh_padding = 0;
+    size_t kh_padding_prf = 0;
+    size_t iwb = 0;
+    size_t iwb_prf = 0;
+    size_t owb = 0;
+    size_t owb_prf = 0;
+    size_t ohb = 0;
+    size_t kw_padding = 0;
+    size_t channel = 0;
+    size_t channel_prf = 0;
+    size_t ic_blocks = 0;
+    size_t oc_blocks = 0;
+    size_t ur_w = 0;
+    size_t ur_str_w = 0;
+    size_t ch_blocks = 0;
+    size_t ch_blocks_prf = 0;
+    size_t reduce_work = 0;
+    size_t reduce_work_prf = 0;
+    size_t load_work = 0;
+    size_t load_work_prf = 0;
+    size_t l_overflow = 0;
+    size_t r_overflow = 0;
+    size_t t_overflow = 0;
+    size_t b_overflow = 0;
+    size_t f_overflow = 0;
+    size_t back_overflow = 0;
+    size_t last_h = 0;
+    size_t tail = 0;
+    size_t current_iw = 0;
+    size_t is_osb = 0;
+    int flags = 0;
+    int flags_prf = 0;
+    int oc_flag = 0;
+    size_t last_ic_block = 0;
+    size_t last_oc_block = 0;
 };
 
-struct jit_deconv_call_s {
-    const void *src; /* hack, non-const for backward_data */
-    const void *dst; /* hack, non-const for forward */
-    const void *filt; /* hack, non-const for backward_weights */
-    const void *bias; /* hack, non-const for backward_bias */
-    const void *scales;
-    const void *dst_scale;
-    const void *compensation;
-    const int32_t *zp_src_pad_str_compensation;
-    const int32_t *zp_compensation;
-    const int32_t *src_zero_point;
-    const int32_t *dst_zero_point;
+struct jit_deconv_args_t {
+    const void *src = nullptr; /* hack, non-const for backward_data */
+    const void *dst = nullptr; /* hack, non-const for forward */
+    const void *filt = nullptr; /* hack, non-const for backward_weights */
+    const void *bias = nullptr; /* hack, non-const for backward_bias */
+    const void *scales = nullptr;
+    const void *dst_scale = nullptr;
+    const void *compensation = nullptr;
+    const int32_t *zp_src_pad_str_compensation = nullptr;
+    const int32_t *zp_compensation = nullptr;
+    const int32_t *src_zero_point = nullptr;
+    const int32_t *dst_zero_point = nullptr;
 
     /*
      * ptr to table of void * elements that are pointers to post_op binary
      * src1 tensors
      */
-    const void *post_ops_binary_rhs_arg_vec;
-    const void *dst_orig; /* pointer to dst memory (no offset) */
-    size_t t_overflow;
-    size_t b_overflow;
-    size_t f_overflow;
-    size_t back_overflow;
-    size_t kh_padding;
-    size_t kd_padding;
-    size_t oc_blocks;
+    const void *post_ops_binary_rhs_arg_vec = nullptr;
+    const void *dst_orig = nullptr; /* pointer to dst memory (no offset) */
+    size_t t_overflow = 0;
+    size_t b_overflow = 0;
+    size_t f_overflow = 0;
+    size_t back_overflow = 0;
+    size_t kh_padding = 0;
+    size_t kd_padding = 0;
+    size_t oc_blocks = 0;
 };
 
-struct jit_dw_conv_call_s {
-    const void *input;
-    const void *output;
-    const void *filter;
-    const void *bias;
-    size_t kh_count;
-    size_t oh_count;
-    size_t oh_index;
-    size_t filter_pad_off;
-    unsigned char
-            exec_flags; /* Flags passed by driver execution to inner kernel */
+struct jit_dw_conv_args_t {
+    const void *input = nullptr;
+    const void *output = nullptr;
+    const void *filter = nullptr;
+    const void *bias = nullptr;
+    size_t kh_count = 0;
+    size_t oh_count = 0;
+    size_t oh_index = 0;
+    size_t filter_pad_off = 0;
+    // Flags passed by driver execution to inner kernel
+    unsigned char exec_flags = 0;
 };
 
 struct jit_1x1_conv_conf_t {
@@ -451,38 +486,47 @@ struct jit_1x1_conv_conf_t {
     bool dst_zero_point;
     bool zp_src_is_common; // common, otherwise (TODO) per-channel
 
-    bool dst_scale;
+    bool dst_scale; // TODO: delete me
+
+    bool with_src_scales = false;
+    bool with_wei_scales = false;
+    bool with_dst_scales = false;
 
     cpu_isa_t isa;
     bool uses_permw_transposition;
 };
 
-struct jit_1x1_conv_call_s {
-    const void *bcast_data;
-    const void *load_data;
-    const void *output_data;
-    const void *bias_data; // used in forward and backward_weights only
-    const void *acc_s32;
-    const void *scales;
-    const void *compensation;
-    const void *store_buffer;
-    const int32_t *zp_compensation;
-    const int32_t *src_zero_point;
-    const int32_t *dst_zero_point;
-    const void *dst_scale;
+struct jit_1x1_conv_args_t {
+    const void *bcast_data = nullptr;
+    const void *load_data = nullptr;
+    const void *output_data = nullptr;
+    // Used in forward and backward_weights only
+    const void *bias_data = nullptr;
+    const void *acc_s32 = nullptr;
+    const void *compensation = nullptr;
+    const void *store_buffer = nullptr;
+    const int32_t *zp_compensation = nullptr;
+    const int32_t *src_zero_point = nullptr;
+    const int32_t *dst_zero_point = nullptr;
+    const void *scales = nullptr; // TODO: delete me
+    const void *dst_scale = nullptr; // TODO: delete me
+
+    const void *src_scales = nullptr;
+    const void *wei_scales = nullptr;
+    const void *dst_scales = nullptr;
 
     // ptr to table of void * elements that are pointers to
     // post_op binary src1 tensors
-    const void *post_ops_binary_rhs_arg_vec;
-    const void *dst_orig; // pointer to dst memory (no offset)
+    const void *post_ops_binary_rhs_arg_vec = nullptr;
+    const void *dst_orig = nullptr; // pointer to dst memory (no offset)
 
-    size_t load_dim;
-    size_t bcast_dim;
-    size_t reduce_dim;
+    size_t load_dim = 0;
+    size_t bcast_dim = 0;
+    size_t reduce_dim = 0;
 
-    size_t output_stride; // used in backward_weights only
+    size_t output_stride = 0; // used in backward_weights only
 
-    size_t first_last_flag;
+    size_t first_last_flag = 0;
 };
 
 struct jit_pool_conf_t {
@@ -526,30 +570,32 @@ struct jit_pool_conf_t {
     bool with_binary;
     int nthr;
     memory_desc_t tmp_md;
+    bool needs_f32_accum_for_bf16;
+    dim_t f32_accum_block_size;
 };
 
-struct jit_pool_call_s {
-    const void *src;
-    const void *dst;
-    const void *indices;
-    const void *src_prf;
-    const void *dst_prf;
-    const void *indices_prf;
-    const void *post_ops_binary_rhs_arg_vec;
-    const void *dst_orig;
-    const void *dst_po_helper;
-    size_t zero_ih;
-    size_t zero_id;
-    const void *zero_ptr;
-    size_t kd_padding;
-    size_t kh_padding;
-    size_t kh_padding_shift;
-    size_t kd_padding_shift;
-    size_t kw_padding;
-    const void *init_value;
-    float ker_area_h;
-    size_t ur_bc; // contains number of channel blocks to processing
-    size_t b_c; // contains number of channel blocks already processed
+struct jit_uni_pooling_args_t {
+    const void *src = nullptr;
+    const void *dst = nullptr;
+    const void *indices = nullptr;
+    const void *src_prf = nullptr;
+    const void *dst_prf = nullptr;
+    const void *indices_prf = nullptr;
+    const void *post_ops_binary_rhs_arg_vec = nullptr;
+    const void *dst_orig = nullptr;
+    const void *dst_po_helper = nullptr;
+    size_t zero_ih = 0;
+    size_t zero_id = 0;
+    const void *zero_ptr = nullptr;
+    size_t kd_padding = 0;
+    size_t kh_padding = 0;
+    size_t kh_padding_shift = 0;
+    size_t kd_padding_shift = 0;
+    size_t kw_padding = 0;
+    const void *init_value = nullptr;
+    float ker_area_h = 0.f;
+    size_t ur_bc = 0; // contains number of channel blocks to processing
+    size_t b_c = 0; // contains number of channel blocks already processed
 };
 
 struct jit_resampling_conf_t {
@@ -588,7 +634,7 @@ struct jit_resampling_conf_t {
 
     cpu_isa_t isa = isa_undef;
 
-    post_ops_t post_ops = post_ops_t();
+    post_ops_t post_ops;
     bool with_postops = false;
     bool with_eltwise = false;
     bool with_binary = false;
@@ -596,7 +642,7 @@ struct jit_resampling_conf_t {
     std::queue<float> sum_scales;
 };
 
-struct jit_resampling_call_s {
+struct jit_uni_resampling_args_t {
     size_t batch_of_sp_points_to_process = 0;
 
     const void *src = nullptr;
@@ -636,8 +682,8 @@ struct jit_brdgmm_conv_conf_t {
 
     bool with_bias;
     bool with_post_ops;
-    bool with_scale;
     bool is_oc_scale;
+    bool with_dst_scales;
 
     data_type_t src_dt;
     data_type_t wei_dt;
@@ -661,6 +707,7 @@ struct jit_brdgmm_conv_conf_t {
 enum conv_brgemm_loop_order_t {
     loop_ndhwgc,
     loop_ngcdhw,
+    loop_gcndhw,
 };
 
 enum conv_brgemm_exec_type_t {
@@ -749,11 +796,14 @@ struct jit_brgemm_conv_conf_t {
 
     bool use_buffer;
     dim_t buffer_size;
+    dim_t comp_ow_size;
     dim_t ker_ranges_size;
     dim_t comp_a_buffer_size;
     dim_t s8s8_comp_buffer_size;
 
-    bool with_scales;
+    bool with_src_scales;
+    bool with_wei_scales;
+    bool with_dst_scales;
     int is_ic_scale, is_oc_scale;
 
     int LDA, LDB, LDC, LDD;
@@ -792,9 +842,12 @@ struct jit_brgemm_conv_conf_t {
     bool dst_zero_point;
     bool req_brg_comp_pad;
     bool req_cal_comp_pad;
-    bool is_bf32;
+    bool is_bf32 {false};
+    bool is_tf32 {false};
     bool is_fp8 {false};
     bool is_fp8_convert {false};
+    bool is_f32_f16 {false};
+    bool is_f32_bf16 {false};
     bool comp_with_vpads;
 
     int nthr_mb, nthr_g, nthr_oc_b, nthr_ic_b, nthr_oh;
@@ -816,6 +869,8 @@ struct jit_brgemm_conv_conf_t {
     int vnni_block {1};
     bool has_uneven_iw;
     int trans_dim_koef {1};
+    bool extendable_k = false;
+    brgemm_kernel_loop_order_t brgemm_kernel_loop_order {brgemm_lo_default};
 };
 
 struct jit_shuffle_conf_t {
@@ -841,13 +896,13 @@ struct jit_shuffle_conf_t {
     cpu_isa_t isa = isa_undef;
 };
 
-struct jit_shuffle_call_s {
+struct jit_uni_shuffle_args_t {
     const void *src = nullptr;
     void *dst = nullptr;
     const void *input_off_ptr = nullptr;
 
-    dim_t cb_loop_size
-            = 0; // number of loop iterations over corresponding C batches
+    // Number of loop iterations over corresponding C batches.
+    dim_t cb_loop_size = 0;
     bool is_padded_block = false;
 };
 
@@ -864,8 +919,8 @@ enum class binary_bcast_t : unsigned {
 struct jit_binary_conf_t {
     binary_op_t op_type = binary_op_t::none;
     binary_bcast_t bcast_type = binary_bcast_t::none;
-    bool do_scale_src0 = false;
-    bool do_scale_src1 = false;
+    bool with_src0_scales = false;
+    bool with_src1_scales = false;
     bool do_sum = false;
     bool with_eltwise = false;
     bool with_binary = false;
@@ -878,6 +933,7 @@ struct jit_binary_conf_t {
     bool is_i8 = false;
     bool is_bf16 = false;
     bool is_f16 = false;
+    bool is_ternary_op = false;
     bool is_src_different_layouts = false;
     dim_t outer_dims = 1;
     int src1_stride = 1;
@@ -886,17 +942,23 @@ struct jit_binary_conf_t {
 
     data_type_t src0_type = data_type::undef;
     data_type_t src1_type = data_type::undef;
+    data_type_t src2_type = data_type::undef;
     data_type_t dst_type = data_type::undef;
 };
 
-struct jit_binary_call_s {
+struct jit_uni_binary_args_t {
     // keep all sizes at 8 bytes -- jit code expects this
-    const void *src0, *src1, *dst, *indices;
-    const float *scales_src0, *scales_src1;
-    size_t spat_offt_count;
-    const void *post_ops_binary_rhs_arg_vec;
-    size_t src1_stride_range;
-    const void *dst_orig;
+    const void *src0 = nullptr;
+    const void *src1 = nullptr;
+    const void *src2 = nullptr;
+    const void *dst = nullptr;
+    const void *indices = nullptr;
+    const void *scales_src0 = nullptr;
+    const void *scales_src1 = nullptr;
+    size_t spat_offt_count = 0;
+    const void *post_ops_binary_rhs_arg_vec = nullptr;
+    size_t src1_stride_range = 0;
+    const void *dst_orig = nullptr;
 };
 
 struct jit_reduction_conf_t {
@@ -916,7 +978,7 @@ struct jit_reduction_conf_t {
 
     bool is_saturation_needed = false;
 
-    post_ops_t post_ops = post_ops_t();
+    post_ops_t post_ops;
     bool with_postops = false;
     bool with_eltwise = false;
     bool with_binary = false;
@@ -924,7 +986,7 @@ struct jit_reduction_conf_t {
     std::queue<float> sum_scales;
 };
 
-struct jit_reduction_call_s {
+struct jit_uni_reduction_args_t {
     const void *src = nullptr;
     void *dst = nullptr;
     const void *post_ops_binary_rhs_arg_vec = nullptr;
