@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,7 +32,7 @@
 #include "cpu/x64/jit_uni_1x1_conv_utils.hpp"
 #include "cpu/x64/jit_uni_x8s8s32x_1x1_conv_kernel.hpp"
 
-#define GET_OFF(field) offsetof(jit_1x1_conv_call_s, field)
+#define GET_OFF(field) offsetof(jit_1x1_conv_args_t, field)
 
 namespace dnnl {
 namespace impl {
@@ -45,10 +45,11 @@ using namespace Xbyak;
 using namespace injector_utils;
 
 template <cpu_isa_t isa, typename Vmm>
-_jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::_jit_uni_x8s8s32x_1x1_conv_kernel(
-        const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
-        const memory_desc_t &dst_md)
-    : jit_generator(jit_name(), isa), jcp(ajcp), attr_(attr) {
+jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa,
+        Vmm>::jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t(const jit_1x1_conv_conf_t
+                                                             &ajcp,
+        const primitive_attr_t &attr, const memory_desc_t &dst_md)
+    : jit_generator_t(jit_name(), isa), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -69,14 +70,15 @@ _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::_jit_uni_x8s8s32x_1x1_conv_kernel(
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::cvt2ps(data_type_t type_in,
-        const Vmm &vmm_in, const Reg64 &reg, int offset, int load_size) {
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::cvt2ps(
+        data_type_t type_in, const Vmm &vmm_in, const Reg64 &reg, int offset,
+        int load_size) {
     load_data(type_in, vmm_in, reg, offset, load_size);
     if (type_in != data_type::f32) uni_vcvtdq2ps(vmm_in, vmm_in);
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::bcast_loop(
         int load_loop_blk) {
     mov(aux1_reg_bcast_data, reg_bcast_data);
     mov(aux_reg_bcast_data, reg_bcast_data);
@@ -113,7 +115,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
 }
 
 template <cpu_isa_t isa, typename Vmm>
-int _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::output_ptr(
+int jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::output_ptr(
         const int i_load, const int i_ur) {
     const size_t ur_stride = jcp.with_dw_conv
             ? jcp.nb_load_blocking * jcp.oc_block * i_ur
@@ -123,7 +125,7 @@ int _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::output_ptr(
 };
 
 template <cpu_isa_t isa, typename Vmm>
-int _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::vreg_accum_idx(
+int jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::vreg_accum_idx(
         const int load_loop_blk, const int i_load, const int i_ur) {
     const int vmm_idx = i_ur * load_loop_blk + i_load;
     assert(vmm_idx < ker_max_reg_idx);
@@ -131,7 +133,7 @@ int _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::vreg_accum_idx(
 };
 
 template <cpu_isa_t isa, typename Vmm>
-Vmm _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::vreg_accum(
+Vmm jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::vreg_accum(
         const int load_loop_blk, const int i_load, const int i_ur) {
     return Vmm(vreg_accum_idx(load_loop_blk, i_load, i_ur));
 };
@@ -144,7 +146,7 @@ void iterate(const int ur, const int load_loop_blk, const F &f) {
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_sum(const int ur,
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::apply_sum(const int ur,
         const int load_loop_blk, const bool mask_flag_in,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
 
@@ -186,8 +188,8 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_sum(const int ur,
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_postops(const int ur,
-        const int load_loop_blk, const bool mask_flag_in,
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::apply_postops(
+        const int ur, const int load_loop_blk, const bool mask_flag_in,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
 
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
@@ -241,7 +243,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_postops(const int ur,
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::reduce_loop(
         int load_loop_blk, int ur, bool wraparound) {
 
     // use 0x10001 to represent 2 words of 0x1
@@ -304,7 +306,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
         const int32_t *p_sum_zp
                 = (sum_idx != -1) ? &p.entry_[sum_idx].sum.zero_point : nullptr;
         mov(ptr[rsp + reg_bcast_data_off], reg_bcast_data);
-        mov(reg_ptr_scales, ptr[rsp + reg_ptr_sum_scale_off]);
         if (p_sum_scale && *p_sum_scale != 1.f) {
             mov(ptr[rsp + reg_load_data_off], reg_load_data);
             mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
@@ -319,14 +320,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
             }
             const bool mask_flag = mask_flag_in && i_load == load_loop_blk - 1;
             const int load_size = mask_flag ? get_tail_size() : simd_w;
-            const auto ptr_scales_offset
-                    = jcp.is_oc_scale * (sizeof(float) * jcp.oc_block * i_load);
-            if (jcp.with_bias) {
-                if (jcp.signed_input || jcp.dst_scale)
-                    mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
-                cvt2ps(jcp.bia_dt, vmm_bias, reg_bias_data,
-                        jcp.typesize_bia * jcp.oc_block * i_load, load_size);
-            }
             if (jcp.signed_input) {
                 mov(reg_comp_data, ptr[rsp + reg_comp_data_off]);
                 cvt2ps(data_type::s32, vmm_comp, reg_comp_data,
@@ -342,12 +335,75 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
                 uni_vcvtdq2ps(vmm_zp_comp, vmm_zp_comp);
             }
 
-            if (mask_flag) {
-                uni_vpxor(vmm_scale, vmm_scale, vmm_scale);
-                cvt2ps(data_type::f32, vmm_scale, reg_ptr_scales,
-                        ptr_scales_offset, get_tail_size());
-            } else {
-                uni_vmovups(vmm_scale, ptr[reg_ptr_scales + ptr_scales_offset]);
+            // TODO: scales support is done not in the most optimal way.
+            // If there're two free Vmm registers, one can be used to store
+            // scale_adjust value permanently, the second one can re-use data
+            // from it and multiply by src_scale that can be obtained at the
+            // point of scales loading. Then it can be used when multiplying
+            // by wei_scales. And further re-used for dst scales to avoid
+            // reading from the same address, but reading from the Vmm instead.
+            // This would save 1st and 3rd sections for every output Vmm.
+            //
+            // If only one Vmm is found, it will add scale_adjust overhead per
+            // src_scale loading, but the second part of the idea holds.
+            //
+            // Note: attempts to identify these Vmms were not taken.
+
+            // `avx2` is less flexible ISA in terms of tail and broadcast handling.
+            // Thus, need to save scales values in Vmm registers.
+            bool is_vmm_scales_set = false;
+            if (jcp.with_src_scales) {
+                mov(reg_src_scales, ptr[rsp + reg_src_scales_off]);
+                uni_vbroadcastss(vmm_scales, ptr[reg_src_scales]);
+                is_vmm_scales_set = true;
+            }
+            if (jcp.with_wei_scales) {
+                mov(reg_wei_scales, ptr[rsp + reg_wei_scales_off]);
+
+                if (!jcp.is_oc_scale) {
+                    uni_vbroadcastss(vmm_scales_tmp, ptr[reg_wei_scales]);
+                } else {
+                    int scale_offset = jcp.is_oc_scale
+                            * (sizeof(float) * jcp.oc_block * i_load);
+                    if (mask_flag) {
+                        uni_vpxor(
+                                vmm_scales_tmp, vmm_scales_tmp, vmm_scales_tmp);
+                        cvt2ps(data_type::f32, vmm_scales_tmp, reg_wei_scales,
+                                scale_offset, get_tail_size());
+                    } else {
+                        uni_vmovups(vmm_scales_tmp,
+                                ptr[reg_wei_scales + scale_offset]);
+                    }
+                }
+                if (is_vmm_scales_set) {
+                    uni_vmulps(vmm_scales, vmm_scales, vmm_scales_tmp);
+                } else {
+                    uni_vmovups(vmm_scales, vmm_scales_tmp);
+                }
+                is_vmm_scales_set = true;
+            }
+            if (jcp.wei_adj_scale != 1.f) {
+                mov(reg_scale_adjust, float2int(1.f / jcp.wei_adj_scale));
+                auto vmm_scale_adjust = vmm_scales_tmp;
+                auto xmm_scale_adjust = Xmm(vmm_scale_adjust.getIdx());
+                uni_vmovq(xmm_scale_adjust, reg_scale_adjust);
+                uni_vbroadcastss(vmm_scale_adjust, xmm_scale_adjust);
+                if (is_vmm_scales_set) {
+                    uni_vmulps(vmm_scales, vmm_scales, vmm_scale_adjust);
+                } else {
+                    uni_vmovups(vmm_scales, vmm_scale_adjust);
+                }
+                is_vmm_scales_set = true;
+            }
+
+            // The order of this load is important. `vmm_bias` is used as a
+            // temporary vector register for scales. Load bias data into it
+            // after scales are processed.
+            if (jcp.with_bias) {
+                if (jcp.signed_input || jcp.with_dst_scales)
+                    mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
+                cvt2ps(jcp.bia_dt, vmm_bias, reg_bias_data,
+                        jcp.typesize_bia * jcp.oc_block * i_load, load_size);
             }
 
             for (int i_ur = 0; i_ur < ur; ++i_ur) {
@@ -356,7 +412,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
                 if (jcp.signed_input) uni_vaddps(r, r, vmm_comp);
                 if (jcp.src_zero_point) uni_vaddps(r, r, vmm_zp_comp);
 
-                uni_vmulps(r, r, vmm_scale);
+                if (is_vmm_scales_set) uni_vmulps(r, r, vmm_scales);
 
                 if (jcp.with_bias) uni_vaddps(r, r, vmm_bias);
             }
@@ -364,15 +420,15 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
 
         apply_postops(ur, load_loop_blk, mask_flag_in, p_sum_scale, p_sum_zp);
 
-        if (jcp.dst_scale) {
-            mov(reg_ptr_dst_scale, ptr[rsp + reg_dst_scale_off]);
-            uni_vmovups(vmm_dst_scale, ptr[reg_ptr_dst_scale]);
+        if (jcp.with_dst_scales) {
+            mov(reg_dst_scales, ptr[rsp + reg_dst_scales_off]);
+            uni_vbroadcastss(vmm_dst_scales, ptr[reg_dst_scales]);
 
             /* Apply dst scale to accumulator */
             for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
                 for (int i_ur = 0; i_ur < ur; ++i_ur) {
                     const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-                    uni_vmulps(r, r, vmm_dst_scale);
+                    uni_vmulps(r, r, vmm_dst_scales);
                 }
             }
         }
@@ -511,7 +567,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
 }
 
 template <cpu_isa_t isa, typename Vmm>
-void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
+void jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<isa, Vmm>::generate() {
     preamble();
 
     sub(rsp, stack_space_needed);
@@ -528,17 +584,23 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         mov(reg_src_zero_point, ptr[param1 + GET_OFF(src_zero_point)]);
         mov(ptr[rsp + reg_src_zero_point_off], reg_src_zero_point);
     }
-    if (jcp.dst_scale) {
+    if (jcp.with_src_scales) {
+        mov(reg_src_scales, ptr[param1 + GET_OFF(src_scales)]);
+        mov(ptr[rsp + reg_src_scales_off], reg_src_scales);
+    }
+    if (jcp.with_wei_scales) {
+        mov(reg_wei_scales, ptr[param1 + GET_OFF(wei_scales)]);
+        mov(ptr[rsp + reg_wei_scales_off], reg_wei_scales);
+    }
+    if (jcp.with_dst_scales) {
         if (!jcp.signed_input) mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
-        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
-        mov(ptr[rsp + reg_dst_scale_off], reg_ptr_dst_scale);
+        mov(reg_dst_scales, ptr[param1 + GET_OFF(dst_scales)]);
+        mov(ptr[rsp + reg_dst_scales_off], reg_dst_scales);
     }
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         mov(ptr[rsp + reg_dst_zero_point_off], reg_dst_zero_point);
     }
-    mov(reg_ptr_scales, ptr[param1 + GET_OFF(scales)]);
-    mov(ptr[rsp + reg_ptr_sum_scale_off], reg_ptr_scales);
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
     mov(reg_load_data, ptr[param1 + GET_OFF(load_data)]);
     mov(reg_output_data, ptr[param1 + GET_OFF(output_data)]);
@@ -553,11 +615,11 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         if (jcp.with_bias) {
-            if (jcp.signed_input || jcp.dst_scale)
+            if (jcp.signed_input || jcp.with_dst_scales)
                 mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
             add(reg_bias_data,
                     load_loop_blk * jcp.load_block * jcp.typesize_bia);
-            if (jcp.signed_input || jcp.dst_scale)
+            if (jcp.signed_input || jcp.with_dst_scales)
                 mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
         }
         if (jcp.signed_input) {
@@ -573,11 +635,13 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
             mov(ptr[rsp + reg_zp_compensation_off], reg_zp_compensation);
         }
         mov(ptr[rsp + reg_bcast_data_off], reg_bcast_data);
-        mov(reg_ptr_scales, ptr[rsp + reg_ptr_sum_scale_off]);
-        add(reg_ptr_scales,
-                jcp.is_oc_scale * load_loop_blk * jcp.load_block
-                        * sizeof(float));
-        mov(ptr[rsp + reg_ptr_sum_scale_off], reg_ptr_scales);
+        if (jcp.with_wei_scales) {
+            mov(reg_wei_scales, ptr[rsp + reg_wei_scales_off]);
+            add(reg_wei_scales,
+                    jcp.is_oc_scale * load_loop_blk * jcp.load_block
+                            * sizeof(float));
+            mov(ptr[rsp + reg_wei_scales_off], reg_wei_scales);
+        }
         mov(reg_bcast_data, ptr[rsp + reg_bcast_data_off]);
         add(reg_output_data, load_loop_blk * jcp.load_block * jcp.typesize_out);
         sub(reg_load_loop_work, load_loop_blk * jcp.load_loop_iter_step);
@@ -632,7 +696,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
 }
 
 template <cpu_isa_t isa>
-status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
+status_t jit_uni_x8s8s32x_1x1_conv_kernel_t<isa>::init_conf(
         jit_1x1_conv_conf_t &jcp, const convolution_desc_t &cd,
         const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
         const memory_desc_wrapper &dst_d, const memory_desc_wrapper &bias_d,
@@ -640,6 +704,11 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(isa)) return status::unimplemented;
 
+    // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
+    // TODO: change data type of jcp fields to size_t
+    VDISPATCH_CONV_IC(!has_large_size(cd, src_d, weights_d, dst_d),
+            VERBOSE_BAD_PARAM, "large size is not supported");
+
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
     const bool dt_not_ok
             = !one_of(src_d.data_type(), data_type::u8, data_type::s8)
@@ -695,11 +764,11 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     const int sum_ind = post_ops.find(primitive_kind::sum, 0, dw_conv_ind);
     jcp.with_sum = sum_ind != -1;
 
-    const auto zp = attr.zero_points_;
+    const auto &zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
     jcp.zp_src_is_common
-            = zp.common(DNNL_ARG_SRC); // otherwise, it's per-channel
+            = zp.get_mask(DNNL_ARG_SRC) == 0; // otherwise, it's per-channel
     assert(IMPLICATION(jcp.src_zero_point, jcp.zp_src_is_common));
 
     VDISPATCH_CONV_IC(
@@ -708,8 +777,8 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
 
     format_tag_t dat_tag = utils::pick(
             ndims - 3, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag);
 
     bool args_ok = true && jcp.ngroups == 1 && jcp.src_tag == dat_tag
             && jcp.dst_tag == dat_tag;
@@ -908,10 +977,11 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     // miniumum size of load dim chunk for work distribution within threads
     jcp.nb_load_chunk = 1;
 
-    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
-    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
-    jcp.is_oc_scale = wei_scales.mask_ != 0;
-    jcp.dst_scale = !dst_scales.has_default_values();
+    jcp.is_oc_scale = attr.scales_.get_mask(DNNL_ARG_WEIGHTS) > 0;
+    jcp.with_src_scales = !attr.scales_.get(DNNL_ARG_SRC).has_default_values();
+    jcp.with_wei_scales
+            = !attr.scales_.get(DNNL_ARG_WEIGHTS).has_default_values();
+    jcp.with_dst_scales = !attr.scales_.get(DNNL_ARG_DST).has_default_values();
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
@@ -922,22 +992,22 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
 }
 
 template <cpu_isa_t isa>
-void jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_scratchpad(
+void jit_uni_x8s8s32x_1x1_conv_kernel_t<isa>::init_scratchpad(
         memory_tracking::registrar_t &scratchpad,
         const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr) {
     using namespace dnnl::impl::memory_tracking::names;
 
-    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
-    const dim_t scales_count
-            = wei_mask == 0 ? 1 : static_cast<dim_t>(jcp.oc) * jcp.ngroups;
-    const dim_t count = nstl::max<dim_t>(scales_count, 8);
-    scratchpad.book<float>(key_conv_adjusted_scales, count);
+    if (jcp.with_dst_scales) {
+        // See brgemm_types.hpp comment for `with_dst_scales`.
+        scratchpad.book(key_conv_dst_scales,
+                static_cast<size_t>(jcp.nthr) * sizeof(float), 4096);
+    }
 }
 
-template struct _jit_uni_x8s8s32x_1x1_conv_kernel<avx2, Ymm>;
-template struct _jit_uni_x8s8s32x_1x1_conv_kernel<sse41, Xmm>;
-template struct jit_uni_x8s8s32x_1x1_conv_kernel<avx2>;
-template struct jit_uni_x8s8s32x_1x1_conv_kernel<sse41>;
+template struct jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<avx2, Ymm>;
+template struct jit_uni_x8s8s32x_1x1_conv_kernel_vmm_t<sse41, Xmm>;
+template struct jit_uni_x8s8s32x_1x1_conv_kernel_t<avx2>;
+template struct jit_uni_x8s8s32x_1x1_conv_kernel_t<sse41>;
 } // namespace x64
 } // namespace cpu
 } // namespace impl
