@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,9 +16,23 @@
 
 #include "gpu/gpu_impl_list.hpp"
 
-#include "gpu/intel/ocl/gen9_binary.hpp"
-#include "gpu/intel/ocl/multi_po_reorder_binary.hpp"
-#include "gpu/intel/ocl/ref_binary.hpp"
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_INTEL
+#include "gpu/intel/binary/multi_po_reorder.hpp"
+#include "gpu/intel/binary/simple.hpp"
+#include "gpu/intel/binary/xe.hpp"
+#endif
+
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_NVIDIA
+#include "gpu/nvidia/cudnn_binary.hpp"
+#endif
+
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_AMD
+#include "gpu/amd/miopen_binary.hpp"
+#endif
+
+#ifdef GENERIC_SYCL_KERNELS_ENABLED
+#include "gpu/generic/sycl/ref_binary.hpp"
+#endif
 
 namespace dnnl {
 namespace impl {
@@ -28,9 +42,12 @@ namespace {
 
 // clang-format off
 constexpr impl_list_item_t impl_list[] = REG_BINARY_P({
-        INSTANCE(intel::ocl::multi_po_reorder_binary)
-        INSTANCE(intel::ocl::gen9_binary_t)
-        INSTANCE(intel::ocl::ref_binary_t)
+        GPU_INSTANCE_INTEL(intel::binary::multi_po_reorder_t)
+        GPU_INSTANCE_INTEL(intel::binary::xe_t)
+        GPU_INSTANCE_INTEL(intel::binary::simple_t)
+        GPU_INSTANCE_NVIDIA(nvidia::cudnn_binary_t)
+        GPU_INSTANCE_AMD(amd::miopen_binary_t)
+        GPU_INSTANCE_GENERIC_SYCL(generic::sycl::ref_binary_t)
         nullptr,
 });
 // clang-format on
