@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,13 +17,13 @@
 #include "gpu/gpu_impl_list.hpp"
 
 #if DNNL_GPU_VENDOR == DNNL_VENDOR_INTEL
-#include "gpu/intel/ocl/reduction/atomic_reduction.hpp"
-#include "gpu/intel/ocl/reduction/combined_reduction.hpp"
-#include "gpu/intel/ocl/reduction/ref_reduction.hpp"
-#include "gpu/intel/ocl/reduction/reusable_ref_reduction.hpp"
+#include "gpu/intel/reduction/atomic.hpp"
+#include "gpu/intel/reduction/combined.hpp"
+#include "gpu/intel/reduction/ref.hpp"
+#include "gpu/intel/reduction/reusable_ref.hpp"
 
 #ifdef DNNL_DEV_MODE
-#include "gpu/intel/jit/jit_reduction.hpp"
+#include "gpu/intel/reduction/jit.hpp"
 #endif
 
 #endif
@@ -36,6 +36,11 @@
 #include "gpu/amd/miopen_reduction.hpp"
 #endif
 
+#ifdef GENERIC_SYCL_KERNELS_ENABLED
+#include "gpu/generic/sycl/ref_reduction.hpp"
+#include "gpu/generic/sycl/simple_reduction.hpp"
+#endif
+
 namespace dnnl {
 namespace impl {
 namespace gpu {
@@ -44,13 +49,15 @@ namespace {
 
 // clang-format off
 constexpr impl_list_item_t impl_list[] = REG_REDUCTION_P({
-        GPU_INSTANCE_INTEL_DEVMODE(intel::jit::jit_reduction_t)
-        GPU_INSTANCE_INTEL(intel::ocl::atomic_reduction_t)
-        GPU_INSTANCE_INTEL(intel::ocl::combined_reduction_t)
-        GPU_INSTANCE_INTEL(intel::ocl::ref_reduction_t)
-        GPU_INSTANCE_INTEL(intel::ocl::reusable_ref_reduction_t)
+        GPU_INSTANCE_INTEL_DEVMODE(intel::reduction::gen_t)
+        GPU_INSTANCE_INTEL(intel::reduction::atomic_t)
+        GPU_INSTANCE_INTEL(intel::reduction::combined_t)
+        GPU_INSTANCE_INTEL(intel::reduction::ref_t)
+        GPU_INSTANCE_INTEL(intel::reduction::reusable_ref_t)
         GPU_INSTANCE_NVIDIA(nvidia::cudnn_reduction_t)
         GPU_INSTANCE_AMD(amd::miopen_reduction_t)
+        GPU_INSTANCE_GENERIC_SYCL(generic::sycl::ref_reduction_t)
+        GPU_INSTANCE_GENERIC_SYCL(generic::sycl::simple_reduction_t)
         nullptr,
 });
 // clang-format on
