@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2024 Intel Corporation
+* Copyright 2024-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,8 +18,10 @@
 
 #include "gpu/gpu_impl_list.hpp"
 
-#include "gpu/intel/ocl/micro_sdpa.hpp"
-#include "gpu/intel/ocl/ref_sdpa.hpp"
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_INTEL
+#include "gpu/intel/sdpa/micro.hpp"
+#include "gpu/intel/sdpa/ref.hpp"
+#endif
 
 namespace dnnl {
 namespace impl {
@@ -29,8 +31,8 @@ namespace {
 
 // clang-format off
 constexpr impl_list_item_t impl_list[] = REG_SDPA_P({
-        GPU_INSTANCE_INTEL(intel::ocl::micro_sdpa_t)
-        GPU_INSTANCE_INTEL_DEVMODE(intel::ocl::ref_sdpa_t)
+        GPU_INSTANCE_INTEL(intel::sdpa::micro_t)
+        GPU_INSTANCE_INTEL_DEVMODE(intel::sdpa::ref_t)
         nullptr,
 });
 // clang-format on
