@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 
 #include "gpu/gpu_impl_list.hpp"
 
-#include "gpu/intel/ocl/ref_zero_pad.hpp"
+#if DNNL_GPU_VENDOR == DNNL_VENDOR_INTEL
+#include "gpu/intel/zeropad/simple.hpp"
+#endif
 
 namespace dnnl {
 namespace impl {
@@ -26,7 +28,7 @@ namespace {
 
 // clang-format off
 constexpr impl_list_item_t impl_list[] = {
-        INSTANCE(intel::ocl::ref_zero_pad_t)
+        GPU_INSTANCE_INTEL(intel::zeropad::simple_t)
         nullptr,
 };
 // clang-format on
