@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 #include "gpu/intel/compute/device_info.hpp"
 #include "gpu/intel/jit/utils/ngen_type_bridge.hpp"
 #include "gpu/intel/utils.hpp"
+#include "ngen.hpp"
 
 #ifdef DNNL_WITH_SYCL
 #include "gpu/intel/sycl/engine.hpp"
@@ -39,12 +40,11 @@ uint64_t get_future_extensions(
 
     uint64_t extensions = 0;
     switch (gpu_arch) {
-        case gpu_arch_t::gen9:
-        case gpu_arch_t::gen11: break;
         case gpu_arch_t::xe_hp:
         case gpu_arch_t::xe_hpg:
         case gpu_arch_t::xe2:
         case gpu_arch_t::xe_hpc:
+        case gpu_arch_t::xe3:
             extensions |= (uint64_t)device_ext_t::intel_global_float_atomics;
             extensions
                     |= (uint64_t)device_ext_t::intel_variable_eu_thread_count;
@@ -64,11 +64,47 @@ uint64_t get_future_extensions(
     return extensions;
 }
 
+int device_info_t::stepping_id() const {
+    ngen::Product p = jit::get_ngen_product(*this);
+    return p.stepping;
+}
+
+bool device_info_t::is_integrated() const {
+    ngen::Product p = jit::get_ngen_product(*this);
+    return p.type == ngen::PlatformType::Integrated;
+}
+
+std::string device_info_t::get_cl_ext_options() const {
+    using namespace compute;
+
+    std::string opts;
+    for (uint64_t i_ext = 1; i_ext < (uint64_t)device_ext_t::last;
+            i_ext <<= 1) {
+        auto ext = (device_ext_t)i_ext;
+
+        // Use real GPU extensions
+        if (!has(ext)) continue;
+
+        // These extensions are not handled properly by the OpenCL runtime.
+        // Pass macros for them manually.
+        if (utils::one_of(ext, device_ext_t::intel_global_float_atomics,
+                    device_ext_t::intel_subgroup_matrix_multiply_accumulate,
+                    device_ext_t::
+                            intel_subgroup_split_matrix_multiply_accumulate,
+                    device_ext_t::intel_global_float_atomics,
+                    device_ext_t::future_bf16_cvt,
+                    device_ext_t::intel_dot_accumulate))
+            opts += std::string("-D") + ext2cl_str(ext) + " ";
+    }
+    return opts;
+}
+
 bool device_info_t::mayiuse_sub_group(int size) const {
     switch (gpu_arch()) {
-        case gpu_arch_t::xe2:
-        case gpu_arch_t::xe_hpc: return utils::one_of(size, 16, 32);
-        default: return utils::one_of(size, 8, 16, 32);
+        case gpu_arch_t::xe_lp:
+        case gpu_arch_t::xe_hp:
+        case gpu_arch_t::xe_hpg: return utils::one_of(size, 8, 16, 32);
+        default: return utils::one_of(size, 16, 32);
     }
 }
 
@@ -102,10 +138,9 @@ bool device_info_t::has_native(data_type_t type) const {
 
 int device_info_t::max_eus_per_wg(gpu_arch_t gpu_arch) {
     switch (gpu_arch) {
-        case gpu::intel::compute::gpu_arch_t::gen9:
-        case gpu::intel::compute::gpu_arch_t::gen11:
+        case gpu::intel::compute::gpu_arch_t::xe_hpc:
         case gpu::intel::compute::gpu_arch_t::xe2:
-        case gpu::intel::compute::gpu_arch_t::xe_hpc: return 8;
+        case gpu::intel::compute::gpu_arch_t::xe3: return 8;
         case gpu::intel::compute::gpu_arch_t::xe_lp:
         case gpu::intel::compute::gpu_arch_t::xe_hp:
         case gpu::intel::compute::gpu_arch_t::xe_hpg: return 16;
@@ -116,10 +151,9 @@ int device_info_t::max_eus_per_wg(gpu_arch_t gpu_arch) {
 
 int device_info_t::max_subgroup_size(gpu_arch_t gpu_arch) {
     switch (gpu_arch) {
-        case gpu::intel::compute::gpu_arch_t::gen9: return 16;
-        case gpu::intel::compute::gpu_arch_t::gen11:
+        case gpu::intel::compute::gpu_arch_t::xe_hpc:
         case gpu::intel::compute::gpu_arch_t::xe2:
-        case gpu::intel::compute::gpu_arch_t::xe_hpc: return 32;
+        case gpu::intel::compute::gpu_arch_t::xe3: return 32;
         case gpu::intel::compute::gpu_arch_t::xe_lp:
         case gpu::intel::compute::gpu_arch_t::xe_hp:
         case gpu::intel::compute::gpu_arch_t::xe_hpg:
@@ -135,21 +169,21 @@ int device_info_t::grf_size(gpu_arch_t gpu_arch) {
 
 int device_info_t::min_subgroup_size() const {
     switch (gpu_arch()) {
-        case gpu_arch_t::gen9:
-        case gpu_arch_t::gen11:
         case gpu_arch_t::xe_lp:
         case gpu_arch_t::xe_hp:
         case gpu_arch_t::xe_hpg: return 8;
         case gpu_arch_t::xe_hpc:
-        case gpu_arch_t::xe2: return 16;
+        case gpu_arch_t::xe2:
+        case gpu_arch_t::xe3: return 16;
         default: return 0;
     }
 }
 
 int device_info_t::max_exec_size(gpu_arch_t gpu_arch) {
     switch (gpu_arch) {
+        case gpu::intel::compute::gpu_arch_t::xe_hpc:
         case gpu::intel::compute::gpu_arch_t::xe2:
-        case gpu::intel::compute::gpu_arch_t::xe_hpc: return 128;
+        case gpu::intel::compute::gpu_arch_t::xe3: return 128;
         default: return 64;
     }
     return 64;
@@ -177,13 +211,12 @@ size_t device_info_t::max_wg_size(
 
 int device_info_t::threads_per_eu(gpu_arch_t gpu_arch, bool large_grf_mode) {
     switch (gpu_arch) {
-        case gpu::intel::compute::gpu_arch_t::gen9:
-        case gpu::intel::compute::gpu_arch_t::gen11:
         case gpu::intel::compute::gpu_arch_t::xe_lp: return 7;
         case gpu::intel::compute::gpu_arch_t::xe_hp:
         case gpu::intel::compute::gpu_arch_t::xe_hpg:
-        case gpu::intel::compute::gpu_arch_t::xe2:
         case gpu::intel::compute::gpu_arch_t::xe_hpc:
+        case gpu::intel::compute::gpu_arch_t::xe2:
+        case gpu::intel::compute::gpu_arch_t::xe3:
             return large_grf_mode ? 4 : 8;
         case gpu::intel::compute::gpu_arch_t::unknown: return 7;
     }
@@ -193,17 +226,14 @@ int device_info_t::threads_per_eu(gpu_arch_t gpu_arch, bool large_grf_mode) {
 int device_info_t::max_slm_size(gpu_arch_t gpu_arch) {
     int slm_size = 0; // SLM size per SS or DSS.
     switch (gpu_arch) {
-        case gpu::intel::compute::gpu_arch_t::gen9:
-        case gpu::intel::compute::gpu_arch_t::gen11:
         case gpu::intel::compute::gpu_arch_t::xe_lp:
             slm_size = (1 << 16);
             break;
         case gpu::intel::compute::gpu_arch_t::xe_hp:
+        case gpu::intel::compute::gpu_arch_t::xe_hpg:
         case gpu::intel::compute::gpu_arch_t::xe_hpc:
         case gpu::intel::compute::gpu_arch_t::xe2:
-        case gpu::intel::compute::gpu_arch_t::xe_hpg:
-            slm_size = (1 << 17);
-            break;
+        case gpu::intel::compute::gpu_arch_t::xe3: slm_size = (1 << 17); break;
         case gpu::intel::compute::gpu_arch_t::unknown: assert(!"not expected");
     }
     return slm_size;
@@ -228,13 +258,12 @@ int device_info_t::max_slm_size_per_tg(
 
 size_t device_info_t::icache_size() const {
     switch (gpu_arch_) {
-        case gpu::intel::compute::gpu_arch_t::gen9:
-        case gpu::intel::compute::gpu_arch_t::gen11:
         case gpu::intel::compute::gpu_arch_t::xe_lp:
         case gpu::intel::compute::gpu_arch_t::xe_hp: return 48 * 1024;
         case gpu::intel::compute::gpu_arch_t::xe_hpg: return 96 * 1024;
         case gpu::intel::compute::gpu_arch_t::xe_hpc: return 80 * 1024;
         case gpu::intel::compute::gpu_arch_t::xe2: return 96 * 1024;
+        case gpu::intel::compute::gpu_arch_t::xe3: return 96 * 1024;
         case gpu::intel::compute::gpu_arch_t::unknown: assert(!"not expected");
     }
     return 0;
@@ -267,34 +296,34 @@ status_t device_info_t::init_attributes_common(impl::engine_t *engine) {
 status_t device_info_t::init_serialized_device_info(
         const std::vector<uint8_t> &cache_blob) {
     if (!cache_blob.empty()) {
-        serialized_device_info_.write(cache_blob.data(), cache_blob.size());
+        serialized_device_info_.append_array(
+                cache_blob.size(), cache_blob.data());
         return status::success;
     }
 
-    serialized_device_info_.write(&gpu_arch_);
-    serialized_device_info_.write(&gpu_product_family_);
-    serialized_device_info_.write(&stepping_id_);
-    serialized_device_info_.write(&ip_version_);
-    serialized_device_info_.write(&runtime_version_.major);
-    serialized_device_info_.write(&runtime_version_.minor);
-    serialized_device_info_.write(&runtime_version_.build);
-    serialized_device_info_.write(hw_threads_, 2);
-    serialized_device_info_.write(&eu_count_);
-    serialized_device_info_.write(&max_eus_per_wg_);
-    serialized_device_info_.write(&max_subgroup_size_);
-    serialized_device_info_.write(&max_exec_size_);
-    serialized_device_info_.write(&max_wg_size_);
-    serialized_device_info_.write(&l3_cache_size_);
-    serialized_device_info_.write(&extensions_);
-    serialized_device_info_.write(&native_extensions_);
-    serialized_device_info_.write(&mayiuse_systolic_);
-    serialized_device_info_.write(&mayiuse_ngen_kernels_);
-    serialized_device_info_.write(&mayiuse_system_memory_allocators_);
-    serialized_device_info_.write(&mayiuse_non_uniform_work_groups_);
+    serialized_device_info_.append(gpu_arch_);
+    serialized_device_info_.append(gpu_product_);
+    serialized_device_info_.append(ip_version_);
+    serialized_device_info_.append(runtime_version_.major);
+    serialized_device_info_.append(runtime_version_.minor);
+    serialized_device_info_.append(runtime_version_.build);
+    serialized_device_info_.append_array(2, hw_threads_);
+    serialized_device_info_.append(eu_count_);
+    serialized_device_info_.append(max_eus_per_wg_);
+    serialized_device_info_.append(max_subgroup_size_);
+    serialized_device_info_.append(max_exec_size_);
+    serialized_device_info_.append(max_wg_size_);
+    serialized_device_info_.append(l3_cache_size_);
+    serialized_device_info_.append(extensions_);
+    serialized_device_info_.append(native_extensions_);
+    serialized_device_info_.append(mayiuse_systolic_);
+    serialized_device_info_.append(mayiuse_ngen_kernels_);
+    serialized_device_info_.append(mayiuse_system_memory_allocators_);
+    serialized_device_info_.append(mayiuse_non_uniform_work_groups_);
 
     const size_t name_size = name_.size();
-    serialized_device_info_.write(&name_size);
-    serialized_device_info_.write(name_.data(), name_size);
+    serialized_device_info_.append(name_size);
+    serialized_device_info_.append_array(name_size, name_.data());
 
     return status::success;
 }
@@ -312,8 +341,7 @@ status_t device_info_t::init_from_cache_blob(
     pos += sizeof(expected_type);
 
     DESERIALIZE(gpu_arch_, compute::gpu_arch_t);
-    DESERIALIZE(gpu_product_family_, int);
-    DESERIALIZE(stepping_id_, int);
+    DESERIALIZE(gpu_product_, compute::gpu_product_t);
     DESERIALIZE(ip_version_, uint32_t);
     DESERIALIZE(runtime_version_.major, int);
     DESERIALIZE(runtime_version_.minor, int);
@@ -355,6 +383,11 @@ void device_info_t::fixup_l3_cache_size() {
     }
 }
 
+static_assert(std::is_trivially_copyable<ngen::Product>(),
+        "ngen::Product cannot safely be copied into gpu_product_t");
+static_assert(sizeof(ngen::Product) == sizeof(compute::gpu_product_t),
+        "ngen::Product cannot safely be copied into gpu_product_t");
+
 } // namespace compute
 } // namespace intel
 } // namespace gpu
