@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@
 #include <string.h>
 
 #include "common/c_types_map.hpp"
-#include "common/serialization_stream.hpp"
+#include "common/serialization.hpp"
 #include "common/utils.hpp"
 #include "common/z_magic.hpp"
 
@@ -36,27 +36,23 @@ namespace gpu {
 namespace intel {
 namespace compute {
 
-enum class gpu_arch_t {
-    unknown,
-    gen9,
-    gen11,
-    xe_lp,
-    xe_hp,
-    xe_hpg,
-    xe_hpc,
-    xe2,
+enum class gpu_arch_t { unknown, xe_lp, xe_hp, xe_hpg, xe_hpc, xe2, xe3 };
+
+// Memory for storing ngen::Product to avoid directly including nGEN because of
+// header dependencies outside of src/gpu/intel.
+struct alignas(int) gpu_product_t {
+    unsigned char data[12];
 };
 
 static inline std::string to_string(gpu_arch_t arch) {
 #define CASE(_case) \
     if (arch == gpu_arch_t::_case) return STRINGIFY(_case)
-    CASE(gen9);
-    CASE(gen11);
     CASE(xe_lp);
     CASE(xe_hp);
     CASE(xe_hpg);
     CASE(xe_hpc);
     CASE(xe2);
+    CASE(xe3);
     return "unknown";
 #undef CASE
 }
@@ -64,14 +60,12 @@ static inline std::string to_string(gpu_arch_t arch) {
 static inline gpu_arch_t str2gpu_arch(const char *str) {
 #define CASE(_case) \
     if (!strcmp(STRINGIFY(_case), str)) return gpu_arch_t::_case
-
-    CASE(gen9);
-    CASE(gen11);
     CASE(xe_lp);
     CASE(xe_hp);
     CASE(xe_hpg);
     CASE(xe_hpc);
     CASE(xe2);
+    CASE(xe3);
     return gpu_arch_t::unknown;
 #undef CASE
 }
@@ -89,13 +83,12 @@ enum class device_ext_t : uint64_t {
     khr_local_int32_base_atomics      = 1ull << 6,
     khr_local_int32_extended_atomics  = 1ull << 7,
     ext_float_atomics                 = 1ull << 8,
-    // Intel specific Gen9+
-    intel_subgroups              = 1ull << 16,
-    intel_required_subgroup_size = 1ull << 17,
-    intel_subgroups_char         = 1ull << 18,
-    intel_subgroups_short        = 1ull << 19,
-    intel_subgroups_long         = 1ull << 20,
     // Intel specific Xe_LP+
+    intel_subgroups               = 1ull << 16,
+    intel_required_subgroup_size  = 1ull << 17,
+    intel_subgroups_char          = 1ull << 18,
+    intel_subgroups_short         = 1ull << 19,
+    intel_subgroups_long          = 1ull << 20,
     intel_subgroup_local_block_io = 1ull << 21,
     intel_dot_accumulate          = 1ull << 22,
     // Intel specific Xe_HP+
@@ -191,11 +184,15 @@ public:
         return status::success;
     }
 
+    std::string get_cl_ext_options() const;
+
     bool has(device_ext_t ext) const { return extensions_ & (uint64_t)ext; }
     bool has_native(native_ext_t ext) const { return native_extensions_ & (uint64_t)ext; }
     gpu_arch_t gpu_arch() const { return gpu_arch_; }
-    int gpu_product_family() const { return gpu_product_family_; }
-    int stepping_id() const { return stepping_id_; }
+    const gpu_product_t &gpu_product() const {return gpu_product_;}
+    int stepping_id() const;
+    uint64_t native_extensions() const { return native_extensions_; }
+    bool is_integrated() const;
     uint32_t ip_version() const { return ip_version_; }
     int max_eus_per_wg() const { return max_eus_per_wg_; }
     static int max_eus_per_wg(gpu_arch_t gpu_arch);
@@ -271,9 +268,8 @@ protected:
     virtual status_t init_extensions(impl::engine_t *engine) = 0;
     virtual status_t init_attributes(impl::engine_t *engine) = 0;
 
-    compute::gpu_arch_t gpu_arch_ = compute::gpu_arch_t::unknown;
-    int gpu_product_family_ = 0;
-    int stepping_id_ = 0;
+    gpu_arch_t gpu_arch_ = compute::gpu_arch_t::unknown;
+    gpu_product_t gpu_product_ = {};
     uint32_t ip_version_ = 0;
     bool mayiuse_systolic_ = false;
     bool mayiuse_ngen_kernels_ = false;
@@ -313,7 +309,8 @@ private:
 };
 
 } // namespace compute
-}} // namespace gpu
+} // namespace intel
+} // namespace gpu
 } // namespace impl
 } // namespace dnnl
 
