@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,13 +14,18 @@
 * limitations under the License.
 *******************************************************************************/
 
+#ifdef ENABLE_LLVM_WCONVERSION
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wimplicit-int-conversion"
+#endif
+
 #include "gpu/intel/jit/binary_format.hpp"
 
 #include "common/utils.hpp"
-#include "gpu/intel/compute/compute_engine.hpp"
-#include "gpu/intel/compute/compute_stream.hpp"
 #include "gpu/intel/compute/utils.hpp"
-#include "gpu/intel/jit/jit_generator.hpp"
+#include "gpu/intel/engine.hpp"
+#include "gpu/intel/jit/generator.hpp"
+#include "gpu/intel/stream.hpp"
 
 #define MAGIC0 0xBEEFCAFEu
 #define MAGIC1 0x3141592653589793ull
@@ -43,11 +48,12 @@ namespace jit {
 using namespace ngen;
 
 template <HW hw>
-class binary_format_kernel_t : public jit_generator<hw> {
+class binary_format_kernel_t : public generator_t<hw> {
     NGEN_FORWARD_OPENCL(hw);
 
 public:
-    binary_format_kernel_t() {
+    binary_format_kernel_t()
+        : generator_t<hw>(debug_config_t {GENERATOR_NAME, GENERATOR_LINE}) {
 
         auto low_half = [](uint64_t q) -> uint32_t { return q & 0xFFFFFFFF; };
         auto high_half = [](uint64_t q) -> uint32_t { return q >> 32; };
@@ -150,7 +156,7 @@ public:
     }
 
     static compute::kernel_t make_kernel(
-            compute::compute_engine_t *engine, bool *skip_check) {
+            intel::engine_t *engine, bool *skip_check) {
         compute::kernel_t kernel;
 
         *skip_check = false;
@@ -158,21 +164,12 @@ public:
         if (hw != HW::Unknown) {
             binary_format_kernel_t<hw> binary_format_kernel;
 
-            auto status
-                    = engine->create_kernel(&kernel, &binary_format_kernel, {});
+            auto status = engine->create_kernel(&kernel, &binary_format_kernel);
 
             if (status != status::success) return nullptr;
             *skip_check = binary_format_kernel.binaryIsZebin();
         } else {
             switch (engine->device_info()->gpu_arch()) {
-                case compute::gpu_arch_t::gen9:
-                    kernel = binary_format_kernel_t<HW::Gen9>::make_kernel(
-                            engine, skip_check);
-                    break;
-                case compute::gpu_arch_t::gen11:
-                    kernel = binary_format_kernel_t<HW::Gen11>::make_kernel(
-                            engine, skip_check);
-                    break;
                 case compute::gpu_arch_t::xe_lp:
                     kernel = binary_format_kernel_t<HW::XeLP>::make_kernel(
                             engine, skip_check);
@@ -193,7 +190,16 @@ public:
                     kernel = binary_format_kernel_t<HW::Xe2>::make_kernel(
                             engine, skip_check);
                     break;
-                case compute::gpu_arch_t::unknown: kernel = nullptr; break;
+                case compute::gpu_arch_t::xe3:
+                    kernel = binary_format_kernel_t<HW::Xe3>::make_kernel(
+                            engine, skip_check);
+                    break;
+                case compute::gpu_arch_t::unknown:
+                    VWARN(common, runtime,
+                            "unknown gpu platform - optimizations are disabled "
+                            "for binary format kernel");
+                    kernel = nullptr;
+                    break;
             }
         }
         return kernel;
@@ -203,14 +209,30 @@ public:
 status_t gpu_supports_binary_format(bool *ok, impl::engine_t *engine) {
     *ok = false;
 
-    auto gpu_engine = utils::downcast<compute::compute_engine_t *>(engine);
-    if (!gpu_engine) return status::invalid_arguments;
+#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
+    if (engine->runtime_kind() == runtime_kind::ocl) {
+        // Here we are doing a check for a temporary OpenCL engine while the
+        // GPU runtime is SYCL. nGEN-based kernels are all SYCL based in the
+        // current build so just exit here.
+        // This check should be already done for the "parent" SYCL engine so
+        // any potential errors should have been caught before this point.
+        *ok = true;
+        return status::success;
+    }
+#endif
+
+    auto gpu_engine = utils::downcast<intel::engine_t *>(engine);
+
+    if (!gpu_engine) {
+        VERROR(common, runtime, "bad engine kind, expected a gpu engine");
+        return status::invalid_arguments;
+    }
 
     impl::stream_t *stream_generic;
     auto status = gpu_engine->get_service_stream(stream_generic);
     if (status != status::success) return status::runtime_error;
 
-    auto stream = utils::downcast<compute::compute_stream_t *>(stream_generic);
+    auto stream = utils::downcast<intel::stream_t *>(stream_generic);
     if (!stream) return status::invalid_arguments;
 
     bool skip_check = false;
@@ -223,6 +245,8 @@ status_t gpu_supports_binary_format(bool *ok, impl::engine_t *engine) {
         return status::success;
     }
 
+    VWARN(common, runtime, "binary kernel is not in zebin format");
+
     // Binary kernel check.
     uint32_t magic0 = MAGIC0;
     uint64_t magic1 = MAGIC1;
@@ -236,28 +260,44 @@ status_t gpu_supports_binary_format(bool *ok, impl::engine_t *engine) {
     std::unique_ptr<memory_storage_t> magic_buf, result_buf;
 
     status = engine->create_memory_storage(&storage, sizeof(int32_t));
-    if (status != status::success) return status::runtime_error;
+    if (status != status::success) {
+        VERROR(common, runtime,
+                "failed to create memory storage during binary kernel check");
+        return status::runtime_error;
+    }
     magic_buf.reset(storage);
 
     status = engine->create_memory_storage(&storage, sizeof(int32_t));
-    if (status != status::success) return status::runtime_error;
+    if (status != status::success) {
+        VERROR(common, runtime,
+                "failed to create memory storage during binary kernel check");
+        return status::runtime_error;
+    }
     result_buf.reset(storage);
 
     void *magic_host = nullptr;
-    magic_buf->map_data(&magic_host, nullptr, sizeof(int32_t));
-    if (!magic_host) return status::runtime_error;
+    status = magic_buf->map_data(&magic_host, nullptr, sizeof(int32_t));
+    if (!magic_host || status != status::success) {
+        VERROR(common, runtime,
+                "failed to map data during binary kernel check");
+        return status::runtime_error;
+    }
 
     *reinterpret_cast<uint32_t *>(magic_host) = magic_ptr;
 
-    magic_buf->unmap_data(magic_host, nullptr);
+    CHECK(magic_buf->unmap_data(magic_host, nullptr));
 
     void *result_host = nullptr;
-    result_buf->map_data(&result_host, nullptr, sizeof(int32_t));
-    if (!result_host) return status::runtime_error;
+    status = result_buf->map_data(&result_host, nullptr, sizeof(int32_t));
+    if (!result_host || status != status::success) {
+        VERROR(common, runtime,
+                "failed to map data during binary kernel check");
+        return status::runtime_error;
+    }
 
     *reinterpret_cast<uint32_t *>(result_host) = 0;
 
-    result_buf->unmap_data(result_host, nullptr);
+    CHECK(result_buf->unmap_data(result_host, nullptr));
 
     compute::kernel_arg_list_t arg_list;
     arg_list.set(0, magic0);
@@ -266,30 +306,42 @@ status_t gpu_supports_binary_format(bool *ok, impl::engine_t *engine) {
     arg_list.set(3, magic3);
     arg_list.set(4, magic4);
     arg_list.set(5, magic5);
-    arg_list.set(6, *magic_buf.get());
-    arg_list.set(7, *result_buf.get());
+    arg_list.set(6, *magic_buf);
+    arg_list.set(7, *result_buf);
 
     compute::range_t gws = {MAGICSIZEX, MAGICSIZEY, MAGICSIZEZ};
     compute::range_t lws = {MAGICSIZEX, MAGICSIZEY, MAGICSIZEZ};
 
     auto nd_range = compute::nd_range_t(gws, lws);
 
-    auto compute_stream = utils::downcast<compute::compute_stream_t *>(stream);
+    auto compute_stream = utils::downcast<intel::stream_t *>(stream);
     status = kernel.parallel_for(*stream, nd_range, arg_list,
             compute_stream->ctx().get_deps(), compute_stream->ctx().get_deps());
 
-    if (status != status::success) return status::runtime_error;
+    if (status != status::success) {
+        VERROR(common, runtime,
+                "failed to execute kernel during binary kernel check");
+        return status::runtime_error;
+    }
 
     status = stream->wait();
-    if (status != status::success) return status::runtime_error;
+    if (status != status::success) {
+        VERROR(common, runtime,
+                "failed to execute stream during binary kernel check");
+        return status::runtime_error;
+    }
 
     result_host = nullptr;
-    result_buf->map_data(&result_host, nullptr, sizeof(int32_t));
-    if (!result_host) return status::runtime_error;
+    status = result_buf->map_data(&result_host, nullptr, sizeof(int32_t));
+    if (!result_host || status != status::success) {
+        VERROR(common, runtime,
+                "failed to map data during binary kernel check");
+        return status::runtime_error;
+    }
 
     auto result = *reinterpret_cast<uint32_t *>(result_host);
 
-    result_buf->unmap_data(result_host, nullptr);
+    CHECK(result_buf->unmap_data(result_host, nullptr));
 
     *ok = (result != 0);
 
@@ -301,3 +353,7 @@ status_t gpu_supports_binary_format(bool *ok, impl::engine_t *engine) {
 } // namespace gpu
 } // namespace impl
 } // namespace dnnl
+
+#ifdef ENABLE_LLVM_WCONVERSION
+#pragma clang diagnostic pop
+#endif
