@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,21 +16,15 @@
 
 #include "gpu/intel/jit/codegen/bank_conflict_allocation.hpp"
 
-#include "common/verbose.hpp"
 #include "gpu/intel/jit/ir/fma.hpp"
 #include "gpu/intel/jit/utils/utils.hpp"
+#include "gpu/intel/logging.hpp"
 
-#include <sstream>
+#include <numeric>
 #include <string>
 #include <vector>
 #include <initializer_list>
 
-#if defined(__GNUC__) && __GNUC__ == 7
-// GCC 7.x issues a false positive warning 'array subscript is above array bounds'
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Warray-bounds"
-#endif
-
 namespace dnnl {
 namespace impl {
 namespace gpu {
@@ -49,7 +43,7 @@ struct hw_context_t {
                 reg_bank_stride = i;
                 break;
             }
-        ir_assert(reg_bank_stride != -1);
+        gpu_assert(reg_bank_stride != -1);
 
         bank_masks.resize(ngen::Bundle::bank_count(hw));
         bundle_masks.resize(ngen::Bundle::bundle_count(hw));
@@ -63,10 +57,10 @@ struct hw_context_t {
             } else {
                 // Ensure bank/bundle pattern is repeated.
                 int j = (i % 64);
-                ir_assert((bank_masks[bank] & (1ull << j)) != 0);
-                //XeLP and Gen9 only have two bundles
+                gpu_assert((bank_masks[bank] & (1ull << j)) != 0);
+                // XeLP only has two bundles
                 if (hw > ngen::HW::XeLP)
-                    ir_assert((bundle_masks[bundle] & (1ull << j)) != 0);
+                    gpu_assert((bundle_masks[bundle] & (1ull << j)) != 0);
             }
         }
     }
@@ -83,15 +77,13 @@ struct hw_context_t {
 
     int hw_simd() const {
         switch (hw) {
-            case ngen::HW::Gen9:
-            case ngen::HW::Gen10:
-            case ngen::HW::Gen11:
             case ngen::HW::XeLP:
             case ngen::HW::XeHP:
             case ngen::HW::XeHPG: return 8;
+            case ngen::HW::XeHPC:
             case ngen::HW::Xe2:
-            case ngen::HW::XeHPC: return 16;
-            default: ir_error_not_expected();
+            case ngen::HW::Xe3: return 16;
+            default: gpu_error_not_expected();
         }
         return -1;
     }
@@ -252,7 +244,7 @@ struct reg_mask_t {
 
     std::string str() const {
         UNUSED(&reg_mask_t::str);
-        std::ostringstream oss;
+        ostringstream_t oss;
         for (int i = hw_ctx->regs - 1; i >= 0; i--) {
             oss << (test(i) ? "1" : "0");
         }
@@ -340,7 +332,7 @@ struct reg_block_mask_t {
 
     std::string str() const {
         UNUSED(&reg_block_mask_t::str);
-        std::ostringstream oss;
+        ostringstream_t oss;
         for (int i = 0; i < regs; i++) {
             oss << "#" << i << " mask: " << masks[i].str();
             if (i != regs - 1) oss << std::endl;
@@ -367,6 +359,7 @@ struct reg_t {
         return block->masks[off].bank();
     }
 
+    // NOLINTNEXTLINE(readability-make-member-function-const)
     void exclude(const reg_mask_t &mask) {
         if (is_empty()) return;
         block->masks[off].subtract(mask);
@@ -379,7 +372,7 @@ struct reg_t {
     std::string str() const {
         UNUSED(&reg_t::str);
         if (is_empty()) return "null";
-        std::ostringstream oss;
+        ostringstream_t oss;
         if (block->is_assigned()) {
             int reg = block->masks[off].bsf();
             oss << "r" << reg;
@@ -402,7 +395,7 @@ struct reg_buf_mask_t {
     reg_buf_mask_t(const hw_context_t *hw_ctx, int regs, int block_regs = 0)
         : hw_ctx(hw_ctx), regs(regs), block_regs(block_regs) {
         if (block_regs == 0) this->block_regs = regs;
-        ir_assert(regs % this->block_regs == 0);
+        gpu_assert(regs % this->block_regs == 0);
         for (int i = 0; i < nblocks(); i++) {
             blocks.emplace_back(hw_ctx, this->block_regs);
         }
@@ -414,7 +407,7 @@ struct reg_buf_mask_t {
     int nblocks() const { return regs / block_regs; }
 
     reg_t get_reg(int off_bytes) {
-        ir_assert(off_bytes < size());
+        gpu_assert(off_bytes < size());
         off_bytes /= hw_ctx->reg_size;
         int block_idx = off_bytes / block_regs;
         int reg_idx = off_bytes % block_regs;
@@ -453,7 +446,7 @@ struct search_context_t {
         saved_blocks.resize(nblocks() * nblocks());
     }
 
-    int nblocks() { return int(blocks.size()); }
+    int nblocks() const { return int(blocks.size()); }
 
     void set_check_bundles(bool value = true) { check_bundles = value; }
 
@@ -463,7 +456,7 @@ struct search_context_t {
 
     // Saves block masks for the current recursion level.
     void save_blocks() {
-        ir_assert(saved_block_idx + nblocks() <= int(saved_blocks.size()));
+        gpu_assert(saved_block_idx + nblocks() <= int(saved_blocks.size()));
         for (int i = 0; i < nblocks(); i++) {
             saved_blocks[saved_block_idx + i] = *blocks[i];
         }
@@ -474,7 +467,7 @@ struct search_context_t {
     // Restores saved block masks.
     void restore_blocks() {
         saved_block_idx -= nblocks();
-        ir_assert(saved_block_idx >= 0);
+        gpu_assert(saved_block_idx >= 0);
         for (int i = 0; i < nblocks(); i++) {
             *blocks[i] = saved_blocks[saved_block_idx + i];
         }
@@ -639,9 +632,10 @@ reg_mask_t create_available_reg_mask(
 } // namespace
 
 bank_conflict_allocation_t bank_conflict_allocation_t::create(
-        reg_allocator_t &ra, int regs, const bank_conflict_attr_t &attr) {
+        reg_allocator_t &ra, const bank_conflict_attr_t &attr) {
+    int regs = ra.getRegisterCount();
     hw_context_t hw_ctx(ra.hardware(), regs);
-    ir_assert(regs <= reg_mask_t::max_regs);
+    gpu_assert(regs <= reg_mask_t::max_regs);
 
     bool is_dpas = false;
     bool is_dp4a = false;
@@ -659,7 +653,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
             auto &mad = func.as<mad_t>();
             is_f64 = mad.dst_type.is_f64();
         } else {
-            ir_error_not_expected();
+            gpu_error_not_expected();
         }
     }
 
@@ -687,7 +681,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
 
     auto create_reg = [&](const expr_t &e, int src_idx, int off_bytes) {
         if (is_zero(e)) return reg_t();
-        auto base = get_base(e);
+        const auto &base = get_base(e);
         int off = 0;
         if (!is_var(e)) off = to_cpp<int>(e.as<ptr_t>().off);
         off += off_bytes;
@@ -697,7 +691,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
                 return buf_masks[i].get_reg(off);
             }
         }
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return reg_t();
     };
 
@@ -719,7 +713,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
             src0 = dpas_t::arg_src0(call);
             src1 = dpas_t::arg_src1(call);
             src2 = dpas_t::arg_src2(call);
-            if (!dpas.is_dp4a()) ir_assert(simd == hw_simd);
+            if (!dpas.is_dp4a()) gpu_assert(simd == hw_simd);
         } else if (call.func.is<mad_t>()) {
             auto &mad = call.func.as<mad_t>();
             simd = mad.exec_size;
@@ -730,7 +724,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
             src1 = mad_t::arg_src1(call);
             src2 = mad_t::arg_src2(call);
         } else {
-            ir_error_not_expected();
+            gpu_error_not_expected();
         }
         for (int off = 0; off < simd; off += hw_simd) {
             auto _src0 = create_reg(src0, 0, off * src0_stride_bytes);
@@ -743,7 +737,7 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
     std::vector<reg_block_mask_t *> blocks;
 
     for (size_t i = 0; i < bufs.size(); i++)
-        ir_assert(buf_src_idx[i] != -1)
+        gpu_assert(buf_src_idx[i] != -1)
                 << "Buffer is not referenced: " << bufs[i];
 
     // Heuristic: search for register blocks in this order: src1, src2, src0.
@@ -775,8 +769,8 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
         ctx.reset_steps();
         ctx.set_check_bundles(check_bundles);
 
-        ir_assert(ctx.saved_block_idx == 0);
-        ir_assert(ctx.reg_mask == reg_mask);
+        gpu_assert(ctx.saved_block_idx == 0);
+        gpu_assert(ctx.reg_mask == reg_mask);
 
 #ifdef DNNL_DEV_MODE
         double search_time = get_msec();
@@ -784,12 +778,12 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
         found = search(ctx);
 #ifdef DNNL_DEV_MODE
         search_time = get_msec() - search_time;
-        ir_trace() << "Bank conflict allocation:" << std::endl;
-        ir_trace() << "    Search time: " << search_time << " ms" << std::endl;
-        ir_trace() << "    Status: " << (found ? "OK" : "FAIL") << std::endl;
-        ir_trace() << "    Steps: " << ctx.steps << std::endl;
-        ir_trace() << "    Bundle check: "
-                   << ir_utils::to_string(ctx.check_bundles) << std::endl;
+        gpu_trace() << "Bank conflict allocation:";
+        gpu_trace() << "    Search time: " << search_time << " ms";
+        gpu_trace() << "    Status: " << (found ? "OK" : "FAIL");
+        gpu_trace() << "    Steps: " << ctx.steps;
+        gpu_trace() << "    Bundle check: "
+                    << ir_utils::to_string(ctx.check_bundles);
 #endif
         if (found) break;
     }
@@ -815,7 +809,8 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
             ngen::Bundle bundle;
             // Choose the opposite bank for src0 or src2.
             if (is_src02 && bank != -1)
-                bundle = ngen::Bundle(1 - bank, ngen::Bundle::any);
+                bundle = ngen::Bundle(
+                        into<int8_t>(1 - bank), ngen::Bundle::any);
             auto &mask = buf_masks[i].blocks[0].masks[0];
             auto range = ra.alloc_range(regs, bundle);
             int base = range[0].getBase();
@@ -847,7 +842,3 @@ bank_conflict_allocation_t bank_conflict_allocation_t::create(
 } // namespace gpu
 } // namespace impl
 } // namespace dnnl
-
-#if defined(__GNUC__) && __GNUC__ == 7
-#pragma GCC diagnostic pop
-#endif
