@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <numeric>
 #include <random>
 
 #include "gpu/intel/jit/ir/blocking.hpp"
@@ -25,10 +26,10 @@ namespace gpu {
 namespace intel {
 namespace jit {
 
-std::vector<int> tile_info_t::iter_blocks(int size) const {
+std::vector<int> tile_info_t::iter_blocks(dim_t size) const {
     if (!any(flags & tile_flags_t::iter)) return {1};
     std::vector<int> ret;
-    int lo = std::min(size, (int)min_iter_blk);
+    int lo = static_cast<int>(std::min<dim_t>(size, min_iter_blk));
     int hi = max_iter_blk;
     uint32_t pow2_seen = 0;
     // Step 1. Check the divisors.
@@ -54,19 +55,19 @@ std::vector<int> tile_info_t::iter_blocks(int size) const {
     return ret;
 }
 
-std::vector<int> tile_info_t::thread_group_blocks(int size) const {
+std::vector<int> tile_info_t::thread_group_blocks(dim_t size) const {
     std::vector<int> ret;
     int bound = any(flags & tile_flags_t::thread_group) ? max_thread_group_blk
                                                         : 1;
     for (int i = 1; i <= bound; i *= 2) {
-        int size_padded = utils::rnd_up(size, i);
+        dim_t size_padded = utils::rnd_up(size, i);
         double eff = (double)size / size_padded;
         if (eff >= 0.75) ret.push_back(i);
     }
     return ret;
 }
 
-std::vector<int> tile_info_t::loop_blocks(int size, int iter_blk) const {
+std::vector<dim_t> tile_info_t::loop_blocks(dim_t size, int iter_blk) const {
     if (!any(flags & tile_flags_t::loop)) return {1};
     if (any(flags & tile_flags_t::loop_span)) return {size};
     if (any(flags & tile_flags_t::loop_iter_unroll)) {
@@ -76,33 +77,33 @@ std::vector<int> tile_info_t::loop_blocks(int size, int iter_blk) const {
     return get_loop_blocks(size);
 }
 
-std::vector<int> tile_info_t::get_factors(int n) {
-    std::vector<int> ret;
-    int n_sqrt = (int)std::sqrt(n);
-    for (int i = 1; i <= n_sqrt; i++) {
+std::vector<dim_t> tile_info_t::get_factors(dim_t n) {
+    std::vector<dim_t> ret;
+    dim_t n_sqrt = std::sqrt(n);
+    for (dim_t i = 1; i <= n_sqrt; i++) {
         if (n % i == 0) ret.push_back(i);
     }
-    int lo = n_sqrt;
+    dim_t lo = n_sqrt;
     if (n_sqrt * n_sqrt == n) lo--;
-    for (int i = lo; i >= 1; i--) {
+    for (dim_t i = lo; i >= 1; i--) {
         if (n % i == 0) ret.push_back(n / i);
     }
     return ret;
 }
 
-std::vector<int> tile_info_t::get_loop_blocks(int n) {
+std::vector<dim_t> tile_info_t::get_loop_blocks(dim_t n) {
     const int step = 4;
     int steps = (int)(std::log((float)n) / std::log((float)step));
     auto factors = get_factors(n);
     if (factors.size() >= (size_t)steps) return factors;
 
-    std::vector<int> ret;
+    std::vector<dim_t> ret;
     ret.reserve(steps);
     for (int i = 1; i <= n; i *= step) {
         int a = i;
         int b = i * step;
         bool found = false;
-        for (int j : factors) {
+        for (dim_t j : factors) {
             if (a <= j && j < b) {
                 found = true;
                 ret.push_back(j);
@@ -114,30 +115,22 @@ std::vector<int> tile_info_t::get_loop_blocks(int n) {
     return ret;
 }
 
-namespace levels {
-level_t loop(level_kind_t::loop);
-level_t thread_group(level_kind_t::thread_group);
-level_t iter(level_kind_t::iter);
-} // namespace levels
-
 void get_level_tiles(
-        int size, const tile_info_t &info, std::vector<level_tile_t> &ret) {
+        dim_t size, const tile_info_t &info, std::vector<level_tile_t> &ret) {
     ret.clear();
     auto iter_blocks = info.iter_blocks(size);
     for (int iter : iter_blocks) {
-        int tg_size = utils::div_up(size, iter);
+        dim_t tg_size = utils::div_up(size, iter);
         auto tg_blocks = info.thread_group_blocks(tg_size);
         for (int tg : tg_blocks) {
-            int loop_size = utils::div_up(size, tg * iter);
+            dim_t loop_size = utils::div_up(size, tg * iter);
             auto loop_blocks = info.loop_blocks(loop_size, iter);
-            for (int loop : loop_blocks) {
+            for (dim_t loop : loop_blocks) {
                 level_tile_t t;
-                if (any(info.flags & tile_flags_t::loop))
-                    t[levels::loop] = loop;
+                if (any(info.flags & tile_flags_t::loop)) t.loop = loop;
                 if (any(info.flags & tile_flags_t::thread_group))
-                    t[levels::thread_group] = tg;
-                if (any(info.flags & tile_flags_t::iter))
-                    t[levels::iter] = iter;
+                    t.thread_group = tg;
+                if (any(info.flags & tile_flags_t::iter)) t.iter = iter;
                 ret.push_back(t);
             }
         }
@@ -146,7 +139,7 @@ void get_level_tiles(
 
 void params_generator_t::set_params(prim_config_t &cfg) {
     auto &params = params_vec_[cur_idx_];
-    ir_trace() << "set params #" << cur_idx_ << ": " << params << std::endl;
+    gpu_trace() << "set params #" << cur_idx_ << ": " << params;
     cfg.set_params(params);
 }
 
@@ -167,7 +160,7 @@ std::string to_string(tiler_mode_t mode) {
         CASE(tune);
 #undef CASE
     }
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return "(unknown)";
 }
 
@@ -204,18 +197,18 @@ std::vector<blocking_t> level_tile_set_t::sample(int target,
             set(blk, dims_[i], try_tiles[i]);
         }
         if (!is_ok(blk)) continue;
-        ret.push_back(blk);
+        ret.push_back(std::move(blk));
         if ((int)ret.size() >= target) break;
     }
     return ret;
 }
 
 void level_tile_set_t::set(
-        blocking_t &blk, const prb_dim_t &dim, const level_tile_t &tile) {
-    if (tile.has(levels::loop)) blk.set_loop(dim, tile[levels::loop]);
-    if (tile.has(levels::thread_group))
-        blk.set_thread_group(dim, tile[levels::thread_group]);
-    if (tile.has(levels::iter)) blk.set_iter(dim, tile[levels::iter]);
+        blocking_t &blk, const pvar_t &dim, const level_tile_t &tile) {
+    if (tile.has(level_t::loop)) blk.set_loop(dim, tile.loop);
+    if (tile.has(level_t::thread_group))
+        blk.set_thread_group(dim, tile.thread_group);
+    if (tile.has(level_t::iter)) blk.set_iter(dim, tile.iter);
 }
 
 void level_tile_set_t::product_impl(int idx, std::vector<int> &cur_idxs,
@@ -271,7 +264,7 @@ void blocking_generator_t::generate_all(int vec_size, blocking_checker_t &chk,
 
 void blocking_generator_t::generate_sample(int vec_size,
         const blocking_checker_t &chk, const level_tile_set_t &level_tile_set) {
-    ir_assert(false);
+    gpu_assert(false);
     int target_size = 1;
     auto is_ok = [&](const blocking_t &blk) { return chk.is_ok(blk); };
     auto ts_blockings = level_tile_set.sample(target_size, is_ok, vec_size);
@@ -297,7 +290,7 @@ params_generator_t::params_generator_t(int tune_level, int simd_size,
         const std::vector<level_tile_set_t> &level_tile_sets, int idx) {
     append_params(params_vec_, level_tile_sets, chk, tune_level, simd_size);
     if (idx != -1) {
-        ir_assert(idx >= 0 && idx < configs());
+        gpu_assert(idx >= 0 && idx < configs());
         std::vector<blocking_params_t> temp_vec;
         temp_vec.swap(params_vec_);
         append_params(params_vec_, temp_vec[idx]);
@@ -350,7 +343,7 @@ const tiler_params_t &tiler_params() {
                 continue;
             }
             auto sub_opts = gpu_utils::split(opt, ":");
-            ir_assert((int)sub_opts.size() == 2);
+            gpu_assert((int)sub_opts.size() == 2);
             auto &key = sub_opts[0];
             auto &value = sub_opts[1];
             if (key == "tune_iters") {
@@ -359,17 +352,17 @@ const tiler_params_t &tiler_params() {
                 ret.mode = tiler_mode_t::env_tiler;
                 ret.env_params_idx = std::stoi(value);
             } else {
-                ir_error_not_expected();
+                gpu_error_not_expected();
             }
         }
         bool do_tune = (ret.mode == tiler_mode_t::tune);
-        ir_assert(do_tune == (ret.tune_iters != 0));
+        gpu_assert(do_tune == (ret.tune_iters != 0));
         return ret;
     }();
     return params;
 }
 
-tile_to_vec_t::tile_to_vec_t(const std::vector<std::vector<prb_tile_t>> &tiles,
+tile_to_vec_t::tile_to_vec_t(const std::vector<std::vector<tile_t>> &tiles,
         const std::vector<int> &_ids) {
     if (tiles.empty()) return;
     int ntiles = (int)tiles.size();
@@ -380,7 +373,7 @@ tile_to_vec_t::tile_to_vec_t(const std::vector<std::vector<prb_tile_t>> &tiles,
         ids.resize(ntiles);
         std::iota(ids.begin(), ids.end(), 0);
     }
-    ir_assert(ids.size() == tiles.size());
+    gpu_assert(ids.size() == tiles.size());
     int max_id = 0;
     for (int i = 0; i < ntiles; i++) {
         for (int j = 0; j < nsubtiles; j++) {
