@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,9 @@
 
 #include "gpu/intel/jit/ir/core.hpp"
 #include "gpu/intel/jit/ir/problem.hpp"
+#include "gpu/intel/logging.hpp"
+
+#include <set>
 
 namespace dnnl {
 namespace impl {
@@ -29,32 +32,32 @@ namespace jit {
 class blocking_t {
 public:
     int simd() const { return simd_; }
-    const prb_tile_t &loop() const { return loop_; }
-    const prb_tile_t &thread_group() const { return thread_group_; }
-    const prb_tile_t &iter() const { return iter_; }
+    const tile_t &loop() const { return loop_; }
+    const tile_t &thread_group() const { return thread_group_; }
+    const tile_t &iter() const { return iter_; }
 
-    int loop_dim(const prb_dim_t &d) const { return loop_[d]; }
-    int thread_group_dim(const prb_dim_t &d) const { return thread_group_[d]; }
-    int iter_dim(const prb_dim_t &d) const { return iter_[d]; }
+    dim_t loop_dim(const pvar_t &d) const { return loop_[d]; }
+    dim_t thread_group_dim(const pvar_t &d) const { return thread_group_[d]; }
+    dim_t iter_dim(const pvar_t &d) const { return iter_[d]; }
 
     void set_simd(int simd) { simd_ = simd; }
-    void set_loop(const prb_dim_t &d, int value) { loop_[d] = value; }
-    void set_thread_group(const prb_dim_t &d, int value) {
+    void set_loop(const pvar_t &d, dim_t value) { loop_[d] = value; }
+    void set_thread_group(const pvar_t &d, dim_t value) {
         thread_group_[d] = value;
     }
-    void set_iter(const prb_dim_t &d, int value) { iter_[d] = value; }
+    void set_iter(const pvar_t &d, dim_t value) { iter_[d] = value; }
 
     bool is_empty() const {
         return loop_.is_empty() && thread_group_.is_empty() && iter_.is_empty();
     }
     bool is_spatial() const {
-        for (auto d : {prb_dims::iw, prb_dims::ow}) {
+        for (const auto &d : {pvars::iw, pvars::ow}) {
             if (iter_.has(d) && iter_[d] != 1) return true;
         }
         return false;
     }
 
-    void unset(const prb_dim_t &d) {
+    void unset(const pvar_t &d) {
         if (loop_.has(d)) loop_[d] = 1;
         if (thread_group_.has(d)) thread_group_[d] = 1;
         if (iter_.has(d)) iter_[d] = 1;
@@ -68,11 +71,11 @@ public:
     void stringify(std::ostream &out) const {
         out << "simd=" << simd_;
         out << " l=";
-        loop_.stringify(out);
+        out << loop_.str();
         out << " T=";
-        thread_group_.stringify(out);
+        out << thread_group_.str();
         out << " i=";
-        iter_.stringify(out);
+        out << iter_.str();
     }
 
     void parse(std::istream &in) {
@@ -91,7 +94,7 @@ public:
     }
 
     std::string str(bool csv = false) const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         if (csv) {
             oss << simd_;
             oss << "," << loop_;
@@ -107,15 +110,15 @@ public:
     }
 
     // Returns the ratio of all operations (with padding) to "useful" operations
-    float get_efficiency(const prb_tile_t &shape) const {
-        float ret = 1;
+    double get_efficiency(const tile_t &shape) const {
+        double ret = 1;
         for (auto &d : shape) {
-            int loop = loop_.get(d, 1);
-            int tg = thread_group_.get(d, 1);
-            int iter = iter_.get(d, 1);
-            int size = shape[d];
-            int size_padded = utils::rnd_up(size, loop * tg * iter);
-            if (size_padded != size) ret *= float(size) / size_padded;
+            dim_t loop = loop_.get(d, 1);
+            dim_t tg = thread_group_.get(d, 1);
+            dim_t iter = iter_.get(d, 1);
+            dim_t size = shape[d];
+            dim_t size_padded = utils::rnd_up(size, loop * tg * iter);
+            if (size_padded != size) ret *= double(size) / size_padded;
         }
         return ret;
     }
@@ -124,9 +127,9 @@ public:
 
 private:
     int simd_ = 0;
-    prb_tile_t loop_;
-    prb_tile_t thread_group_;
-    prb_tile_t iter_;
+    tile_t loop_;
+    tile_t thread_group_;
+    tile_t iter_;
 };
 
 struct blocking_hash_t {
@@ -148,26 +151,7 @@ enum class tile_flags_t : uint32_t {
     loop_iter_unroll = (1 << 4),
 };
 
-inline tile_flags_t operator&(tile_flags_t a, tile_flags_t b) {
-    auto _a = static_cast<uint32_t>(a);
-    auto _b = static_cast<uint32_t>(b);
-    return static_cast<tile_flags_t>(_a & _b);
-}
-
-inline tile_flags_t operator|(tile_flags_t a, tile_flags_t b) {
-    auto _a = static_cast<uint32_t>(a);
-    auto _b = static_cast<uint32_t>(b);
-    return static_cast<tile_flags_t>(_a | _b);
-}
-
-inline tile_flags_t operator~(tile_flags_t a) {
-    auto _a = static_cast<uint32_t>(a);
-    return static_cast<tile_flags_t>(~_a);
-}
-
-inline bool any(tile_flags_t a) {
-    return a != tile_flags_t::undef;
-}
+GPU_DEFINE_BIT_MASK_ENUM_OPS(tile_flags_t)
 
 // Divisibility restrictions for a prb dimension.
 struct div_info_t {
@@ -183,7 +167,7 @@ struct div_info_t {
         unroll_unit = math::lcm(unroll_unit, new_unit);
     }
 
-    bool is_iter_ok(int blk) const {
+    bool is_iter_ok(dim_t blk) const {
         if (iter_unit != 1 && blk % iter_unit != 0) return false;
         if (iter_unit != 1 && !math::is_pow2(blk)) return false;
         return true;
@@ -193,7 +177,7 @@ struct div_info_t {
 // Blocking restrictions for a prb dimension.
 struct tile_info_t {
     tile_info_t() = default;
-    tile_info_t(const prb_dim_t &dim) : dim(dim) {}
+    tile_info_t(const pvar_t &dim) : dim(dim) {}
     void add(tile_flags_t f) { flags = flags | f; }
     void remove(tile_flags_t f) { flags = flags & ~f; }
     void set_iter_unit(int unit) { div_info.set_iter_unit(unit); }
@@ -203,20 +187,20 @@ struct tile_info_t {
         if (pow2_block != 0) min_iter_pow2_blk = pow2_block;
     }
 
-    std::vector<int> iter_blocks(int size) const;
-    std::vector<int> thread_group_blocks(int size) const;
-    std::vector<int> loop_blocks(int size, int iter_blk) const;
+    std::vector<int> iter_blocks(dim_t size) const;
+    std::vector<int> thread_group_blocks(dim_t size) const;
+    std::vector<dim_t> loop_blocks(dim_t size, int iter_blk) const;
 
-    static bool block_ok(int size, int blk, int target_eff) {
-        int size_padded = utils::rnd_up(size, blk);
+    static bool block_ok(dim_t size, int blk, int target_eff) {
+        dim_t size_padded = utils::rnd_up(size, blk);
         double eff = size / (double)size_padded;
         return eff * 100 >= target_eff;
     }
 
-    static std::vector<int> get_factors(int n);
-    static std::vector<int> get_loop_blocks(int n);
+    static std::vector<dim_t> get_factors(dim_t n);
+    static std::vector<dim_t> get_loop_blocks(dim_t n);
 
-    prb_dim_t dim;
+    pvar_t dim;
     tile_flags_t flags = tile_flags_t::undef;
     div_info_t div_info;
 
@@ -231,45 +215,49 @@ struct tile_info_t {
     static const int default_max_thread_group_blk = 16;
 };
 
-// Tile level kinds.
-enum class level_kind_t {
+// Tile levels.
+enum class level_t {
     undef = 0,
     loop,
     thread_group,
     iter,
-    _max,
 };
 
-inline std::string to_string(level_kind_t kind) {
-    std::ostringstream oss;
-    switch (kind) {
-#define CASE(name, value) \
-    case level_kind_t::name: return value
-        CASE(loop, "l");
-        CASE(thread_group, "T");
-        CASE(iter, "i");
-#undef CASE
-        default: ir_error_not_expected();
+class level_tile_t {
+public:
+    bool has(level_t level) const {
+        switch (level) {
+            case level_t::loop: return loop != 0;
+            case level_t::thread_group: return thread_group != 0;
+            case level_t::iter: return iter != 0;
+            default: gpu_error_not_expected();
+        }
+        return false;
     }
-    return oss.str();
-}
 
-using level_t = map_key_t<level_kind_t>;
-using level_tile_t = tile_t<level_t>;
+    std::string str() const {
+        if (utils::everyone_is(0, loop, thread_group, iter)) return "x";
+        ostringstream_t oss;
+        if (loop != 0) oss << "l" << loop;
+        if (thread_group != 0) oss << "T" << thread_group;
+        if (iter != 0) oss << "i" << iter;
+        return oss.str();
+    }
 
-namespace levels {
-extern level_t loop;
-extern level_t thread_group;
-extern level_t iter;
-}; // namespace levels
+    IR_DEFINE_DUMP()
+
+    dim_t loop = 0;
+    int thread_group = 0;
+    int iter = 0;
+};
 
 void get_level_tiles(
-        int size, const tile_info_t &info, std::vector<level_tile_t> &ret);
+        dim_t size, const tile_info_t &info, std::vector<level_tile_t> &ret);
 
 class level_tile_set_t {
 public:
     level_tile_set_t(const std::vector<std::vector<level_tile_t>> &tiles,
-            const std::vector<int> &deps, const std::vector<prb_dim_t> &dims)
+            const std::vector<int> &deps, const std::vector<pvar_t> &dims)
         : tiles_(tiles), deps_(deps), dims_(dims) {}
 
     int count() const;
@@ -280,7 +268,7 @@ public:
 
 private:
     static void set(
-            blocking_t &blk, const prb_dim_t &dim, const level_tile_t &tile);
+            blocking_t &blk, const pvar_t &dim, const level_tile_t &tile);
 
     void product_impl(int idx, std::vector<int> &cur_idxs, blocking_t &blk,
             std::vector<blocking_t> &ret) const;
@@ -289,7 +277,7 @@ private:
 
     std::vector<std::vector<level_tile_t>> tiles_;
     std::vector<int> deps_;
-    std::vector<prb_dim_t> dims_;
+    std::vector<pvar_t> dims_;
 };
 
 // Blocking scheme describing recipes to generate blockings.
@@ -298,13 +286,13 @@ public:
     virtual ~blocking_scheme_t() = default;
     blocking_scheme_t() = default;
     blocking_scheme_t(const std::string &s) {
-        ir_assert(s[s.length() - 1] == ']');
+        gpu_assert(s[s.length() - 1] == ']');
         auto parts = gpu_utils::split(s.substr(0, s.length() - 1), "],");
         for (auto &p : parts) {
             auto p_parts = gpu_utils::split(p, ":");
             auto &key = p_parts[0];
             auto &vec = p_parts[1];
-            ir_assert(vec[0] == '[');
+            gpu_assert(vec[0] == '[');
             auto s_dims
                     = gpu_utils::split(vec.substr(1, vec.length() - 1), ",");
             for (auto &s : s_dims)
@@ -313,7 +301,7 @@ public:
     }
 
     virtual level_tile_set_t make_level_tile_set(
-            const prb_tile_t &padded_shape) const {
+            const tile_t &padded_shape) const {
         const auto all_dims = dims();
         const int ndims = int(all_dims.size());
         const std::vector<int> deps(ndims, -1);
@@ -326,36 +314,30 @@ public:
         return level_tile_set_t(tiles, deps, all_dims);
     }
 
-    tile_info_t &tile_info(const prb_dim_t &d) {
-        auto it = tile_infos_.find(d.id());
+    tile_info_t &tile_info(const pvar_t &d) {
+        auto it = tile_infos_.find(d);
         if (it != tile_infos_.end()) return it->second;
-        auto &info = tile_infos_[d.id()];
+        auto &info = tile_infos_[d];
         info = tile_info_t(d);
         return info;
     }
 
-    const tile_info_t &tile_info(const prb_dim_t &d) const {
-        return tile_infos_.at(d.id());
+    const tile_info_t &tile_info(const pvar_t &d) const {
+        return tile_infos_.at(d);
     }
 
-    std::vector<prb_dim_t> dims() const {
-        std::vector<prb_dim_t> all_dims;
-        for (int i = 0; i < prb_dim_t::max_id(); i++) {
-            auto d = prb_dim_t::from_id(i);
-            if (d.is_undef()) continue;
-            all_dims.push_back(d);
-        }
-        std::vector<prb_dim_t> ret;
-        for (auto &d : all_dims) {
-            if (loop_.has(d) || thread_group_.has(d) || iter_.has(d)) {
-                ret.push_back(d);
+    std::vector<pvar_t> dims() const {
+        std::set<pvar_t> dims;
+        for (auto *t : {&loop_, &thread_group_, &iter_}) {
+            for (auto &d : t->keys()) {
+                dims.insert(d);
             }
         }
-        return ret;
+        return std::vector<pvar_t>(dims.begin(), dims.end());
     }
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "l:" << loop_;
         oss << " T:" << thread_group_;
         oss << " i:" << iter_;
@@ -366,11 +348,11 @@ public:
 
 private:
     void set(const std::string &s_tile, const std::string &_s_dim) {
-        ir_assert(!_s_dim.empty());
+        gpu_assert(!_s_dim.empty());
         bool no_min_check = (_s_dim[0] == '#');
-        auto s_dim = no_min_check ? _s_dim.substr(1) : _s_dim;
-        auto d = prb_dim_t::from_name(s_dim);
-        if (no_min_check) ir_assert(s_tile == "i");
+        const auto &s_dim = no_min_check ? _s_dim.substr(1) : _s_dim;
+        auto d = pvar_t(s_dim);
+        if (no_min_check) gpu_assert(s_tile == "i");
         if (s_tile == "i") {
             add_iter_dim(d);
             if (no_min_check) tile_info(d).set_min_iter_block(1);
@@ -383,43 +365,43 @@ private:
         } else if (s_tile == "li") {
             add_loop_dim_with_iter_unroll(d);
         } else {
-            ir_error_not_expected() << s_tile;
+            gpu_error_not_expected() << s_tile;
         }
     }
 
-    void add_loop_dim(const prb_dim_t &d) {
+    void add_loop_dim(const pvar_t &d) {
         loop_[d] = 1;
         auto &info = tile_info(d);
         info.add(tile_flags_t::loop);
     }
 
-    void add_loop_dim_with_span(const prb_dim_t &d) {
+    void add_loop_dim_with_span(const pvar_t &d) {
         add_loop_dim(d);
         tile_info(d).add(tile_flags_t::loop_span);
     }
 
-    void add_loop_dim_with_iter_unroll(const prb_dim_t &d) {
+    void add_loop_dim_with_iter_unroll(const pvar_t &d) {
         add_loop_dim(d);
         tile_info(d).add(tile_flags_t::loop_iter_unroll);
     }
 
-    void add_thread_group_dim(const prb_dim_t &d) {
+    void add_thread_group_dim(const pvar_t &d) {
         thread_group_[d] = 1;
         auto &info = tile_info(d);
         info.add(tile_flags_t::thread_group);
     }
 
-    void add_iter_dim(const prb_dim_t &d) {
+    void add_iter_dim(const pvar_t &d) {
         iter_[d] = 1;
         auto &info = tile_info(d);
         info.add(tile_flags_t::iter);
     }
 
 protected:
-    prb_tile_t loop_;
-    prb_tile_t thread_group_;
-    prb_tile_t iter_;
-    std::unordered_map<int, tile_info_t> tile_infos_;
+    tile_t loop_;
+    tile_t thread_group_;
+    tile_t iter_;
+    std::map<pvar_t, tile_info_t> tile_infos_;
 };
 
 template <class blocking_scheme_kind>
@@ -487,7 +469,7 @@ public:
     void set_id(int id) { id_ = id; }
 
     void stringify(std::ostream &out) const {
-        blocking_.stringify(out);
+        out << blocking_.str();
         if (bufs_hint_ != -1) out << " bufs=" << bufs_hint_;
     }
 
@@ -502,7 +484,7 @@ public:
     }
 
     std::string str(bool csv = false) const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         if (csv) {
             oss << blocking_.str(csv);
             oss << "," << bufs_hint_;
@@ -545,6 +527,7 @@ public:
     const std::vector<blocking_params_t> &params_vec() const {
         return params_vec_;
     }
+    std::vector<blocking_params_t> &params_vec() { return params_vec_; }
 
     bool is_empty() const { return params_vec_.empty(); }
 
@@ -555,14 +538,14 @@ public:
     int cur_index() const { return cur_idx_; }
 
     void set_cur_index(int idx) {
-        ir_assert(idx < configs());
+        gpu_assert(idx < configs());
         cur_idx_ = idx;
     }
 
     const blocking_params_t &cur_params() const { return at(cur_idx_); }
 
     const blocking_params_t &at(int idx) const {
-        ir_assert(idx >= 0 && idx < configs());
+        gpu_assert(idx >= 0 && idx < configs());
         return params_vec_[idx];
     }
 
@@ -572,8 +555,8 @@ public:
 
     template <typename KeyFuncT>
     void sort(int beg, int end, const KeyFuncT &key_func) {
-        ir_assert(beg >= 0 && beg < configs());
-        ir_assert(end >= beg && end <= configs());
+        gpu_assert(beg >= 0 && beg < configs());
+        gpu_assert(end >= beg && end <= configs());
         std::sort(params_vec_.begin() + beg, params_vec_.begin() + end,
                 [&](const blocking_params_t &a, const blocking_params_t &b) {
                     return key_func(a) < key_func(b);
@@ -582,7 +565,7 @@ public:
 
     template <typename PredicateFuncT>
     void remove_if(const PredicateFuncT &func) {
-        ir_assert(cur_idx_ == -1);
+        gpu_assert(cur_idx_ == -1);
         params_vec_.erase(
                 std::remove_if(params_vec_.begin(), params_vec_.end(), func),
                 params_vec_.end());
@@ -596,7 +579,7 @@ public:
         table_t table("List of configs", headers);
         for (int i = 0; i < configs(); i++) {
             auto &params = params_vec_[i];
-            ir_trace() << "params #" << i << ": " << params << std::endl;
+            gpu_trace() << "params #" << i << ": " << params;
         }
     }
 
@@ -657,7 +640,7 @@ const tiler_params_t &tiler_params();
 class tile_to_vec_t {
 public:
     tile_to_vec_t() = default;
-    tile_to_vec_t(const std::vector<std::vector<prb_tile_t>> &tiles,
+    tile_to_vec_t(const std::vector<std::vector<tile_t>> &tiles,
             const std::vector<int> &ids = {});
 
     float dist(int id0, int id1) const {
@@ -666,7 +649,7 @@ public:
         float ret = 0;
         // Use L1 distance between coordinates.
         for (int i = 0; i < (int)v0.size(); i++) {
-            ret += std::abs(v0[i] - v1[i]);
+            ret += float(std::abs(v0[i] - v1[i]));
         }
         return ret;
     }
@@ -676,64 +659,63 @@ private:
     struct indexed_tile_t {
         struct indexed_dim_t {
             indexed_dim_t() = default;
-            indexed_dim_t(const prb_dim_t &dim) : dim_(dim) {}
+            indexed_dim_t(const pvar_t &dim) : dim_(dim) {}
             bool is_empty() const { return values_.empty(); }
-            const prb_dim_t &dim() const { return dim_; }
+            const pvar_t &dim() const { return dim_; }
 
-            void add(int value) { values_.emplace(value, -1); }
+            void add(dim_t value) { values_.emplace(value, dim_idx::invalid); }
 
             void finalize() {
-                int idx = 0;
+                dim_idx_t idx = 0;
                 add(1);
                 for (auto &kv : values_) {
                     kv.second = idx++;
                 }
             }
 
-            int to_index(int value) const {
+            dim_idx_t to_index(dim_t value) const {
                 auto it = values_.find(value);
-                ir_assert(it != values_.end());
+                gpu_assert(it != values_.end());
                 return it->second;
             }
 
-            prb_dim_t dim_;
-            std::map<int, int> values_;
+            pvar_t dim_;
+            std::map<dim_t, dim_idx_t> values_;
         };
 
-        indexed_tile_t() {
-            for (int i = 0; i < prb_dim_t::max_id(); i++) {
-                auto d = prb_dim_t::from_id(i);
-                dim_mappers_[i] = indexed_dim_t(d);
+        void add(const pvar_t &d, dim_t value) {
+            if (dim_mappers_.count(d) == 0) {
+                dim_mappers_[d] = indexed_dim_t(d);
             }
+            dim_mappers_[d].add(value);
         }
 
-        void add(prb_dim_t d, int value) { dim_mappers_[d.id()].add(value); }
-
-        void add(const prb_tile_t &t) {
+        void add(const tile_t &t) {
             for (auto &d : t) {
                 add(d, t[d]);
             }
         }
 
         void finalize() {
-            for (auto &d : dim_mappers_)
-                if (!d.is_empty()) d.finalize();
+            for (auto &kv : dim_mappers_)
+                if (!kv.second.is_empty()) kv.second.finalize();
         }
 
-        int to_index(const prb_dim_t &d, int value) const {
-            return dim_mappers_[d.id()].to_index(value);
+        dim_idx_t to_index(const pvar_t &d, dim_t value) const {
+            return dim_mappers_.at(d).to_index(value);
         }
 
-        std::vector<int> to_index(const prb_tile_t &t) const {
-            std::vector<int> ret;
-            for (auto &m : dim_mappers_) {
+        std::vector<dim_idx_t> to_index(const tile_t &t) const {
+            std::vector<dim_idx_t> ret;
+            for (auto &kv : dim_mappers_) {
+                auto &m = kv.second;
                 if (m.is_empty()) continue;
                 ret.push_back(to_index(m.dim(), t.get(m.dim(), 1)));
             }
             return ret;
         }
 
-        std::array<indexed_dim_t, prb_dim_t::max_id()> dim_mappers_;
+        std::unordered_map<pvar_t, indexed_dim_t> dim_mappers_;
     };
 
     std::vector<std::vector<int>> vecs_;
