@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@
 
 #include <algorithm>
 
-#include "gpu/intel/jit/ir/linear_expr.hpp"
 #include "gpu/intel/jit/pass/simplify.hpp"
 
 namespace dnnl {
@@ -30,55 +29,22 @@ namespace jit {
 expr_t const_fold_non_recursive(const expr_t &expr);
 object_t const_fold(const object_t &obj);
 
-int type_t::size() const {
-    if (is_ptr()) return sizeof(uint64_t);
-
-    if (is_bool()) return utils::div_up(elems(), 8);
-
-    if (elems() != 1) return elems() * scalar().size();
-
-    switch (kind()) {
-        case type_kind_t::u8:
-        case type_kind_t::s8:
-        case type_kind_t::bf8:
-        case type_kind_t::hf8:
-        case type_kind_t::byte: return 1;
-        case type_kind_t::u16:
-        case type_kind_t::s16:
-        case type_kind_t::bf16:
-        case type_kind_t::f16: return 2;
-        case type_kind_t::u32:
-        case type_kind_t::s32:
-        case type_kind_t::tf32:
-        case type_kind_t::f32:
-        case type_kind_t::dword: return 4;
-        case type_kind_t::f64:
-        case type_kind_t::u64:
-        case type_kind_t::s64:
-        case type_kind_t::qword: return 8;
-        case type_kind_t::oword: return 16;
-        case type_kind_t::hword: return 32;
-        default: ir_error_not_expected();
-    }
-    return 0;
-}
-
 data_type_t to_dnnl(const type_t &type) {
-    ir_assert(type.elems() == 1) << type;
-    ir_assert(!type.is_ptr() == 1) << type;
-    switch (type.kind()) {
-        case type_kind_t::bf8: return data_type::f8_e5m2;
-        case type_kind_t::hf8: return data_type::f8_e4m3;
-        case type_kind_t::bf16: return data_type::bf16;
-        case type_kind_t::f16: return data_type::f16;
-        case type_kind_t::tf32: return data_type::tf32;
-        case type_kind_t::f32: return data_type::f32;
-        case type_kind_t::f64: return data_type::f64;
-        case type_kind_t::s32: return data_type::s32;
-        case type_kind_t::s8: return data_type::s8;
-        case type_kind_t::u8: return data_type::u8;
-        default: ir_error_not_expected();
-    }
+    gpu_assert(type.elems() == 1) << type;
+    gpu_assert(!type.is_ptr() == 1) << type;
+    if (type.is_f4_e3m0()) return data_type::f4_e3m0;
+    if (type.is_f4_e2m1()) return data_type::f4_e2m1;
+    if (type.is_bf8()) return data_type::f8_e5m2;
+    if (type.is_hf8()) return data_type::f8_e4m3;
+    if (type.is_bf16()) return data_type::bf16;
+    if (type.is_f16()) return data_type::f16;
+    if (type.is_tf32()) return data_type::tf32;
+    if (type.is_f32()) return data_type::f32;
+    if (type.is_f64()) return data_type::f64;
+    if (type.is_s32()) return data_type::s32;
+    if (type.is_s8()) return data_type::s8;
+    if (type.is_u8()) return data_type::u8;
+    gpu_error_not_expected();
     return data_type::undef;
 }
 
@@ -104,6 +70,8 @@ std::string to_string(op_kind_t kind) {
         case op_kind_t::_ne: return "!=";
 
         case op_kind_t::_and: return "&&";
+        case op_kind_t::_or: return "||";
+        case op_kind_t::_xor: return "^";
 
         case op_kind_t::_add3: return "add3";
         case op_kind_t::_mad: return "mad";
@@ -112,7 +80,7 @@ std::string to_string(op_kind_t kind) {
         case op_kind_t::_idiv: return "idiv";
         case op_kind_t::_imod: return "imod";
 
-        default: ir_error_not_expected() << "Unknown op_kind_t value.";
+        default: gpu_error_not_expected() << "Unknown op_kind_t value.";
     }
     return "";
 }
@@ -138,6 +106,8 @@ bool is_commutative_op(op_kind_t op_kind) {
         case op_kind_t::_eq:
         case op_kind_t::_ne:
         case op_kind_t::_and:
+        case op_kind_t::_or:
+        case op_kind_t::_xor:
         case op_kind_t::_add3: return true;
         default: return false;
     }
@@ -151,7 +121,7 @@ op_kind_t negate_cmp_op(op_kind_t op_kind) {
         case op_kind_t::_lt: return op_kind_t::_gt;
         case op_kind_t::_eq: return op_kind_t::_eq;
         case op_kind_t::_ne: return op_kind_t::_ne;
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return op_kind_t::undef;
 }
@@ -161,48 +131,56 @@ type_t unary_op_type(op_kind_t op_kind, const expr_t &a) {
         case op_kind_t::_minus: {
             auto &t = a.type();
             if (!t.is_int()) return t;
-            if (t.size() < int(sizeof(int32_t))) return type_t::s32(t.elems());
+            if (t.size() < int(sizeof(int32_t)))
+                return type_t::s32(t.elems(), a.type().attr());
             return t;
         }
         default:
-            ir_error_not_expected() << "Unknown op_kind_t value: " << op_kind;
+            gpu_error_not_expected() << "Unknown op_kind_t value: " << op_kind;
     }
     return type_t::undef();
 }
 
-type_t common_int_type(const type_t &_a, const type_t &_b) {
-    ir_assert(_a.is_int() && _b.is_int()) << "Unexpected types.";
+type::attr_t common_attr(const type_t &a, const type_t &b) {
+    gpu_assert(!a.is_ptr() && !b.is_ptr());
+    return (a.attr() | b.attr()) & ~type::attr_t::mut;
+}
 
-    int elems = _a.elems();
+type_t common_type(const type_t &base, const type_t &a, const type_t &b) {
+    auto attr = common_attr(a, b);
+    int elems = std::max(a.elems(), b.elems());
+    return base[elems].with_attr(attr);
+}
+
+type_t common_int_type_impl(const type_t &_a, const type_t &_b) {
+    gpu_assert(_a.is_int() && _b.is_int()) << "Unexpected types.";
 
     // Promote to s32 first.
     type_t a = _a.size() < int(sizeof(int32_t)) ? type_t::s32() : _a;
     type_t b = _b.size() < int(sizeof(int32_t)) ? type_t::s32() : _b;
-    a = a.scalar();
-    b = b.scalar();
+    a = a.base();
+    b = b.base();
 
     // Integer promotion, follow C++ rules.
     int common_bits = 8 * std::max(a.size(), b.size());
     if (a.is_signed() == b.is_signed()) {
-        if (a.is_signed()) return type_t::s(common_bits, elems);
-        return type_t::u(common_bits, elems);
+        if (a.is_signed()) return type_t::s(common_bits);
+        return type_t::u(common_bits);
     }
 
-    if (a.size() >= b.size() && a.is_unsigned())
-        return type_t::u(common_bits, elems);
-    if (b.size() >= a.size() && b.is_unsigned())
-        return type_t::u(common_bits, elems);
-    if (a.size() > b.size() && a.is_signed())
-        return type_t::s(common_bits, elems);
-    if (b.size() > a.size() && b.is_signed())
-        return type_t::s(common_bits, elems);
+    if (a.size() >= b.size() && a.is_unsigned()) return type_t::u(common_bits);
+    if (b.size() >= a.size() && b.is_unsigned()) return type_t::u(common_bits);
+    if (a.size() > b.size() && a.is_signed()) return type_t::s(common_bits);
+    if (b.size() > a.size() && b.is_signed()) return type_t::s(common_bits);
 
-    return type_t::u(common_bits, elems);
+    return type_t::u(common_bits);
 }
 
-type_t common_type(const type_t &a, const type_t &b) {
-    ir_assert(a.elems() == b.elems())
-            << "Types must have the same number of components.";
+type_t common_int_type(const type_t &a, const type_t &b) {
+    return common_type(common_int_type_impl(a, b), a, b);
+}
+
+type_t common_type_impl(const type_t &a, const type_t &b) {
     if (a.is_undef() || b.is_undef()) return type_t::undef();
     if (a.is_fp() && !b.is_fp()) return a;
     if (!a.is_fp() && b.is_fp()) return b;
@@ -211,6 +189,10 @@ type_t common_type(const type_t &a, const type_t &b) {
     return common_int_type(a, b);
 }
 
+type_t common_type(const type_t &a, const type_t &b) {
+    return common_type(common_type_impl(a, b), a, b);
+}
+
 type_t common_type(const expr_t &a, const expr_t &b) {
     return common_type(a.type(), b.type());
 }
@@ -218,20 +200,26 @@ type_t common_type(const expr_t &a, const expr_t &b) {
 type_t binary_op_type(op_kind_t op_kind, const type_t &a, const type_t &b,
         const expr_t &a_expr = expr_t(), const expr_t &b_expr = expr_t()) {
     if (a.is_undef() || b.is_undef()) return type_t::undef();
-    ir_assert(a.elems() == b.elems())
-            << "Types must have the same number of components.";
-    if (is_cmp_op(op_kind)) return type_t::_bool(a.elems());
+    int elems = std::max(a.elems(), b.elems());
+
+    type::attr_t attr = common_attr(a, b);
+    if (is_cmp_op(op_kind)) return type_t::_bool(elems, attr);
     if (utils::one_of(op_kind, op_kind_t::_shl, op_kind_t::_shr)) {
-        ir_assert(a.is_unsigned())
-                << "a must be unsigned for shift left/right.";
-        return type_t::u32(a.elems());
+        return a[elems].with_attr(attr);
     }
-    if (op_kind == op_kind_t::_and) {
+
+    if (utils::one_of(
+                op_kind, op_kind_t::_and, op_kind_t::_or, op_kind_t::_xor)) {
         if (a == b) return a;
         if (is_const(a_expr)) return b;
         if (is_const(b_expr)) return a;
         return (a.size() >= b.size()) ? a : b;
     }
+    if (utils::one_of(op_kind, op_kind_t::_div, op_kind_t::_mod) && a.is_int()
+            && b.is_int()) {
+        return (a.is_signed() ? type_t::s32() : type_t::u32())[elems].with_attr(
+                attr);
+    }
     return common_type(a, b);
 }
 
@@ -251,13 +239,13 @@ type_t ternary_op_type(
         case op_kind_t::_idiv:
         case op_kind_t::_imod:
             return a.type().is_signed() ? type_t::s32() : type_t::u32();
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return type_t::undef();
 }
 
 type_t nary_op_type(op_kind_t op_kind, const std::vector<expr_t> &args) {
-    ir_assert(!args.empty());
+    gpu_assert(!args.empty());
     if (args.size() == 1) return args[0].type();
 
     auto type = args[0].type();
@@ -268,9 +256,9 @@ type_t nary_op_type(op_kind_t op_kind, const std::vector<expr_t> &args) {
 }
 
 void ptr_t::normalize(expr_t &base, expr_t &off, op_kind_t op_kind) {
-    ir_assert(base.type().is_ptr()) << "base is not a pointer: " << base;
-    ir_assert(off.type().is_int()) << "off is not an integer: " << off;
-    ir_assert(utils::one_of(op_kind, op_kind_t::_add, op_kind_t::_sub))
+    // Normalize (base + off1) + off2 -> base + (off1 + off2)
+    gpu_assert(off.type().is_int()) << "off is not an integer: " << off;
+    gpu_assert(utils::one_of(op_kind, op_kind_t::_add, op_kind_t::_sub))
             << "Can't apply this operation to pointer: " << to_string(op_kind);
 
     if (!base.is<ptr_t>()) {
@@ -294,9 +282,16 @@ void normalize_ptr(const type_t &type, expr_t &base_expr, expr_t &off) {
     if (base_expr.is<ptr_t>()) {
         off = const_fold_non_recursive(base_expr.as<ptr_t>().off + off);
         base_expr = base_expr.as<ptr_t>().base;
+        return;
     }
-    ir_assert(to_cpp<int64_t>(off) % type.scalar().size() == 0)
-            << "Incompatible offset: " << off;
+    gpu_assert(is_const(off)) << "var/ref requires constant offset.";
+    if (auto *ref = base_expr.as_ptr<ref_t>()) {
+        off = const_fold_non_recursive(ref->off + off);
+        base_expr = ref->var;
+        return;
+    }
+    if (base_expr.is<var_t>()) return;
+    gpu_error_not_expected() << "Unexpected expression: " << base_expr.str();
 }
 
 expr_t linear_t::to_expr() const {
@@ -307,77 +302,21 @@ expr_t linear_t::to_expr() const {
     return simplify_rewrite(ret);
 }
 
-expr_t expr_t::operator[](const expr_t &off) const {
-    if (is<shuffle_t>()) {
-        ir_assert(is_const(off)) << "Offset is not constant.";
-        auto &shuffle = as<shuffle_t>();
-        int idx = shuffle.idx[to_cpp<int>(off)];
-        return shuffle.vec[idx];
+static void stmt_seq_flatten(std::vector<stmt_t> &out, const stmt_t &s) {
+    if (auto *seq = s.as_ptr<stmt_seq_t>()) {
+        out.insert(out.end(), seq->vec.begin(), seq->vec.end());
+        return;
     }
-    return shift_ptr(op_kind_t::_add, *this, off);
+    out.push_back(s);
 }
 
-expr_t::expr_t(bool value) : object_t(new bool_imm_t(value)) {}
-expr_t::expr_t(float value) : object_t(new float_imm_t(value)) {}
-expr_t::expr_t(double value)
-    : object_t(new float_imm_t(value, type_t::f64())) {}
-expr_t::expr_t(int16_t value) : object_t(new int_imm_t(value)) {}
-expr_t::expr_t(int32_t value) : object_t(new int_imm_t(value)) {}
-expr_t::expr_t(int64_t value) : object_t(new int_imm_t(value)) {}
-expr_t::expr_t(uint16_t value) : object_t(new int_imm_t(value)) {}
-expr_t::expr_t(uint32_t value) : object_t(new int_imm_t(value)) {}
-expr_t::expr_t(uint64_t value) : object_t(new int_imm_t(value)) {}
-
-expr_t operator-(const expr_t &a) {
-    return const_fold_non_recursive(unary_op_t::make(op_kind_t::_minus, a));
+stmt_t stmt_seq_t::make(const std::vector<stmt_t> &_vec) {
+    std::vector<stmt_t> vec;
+    for (auto &s : _vec)
+        stmt_seq_flatten(vec, s);
+    return stmt_t(new stmt_seq_t(vec));
 }
 
-#define DEFINE_BINARY_OPERATOR(op, op_kind) \
-    expr_t operator op(const expr_t &a, const expr_t &b) { \
-        if (a.type().is_ptr()) return shift_ptr(op_kind, a, b); \
-        return const_fold_non_recursive(binary_op_t::make(op_kind, a, b)); \
-    }
-
-DEFINE_BINARY_OPERATOR(+, op_kind_t::_add)
-DEFINE_BINARY_OPERATOR(-, op_kind_t::_sub)
-DEFINE_BINARY_OPERATOR(*, op_kind_t::_mul)
-DEFINE_BINARY_OPERATOR(/, op_kind_t::_div)
-DEFINE_BINARY_OPERATOR(%, op_kind_t::_mod)
-DEFINE_BINARY_OPERATOR(<<, op_kind_t::_shl)
-DEFINE_BINARY_OPERATOR(>>, op_kind_t::_shr)
-
-DEFINE_BINARY_OPERATOR(==, op_kind_t::_eq)
-DEFINE_BINARY_OPERATOR(!=, op_kind_t::_ne)
-DEFINE_BINARY_OPERATOR(>, op_kind_t::_gt)
-DEFINE_BINARY_OPERATOR(>=, op_kind_t::_ge)
-DEFINE_BINARY_OPERATOR(<, op_kind_t::_lt)
-DEFINE_BINARY_OPERATOR(<=, op_kind_t::_le)
-
-DEFINE_BINARY_OPERATOR(&, op_kind_t::_and)
-
-#undef DEFINE_BINARY_OPERATOR
-
-#define DEFINE_BINARY_ASSIGN_OPERATOR(op) \
-    expr_t &expr_t::operator op##=(const expr_t &rhs) { \
-        auto tmp = (*this)op rhs; \
-        *this = std::move(tmp); \
-        return *this; \
-    }
-
-DEFINE_BINARY_ASSIGN_OPERATOR(+)
-DEFINE_BINARY_ASSIGN_OPERATOR(-)
-DEFINE_BINARY_ASSIGN_OPERATOR(*)
-DEFINE_BINARY_ASSIGN_OPERATOR(/)
-DEFINE_BINARY_ASSIGN_OPERATOR(%)
-DEFINE_BINARY_ASSIGN_OPERATOR(&)
-
-#undef DEFINE_BINARY_ASSIGN_OPERATOR
-
-object_t object_impl_t::_mutate(ir_mutator_t &mutator) const {
-    return *this;
-}
-void object_impl_t::_visit(ir_visitor_t &visitor) const {}
-
 #define DECL_TRAVERSE_LEAF(name) \
     object_t ir_mutator_t::_mutate(const name &obj) { return obj; } \
     void ir_visitor_t::_visit(const name &obj) {}
@@ -385,7 +324,6 @@ void object_impl_t::_visit(ir_visitor_t &visitor) const {}
 DECL_TRAVERSE_LEAF(bool_imm_t)
 DECL_TRAVERSE_LEAF(const_var_t)
 DECL_TRAVERSE_LEAF(float_imm_t)
-DECL_TRAVERSE_LEAF(func_impl_t)
 DECL_TRAVERSE_LEAF(int_imm_t)
 DECL_TRAVERSE_LEAF(var_t)
 
@@ -405,6 +343,28 @@ void ir_visitor_t::_visit(const alloc_t &obj) {
     visit(obj.body);
 }
 
+object_t ir_mutator_t::_mutate(const assign_t &obj) {
+    auto var = mutate(obj.var);
+    auto value = mutate(obj.value);
+
+    if (var.is_same(obj.var) && value.is_same(obj.value)) return obj;
+
+    return assign_t::make(var, value);
+}
+
+void ir_visitor_t::_visit(const assign_t &obj) {
+    visit(obj.var);
+    visit(obj.value);
+}
+
+object_t ir_mutator_t::_mutate(const ref_t &obj) {
+    auto var = mutate(obj.var);
+    if (var.impl() == obj.var.impl()) return obj;
+    return ref_t::make(var, obj.off, obj.elems);
+}
+
+void ir_visitor_t::_visit(const ref_t &obj) {}
+
 object_t ir_mutator_t::_mutate(const binary_op_t &obj) {
     auto a = mutate(obj.a);
     auto b = mutate(obj.b);
@@ -453,16 +413,14 @@ void ir_visitor_t::_visit(const for_t &obj) {
 }
 
 object_t ir_mutator_t::_mutate(const func_call_t &obj) {
-    auto func = mutate(obj.func);
     auto args = mutate(obj.args);
 
-    if (func.is_same(obj.func) && ir_utils::is_same(args, obj.args)) return obj;
+    if (ir_utils::is_same(args, obj.args)) return obj;
 
-    return func_call_t::make(func, args, obj.attr);
+    return func_call_t::make(obj.func, args, obj.attr);
 }
 
 void ir_visitor_t::_visit(const func_call_t &obj) {
-    visit(obj.func);
     visit(obj.args);
 }
 
@@ -521,12 +479,12 @@ void ir_visitor_t::_visit(const let_t &obj) {
 }
 
 object_t ir_mutator_t::_mutate(const linear_t &obj) {
-    ir_error_not_expected();
+    gpu_error_not_expected();
     return obj;
 }
 
 void ir_visitor_t::_visit(const linear_t &obj) {
-    ir_error_not_expected();
+    gpu_error_not_expected();
 }
 
 object_t ir_mutator_t::_mutate(const load_t &obj) {
@@ -582,17 +540,13 @@ void ir_visitor_t::_visit(const stmt_group_t &obj) {
 }
 
 object_t ir_mutator_t::_mutate(const stmt_seq_t &obj) {
-    auto head = mutate(obj.head);
-    auto tail = mutate(obj.tail);
-
-    if (head.is_same(obj.head) && tail.is_same(obj.tail)) return obj;
-
-    return stmt_seq_t::make(head, tail);
+    auto vec = mutate(obj.vec);
+    if (ir_utils::is_same(vec, obj.vec)) return obj;
+    return stmt_seq_t::make(vec);
 }
 
 void ir_visitor_t::_visit(const stmt_seq_t &obj) {
-    visit(obj.head);
-    visit(obj.tail);
+    visit(obj.vec);
 }
 
 object_t ir_mutator_t::_mutate(const store_t &obj) {
@@ -641,21 +595,18 @@ void ir_visitor_t::_visit(const unary_op_t &obj) {
     visit(obj.a);
 }
 
-// Catch missing mutates that are not expected to dispatch to the base
-// mutator
-object_t ir_mutator_t::_mutate(const nary_op_t &obj) {
-    ir_error_not_expected() << "Can't handle type: nary_op_t";
-    return {};
+object_t ir_mutator_t::_mutate(const while_t &obj) {
+    auto cond = mutate(obj.cond);
+    auto body = mutate(obj.body);
+
+    if (cond.is_same(obj.cond) && body.is_same(obj.body)) return obj;
+
+    return while_t::make(cond, body);
 }
-void ir_visitor_t::_visit(const nary_op_t &obj) {
-    ir_error_not_expected() << "Can't handle type: nary_op_t";
-}
-object_t ir_mutator_t::_mutate(const pexpr_t &obj) {
-    ir_error_not_expected() << "Can't handle type: pexpr_t";
-    return {};
-}
-void ir_visitor_t::_visit(const pexpr_t &obj) {
-    ir_error_not_expected() << "Can't handle type: pexpr_t";
+
+void ir_visitor_t::_visit(const while_t &obj) {
+    visit(obj.cond);
+    visit(obj.body);
 }
 
 } // namespace jit
