@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -29,17 +29,20 @@ namespace gpu {
 namespace intel {
 namespace jit {
 
-class eltwise_t : public func_impl_t {
+class eltwise_t : public func_impl_t, public object::info_t<eltwise_t> {
 public:
-    IR_DECL_DERIVED_TYPE_ID(eltwise_t, func_impl_t)
-
+    static func_t make(alg_kind_t alg_kind, float scale, float alpha,
+            float beta, expr_t &seed, ngen::DataType dst_dt) {
+        return func_t(
+                new eltwise_t(alg_kind, scale, alpha, beta, seed, dst_dt));
+    }
     static func_t make(
             alg_kind_t alg_kind, float scale, float alpha, float beta) {
         return func_t(new eltwise_t(alg_kind, scale, alpha, beta));
     }
 
     std::string str() const override {
-        switch (alg_kind) {
+        switch (static_cast<int>(alg_kind)) {
             case alg_kind::eltwise_relu: return "relu";
             case alg_kind::eltwise_tanh: return "tanh";
             case alg_kind::eltwise_elu: return "elu";
@@ -59,6 +62,7 @@ public:
             case alg_kind::eltwise_gelu_tanh: return "gelu_tanh";
             case alg_kind::eltwise_gelu_erf: return "gelu_erf";
             case alg_kind::eltwise_hardswish: return "hardswish";
+            case alg_kind::eltwise_hardsigmoid: return "hardsigmoid";
             case alg_kind::eltwise_relu_use_dst_for_bwd:
                 return "relu_use_dst_for_bwd";
             case alg_kind::eltwise_tanh_use_dst_for_bwd:
@@ -74,7 +78,10 @@ public:
             case alg_kind::eltwise_clip_v2_use_dst_for_bwd:
                 return "clip_v2_use_dst_for_bwd";
             case alg_kind::eltwise_round: return "round";
-            default: ir_error_not_expected();
+            // Note: `eltwise_stochastic_round` is not a part of `enum` which
+            // forces `switch` to iterate over `int`, not `alg_kind_t`.
+            case alg_kind::eltwise_stochastic_round: return "stochastic_round";
+            default: gpu_error_not_expected();
         }
         return "unknown";
     }
@@ -86,14 +93,30 @@ public:
     float scale;
     float alpha;
     float beta;
+    expr_t seed;
+    ngen::DataType dst_dt = ngen::DataType::invalid;
 
 private:
-    eltwise_t(alg_kind_t alg_kind, float scale, float alpha, float beta)
-        : func_impl_t(_type_info())
+    eltwise_t(alg_kind_t alg_kind, float scale, float alpha, float beta,
+            expr_t &seed, ngen::DataType dst_dt)
+        : func_impl_t(get_info())
         , alg_kind(alg_kind)
         , scale(scale)
         , alpha(alpha)
-        , beta(beta) {}
+        , beta(beta)
+        , seed(seed)
+        , dst_dt(dst_dt) {
+        assert(alg_kind == alg_kind::eltwise_stochastic_round);
+    }
+
+    eltwise_t(alg_kind_t alg_kind, float scale, float alpha, float beta)
+        : func_impl_t(get_info())
+        , alg_kind(alg_kind)
+        , scale(scale)
+        , alpha(alpha)
+        , beta(beta) {
+        assert(alg_kind != alg_kind::eltwise_stochastic_round);
+    }
 };
 
 } // namespace jit
