@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -53,7 +53,7 @@ inline std::ostream &operator<<(std::ostream &out, abc_kind_t abc) {
         case abc_kind_t::a: out << "a"; break;
         case abc_kind_t::b: out << "b"; break;
         case abc_kind_t::c: out << "c"; break;
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return out;
 }
@@ -71,7 +71,7 @@ public:
         return it->second;
     }
 
-    bmnk_kind_t bmnk_kind(abc_kind_t abc_kind, int dim_idx) const {
+    bmnk_kind_t bmnk_kind(abc_kind_t abc_kind, size_t dim_idx) const {
         return bmnk_kind(var(abc_kind, dim_idx));
     }
 
@@ -85,10 +85,10 @@ public:
 
     void set_bmnk_kind(const expr_t &var, bmnk_kind_t bmnk_kind) {
         auto ret = bmnk_kinds_.insert({var, bmnk_kind});
-        ir_assert(ret.second) << "Can't set variable twice: " << var;
+        gpu_assert(ret.second) << "Can't set variable twice: " << var;
     }
 
-    const expr_t &var(abc_kind_t abc_kind, int dim_idx) const {
+    const expr_t &var(abc_kind_t abc_kind, size_t dim_idx) const {
         return get_vars(abc_kind)[dim_idx];
     }
 
@@ -118,7 +118,7 @@ private:
             case abc_kind_t::a: return a_vars_;
             case abc_kind_t::b: return b_vars_;
             case abc_kind_t::c: return c_vars_;
-            default: ir_error_not_expected() << "Unknown ABC kind.";
+            default: gpu_error_not_expected() << "Unknown ABC kind.";
         }
         return a_vars_;
     }
@@ -140,51 +140,53 @@ public:
     bmnk_block_mapper_t(const bmnk_mapper_t &bmnk_mapper)
         : bmnk_mapper_(bmnk_mapper) {}
 
-    void push_blocks(abc_kind_t abc_kind, const std::vector<block_t> &blocks) {
+    void push_blocks(
+            abc_kind_t abc_kind, const std::vector<layout_block_t> &blocks) {
         for (auto &b : blocks)
             push_block(abc_kind, b);
     }
 
-    void push_block(abc_kind_t abc_kind, const block_t &b);
+    void push_block(abc_kind_t abc_kind, const layout_block_t &b);
 
     layout_t map_from_bmnk(abc_kind_t abc_kind,
             const std::vector<bmnk_kind_t> &bmnk_kinds,
             const layout_t &bmnk_layout) const;
 
 private:
-    static void pop_size_1_blocks(std::vector<block_t> &blocks) {
-        while (!blocks.empty() && blocks.front().block == 1) {
+    static void pop_size_1_blocks(std::vector<layout_block_t> &blocks) {
+        while (!blocks.empty() && blocks.front().size == 1) {
             blocks.erase(blocks.begin());
         }
     }
 
-    std::vector<block_t> create_prb_blocks(abc_kind_t abc_kind,
-            const std::vector<std::pair<abc_kind_t, block_t>> &mn_blocks)
+    std::vector<layout_block_t> create_prb_blocks(abc_kind_t abc_kind,
+            const std::vector<std::pair<abc_kind_t, layout_block_t>> &mn_blocks)
             const {
-        std::vector<block_t> ret;
+        std::vector<layout_block_t> ret;
         ret.reserve(mn_blocks.size());
         for (auto &p : mn_blocks) {
             auto b = p.second;
-            const auto &var = bmnk_mapper_.var(p.first, b.dim_idx);
-            b.dim_idx = bmnk_mapper_.dim_idx(abc_kind, var);
+            const auto &var = bmnk_mapper_.var(p.first, b.idx);
+            b.idx = bmnk_mapper_.dim_idx(abc_kind, var);
             ret.push_back(b);
         }
         return ret;
     }
 
-    bool pop_block(std::vector<block_t> &bmnk_blocks,
-            std::vector<block_t> &prb_blocks, const block_t &bmnk_block) const;
+    bool pop_block(std::vector<layout_block_t> &bmnk_blocks,
+            std::vector<layout_block_t> &prb_blocks,
+            const layout_block_t &bmnk_block) const;
 
     bmnk_mapper_t bmnk_mapper_;
 
     // Ordered from innermost to outermost.
-    std::vector<std::pair<abc_kind_t, block_t>> b_blocks_;
-    std::vector<std::pair<abc_kind_t, block_t>> m_blocks_;
-    std::vector<std::pair<abc_kind_t, block_t>> n_blocks_;
-    std::vector<std::pair<abc_kind_t, block_t>> k_blocks_;
+    std::vector<std::pair<abc_kind_t, layout_block_t>> b_blocks_;
+    std::vector<std::pair<abc_kind_t, layout_block_t>> m_blocks_;
+    std::vector<std::pair<abc_kind_t, layout_block_t>> n_blocks_;
+    std::vector<std::pair<abc_kind_t, layout_block_t>> k_blocks_;
 };
 
-enum class loop_kind_t : int {
+enum class loop_kind_t : uint32_t {
     undef = 0,
     // Loop is bound to the kernel grid.
     kernel_grid = (1 << 0),
@@ -203,7 +205,7 @@ static std::string to_string(loop_kind_t kind) {
         case loop_kind_t::serial: return "serial";
         case loop_kind_t::tg_grid: return "tg_grid";
         case loop_kind_t::tensorized: return "tensorized";
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return "unknown";
 }
@@ -213,17 +215,7 @@ inline std::ostream &operator<<(std::ostream &out, loop_kind_t kind) {
     return out;
 }
 
-inline loop_kind_t operator&(loop_kind_t a, loop_kind_t b) {
-    return static_cast<loop_kind_t>(static_cast<int>(a) & static_cast<int>(b));
-}
-
-inline loop_kind_t operator|(loop_kind_t a, loop_kind_t b) {
-    return static_cast<loop_kind_t>(static_cast<int>(a) | static_cast<int>(b));
-}
-
-inline bool any(loop_kind_t a) {
-    return a != loop_kind_t::undef;
-}
+GPU_DEFINE_BIT_MASK_ENUM_OPS(loop_kind_t)
 
 const loop_kind_t all_loop_kinds = loop_kind_t::kernel_grid
         | loop_kind_t::serial | loop_kind_t::tg_grid | loop_kind_t::tensorized;
@@ -260,7 +252,7 @@ public:
 
     void set_bound(const expr_t &bound) { bound_ = bound; }
 
-    bool is_bound() const { return !bound_var().is_empty(); }
+    bool is_bound() const { return bool(bound_var()); }
 
     const expr_t &bound_var() const { return bound_var_; }
 
@@ -319,7 +311,7 @@ public:
             return 0;
         }
         if (is_split_parent()) {
-            ir_assert(child_vars_.size() == 2);
+            gpu_assert(child_vars_.size() == 2);
             auto &outer_loop = all_loops.at(child_vars_[0]);
             auto &inner_loop = all_loops.at(child_vars_[1]);
             auto outer_var
@@ -333,7 +325,7 @@ public:
             // Example of "unpacking":
             //     fused_var = (a * b * c * d)
             //     b = (fused_var / (D * C)) % B
-            ir_assert(child_vars_.size() == 1);
+            gpu_assert(child_vars_.size() == 1);
             auto &fused_loop = all_loops.at(child_vars_[0]);
             int nvars = int(fused_loop.parent_vars_.size());
             expr_t denom = 1;
@@ -351,22 +343,22 @@ public:
             }
         }
 
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return expr_t();
     }
 
     std::string str() const {
         using namespace ir_utils;
 
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "var: " << var_;
         oss << " bound: " << bound_;
         oss << " kind: " << kind_;
         if (unroll_factor_ != 1) oss << " unroll: " << unroll_factor_;
         std::vector<std::string> props;
-        if (is_root()) props.push_back("root");
-        if (is_fused_child()) props.push_back("fused");
-        if (is_split_parent()) props.push_back("split");
+        if (is_root()) props.emplace_back("root");
+        if (is_fused_child()) props.emplace_back("fused");
+        if (is_split_parent()) props.emplace_back("split");
         oss << "(" << make_seq_print_helper(props, ", ") << ")";
         return oss.str();
     }
@@ -473,12 +465,12 @@ public:
 
     void set_view(const view_t &view) {
         // Create missing loops.
-        for (int i = 0; i < view.nvdims(); i++) {
+        for (dim_idx_t i = 0; i < view.nvdims(); i++) {
             auto &v = view.vvars()[i];
             dim_t bound = view.vdims()[i];
             if (has_loop(v)) {
                 auto &loop = find_loop(v);
-                ir_assert(bound == to_cpp<dim_t>(loop.bound()))
+                gpu_assert(bound == to_cpp<dim_t>(loop.bound()))
                         << "Inconsistent sizes.";
                 continue;
             }
@@ -486,57 +478,61 @@ public:
         }
     }
 
-    tensor_t tg_view_tile(const view_t &view) const {
-        return view_tile(view, tile_level_t::thread_group);
+    tile_coord_t tg_view_tile_coord(const view_t &view) const {
+        return view_tile_coord(view, tile_level_t::thread_group);
     }
 
-    tensor_t thr_view_tile(const view_t &view, bool is_relative = true) const {
-        auto thr_tile = view_tile(view, tile_level_t::iter);
+    tile_coord_t thr_view_tile_coord(
+            const view_t &view, bool is_relative = true) const {
+        auto thr_tile = view_tile_coord(view, tile_level_t::iter);
         if (is_relative) return thr_tile;
-        return tg_view_tile(view).create_sub_tensor(thr_tile);
+        return tg_view_tile_coord(view).sub(thr_tile);
     }
 
     view_t a_tg_view() const {
-        ir_assert(is_finalized_);
-        return a_view_.create_sub_view(a_tg_tile_);
+        gpu_assert(is_finalized_);
+        return a_view_.create_sub_view(
+                a_tg_tile_coord_.tile, a_tg_tile_coord_.coord);
     }
 
     view_t b_tg_view() const {
-        ir_assert(is_finalized_);
-        return b_view_.create_sub_view(b_tg_tile_);
+        gpu_assert(is_finalized_);
+        return b_view_.create_sub_view(
+                b_tg_tile_coord_.tile, b_tg_tile_coord_.coord);
     }
 
     view_t c_tg_view() const {
-        ir_assert(is_finalized_);
-        return c_view_.create_sub_view(c_tg_tile_);
+        gpu_assert(is_finalized_);
+        return c_view_.create_sub_view(
+                c_tg_tile_coord_.tile, c_tg_tile_coord_.coord);
     }
 
     // Thread group tiles for A, B, C.
-    const tensor_t &a_tg_tile() const { return a_tg_tile_; }
-    const tensor_t &b_tg_tile() const { return b_tg_tile_; }
-    const tensor_t &c_tg_tile() const { return c_tg_tile_; }
+    const tile_coord_t &a_tg_tile_coord() const { return a_tg_tile_coord_; }
+    const tile_coord_t &b_tg_tile_coord() const { return b_tg_tile_coord_; }
+    const tile_coord_t &c_tg_tile_coord() const { return c_tg_tile_coord_; }
 
     // Thread tiles for A, B, C.
-    tensor_t a_thr_tile(bool is_relative = true) const {
-        if (is_relative) return a_thr_tile_;
-        return a_tg_tile_.create_sub_tensor(a_thr_tile_);
+    tile_coord_t a_thr_tile_coord(bool is_relative = true) const {
+        if (is_relative) return a_thr_tile_coord_;
+        return a_tg_tile_coord_.sub(a_thr_tile_coord_);
     }
 
-    tensor_t b_thr_tile(bool is_relative = true) const {
-        if (is_relative) return b_thr_tile_;
-        return b_tg_tile_.create_sub_tensor(b_thr_tile_);
+    tile_coord_t b_thr_tile_coord(bool is_relative = true) const {
+        if (is_relative) return b_thr_tile_coord_;
+        return b_tg_tile_coord_.sub(b_thr_tile_coord_);
     }
 
-    tensor_t c_thr_tile(bool is_relative = true) const {
-        if (is_relative) return c_thr_tile_;
-        return c_tg_tile_.create_sub_tensor(c_thr_tile_);
+    tile_coord_t c_thr_tile_coord(bool is_relative = true) const {
+        if (is_relative) return c_thr_tile_coord_;
+        return c_tg_tile_coord_.sub(c_thr_tile_coord_);
     }
 
-    int var_bound(const expr_t &var) const {
-        return to_cpp<int>(find_loop(var).bound());
+    dim_t var_bound(const expr_t &var) const {
+        return to_cpp<dim_t>(find_loop(var).bound());
     }
 
-    void set_var_bound(const expr_t &var, int bound) {
+    void set_var_bound(const expr_t &var, dim_t bound) {
         return find_loop(var).set_bound(bound);
     }
 
@@ -549,11 +545,11 @@ public:
     //       ...
     //     }
     //   }
-    void split(const expr_t &var, int factor, expr_t &outer_var,
+    void split(const expr_t &var, dim_t factor, expr_t &outer_var,
             expr_t &inner_var, const std::string &outer_name = {},
             const std::string &inner_name = {}) {
         auto &loop = find_loop(var);
-        ir_assert(loop.is_leaf()) << "Can't split, non-leaf loop.";
+        gpu_assert(loop.is_leaf()) << "Can't split, non-leaf loop.";
 
         auto bound = to_cpp<int64_t>(loop.bound());
         if (loop.is_root() && (bound % factor != 0)) {
@@ -562,7 +558,7 @@ public:
             loop.set_bound(bound);
         }
 
-        ir_assert(bound % factor == 0) << "Can't split.";
+        gpu_assert(bound % factor == 0) << "Can't split.";
 
         if (outer_name.empty()) {
             outer_var = create_var({var}, "outer");
@@ -575,7 +571,7 @@ public:
             inner_var = var_t::make(type_t::s32(), inner_name);
         }
 
-        ir_assert(outer_var.as<var_t>().name != inner_var.as<var_t>().name)
+        gpu_assert(outer_var.as<var_t>().name != inner_var.as<var_t>().name)
                 << "Cannot split loops to the same name "
                 << outer_var.as<var_t>().name;
 
@@ -623,8 +619,9 @@ public:
         }
         auto &fused_loop = create_loop(fused_var, fused_bound);
         std::vector<std::reference_wrapper<loop_t>> loop_refs;
+        loop_refs.reserve(vars.size());
         for (auto &v : vars) {
-            loop_refs.push_back(find_loop(v));
+            loop_refs.emplace_back(find_loop(v));
         }
         fused_loop.set_fuse(loop_refs);
         set_bmnk_kind(fused_var, bmnk_kind(vars));
@@ -646,20 +643,21 @@ public:
     // Binds the loop defined by `v` to an external variable.
     void bind(const expr_t &v, const expr_t &bound_var) {
         auto &loop = find_loop(v);
-        ir_assert(loop.is_leaf()) << "Can't bind non-leaf loop: " << v;
+        gpu_assert(loop.is_leaf()) << "Can't bind non-leaf loop: " << v;
         loop.set_bound_var(bound_var);
         loop.set_kind(bound_var_to_loop_kind(bound_var));
 
-        int var_dim = bound_var_to_dim(bound_var);
-        ir_assert(to_cpp<int>(loop.bound()) == var_dim)
-                << "Dimension size doesn't match.";
+        dim_t var_dim = bound_var_to_dim(bound_var);
+        gpu_assert(to_cpp<dim_t>(loop.bound()) == var_dim)
+                << "Dimension size doesn't match, "
+                << to_cpp<dim_t>(loop.bound()) << " != " << var_dim << ".";
     }
 
     // Reorders loops defined by given variables.
     void reorder(const std::vector<expr_t> &ordered_vars) {
         for (auto &v : ordered_vars) {
             auto &loop = find_loop(v);
-            ir_assert(loop.is_leaf()) << "Can't reorder non-leaf loop: " << v;
+            gpu_assert(loop.is_leaf()) << "Can't reorder non-leaf loop: " << v;
         }
         std::vector<bool> found(vars_.size());
         for (size_t i = 0; i < vars_.size(); i++) {
@@ -684,27 +682,27 @@ public:
     //   }
     void set_dynamic_bounds(
             const expr_t &var, const expr_t &init, const expr_t &step) {
-        ir_assert(find_loop(var).is_leaf()) << "Variable is non-leaf: " << var;
+        gpu_assert(find_loop(var).is_leaf()) << "Variable is non-leaf: " << var;
         dynamic_inits_[var] = expand(init);
         dynamic_steps_[var] = expand(step);
     }
 
     bool with_thread_group_k_slicing() const {
-        ir_assert(is_finalized_);
+        gpu_assert(is_finalized_);
         dim_t k_thr = 1;
         dim_t k_tg = 1;
         for (int i = 0; i < bmnk_mapper_.ndims(abc_kind_t::a); i++) {
             if (bmnk_mapper_.bmnk_kind(abc_kind_t::a, i) != bmnk_kind_t::k)
                 continue;
-            k_thr *= a_thr_tile_(i);
-            k_tg *= a_tg_tile_(i);
+            k_thr *= a_thr_tile_coord_.tile[i];
+            k_tg *= a_tg_tile_coord_.tile[i];
         }
-        ir_assert(k_tg % k_thr == 0);
+        gpu_assert(k_tg % k_thr == 0);
         return k_thr < k_tg;
     }
 
     bool with_kernel_grid_k_slicing() const {
-        ir_assert(is_finalized_);
+        gpu_assert(is_finalized_);
         dim_t k_loop = 1;
         dim_t k = 1;
         for (int i = 0; i < bmnk_mapper_.ndims(abc_kind_t::a); i++) {
@@ -726,7 +724,7 @@ public:
                 ret.push_back(loop.var());
                 return;
             }
-            ir_assert(loop.is_fused_child() || loop.is_split_child());
+            gpu_assert(loop.is_fused_child() || loop.is_split_child());
             for (auto &pv : loop.parent_vars()) {
                 walk(pv);
             }
@@ -761,7 +759,7 @@ public:
         }
         if (expand_trivial_vars) {
             for (auto &kv : loops_) {
-                int bound = to_cpp<int>(kv.second.bound());
+                dim_t bound = to_cpp<dim_t>(kv.second.bound());
                 if (bound != 1) continue;
                 if (!contains_object(ret, kv.first)) continue;
                 ret = substitute(ret, kv.first, expr_t(0));
@@ -787,8 +785,8 @@ public:
             bool with_dyn = init_it != dynamic_inits.end();
             auto init = with_dyn ? init_it->second : expr_t(0);
             auto step = with_dyn ? step_it->second : expr_t(1);
-            ir_assert(!with_dyn || step_it != dynamic_steps.end());
-            if (found_vars.count(var) == 0 && to_cpp<int>(loop.bound()) == 1
+            gpu_assert(!with_dyn || step_it != dynamic_steps.end());
+            if (found_vars.count(var) == 0 && to_cpp<dim_t>(loop.bound()) == 1
                     && !with_dyn)
                 continue;
             body = for_t::make(
@@ -798,7 +796,7 @@ public:
 
         for (auto &kv : dynamic_inits) {
             auto &c = kv.second;
-            ir_assert(c.is_empty()) << "Skip condition is not injected: " << c;
+            gpu_assert(c.is_empty()) << "Skip condition is not injected: " << c;
         }
 
         return body;
@@ -825,7 +823,7 @@ private:
             case tile_level_t::loop: return 1;
             case tile_level_t::thread_group: return 2;
             case tile_level_t::iter: return 3;
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return -1;
     }
@@ -836,7 +834,7 @@ private:
             case loop_kind_t::serial: return 1;
             case loop_kind_t::tg_grid: return 2;
             case loop_kind_t::tensorized: return 3;
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
         return -1;
     }
@@ -869,7 +867,7 @@ private:
                     case loop_kind_t::serial: return 0;
                     case loop_kind_t::tensorized:
                         return std::numeric_limits<int>::max();
-                    default: ir_error_not_expected();
+                    default: gpu_error_not_expected();
                 }
                 return -1;
             };
@@ -938,28 +936,28 @@ private:
     }
 
     loop_kind_t bound_var_to_loop_kind(const expr_t &v) const {
-        for (int i = 0; i < kernel_grid_.ndims(); i++) {
+        for (dim_idx_t i = 0; i < kernel_grid_.ndims(); i++) {
             if (kernel_grid_.idx(i).is_same(v)) return loop_kind_t::kernel_grid;
         }
-        for (int i = 0; i < tg_grid_.ndims(); i++) {
+        for (dim_idx_t i = 0; i < tg_grid_.ndims(); i++) {
             if (tg_grid_.idx(i).is_same(v)) return loop_kind_t::tg_grid;
         }
         if (kernel_grid_walk_order_.is_grid_var(v))
             return loop_kind_t::kernel_grid;
-        ir_error_not_expected() << "Unknown external variable: " << v;
+        gpu_error_not_expected() << "Unknown external variable: " << v;
         return loop_kind_t::undef;
     }
 
-    int bound_var_to_dim(const expr_t &v) const {
-        for (int i = 0; i < kernel_grid_.ndims(); i++) {
+    dim_t bound_var_to_dim(const expr_t &v) const {
+        for (dim_idx_t i = 0; i < kernel_grid_.ndims(); i++) {
             if (kernel_grid_.idx(i).is_same(v)) return kernel_grid_.dim(i);
         }
-        for (int i = 0; i < tg_grid_.ndims(); i++) {
+        for (dim_idx_t i = 0; i < tg_grid_.ndims(); i++) {
             if (tg_grid_.idx(i).is_same(v)) return tg_grid_.dim(i);
         }
         if (kernel_grid_walk_order_.is_grid_var(v))
             return kernel_grid_walk_order_.dim_size(v);
-        ir_error_not_expected() << "Unknown external variable: " << v;
+        gpu_error_not_expected() << "Unknown external variable: " << v;
         return -1;
     }
 
@@ -969,12 +967,12 @@ private:
     }
 
     const loop_t &find_loop(const expr_t &var) const {
-        ir_assert(has_loop(var)) << "Var not found: " << var;
+        gpu_assert(has_loop(var)) << "Var not found: " << var;
         return loops_.at(var);
     }
 
     loop_t &find_loop(const expr_t &var) {
-        ir_assert(has_loop(var)) << "Var not found: " << var;
+        gpu_assert(has_loop(var)) << "Var not found: " << var;
         return loops_[var];
     }
 
@@ -989,7 +987,7 @@ private:
             const expr_t &var, const expr_t &bound, bool is_root = false) {
         loop_t loop(var, bound, is_root);
         auto ret = loops_.insert({var, loop});
-        ir_assert(ret.second) << "Variable already exists: " << var;
+        gpu_assert(ret.second) << "Variable already exists: " << var;
         vars_.push_back(var);
         return ret.first->second;
     }
@@ -1021,17 +1019,17 @@ private:
                 split_infos.insert({v, get_split_info(v)});
             }
         }
-        a_tg_tile_ = compute_problem_tile(
+        a_tg_tile_coord_ = compute_problem_tile_coord(
                 a_view_.vvars(), split_infos, tile_level_t::thread_group);
-        b_tg_tile_ = compute_problem_tile(
+        b_tg_tile_coord_ = compute_problem_tile_coord(
                 b_view_.vvars(), split_infos, tile_level_t::thread_group);
-        c_tg_tile_ = compute_problem_tile(
+        c_tg_tile_coord_ = compute_problem_tile_coord(
                 c_view_.vvars(), split_infos, tile_level_t::thread_group);
-        a_thr_tile_ = compute_problem_tile(
+        a_thr_tile_coord_ = compute_problem_tile_coord(
                 a_view_.vvars(), split_infos, tile_level_t::iter);
-        b_thr_tile_ = compute_problem_tile(
+        b_thr_tile_coord_ = compute_problem_tile_coord(
                 b_view_.vvars(), split_infos, tile_level_t::iter);
-        c_thr_tile_ = compute_problem_tile(
+        c_thr_tile_coord_ = compute_problem_tile_coord(
                 c_view_.vvars(), split_infos, tile_level_t::iter);
     }
 
@@ -1058,13 +1056,13 @@ private:
         }
     }
 
-    tensor_t view_tile(const view_t &view, tile_level_t level) const {
+    tile_coord_t view_tile_coord(const view_t &view, tile_level_t level) const {
         object_map_t<expr_t, split_info_t> split_infos;
         for (auto &v : view.vvars()) {
             if (split_infos.count(v) > 0) continue;
             split_infos.insert({v, get_split_info(v)});
         }
-        return compute_problem_tile(view.vvars(), split_infos, level);
+        return compute_problem_tile_coord(view.vvars(), split_infos, level);
     }
 
     split_info_t get_split_info(const expr_t &root_var) const {
@@ -1079,7 +1077,7 @@ private:
                 int level;
                 if (loop.is_fused_parent()) {
                     auto &child_var = loop.child_vars()[0];
-                    ir_assert(find_loop(child_var).is_leaf());
+                    gpu_assert(find_loop(child_var).is_leaf());
                     kind = find_loop(child_var).kind();
                     level = loop_level(child_var);
                 } else {
@@ -1090,15 +1088,15 @@ private:
                 walk_down(loop.child_vars()[0]);
                 walk_down(loop.child_vars()[1]);
             } else {
-                ir_error_not_expected();
+                gpu_error_not_expected();
             }
         };
         walk_down(root_var);
-        ir_assert(ret.is_valid()) << "Invalid loop nest.";
+        gpu_assert(ret.is_valid()) << "Invalid loop nest.";
         return ret;
     }
 
-    tensor_t compute_problem_tile(const std::vector<expr_t> &vars,
+    tile_coord_t compute_problem_tile_coord(const std::vector<expr_t> &vars,
             const object_map_t<expr_t, split_info_t> &split_infos,
             tile_level_t tile_level) const {
         std::vector<dim_t> tile_dims;
@@ -1112,7 +1110,7 @@ private:
             tile_start.push_back(
                     split_info.start(v_expanded, tile_level, with_outer));
         }
-        return tensor_t(tile_dims, tile_start);
+        return tile_coord_t(tile_dims, tile_start);
     }
 
     stmt_t maybe_inject_let_for_fused_vars(
@@ -1150,14 +1148,14 @@ private:
     view_t c_view_;
 
     // Thread group tiles for A, B, C.
-    tensor_t a_tg_tile_;
-    tensor_t b_tg_tile_;
-    tensor_t c_tg_tile_;
+    tile_coord_t a_tg_tile_coord_;
+    tile_coord_t b_tg_tile_coord_;
+    tile_coord_t c_tg_tile_coord_;
 
     // Thread tiles for A, B, C (relative to thread group tiles).
-    tensor_t a_thr_tile_;
-    tensor_t b_thr_tile_;
-    tensor_t c_thr_tile_;
+    tile_coord_t a_thr_tile_coord_;
+    tile_coord_t b_thr_tile_coord_;
+    tile_coord_t c_thr_tile_coord_;
 };
 
 } // namespace jit
