@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #include "gpu/intel/jit/ir/ir.hpp"
 
+#include <numeric>
 #include <sstream>
 
 #include "common/math_utils.hpp"
@@ -39,12 +40,16 @@ class ir_printer_t : public ir_visitor_t {
 public:
     ir_printer_t(std::ostream &out) : out_(out) {}
 
+    void _visit(const assign_t &obj) override {
+        print_indent();
+        out_ << obj.str() << "\n";
+    }
+
     void _visit(const alloc_t &obj) override {
         auto grf_size = 1; // Assume all objects are grf aligned
         auto guard = mem_usage_guard(obj.register_alloc_size(grf_size));
         print_indent();
-        out_ << "alloc " << obj.buf.as<var_t>().name << "[" << obj.size
-             << "] (mem_usage: " << mem_usage_bytes_ << ")\n";
+        out_ << obj.line_str() << "(mem_usage: " << mem_usage_bytes_ << ")\n";
         visit(obj.body);
     }
 
@@ -98,16 +103,12 @@ public:
 
     void _visit(const func_call_t &obj) override {
         print_indent();
-        out_ << obj.func << "(" << make_seq_print_helper(obj.args) << ")";
-        if (!obj.attr.is_empty()) out_ << " " << obj.attr;
-        out_ << "\n";
+        out_ << obj.line_str() << "\n";
     }
 
-    void _visit(const func_impl_t &obj) override { out_ << obj.str(); }
-
     void _visit(const if_t &obj) override {
         print_indent();
-        out_ << "if (" << strip_parens(obj.cond.str()) << ") {\n";
+        out_ << obj.line_str() << " {\n";
         add_indent();
         visit(obj.body);
         remove_indent();
@@ -137,7 +138,7 @@ public:
         int size = obj.register_alloc_size();
         auto guard = mem_usage_guard(size);
         print_indent();
-        out_ << obj.var << "." << obj.var.type() << " = " << obj.value << "\n";
+        out_ << obj.line_str() << "\n";
         visit(obj.body);
     }
 
@@ -207,29 +208,21 @@ public:
         remove_indent();
         print_indent();
         out_ << "}\n";
-        return;
     }
 
     void _visit(const stmt_seq_t &obj) override {
-        visit(obj.head);
-        visit(obj.tail);
+        for (auto &s : obj.vec)
+            visit(s);
     }
 
     void _visit(const store_t &obj) override {
         print_indent();
-        out_ << load_t::make(obj.value.type(), obj.buf, obj.off, obj.stride);
-        out_ << " = " << obj.value;
-        if (!obj.mask.is_empty()) {
-            out_ << ", mask = " << obj.mask.str();
-            if (obj.fill_mask0) out_ << " [FILL]";
-        }
-        out_ << "\n";
+        out_ << obj.line_str() << "\n";
     }
 
     void _visit(const ternary_op_t &obj) override {
         out_ << to_string(obj.op_kind) << "(" << obj.a << ", " << obj.b << ", "
              << obj.c << ")";
-        return;
     }
 
     void _visit(const unary_op_t &obj) override {
@@ -239,6 +232,18 @@ public:
 
     void _visit(const var_t &obj) override { out_ << obj.name; }
 
+    void _visit(const ref_t &obj) override { out_ << obj.str(); }
+
+    void _visit(const while_t &obj) override {
+        print_indent();
+        out_ << obj.line_str() << " {\n";
+        add_indent();
+        visit(obj.body);
+        remove_indent();
+        print_indent();
+        out_ << "}\n";
+    }
+
 private:
     mem_usage_guard_t mem_usage_guard(int size) {
         return mem_usage_guard_t(&mem_usage_bytes_, size);
@@ -278,14 +283,14 @@ public:
 
 #define HANDLE_IR_OBJECT(type) \
     object_t _mutate(const type &obj) override { \
-        if (from_.impl() == (const object_impl_t *)&obj) { \
+        if (from_.impl() == (const impl_t *)&obj) { \
             substitutions_++; \
             return to_; \
         } \
         return ir_mutator_t::_mutate(obj); \
     };
 
-    HANDLE_TRAVERSE_TARGETS()
+    HANDLE_CORE_IR_OBJECTS()
 
 #undef HANDLE_IR_OBJECT
 
@@ -318,8 +323,8 @@ public:
         //
         // tmp0.s32            -> tmp0_0.u64
         // tmp1.s32 = tmp0.s32 -> tmp1.s32 = tmp0_0.u64
-        if (!value.is_empty()) {
-            auto &value_type = expr_t(value).type();
+        if (value) {
+            auto value_type = expr_t(value).type();
             if (var.as<var_t>().type != value_type) {
                 auto var_old = var;
                 var = var_t::make(value_type, var.as<var_t>().name);
@@ -347,7 +352,7 @@ public:
         return _mutate_after(obj); \
     };
 
-    HANDLE_ALL_IR_OBJECTS()
+    HANDLE_CORE_IR_OBJECTS()
 
 #undef HANDLE_IR_OBJECT
 
@@ -364,10 +369,10 @@ public:
         size_t old_size = stmts.size(); \
         ir_visitor_t::_visit(obj); \
         if (stmts.size() > old_size) return; \
-        if (obj.is_stmt()) stmts.push_back(obj); \
+        if (obj.is_stmt()) stmts.emplace_back(obj); \
     }
 
-    HANDLE_ALL_IR_OBJECTS()
+    HANDLE_CORE_IR_OBJECTS()
 
 #undef HANDLE_IR_OBJECT
 
@@ -376,19 +381,19 @@ public:
 
 class alloc_injector_t : public ir_mutator_t {
 public:
-    alloc_injector_t(const stmt_t &root, const std::vector<stmt_t> &allocs,
-            bool put_innermost)
-        : root_(root), put_innermost_(put_innermost), allocs_(allocs) {
+    alloc_injector_t(const stmt_t &root, const std::vector<stmt_t> &allocs)
+        : allocs_(allocs) {
         for (auto &_a : allocs) {
             auto &a = _a.as<alloc_t>();
-            if (a.kind != alloc_kind_t::global) ir_assert(a.size > 0) << _a;
+            if (a.kind != alloc_kind_t::global) gpu_assert(a.size > 0) << _a;
             alloc_map_.insert({a.buf, _a});
+            buf_cur_refs_[a.buf] = 0;
         }
-        mutate(root_);
+        mutate(root);
         buf_total_refs_ = buf_cur_refs_;
         for (auto &kv : buf_cur_refs_)
             kv.second = 0;
-        in_ctor_ = false;
+        in_ctor_ = false; // NOLINT(cppcoreguidelines-prefer-member-initializer)
     }
 
 #define HANDLE_IR_OBJECT(type) \
@@ -406,49 +411,153 @@ private:
     template <typename T>
     object_t mutate_stmt(const T &obj) {
         if (in_ctor_) return ir_mutator_t::_mutate(obj);
-        object_t new_obj = obj;
-        object_set_t<expr_t> undef_bufs;
-        if (put_innermost_) {
-            for (auto &kv : buf_cur_refs_)
-                if (kv.second == 0) undef_bufs.insert(kv.first);
-            new_obj = ir_mutator_t::_mutate(obj);
-        }
-        for (auto &a : allocs_) {
-            auto it = alloc_map_.find(a.as<alloc_t>().buf);
-            auto &buf = it->first;
-            if (it->second.is_empty()) continue; // Already injected.
-            bool do_inject = false;
-            if (put_innermost_) {
-                int cur_refs = buf_cur_refs_[buf];
-                int total_refs = buf_total_refs_[buf];
-                bool was_undef = (undef_bufs.count(buf) != 0);
-                do_inject = was_undef && (cur_refs == total_refs);
-            } else {
-                do_inject = root_.is_same(obj);
+        if (obj.template is<stmt_seq_t>()) { return mutate_stmt_seq(obj); }
+        auto undef_bufs = get_undef_bufs();
+        auto new_obj = ir_mutator_t::_mutate(obj);
+        new_obj = maybe_inject(new_obj, undef_bufs);
+        return new_obj;
+    }
+
+    // Handle stmt_seq_t in a special way:
+    // 1. Walk through the sequence and record the first and the last statement
+    //    where a buffer is referenced
+    // 2. Inject alloc statements according to the usage
+    object_t mutate_stmt_seq(const object_t &obj) {
+        auto stmt_vec = obj.as<stmt_seq_t>().vec;
+        gpu_assert(!stmt_vec.empty());
+        int nstmts = (int)stmt_vec.size();
+        // Mutate statments and record buffer usage in the form: buf: [first, last].
+        object_map_t<expr_t, int> last_undef;
+        object_map_t<expr_t, std::pair<int, int>> entries;
+        for (int i = 0; i < nstmts; i++) {
+            auto &s = stmt_vec[i];
+            for (auto &b : get_undef_bufs()) {
+                auto it = alloc_map_.find(b);
+                if (it == alloc_map_.end() || it->second.is_empty()) continue;
+                last_undef[b] = i;
             }
-            if (do_inject) {
-                auto &a = it->second.as<alloc_t>();
+            s = mutate(s);
+            for (auto &kv : last_undef) {
+                auto &buf = kv.first;
+                if (entries.count(buf) != 0) continue;
+                if (buf_cur_refs_[buf] == buf_total_refs_[buf]) {
+                    entries[buf] = std::make_pair(kv.second, i);
+                }
+            }
+        }
+        // Sort buffers based on the number of statements they span. This is to
+        // inject more local allocations first.
+        std::vector<expr_t> bufs;
+        for (auto &kv : entries) {
+            if (alloc_map_.at(kv.first).is_empty()) continue;
+            bufs.push_back(kv.first);
+        }
+        std::sort(bufs.begin(), bufs.end(),
+                [&](const expr_t &a, const expr_t &b) {
+                    auto &ea = entries.at(a);
+                    auto &eb = entries.at(b);
+                    int a_span = (ea.second - ea.first);
+                    int b_span = (eb.second - eb.first);
+                    if (a_span == b_span)
+                        return a.as<var_t>().name < b.as<var_t>().name;
+                    return a_span < b_span;
+                });
+        // Use union-find to incrementally merge statements based on the common
+        // buffers.
+        std::vector<int> parent(nstmts);
+        std::iota(parent.begin(), parent.end(), 0);
+        std::function<int(int)> _find;
+        std::function<void(int, int)> _union;
+        _find = [&](int i) {
+            if (parent[i] == i) return i;
+            return parent[i] = _find(parent[i]);
+        };
+        _union = [&](int i, int j) {
+            i = _find(i);
+            j = _find(j);
+            parent[j] = i;
+        };
+        std::vector<stmt_t> new_stmt_seq = std::move(stmt_vec);
+        for (auto &buf : bufs) {
+            auto &e = entries.at(buf);
+            stmt_t stmt;
+            for (int i = e.first; i <= e.second; i++) {
+                int idx = _find(i);
+                stmt = stmt.append(new_stmt_seq[idx]);
+                new_stmt_seq[idx] = stmt_t();
+                _union(e.first, i);
+            }
+            auto it = alloc_map_.find(buf);
+            auto &a = it->second.as<alloc_t>();
+            stmt = alloc_t::make(a.buf, a.size, a.kind, a.attrs, stmt);
+            new_stmt_seq[_find(e.first)] = stmt;
+            it->second = stmt_t();
+        }
+        stmt_t new_obj;
+        for (auto &s : new_stmt_seq) {
+            if (s.is_empty()) continue;
+            new_obj = new_obj.append(s);
+        }
+        return std::move(new_obj);
+    }
+
+    object_set_t<expr_t> get_undef_bufs() const {
+        object_set_t<expr_t> ret;
+        for (auto &kv : buf_cur_refs_)
+            if (kv.second == 0) ret.insert(kv.first);
+        return ret;
+    }
+
+    object_t maybe_inject(
+            const object_t &obj, const object_set_t<expr_t> &undef_bufs) {
+        auto new_obj = obj;
+        for (auto &kv : alloc_map_) {
+            if (kv.second.is_empty()) continue;
+            auto &buf = kv.first;
+            auto &a = kv.second.as<alloc_t>();
+            if (do_inject(buf, undef_bufs)) {
                 new_obj = alloc_t::make(
                         a.buf, a.size, a.kind, a.attrs, new_obj);
-                it->second = stmt_t();
+                kv.second = stmt_t();
             }
         }
         return new_obj;
     }
 
+    bool do_inject(
+            const expr_t &buf, const object_set_t<expr_t> &undef_bufs) const {
+        if (buf.is_empty()) return false; // Already injected.
+        int cur_refs = buf_cur_refs_.at(buf);
+        int total_refs = buf_total_refs_.at(buf);
+        bool was_undef = (undef_bufs.count(buf) != 0);
+        return was_undef && (cur_refs == total_refs);
+    }
+
     bool in_ctor_ = true;
-    const stmt_t &root_;
-    bool put_innermost_;
     std::vector<stmt_t> allocs_;
     object_map_t<expr_t, stmt_t> alloc_map_;
     object_map_t<expr_t, int> buf_total_refs_;
     object_map_t<expr_t, int> buf_cur_refs_;
 };
 
+class alloc_remover_t : public ir_mutator_t {
+public:
+    alloc_remover_t(std::vector<stmt_t> &allocs) : allocs_(allocs) {}
+
+    object_t _mutate(const alloc_t &obj) override {
+        allocs_.push_back(
+                alloc_t::make(obj.buf, obj.size, obj.kind, obj.attrs));
+        return mutate(obj.body);
+    }
+
+private:
+    std::vector<stmt_t> &allocs_;
+};
+
 } // namespace
 
-std::string object_impl_t::str() const {
-    std::ostringstream oss;
+std::string object::impl_t::str() const {
+    ostringstream_t oss;
     ir_printer_t printer(oss);
     printer.visit(this);
     return oss.str();
@@ -459,7 +568,7 @@ object_t substitute(const object_t &root, const object_t &from,
     if (to.is_same(from)) return root;
     substitute_mutator_t sm(from, to);
     auto ret = sm.mutate(root);
-    ir_assert(sm.substitutions() <= max_substitutions)
+    gpu_assert(sm.substitutions() <= max_substitutions)
             << "Unexpected number of substitutions.";
     return ret;
 }
@@ -469,7 +578,7 @@ object_t substitute_with_different_type(const object_t &root,
     if (to.is_same(from)) return root;
     substitute_and_type_mutator_t sm(from, to);
     auto ret = sm.mutate(root);
-    ir_assert(sm.substitutions() <= max_substitutions)
+    gpu_assert(sm.substitutions() <= max_substitutions)
             << "Unexpected number of substitutions.";
     return ret;
 }
@@ -481,8 +590,26 @@ std::vector<stmt_t> flatten_statements(const stmt_t &root) {
 }
 
 stmt_t inject_alloc_stmts(const stmt_t &stmt, const std::vector<stmt_t> &allocs,
-        bool put_innermost) {
-    alloc_injector_t injector(stmt, allocs, put_innermost);
+        bool put_innermost, bool update_existing) {
+    if (update_existing)
+        gpu_assert(put_innermost)
+                << "update_existing can be used only with put_innermost.";
+    if (!put_innermost) {
+        auto ret = stmt;
+        for (auto &_a : allocs) {
+            auto &a = _a.as<alloc_t>();
+            ret = alloc_t::make(a.buf, a.size, a.kind, a.attrs, ret);
+        }
+        return ret;
+    }
+    if (update_existing) {
+        std::vector<stmt_t> _allocs;
+        alloc_remover_t remover(_allocs);
+        auto _stmt = remover.mutate(stmt);
+        _allocs.insert(_allocs.end(), allocs.begin(), allocs.end());
+        return inject_alloc_stmts(_stmt, _allocs, put_innermost);
+    }
+    alloc_injector_t injector(stmt, allocs);
     return injector.mutate(stmt);
 }
 
@@ -503,6 +630,121 @@ stmt_t inject_let_stmts(const stmt_t &stmt, const std::vector<stmt_t> &lets) {
     return ret;
 }
 
+class var_counter_t : public ir_visitor_t {
+public:
+    var_counter_t(const object_set_t<expr_t> &vars) {
+        for (auto &v : vars) {
+            counts[v] = 0;
+        }
+    }
+
+    void _visit(const var_t &obj) override {
+        auto it = counts.find(obj);
+        if (it == counts.end()) return;
+        it->second++;
+    }
+
+    object_map_t<expr_t, int> counts;
+};
+
+object_map_t<expr_t, int> count_vars(
+        const stmt_t &stmt, const object_set_t<expr_t> &vars) {
+    var_counter_t counter(vars);
+    counter.visit(stmt);
+    return counter.counts;
+}
+
+class let_injector_t : public ir_mutator_t {
+public:
+    object_t _mutate(const stmt_seq_t &obj) override {
+        auto new_obj = ir_mutator_t::_mutate(obj);
+        auto &stmt_vec = new_obj.as<stmt_seq_t>().vec;
+        int nstmts = (int)stmt_vec.size();
+        // 1. Collect total var references for dangling lets.
+        object_set_t<expr_t> let_vars;
+        for (auto &s : stmt_vec) {
+            if (is_dangling_let(s)) {
+                auto &var = s.as<let_t>().var;
+                let_vars.insert(var);
+            }
+        }
+        if (let_vars.empty()) return new_obj;
+        auto total_refs = count_vars(new_obj, let_vars);
+
+        // 2. Find scopes for dangling lets.
+        object_map_t<expr_t, stmt_t> var2let;
+        object_map_t<stmt_t, int> let_scope_ends;
+        object_map_t<expr_t, int> cur_refs;
+        for (auto &v : let_vars)
+            cur_refs[v] = 0;
+        for (int i = 0; i < nstmts; i++) {
+            auto &s = stmt_vec[i];
+            if (is_dangling_let(s)) {
+                var2let[s.as<let_t>().var] = s;
+                let_scope_ends[s] = i;
+            }
+            for (auto &kv : count_vars(s, let_vars)) {
+                auto &var = kv.first;
+                cur_refs[var] += kv.second;
+                if (cur_refs[var] == total_refs[var]) {
+                    let_vars.erase(var);
+                    let_scope_ends[var2let.at(var)] = i;
+                }
+            }
+        }
+
+        // 3. Nest let statements according to the scopes.
+        std::vector<entry_t> entries;
+        entries.emplace_back();
+        for (int i = 0; i < nstmts; i++) {
+            auto &s = stmt_vec[i];
+            if (is_dangling_let(s)) {
+                entry_t e;
+                e.let_stmt = s;
+                entries.push_back(e);
+            } else {
+                entries.back().append(s);
+            }
+            while (!entries.empty()) {
+                auto &last = entries.back();
+                if (last.let_stmt.is_empty()) break;
+                int end = let_scope_ends.at(last.let_stmt);
+                if (end > i) break;
+                auto new_stmt = last.make_let();
+                entries.pop_back();
+                entries.back().append(new_stmt);
+            }
+        }
+
+        stmt_t ret;
+        for (auto &e : entries) {
+            gpu_assert(e.let_stmt.is_empty()) << e.let_stmt;
+            ret = ret.append(e.body);
+        }
+
+        return std::move(ret);
+    }
+
+private:
+    static bool is_dangling_let(const stmt_t &s) {
+        auto *let = s.as_ptr<let_t>();
+        return let && let->body.is_empty();
+    }
+
+    struct entry_t {
+        stmt_t body;
+        stmt_t let_stmt;
+
+        stmt_t make_let() const { return replace_stmt_body(let_stmt, body); }
+
+        void append(const stmt_t &s) { body = body.append(s); }
+    };
+};
+
+stmt_t inject_dangling_let_stmts(const stmt_t &stmt) {
+    return let_injector_t().mutate(stmt);
+}
+
 std::vector<expr_t> split_by_and(const expr_t &e) {
     auto *binary = e.as_ptr<binary_op_t>();
     if (!binary || binary->op_kind != op_kind_t::_and) return {e};
@@ -514,33 +756,23 @@ std::vector<expr_t> split_by_and(const expr_t &e) {
 }
 
 expr_t abs(const expr_t &e) {
-    ir_assert(is_const(e)) << e;
+    gpu_assert(is_const(e)) << e;
     if (to_cpp<bool>(e >= 0)) return e;
     return -e;
 }
 
+expr_t max(const expr_t &a, const expr_t &b) {
+    return binary_op_t::make(op_kind_t::_max, a, b);
+}
+
+expr_t min(const expr_t &a, const expr_t &b) {
+    return binary_op_t::make(op_kind_t::_min, a, b);
+}
+
 expr_t cast(const expr_t &e, const type_t &type, bool saturate) {
     return const_fold(cast_t::make(type, e, saturate));
 }
 
-bool is_zero(const expr_t &e) {
-    if (e.is_empty()) return false;
-    if (!e.type().is_scalar() || e.type().is_ptr()) return false;
-    return e.is_equal(to_expr(0, e.type()));
-}
-
-bool is_one(const expr_t &e) {
-    if (e.is_empty()) return false;
-    if (!e.type().is_scalar() || e.type().is_ptr()) return false;
-    return e.is_equal(to_expr(1, e.type()));
-}
-
-bool is_minus_one(const expr_t &e) {
-    if (e.is_empty()) return false;
-    if (!e.type().is_scalar() || e.type().is_ptr()) return false;
-    return e.is_equal(to_expr(-1, e.type()));
-}
-
 bool is_const_broadcast(const expr_t &e) {
     auto *shuffle = e.as_ptr<shuffle_t>();
     if (!shuffle) return false;
@@ -554,26 +786,26 @@ bool is_const_broadcast(const expr_t &e, const expr_t &value) {
 }
 
 expr_t make_buffer(const std::string &name) {
-    return var_t::make(type_t::byte_ptr(), name);
+    return var_t::make(type_t::byte(type::attr_t::ptr), name);
 }
 
 // Returns number of occurrences of `obj` in `root` (based on identity equality).
 int count_object(const object_t &root, const object_t &obj) {
-    ir_assert(!obj.is_empty());
+    gpu_assert(obj);
 
     std::vector<object_t> found;
     do {
 #define HANDLE_IR_OBJECT(type) \
-    if (obj.dispatch_type_id() == type::_dispatch_type_id()) { \
+    if (obj.is<type>()) { \
         found = find_objects<type>(root); \
         break; \
     }
 
-        HANDLE_ALL_IR_OBJECTS()
+        HANDLE_CORE_IR_OBJECTS()
 
 #undef HANDLE_IR_OBJECT
 
-        ir_error_not_expected() << obj;
+        gpu_error_not_expected() << obj;
     } while (false);
 
     int ret = 0;
@@ -583,7 +815,7 @@ int count_object(const object_t &root, const object_t &obj) {
 }
 
 bool contains_object(const object_t &root, const object_t &obj) {
-    ir_assert(is_var(obj)) << obj;
+    gpu_assert(is_var(obj)) << obj;
     return count_object(root, obj) > 0;
 }
 
@@ -592,18 +824,17 @@ std::vector<stmt_t> find_stmt_groups(
     auto groups = find_objects<stmt_group_t>(root);
     std::vector<stmt_t> ret;
     for (auto &g : groups) {
-        if (g.as<stmt_group_t>().label == label) ret.push_back(g);
+        if (g.as<stmt_group_t>().label == label) ret.emplace_back(g);
     }
     return ret;
 }
 
-utils::optional_t<stmt_t> find_stmt_group(
-        const object_t &root, const stmt_label_t &label) {
+stmt_t find_stmt_group(const object_t &root, const stmt_label_t &label) {
     auto groups = find_stmt_groups(root, label);
     if (groups.size() == 1)
         return groups[0];
     else
-        return utils::nullopt;
+        return {};
 }
 
 class stmt_group_remover_t : public ir_mutator_t {
@@ -628,6 +859,9 @@ stmt_t get_stmt_body(const stmt_t &stmt) {
     auto *_for = stmt.as_ptr<for_t>();
     if (_for) return _for->body;
 
+    auto *_while = stmt.as_ptr<while_t>();
+    if (_while) return _while->body;
+
     auto *let = stmt.as_ptr<let_t>();
     if (let) return let->body;
 
@@ -650,6 +884,9 @@ stmt_t replace_stmt_body(const stmt_t &stmt, const stmt_t &new_body) {
                 _for->step, _for->unroll);
     }
 
+    auto *_while = stmt.as_ptr<while_t>();
+    if (_while) { return while_t::make(_while->cond, new_body); }
+
     auto *let = stmt.as_ptr<let_t>();
     if (let) { return let_t::make(let->var, let->value, new_body); }
 
@@ -697,24 +934,8 @@ int get_peak_regs(
     return utils::div_up(visitor.peak_regs(), grf_size);
 }
 
-class has_send_atomics_visitor_t : public ir_visitor_t {
-public:
-    void _visit(const func_call_t &obj) override {
-        auto *send = obj.func.as_ptr<send_t>();
-        if (send && send->is_atomic()) found = true;
-    }
-
-    bool found = false;
-};
-
-bool has_send_atomics(const stmt_t &s) {
-    has_send_atomics_visitor_t visitor;
-    visitor.visit(s);
-    return visitor.found;
-}
-
 bool relation_t::implies(const relation_t &other) const {
-    ir_assert(var().is_same(other.var()));
+    gpu_assert(var().is_same(other.var()));
 
     if (op_kind() != other.op_kind()) return false;
 
@@ -730,19 +951,19 @@ bool relation_t::implies(const relation_t &other) const {
         // (x <= A) && (A <= B) => (x <= B)
         case op_kind_t::_lt:
         case op_kind_t::_le: return A <= B;
-        default: ir_error_not_expected() << "Not implemented: " << expr_;
+        default: gpu_error_not_expected() << "Not implemented: " << expr_;
     }
     return false;
 }
 
 relation_t relation_t::transform(
-        const linear_transform_t &t, const expr_t &new_var) {
-    ir_assert(t.a == 1) << "Not implemented.";
+        const linear_transform_t &t, const expr_t &new_var) const {
+    gpu_assert(t.a == 1) << "Not implemented.";
     return relation_t(binary_op_t::make(op_kind(), new_var, rhs() + t.b));
 }
 
 expr_t relation_t::normalize(const expr_t &e) {
-    ir_assert(is_relation_constraint(e)) << e;
+    gpu_assert(is_relation_constraint(e)) << e;
     auto &op = e.as<binary_op_t>();
 
     auto op_kind = op.op_kind;
@@ -786,7 +1007,7 @@ int64_t bound_finder_base_t::find_bound_impl(
 
     auto *unary = e.as_ptr<unary_op_t>();
     if (unary) {
-        ir_assert(unary->op_kind == op_kind_t::_minus) << e;
+        gpu_assert(unary->op_kind == op_kind_t::_minus) << e;
         auto a = find_bound_impl(unary->a, !is_low);
         if (!is_good_bound(a)) return def_bound;
         return -a;
@@ -830,7 +1051,7 @@ int64_t bound_finder_base_t::find_bound_impl(
                 if (!is_const(binary->b)) return def_bound;
 
                 auto b = to_cpp<int64_t>(binary->b);
-                ir_assert(b != 0);
+                gpu_assert(b != 0);
 
                 auto a = find_bound_impl(binary->a, b > 0 ? is_low : !is_low);
                 if (!is_good_bound(a)) return def_bound;
@@ -927,7 +1148,7 @@ bool is_linear_var_transform(const expr_t &e, linear_transform_t &t) {
     if (a.is_same(var) && is_const(b)) {
         t.x = var;
         t.a = 1;
-        t.b = (is_sub ? -1 : 1) * to_cpp<int>(b);
+        t.b = (is_sub ? -1 : 1) * to_cpp<int64_t>(b);
         return true;
     }
 
@@ -935,7 +1156,7 @@ bool is_linear_var_transform(const expr_t &e, linear_transform_t &t) {
     if (is_const(a) && b.is_same(var)) {
         t.x = var;
         t.a = (is_sub ? -1 : 1);
-        t.b = to_cpp<int>(a);
+        t.b = to_cpp<int64_t>(a);
         return true;
     }
 
@@ -965,8 +1186,16 @@ void constraint_set_t::add_constraint(const expr_t &e) {
         return;
     }
 
-    // Propagate constraints from y for (x == y) equalities.
     auto *binary_op = e.as_ptr<binary_op_t>();
+
+    // Propagate constraints from chained ands like a & b & c
+    if (binary_op && binary_op->op_kind == op_kind_t::_and) {
+        add_constraint(binary_op->a);
+        add_constraint(binary_op->b);
+        return;
+    }
+
+    // Propagate constraints from y for (x == y) equalities.
     if (binary_op && binary_op->op_kind == op_kind_t::_eq) {
         auto &a = binary_op->a;
         auto &b = binary_op->b;
@@ -994,14 +1223,14 @@ void constraint_set_t::add_constraint(const expr_t &e) {
 }
 
 bool constraint_set_t::is_single_value(const expr_t &e, expr_t &value) const {
-    ir_assert(is_var(e)) << e;
+    gpu_assert(is_var(e)) << e;
     auto it = relations_.find(e);
     if (it == relations_.end()) return false;
 
     expr_t lo;
     expr_t hi;
     for (auto &rel : it->second) {
-        ir_assert(is_const(rel.rhs())) << rel;
+        gpu_assert(is_const(rel.rhs())) << rel;
         bool do_break = false;
         switch (rel.op_kind()) {
             case op_kind_t::_eq:
@@ -1012,22 +1241,26 @@ bool constraint_set_t::is_single_value(const expr_t &e, expr_t &value) const {
             case op_kind_t::_gt: {
                 auto cur_lo = (rel.op_kind() == op_kind_t::_ge ? rel.rhs()
                                                                : rel.rhs() + 1);
-                if (lo.is_empty() || to_cpp<bool>(cur_lo > lo)) { lo = cur_lo; }
+                if (lo.is_empty() || to_cpp<bool>(cur_lo > lo)) {
+                    lo = std::move(cur_lo);
+                }
                 break;
             }
             case op_kind_t::_le:
             case op_kind_t::_lt: {
                 auto cur_hi = (rel.op_kind() == op_kind_t::_le ? rel.rhs()
                                                                : rel.rhs() - 1);
-                if (hi.is_empty() || to_cpp<bool>(cur_hi < hi)) { hi = cur_hi; }
+                if (hi.is_empty() || to_cpp<bool>(cur_hi < hi)) {
+                    hi = std::move(cur_hi);
+                }
                 break;
             }
-            default: ir_error_not_expected() << rel;
+            default: gpu_error_not_expected() << rel;
         }
         if (do_break) break;
     }
-    bool ret = !lo.is_empty() && lo.is_equal(hi);
-    if (ret) value = lo;
+    bool ret = lo && lo.is_equal(hi);
+    if (ret) value = std::move(lo);
     return ret;
 }
 
@@ -1035,7 +1268,7 @@ bool constraint_set_t::can_prove_impl(
         const expr_t &_e, bool do_simplify) const {
     auto e = _e;
     if (is_const(e)) {
-        ir_assert(e.type() == type_t::_bool()) << e;
+        gpu_assert(e.type() == type_t::_bool()) << e;
         return to_cpp<bool>(e);
     }
 
@@ -1045,7 +1278,7 @@ bool constraint_set_t::can_prove_impl(
         e = simplify_cmp_reduce_lhs_rhs(e);
         e = simplify(e);
         if (is_const(e)) {
-            ir_assert(e.type() == type_t::_bool()) << e;
+            gpu_assert(e.type() == type_t::_bool()) << e;
             return to_cpp<bool>(e);
         }
     }
