@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,10 +18,10 @@
 #define GPU_INTEL_JIT_IR_IR_HPP
 
 #include <algorithm>
+#include <cstdint>
 #include <map>
 #include <vector>
 
-#include "common/optional.hpp"
 #include "gpu/intel/jit/ir/core.hpp"
 #include "gpu/intel/jit/ir/hw.hpp"
 
@@ -35,12 +35,12 @@ class constraint_set_t;
 
 class ir_context_t {
 public:
-    ir_context_t(const exec_config_t &exec_cfg, constraint_set_t &cset)
-        : exec_cfg_(exec_cfg), cset_(cset) {}
+    ir_context_t(const kernel::options_t &options, constraint_set_t &cset)
+        : options_(options), cset_(cset) {}
 
-    const exec_config_t &exec_cfg() const { return exec_cfg_; }
+    const kernel::options_t &options() const { return options_; }
 
-    const hw_t &hw() const { return exec_cfg().hw(); }
+    const hw_t &hw() const { return options().hw(); }
 
     int grf_size() const { return hw().grf_size(); }
 
@@ -54,15 +54,22 @@ public:
     }
 
     std::string create_tmp_name(const std::string &prefix = "tmp") {
-        int &id = prefix_ids_[prefix];
-        auto name = prefix + "_" + std::to_string(id);
-        id++;
+        auto name = prefix;
+        if (all_names_.count(prefix) != 0) {
+            int &id = prefix_ids_[prefix];
+            do {
+                id++;
+                name = prefix + "_" + std::to_string(id);
+            } while (all_names_.count(name) != 0);
+        }
+        all_names_.insert(name);
         return name;
     }
 
 private:
-    exec_config_t exec_cfg_;
+    kernel::options_t options_;
     constraint_set_t &cset_;
+    std::unordered_set<std::string> all_names_;
     std::unordered_map<std::string, int> prefix_ids_;
 };
 
@@ -84,7 +91,7 @@ public:
         }
 
         std::string str() const {
-            std::ostringstream oss;
+            ostringstream_t oss;
             oss << "buf: " << buf << " size: " << size;
             return oss.str();
         }
@@ -116,15 +123,19 @@ public:
         }
         // Ensure large GRF buffers are aligned to a register boundary.
         if (!entry_ptr->is_slm() && entry_ptr->size > ir_ctx_->grf_size()) {
-            ir_assert(entry_ptr->size % ir_ctx_->grf_size() == 0);
+            gpu_assert(entry_ptr->size % ir_ctx_->grf_size() == 0);
         }
         return entry_ptr->buf;
     }
 
+    bool has(const std::string &name) const {
+        return entries_.find(name) != entries_.end();
+    }
+
     entry_t find(const std::string &name, bool allow_empty = false) const {
         auto it = entries_.find(name);
         if (it != entries_.end()) return it->second;
-        if (!allow_empty) ir_error_not_expected() << "Not found: " << name;
+        if (!allow_empty) gpu_error_not_expected() << "Not found: " << name;
         return entry_t();
     }
 
@@ -134,7 +145,7 @@ public:
 
     const entry_t &find_ref(const std::string &name) const {
         auto it = entries_.find(name);
-        ir_assert(it != entries_.end());
+        gpu_assert(it != entries_.end());
         return it->second;
     }
 
@@ -144,7 +155,7 @@ public:
 
     entry_t &find_ref(const std::string &name) {
         auto it = entries_.find(name);
-        ir_assert(it != entries_.end());
+        gpu_assert(it != entries_.end());
         return it->second;
     }
 
@@ -189,19 +200,19 @@ class alloc_updater_t : public ir_mutator_t {
 public:
     void resize(const expr_t &buf, int new_size) {
         auto ret = resizes_.insert({buf, new_size});
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
     void add_attr(const expr_t &buf, const alloc_attr_t &attr) {
         auto ret = attrs_.insert({buf, attr});
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
     void remove(const expr_t &buf) {
         auto ret = removes_.insert(buf);
-        ir_assert(ret.second) << buf;
+        gpu_assert(ret.second) << buf;
         MAYBE_UNUSED(ret);
     }
 
@@ -218,7 +229,7 @@ public:
             } else if (!new_stmt.is_same(old_stmt)) {
                 auto &new_a = new_stmt.as<alloc_t>();
                 auto &entry = buf_mgr.find_ref(e.buf);
-                ir_assert(entry.attrs.empty());
+                gpu_assert(entry.attrs.empty());
                 entry.size = new_a.size;
                 entry.attrs = new_a.attrs;
                 continue;
@@ -284,8 +295,11 @@ private:
 // - If put_innermost is false, then `stmt` is nested to all allocations
 // - If put_innermost is true, then every allocation is injected as innermost
 //   as possible
+// - If update_existing is true, then all existing alloc statements are removed
+//   and reinserted in the innermost position. This flag requires put_innermost
+//   to be set to true.
 stmt_t inject_alloc_stmts(const stmt_t &stmt, const std::vector<stmt_t> &allocs,
-        bool put_innermost = false);
+        bool put_innermost = false, bool update_existing = false);
 
 // Similar to the previous function but allocations are taken from the buffer
 // manager, allocations are injected at innermost possible scope.
@@ -295,12 +309,33 @@ stmt_t inject_alloc_stmts(const stmt_t &stmt, const buffer_manager_t &buf_mgr);
 // all let statements.
 stmt_t inject_let_stmts(const stmt_t &stmt, const std::vector<stmt_t> &lets);
 
+// Injects dangling let statements (having empty body) in the innermost
+// possible scope. This allows to use declare and use variables on-the-fly, in
+// the imperative manner.
+//
+// Example:
+//     let x = 1 {}
+//     let y = (x + 1) {}
+//     let z = (y + 1) {}
+//     store(..., z)
+//
+// After injection:
+//     let x = 1 {
+//       let y = (x + 1) }
+//         let z = (y + 1) {
+//           store(..., z)
+//         }
+//       }
+//     }
+stmt_t inject_dangling_let_stmts(const stmt_t &stmt);
+
 template <typename T>
 struct expr_cast_helper_t {
     static T call(const expr_t &e) { return to_cpp<T>(e); }
 
     static std::vector<T> call(const std::vector<expr_t> &exprs) {
         std::vector<T> ret;
+        ret.reserve(exprs.size());
         for (auto &e : exprs)
             ret.push_back(to_cpp<T>(e));
         return ret;
@@ -320,6 +355,7 @@ struct expr_cast_helper_t<expr_t> {
             = typename std::enable_if<std::is_arithmetic<U>::value>::type>
     static std::vector<expr_t> call(const std::vector<U> &vec) {
         std::vector<expr_t> ret;
+        ret.reserve(vec.size());
         for (auto &v : vec)
             ret.push_back(to_expr(v));
         return ret;
@@ -353,6 +389,7 @@ std::vector<std::pair<KeyT, ValueT>> sort_var_map(
         const std::unordered_map<KeyT, ValueT, HashT, EqualT> &map,
         const CompareT &compare) {
     std::vector<std::pair<KeyT, ValueT>> ret;
+    ret.reserve(map.size());
     for (auto &kv : map)
         ret.emplace_back(kv);
     std::sort(ret.begin(), ret.end(), compare);
@@ -370,34 +407,60 @@ std::vector<std::pair<KeyT, expr_t>> sort_var_map_by_value(
             });
 }
 
+template <typename ValueT, typename HashT, typename EqualT>
+std::vector<std::pair<expr_t, ValueT>> sort_var_map_by_key(
+        const std::unordered_map<expr_t, ValueT, HashT, EqualT> &map) {
+    return sort_var_map(map,
+            [](const std::pair<expr_t, ValueT> &a,
+                    const std::pair<expr_t, ValueT> &b) {
+                return a.first.template as<var_t>().name
+                        < b.first.template as<var_t>().name;
+            });
+}
+
+template <typename T>
+object_set_t<object_t> find_unique_objects(const object_t &root);
+
 class alloc_manager_t {
 public:
     alloc_manager_t(const stmt_t &root) {
+        auto name_sort = [](const expr_t &a, const expr_t &b) {
+            return a.as<var_t>().name < b.as<var_t>().name;
+        };
+
+        auto vars = find_unique_objects<var_t>(root);
+        all_vars_.insert(all_vars_.end(), vars.begin(), vars.end());
+
+        auto calls = find_objects<func_call_t>(root);
+        for (auto &_c : calls) {
+            if (!is_func_call<builtin_t>(_c)) continue;
+            auto &c = _c.as<func_call_t>();
+            auto &builtin = c.func.as<builtin_t>();
+            if (builtin.name != "alloc") continue;
+            alloc_vars_.push_back(c.args[0]);
+        }
+
         auto allocs = find_objects<alloc_t>(root);
         for (auto &_a : allocs) {
             auto &a = _a.as<alloc_t>();
             auto ret = buf2alloc_.insert({a.buf, _a});
             buffers_.push_back(a.buf);
-            ir_assert(ret.second) << "Buffer already exists: " << a.buf;
+            gpu_assert(ret.second) << "Buffer already exists: " << a.buf;
             MAYBE_UNUSED(ret);
         }
-
         // Sort buffers by name.
-        std::sort(buffers_.begin(), buffers_.end(),
-                [](const expr_t &a, const expr_t &b) {
-                    return a.as<var_t>().name < b.as<var_t>().name;
-                });
+        std::sort(buffers_.begin(), buffers_.end(), name_sort);
     }
 
     const std::vector<expr_t> &buffers() const { return buffers_; }
 
+    expr_t find_var(const std::string &name, bool allow_empty = false) const {
+        return find(all_vars_, name, allow_empty);
+    }
+
     expr_t find_buffer(
             const std::string &name, bool allow_empty = false) const {
-        for (auto &b : buffers())
-            if (b.as<var_t>().name == name) return b;
-
-        if (!allow_empty) ir_error_not_expected() << name;
-        return expr_t();
+        return find(buffers(), name, allow_empty);
     }
 
     std::vector<expr_t> find_buffers(alloc_kind_t kind) const {
@@ -407,50 +470,59 @@ public:
         return ret;
     }
 
-    int alloc_size(const expr_t &buf) const {
+    uint32_t alloc_size(const expr_t &buf) const {
         auto *a = find_alloc(buf);
-        ir_assert(a) << buf;
+        gpu_assert(a) << buf;
         return a->size;
     }
 
     alloc_kind_t alloc_kind(const expr_t &buf) const {
         auto *a = find_alloc(buf);
-        ir_assert(a) << buf;
+        gpu_assert(a) << buf;
         return a->kind;
     }
 
-    int total_size(alloc_kind_t kind) const {
-        int ret = 0;
+    uint32_t slm_size() const {
+        uint32_t ret = 0;
         for (auto &kv : buf2alloc_) {
             auto &a = kv.second.as<alloc_t>();
-            if (a.kind == kind) ret += a.size;
+            if (a.kind == alloc_kind_t::slm) ret += a.size;
+        }
+        for (auto &v : alloc_vars_) {
+            if (!v.type().is_slm()) continue;
+            ret += v.type().size();
         }
         return ret;
     }
 
 private:
+    expr_t find(const std::vector<expr_t> &vars, const std::string &name,
+            bool allow_empty) const {
+        for (auto &v : vars)
+            if (v.as<var_t>().name == name) return v;
+        if (!allow_empty) gpu_error_not_expected() << name;
+        return expr_t();
+    }
+
     const alloc_t *find_alloc(const expr_t &buf) const {
         auto it = buf2alloc_.find(buf);
         if (it == buf2alloc_.end()) return nullptr;
         return it->second.as_ptr<alloc_t>();
     }
 
-    object_map_t<expr_t, stmt_t> buf2alloc_;
+    std::vector<expr_t> alloc_vars_;
+    std::vector<expr_t> all_vars_;
     std::vector<expr_t> buffers_;
-    object_map_t<expr_t, stmt_t> alloc_updates_;
+    object_map_t<expr_t, stmt_t> buf2alloc_;
 };
 
 // IR utility functions.
 expr_t abs(const expr_t &e);
+expr_t max(const expr_t &a, const expr_t &b);
+expr_t min(const expr_t &a, const expr_t &b);
 
 expr_t cast(const expr_t &e, const type_t &type, bool saturate = false);
 
-bool is_zero(const expr_t &e);
-
-bool is_one(const expr_t &e);
-
-bool is_minus_one(const expr_t &e);
-
 bool is_const_broadcast(const expr_t &e);
 
 bool is_const_broadcast(const expr_t &e, const expr_t &value);
@@ -541,8 +613,7 @@ std::vector<stmt_t> find_stmt_groups(
 
 // Returns a statement group matching the label. `root` must have exactly one
 // occurrence.
-utils::optional_t<stmt_t> find_stmt_group(
-        const object_t &root, const stmt_label_t &label);
+stmt_t find_stmt_group(const object_t &root, const stmt_label_t &label);
 
 // Removes all statement groups matching the label.
 object_t remove_stmt_group(const object_t &root, stmt_label_t label);
@@ -563,13 +634,13 @@ public:
         return; \
     }
 
-    void pre_visit(const object_impl_t &obj) override {
+    void pre_visit(const impl_t &obj) override {
         CASE(alloc_t, buf, true);
         CASE(let_t, var, true);
         CASE(for_t, var, true);
     }
 
-    void post_visit(const object_impl_t &obj) override {
+    void post_visit(const impl_t &obj) override {
         CASE(alloc_t, buf, false);
         CASE(let_t, var, false);
         CASE(for_t, var, false);
@@ -592,12 +663,12 @@ private:
 
 class ir_path_t {
 public:
-    void push(const object_impl_t *obj) { path_.push_back(obj); }
+    void push(const object::impl_t *obj) { path_.push_back(obj); }
 
     void pop() { path_.pop_back(); }
 
-    const object_impl_t *back() const {
-        ir_assert(!is_empty());
+    const object::impl_t *back() const {
+        gpu_assert(!is_empty());
         return path_.back();
     }
 
@@ -613,7 +684,7 @@ public:
     }
 
 private:
-    std::vector<const object_impl_t *> path_;
+    std::vector<const object::impl_t *> path_;
 };
 
 // Only for statements that create scope.
@@ -624,8 +695,6 @@ stmt_t replace_stmt_body(const stmt_t &stmt, const stmt_t &new_body);
 int get_peak_regs(const stmt_t &stmt, int grf_size, int external_regs = 0,
         bool skip_let = false);
 
-bool has_send_atomics(const stmt_t &s);
-
 struct mem_usage_guard_t {
     mem_usage_guard_t(int *usage, int *peak_usage, int size)
         : usage(usage), peak_usage(peak_usage), size(size) {
@@ -672,8 +741,8 @@ struct mem_usage_guard_t {
 // where a and b are integer constants.
 struct linear_transform_t {
     expr_t x;
-    int a;
-    int b;
+    int64_t a;
+    int64_t b;
 
     bool is_identity() const { return a == 1 && b == 0; }
 };
@@ -701,7 +770,7 @@ public:
             const linear_transform_t &t, const expr_t &new_var) const;
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << expr_;
         return oss.str();
     }
@@ -742,7 +811,7 @@ public:
     }
 
     bool implies(const modulus_info_t &other) const {
-        ir_assert(var().is_same(other.var()));
+        gpu_assert(var().is_same(other.var()));
 
         int64_t this_mod = to_cpp<int64_t>(mod());
         int64_t other_mod = to_cpp<int64_t>(other.mod());
@@ -751,7 +820,7 @@ public:
     }
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << expr_;
         return oss.str();
     }
@@ -801,7 +870,7 @@ public:
         : relations_(relations) {}
 
     int64_t get_var_bound(const expr_t &e, bool is_low) const override {
-        ir_assert(is_var(e));
+        gpu_assert(is_var(e));
         int64_t def_bound = unlimited_bound(is_low);
         auto it = relations_.find(e);
         if (it == relations_.end()) return def_bound;
@@ -844,9 +913,9 @@ public:
     int max_proven_gcd(const expr_t &var) const;
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "relations:" << (relations_.empty() ? " (empty)\n" : "\n");
-        for (auto &r : relations_) {
+        for (auto &r : sort_var_map_by_key(relations_)) {
             oss << "\t" << r.first << ":";
             bool first = true;
             for (auto &s : r.second) {
@@ -858,7 +927,7 @@ public:
 
         oss << "modulus_info:"
             << (modulus_infos_.empty() ? " (empty)\n" : "\n");
-        for (auto &m : modulus_infos_) {
+        for (auto &m : sort_var_map_by_key(modulus_infos_)) {
             oss << "\t" << m.first << ":";
             bool first = true;
             for (auto &s : m.second) {
@@ -913,12 +982,12 @@ private:
             case op_kind_t::_le: break;
             case op_kind_t::_gt:
                 op_kind = op_kind_t::_ge;
-                ir_assert(b < std::numeric_limits<int64_t>::max());
+                gpu_assert(b < std::numeric_limits<int64_t>::max());
                 b += 1;
                 break;
             case op_kind_t::_lt:
                 op_kind = op_kind_t::_le;
-                ir_assert(b > std::numeric_limits<int64_t>::min());
+                gpu_assert(b > std::numeric_limits<int64_t>::min());
                 b -= 1;
                 break;
             default: return false;
@@ -990,7 +1059,7 @@ inline func_t zero_out_func() {
     return f;
 }
 
-inline stmt_t zero_out(const expr_t &buf, int size) {
+inline stmt_t zero_out(const expr_t &buf, dim_t size) {
     return zero_out_func().call({buf, expr_t(size)});
 }
 
