@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,11 +14,17 @@
 * limitations under the License.
 *******************************************************************************/
 
+#ifdef ENABLE_LLVM_WCONVERSION
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wimplicit-int-conversion"
+#endif
+
 #include "gpu/intel/jit/ir/message.hpp"
 
 #include "gpu/intel/jit/ir/block_2d_utils.hpp"
 #include "gpu/intel/jit/ir/ir.hpp"
 #include "gpu/intel/jit/ir/send_plan.hpp"
+#include "gpu/intel/logging.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -26,34 +32,16 @@ namespace gpu {
 namespace intel {
 namespace jit {
 
-std::ostream &operator<<(std::ostream &out, const send_op_t op) {
-    const char *s = nullptr;
-    switch (op) {
-        case send_op_t::atomic_fadd: s = "atomic_fadd"; break;
-        case send_op_t::atomic_cmpwr: s = "atomic_cmpwr"; break;
-        case send_op_t::load: s = "load"; break;
-        case send_op_t::load_2d: s = "load_2d"; break;
-        case send_op_t::prefetch: s = "prefetch"; break;
-        case send_op_t::prefetch_2d: s = "prefetch_2d"; break;
-        case send_op_t::store: s = "store"; break;
-        case send_op_t::store_2d: s = "store_2d"; break;
-        case send_op_t::undef: s = "undef"; break;
-        default: ir_error_not_expected(); s = "unknown";
-    }
-
-    return out << s;
-}
-
 stmt_t send_t::create_offset_store(const expr_t &header_buf,
         const expr_t &mem_buf, const expr_t &_mem_off,
         bool is_signed_offset) const {
-    ir_assert(is_var(mem_buf));
+    gpu_assert(is_var(mem_buf));
     int header_off = 0;
     int unit_size = 1;
     if (!is_lsc && is_block() && is_slm()) {
         header_off = 2 * address_type().size();
         // Convert byte offset to dwords/owords/hwords offset.
-        unit_size = type.scalar().size();
+        unit_size = type.base().size();
     }
 
     expr_t mem_off = _mem_off;
@@ -64,7 +52,7 @@ stmt_t send_t::create_offset_store(const expr_t &header_buf,
     expr_t off;
     if (is_a64()) {
         off = cast(mem_buf, address_type());
-        if (mem_off.type().is_vector()) {
+        if (mem_off.type().elems() > 1) {
             off = shuffle_t::make_broadcast(off, mem_off.type().elems());
         }
         off += mem_off;
@@ -166,7 +154,7 @@ std::vector<func_t> send_t::get_all(const hw_t &hw, send_op_t op,
                 // Prefer messages with a smaller type as they have less strict
                 // alignment requirements.
                 if (a_sz == b_sz)
-                    return a.type.scalar().size() < b.type.scalar().size();
+                    return a.type.base().size() < b.type.base().size();
                 return a_sz > b_sz;
             });
 
@@ -193,7 +181,7 @@ ngen::CacheSettingsLSC get_cache_settings(const send_t &send, const hw_t &hw) {
     bool is_prefetch = send.is_prefetch() || send.is_prefetch_2d();
     switch (send.cache_hint) {
         case send_cache_hint_t::undef:
-            switch (send.hw.to_ngen()) {
+            switch (send.hw.ngen_hw()) {
                 case ngen::HW::XeHPG:
                     // Use default cache policy on xelpg to avoid suspected driver issue.
                     if (is_store && hw.systolic_support())
@@ -211,6 +199,10 @@ ngen::CacheSettingsLSC get_cache_settings(const send_t &send, const hw_t &hw) {
             break;
         case send_cache_hint_t::load_once:
             ret = ngen::CacheSettingsLSC::L1C_L3C;
+            break;
+        case send_cache_hint_t::hw_default:
+            ret = ngen::CacheSettingsLSC::Default;
+            break;
     }
     return ret;
 }
@@ -221,9 +213,8 @@ class memory_walker_t {
 public:
     memory_walker_t(const constraint_set_t &cset, const view_t &view)
         : view_(view)
-        , type_size_(view.type().size())
         , mask_tensor_(view.create_mask_tensor(cset).reinterpret(view.type()))
-        , full_size_(view.velems() * type_size_) {
+        , full_size_(utils::div_up(view.velems() * view.type().bitsize(), 8)) {
         init_dense_blocks(cset);
         reset();
     }
@@ -239,7 +230,9 @@ public:
 
     int remaining_size() const { return remaining_size_; }
 
-    int remaining_elems() const { return remaining_size_ / type_size_; }
+    int remaining_elems() const {
+        return remaining_size_ * type().packing() / type().size();
+    }
 
     bool is_dense_and_aligned(int off, int size, int alignment) const {
         if (off + size > remaining_size_) return false;
@@ -257,7 +250,8 @@ public:
             int off = i * slot_size;
             // Overflow is fine, expect it to be handled by proper masking.
             if (off >= remaining_size_) return true;
-            if ((slot_size * slots) % type_size_ != 0) return false;
+            if ((slot_size * slots * type().packing()) % type().size() != 0)
+                return false;
             if (!is_dense_and_aligned(off, slot_size, alignment)) return false;
         }
         return true;
@@ -267,7 +261,7 @@ public:
     // granularity and `nmasks` number of masks.
     bool check_mask_size(int off, int size, int mask_size, int nmasks) const {
         auto mask = get_mask(off, size, mask_size, nmasks, /*allow_fail=*/true);
-        return !mask.is_empty();
+        return bool(mask);
     }
 
     expr_t get_offset(int off, expr_t &base, int &off_const) const {
@@ -277,7 +271,7 @@ public:
             return base;
         }
         int block_idx = get_block_index(cur_off_ + off);
-        ir_assert(block_idx >= 0 && block_idx < int(block_offs_.size()));
+        gpu_assert(block_idx >= 0 && block_idx < int(block_offs_.size()));
         base = block_offs_[block_idx];
         auto prev_base = block_offs_[block_idx == 0 ? 0 : block_idx - 1];
         auto get_const_summand = [&](const expr_t &expr) -> int64_t {
@@ -303,66 +297,68 @@ public:
     // Returns a boolean mask expression for the given region to access.
     expr_t get_mask(int off, int size, int mask_size, int nmasks,
             bool allow_fail = false) const {
-        ir_assert(size % mask_size == 0) << "Incompatible mask size.";
+        gpu_assert(size % mask_size == 0) << "Incompatible mask size.";
         auto sub_mask_tensor = create_sub_mask_tensor(off, size);
         sub_mask_tensor = sub_mask_tensor.reinterpret(type_t::u8(mask_size));
         if (sub_mask_tensor.is_empty()) {
             if (allow_fail) return expr_t();
-            ir_error_not_expected();
+            gpu_error_not_expected();
         }
         auto ret = sub_mask_tensor.to_expr(nmasks);
         if (ret.is_empty()) {
             if (allow_fail) return expr_t();
-            ir_error_not_expected() << "Can't create mask.";
+            gpu_error_not_expected() << "Can't create mask.";
         }
         return ret;
     }
 
     // Moves the current position `size` bytes ahead.
     void advance(int size) {
-        ir_assert(size % type_size_ == 0);
+        gpu_assert((size * type().packing()) % type().size() == 0);
         size = std::min(size, remaining_size_);
         cur_off_ += size;
         remaining_size_ -= size;
     }
 
 private:
+    const type_t &type() const { return view_.type(); }
+
     void init_dense_blocks(const constraint_set_t &cset) {
         auto l = view_.create_pseudo_vlayout();
         // Find the maximum innermost dense tile.
         stride_t stride = 1;
-        std::vector<dim_t> dims(l.ndims(), 1);
+        tile_t tile;
         for (auto &b : l.blocks()) {
             if (b.stride != stride) break;
-            dims[b.dim_idx] *= b.block;
-            stride = b.block * b.stride;
+            tile[b.idx] *= b.size;
+            stride = b.size * b.stride;
         }
-        tensor_t tile(dims);
-        dense_block_size_ = tile.elems() * type_size_;
+        dense_block_size_ = tile.elems() * type().size() / type().packing();
         // Split the memory view into dense blocks and precompute block offsets
         // and alignments.
-        view_.for_each_tile(tile, [&](const std::vector<dim_t> &start) {
-            auto off = view_.offset_in_bytes(expr_cast<expr_t>(start));
+        view_.for_each_tile(tile, [&](const icoord_t &start) {
+            auto off = view_.offset_bytes(start);
             off = simplify(off, cset);
 
             const int base_alignment = 128;
             int64_t f = get_max_const_factor(off, cset);
             int alignment = f ? ir_utils::max_pow2_divisor(f) : base_alignment;
 
-            block_offs_.push_back(off);
+            block_offs_.push_back(std::move(off));
             block_alignments_.push_back(alignment);
         });
     }
 
     mask_tensor_t create_sub_mask_tensor(int off, int size) const {
-        ir_assert(off % type_size_ == 0);
-        ir_assert(size % type_size_ == 0);
+        gpu_assert((off * type().packing()) % type().size() == 0);
+        gpu_assert((size * type().packing()) % type().size() == 0);
 
-        std::vector<dim_t> sub_dims = {size / type_size_};
-        layout_t sub_layout(view_.type(), 0, sub_dims);
+        std::vector<dim_t> sub_dims
+                = {(dim_t)size * type().packing() / type().size()};
+        layout_t sub_layout(type(), sub_dims);
         mask_tensor_t sub_mask_tensor(sub_layout);
-        int beg = (cur_off_ + off) / type_size_;
-        int end = (cur_off_ + off + size) / type_size_;
+        int beg = (cur_off_ + off) * type().packing() / type().size();
+        int end = (cur_off_ + off + size) * type().packing() / type().size();
         for (int i = beg; i < end; i++) {
             auto mask = (i < mask_tensor_.elems()) ? mask_tensor_.mask(i)
                                                    : expr_t(false);
@@ -380,7 +376,6 @@ private:
     }
 
     view_t view_;
-    int type_size_;
     mask_tensor_t mask_tensor_;
     std::vector<expr_t> block_offs_;
     std::vector<int> block_alignments_;
@@ -394,10 +389,7 @@ class layout_walker_t {
 public:
     layout_walker_t() = default;
     layout_walker_t(const layout_t &layout, int grf_size)
-        : layout_(layout)
-        , grf_size_(grf_size)
-        , type_size_(layout.type().size())
-        , idxs_(layout.blocks().size()) {}
+        : layout_(layout), grf_size_(grf_size), idxs_(layout.blocks().size()) {}
 
     int offset_bytes() const { return off_bytes_; }
 
@@ -414,12 +406,13 @@ public:
     // - The last element must not cross the layout boundary
     bool can_advance(int stride, int elems, bool is_last_region = false) {
         if (is_last_region) elems = std::min(elems, remaining_elems());
+        const auto stride_bytes
+                = utils::div_up(stride * type().size(), type().packing());
         auto cur_idxs = idxs_;
         int cur_off_bytes = off_bytes_;
         for (int i = 0; i < elems - 1; i++) {
             int next_off_bytes = advance(cur_idxs, cur_off_bytes);
-            if (next_off_bytes - cur_off_bytes != stride * type_size_)
-                return false;
+            if (next_off_bytes - cur_off_bytes != stride_bytes) return false;
             cur_off_bytes = next_off_bytes;
         }
         cur_off_bytes = advance(cur_idxs, cur_off_bytes);
@@ -438,28 +431,27 @@ public:
     }
 
 private:
-    int max_offset_bytes() const {
-        return utils::rnd_up((int)layout_.size(), grf_size_);
-    }
+    const type_t &type() const { return layout_.type(); }
+
+    int max_offset_bytes() const { return (int)size_bytes(layout_, grf_size_); }
 
     int remaining_elems() const { return layout_.elems() - elems_; }
 
     int advance(std::vector<int> &idxs, int off_bytes) const {
         for (size_t i = 0; i < idxs.size(); i++) {
-            if (++idxs[i] < layout_.blocks()[i].block) break;
+            if (++idxs[i] < layout_[i].size) break;
             idxs[i] = 0;
         }
         int off = 0;
         for (size_t i = 0; i < idxs.size(); i++) {
-            int stride = (int)layout_.blocks()[i].stride;
+            int stride = (int)layout_[i].stride;
             off += idxs[i] * stride;
         }
-        return off * type_size_;
+        return utils::div_up(off * type().size(), type().packing());
     }
 
     layout_t layout_;
     int grf_size_;
-    int type_size_;
 
     std::vector<int> idxs_;
     int elems_ = 0;
@@ -481,7 +473,7 @@ access_builder_t::access_builder_t(ir_context_t &ir_ctx, const view_t &mem_view,
     , zero_out_(zero_out) {
     if (send_params.use_send_plan) {
         auto sp = create_send_plan(
-                ir_ctx.exec_cfg(), mem_view, send_params, zero_out);
+                ir_ctx.options(), mem_view, send_params, zero_out);
         if (sp && !sp.is_2d()) send_params.hint_2d = send_2d_hint_t();
         if (!sp) return;
         reg_layout_ = sp.reg_layout();
@@ -511,19 +503,18 @@ void access_builder_t::build() {
     }
     if (!ok && send_op_ == send_op_t::prefetch) {
         // Do not treat as an error, skip prefetch messages during generation.
-        ir_warning() << "Can't generate send decomposition for prefetch."
-                     << std::endl;
+        gpu_warning() << "Can't generate send decomposition for prefetch.";
         return;
     }
-    ir_assert(ok) << "Can't generate send decomposition.";
+    gpu_assert(ok) << "Can't generate send decomposition.";
 }
 
-static bool stride_dimension_ok(const view_t &view, int stride_tidx,
-        int stride_vidx, const std::vector<expr_t> &vstart) {
+static bool stride_dimension_ok(const view_t &view, size_t stride_tidx,
+        size_t stride_vidx, const coord_t &vstart) {
     auto &tdim = view.tdim(stride_tidx);
     auto e = tdim.expr();
-    for (int i = 0; i < tdim.nvargs(); i++) {
-        int vidx = tdim.vidx(i);
+    for (dim_idx_t i = 0; i < tdim.nvargs(); i++) {
+        dim_idx_t vidx = tdim.vidx(i);
         auto &vvar = view.vvars()[vidx];
         if (vidx == stride_vidx) {
             e = substitute(e, vvar, expr_t(0));
@@ -550,7 +541,7 @@ static expr_t try_scalarize(const expr_t &e) {
         return binary_op_t::make(binary->op_kind, a, b);
     }
 
-    ir_error_not_expected() << e;
+    gpu_error_not_expected() << e;
     return expr_t();
 }
 
@@ -570,7 +561,7 @@ static stmt_t try_promote_to_lsc(const stmt_t &_call) {
     send_t::arg_mask(new_args) = std::move(mask);
 
     auto lsc_send = send_t::make(send.hw, send.op, send.address, send.type,
-            send.slots, /*is_lsc=*/true, send.zero_out, send.cache_hint);
+            send.slots, /*is_lsc=*/true, send.fill_buf, send.cache_hint);
     return lsc_send.call(new_args);
 }
 
@@ -578,7 +569,7 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
     auto vlayout = mem_view_.create_pseudo_vlayout();
     auto &hint = send_params.hint_2d;
     // The data may be loaded in a wider data type to get a proper GRF layout.
-    if (!hint.type.is_undef()) vlayout = vlayout.reinterpret(hint.type);
+    if (!hint.type.is_undef()) vlayout = reinterpret(vlayout, hint.type);
 
     bool is_store = (send_op_ == send_op_t::store);
     auto send_type = type_t::u(vlayout.type().size() * 8);
@@ -587,17 +578,17 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
 
     auto &b0 = blocks[0];
     auto &b1 = blocks[1];
-    ir_assert(b0.dim_idx != b1.dim_idx);
+    gpu_assert(b0.idx != b1.idx);
     if (b0.stride != stride_t(1)) return false;
     if (!b1.stride.is_fixed()) return false;
 
-    auto get_tdim_idx = [&](int vdim_idx, int &stride) {
-        int ret = -1;
-        for (int i = 0; i < mem_view_.ntdims(); i++) {
+    auto get_tdim_idx = [&](size_t vdim_idx, int &stride) {
+        size_t ret = dim_idx::invalid;
+        for (dim_idx_t i = 0; i < mem_view_.ntdims(); i++) {
             auto &tdim = mem_view_.tdim(i);
-            for (int j = 0; j < tdim.nvargs(); j++) {
+            for (dim_idx_t j = 0; j < tdim.nvargs(); j++) {
                 if (tdim.vidx(j) == vdim_idx) {
-                    ir_assert(ret == -1);
+                    gpu_assert(ret == dim_idx::invalid);
                     stride = (int)tdim.vstride(j);
                     ret = i;
                 }
@@ -608,31 +599,31 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
 
     int w_tstride = 0;
     int h_tstride = 0;
-    int w_dim_idx = get_tdim_idx(b0.dim_idx, w_tstride);
-    int h_dim_idx = get_tdim_idx(b1.dim_idx, h_tstride);
+    size_t w_dim_idx = get_tdim_idx(b0.idx, w_tstride);
+    size_t h_dim_idx = get_tdim_idx(b1.idx, h_tstride);
 
     if (w_tstride != 1) return false;
 
     auto &tlayout = mem_view_.tlayout();
-    auto get_2d_dim = [&](int tidx) {
-        return tlayout.inner_block(tidx, /*skip_outer=*/false);
+    auto get_2d_dim = [&](size_t tidx) {
+        return inner_block(tlayout, tidx, /*skip_outer=*/false);
     };
 
     int surface_width = 0;
     int surface_height = 0;
-    int surface_pitch = b1.stride;
-    bool is_w_blocked = (get_2d_dim(w_dim_idx) != tlayout.dim(w_dim_idx));
-    bool is_h_blocked = (get_2d_dim(h_dim_idx) != tlayout.dim(h_dim_idx));
+    int surface_pitch = int(b1.stride);
+    bool is_w_blocked = (get_2d_dim(w_dim_idx) != tlayout.elems(w_dim_idx));
+    bool is_h_blocked = (get_2d_dim(h_dim_idx) != tlayout.elems(h_dim_idx));
     // Virtual surface means loading from the innermost block of a block layout
     // which implies no bound checks embedded into 2D block message.
     bool use_virtual_surface = is_w_blocked || is_h_blocked;
     if (use_virtual_surface) {
         if (h_tstride != 1) return false;
-        surface_width = b0.block;
-        surface_height = b1.block;
+        surface_width = b0.size;
+        surface_height = b1.size;
     } else {
-        surface_width = tlayout.dim(w_dim_idx);
-        surface_height = tlayout.dim(h_dim_idx);
+        surface_width = tlayout.elems(w_dim_idx);
+        surface_height = tlayout.elems(h_dim_idx);
         if (surface_height % h_tstride != 0) return false;
         surface_height = surface_height / h_tstride;
     }
@@ -650,7 +641,7 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
     int max_count
             = block_2d_max_count(is_store, transpose, width, mem_type_.size());
     while (try_count <= max_count) {
-        if (b0.block % (try_count * width) != 0) break;
+        if (b0.size % (try_count * width) != 0) break;
         count = try_count;
         try_count *= 2;
     }
@@ -666,43 +657,42 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
                 hint.vnni_permute_factor))
         return false;
 
-    std::vector<dim_t> dims(vlayout.ndims(), 1);
-    dims[b0.dim_idx] = count * width;
-    dims[b1.dim_idx] = height;
-    tensor_t tile(dims);
+    tile_t tile;
+    tile[b0.idx] = count * width;
+    tile[b1.idx] = height;
 
-    reg_layout_ = layout_t(type_factor == 1 ? mem_type_ : send_type, 0,
+    reg_layout_ = layout_t(type_factor == 1 ? mem_type_ : send_type,
             std::vector<dim_t>(vlayout.ndims(), 1));
     int h_inner = vnni ? 4 / send_type.size() : 1;
     int h_outer = ir_utils::safe_divide(height, h_inner);
-    reg_layout_ = reg_layout_.add_outer_block(b1.dim_idx, h_inner);
+    reg_layout_ = reg_layout_.with_block({b1.idx, h_inner});
     if (transpose) {
-        reg_layout_ = reg_layout_.add_outer_block(b1.dim_idx, h_outer);
-        reg_layout_ = reg_layout_.add_outer_block(b0.dim_idx, width);
+        reg_layout_ = reg_layout_.with_block({b1.idx, h_outer});
+        reg_layout_ = reg_layout_.with_block({b0.idx, width});
     } else {
-        reg_layout_ = reg_layout_.add_outer_block(b0.dim_idx, width);
-        reg_layout_ = reg_layout_.add_outer_block(b1.dim_idx, h_outer);
+        reg_layout_ = reg_layout_.with_block({b0.idx, width});
+        reg_layout_ = reg_layout_.with_block({b1.idx, h_outer});
     }
-    reg_layout_ = reg_layout_.add_outer_block(b0.dim_idx, count);
+    reg_layout_ = reg_layout_.with_block({b0.idx, count});
 
     int w_outermost
-            = ir_utils::safe_divide(vlayout.dim(b0.dim_idx), count * width);
-    int h_outermost = ir_utils::safe_divide(vlayout.dim(b1.dim_idx), height);
-    reg_layout_ = reg_layout_.add_outer_block(b0.dim_idx, w_outermost);
-    reg_layout_ = reg_layout_.add_outer_block(b1.dim_idx, h_outermost);
+            = ir_utils::safe_divide(vlayout.elems(b0.idx), count * width);
+    int h_outermost = ir_utils::safe_divide(vlayout.elems(b1.idx), height);
+    reg_layout_ = reg_layout_.with_block({b0.idx, w_outermost});
+    reg_layout_ = reg_layout_.with_block({b1.idx, h_outermost});
 
     if (type_factor != 1) {
         auto blocks = reg_layout_.blocks();
-        reg_layout_ = layout_t(
-                mem_type_, 0, std::vector<dim_t>(vlayout.ndims(), 1));
-        reg_layout_ = reg_layout_.add_outer_block(b0.dim_idx, type_factor);
+        reg_layout_
+                = layout_t(mem_type_, std::vector<dim_t>(vlayout.ndims(), 1));
+        reg_layout_ = reg_layout_.with_block({b0.idx, type_factor});
         for (auto &b : blocks)
-            reg_layout_ = reg_layout_.add_outer_block(b.dim_idx, b.block);
+            reg_layout_ = reg_layout_.with_block({b.idx, b.size});
     }
 
     for (auto &b : blocks) {
-        if (utils::one_of(b.dim_idx, b0.dim_idx, b1.dim_idx)) continue;
-        reg_layout_ = reg_layout_.add_outer_block(b.dim_idx, b.block);
+        if (utils::one_of(b.idx, b0.idx, b1.idx)) continue;
+        reg_layout_ = reg_layout_.with_block({b.idx, b.size});
     }
 
     reg_layout_walker_
@@ -721,38 +711,32 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
     auto &send = _send.as<send_t>();
 
     stmt_ = stmt_t();
-    bool ok = true;
-    auto vstart0 = mem_view_.vstart();
-    vlayout.for_each_tile(tile, [&](const std::vector<dim_t> &start) {
-        if (!ok) return;
-
+    const auto &vstart0 = mem_view_.vstart();
+    for (auto &start : vlayout.iter(tile)) {
         int access_size = send.access_size();
         int access_elems = access_size / mem_type_.size();
 
         // Check mask requirements.
         expr_t mask;
-        if (!check_2d_mask(tensor_t(tile.dims(), start), use_virtual_surface,
-                    w_dim_idx, h_dim_idx, mask)) {
-            ok = false;
-            return;
+        if (!check_2d_mask(tile, start, use_virtual_surface, w_dim_idx,
+                    h_dim_idx, mask)) {
+            return false;
         }
 
         if (!send.is_prefetch_2d()) {
             if (!reg_layout_walker_->can_advance(1, access_elems)) {
-                ok = false;
-                return;
+                return false;
             }
 
             if (!reg_layout_walker_->can_access(send.payload_size())) {
-                ok = false;
-                return;
+                return false;
             }
         }
 
         auto vstart = vstart0;
-        for (int i = 0; i < vlayout.ndims(); i++) {
-            if (start[i] == 0) continue;
-            int factor = (i == b0.dim_idx ? type_factor : 1);
+        for (dim_idx_t i = 0; i < into<dim_idx_t>(vlayout.ndims()); i++) {
+            if (start.get(i) == 0) continue;
+            int factor = (i == b0.idx.index() ? type_factor : 1);
             vstart[i] += factor * start[i];
         }
         auto tstart
@@ -772,12 +756,11 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
 
             if (h_tstride != 1) {
                 if (!stride_dimension_ok(
-                            mem_view_, h_dim_idx, b1.dim_idx, vstart)) {
+                            mem_view_, h_dim_idx, b1.idx, vstart)) {
                     if (send.is_prefetch_2d()) {
                         skip_send = true;
                     } else {
-                        ok = false;
-                        return;
+                        break;
                     }
                 }
                 y /= h_tstride;
@@ -785,20 +768,18 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
         }
 
         auto off = simplify(
-                mem_view_.tlayout().offset_in_bytes(tstart), ir_ctx_->cset());
+                offset_bytes(mem_view_.tlayout(), tstart), ir_ctx_->cset());
 
         // Check alignment requirements.
         int64_t align = get_max_const_factor(off, ir_ctx_->cset());
         if (align % block_2d_base_alignment(ir_ctx_->hw()) != 0) {
-            ok = false;
-            return;
+            return false;
         }
 
         if (!skip_send) {
             if (!ir_ctx_->cset().can_prove(
                         x % block_2d_x_alignment(send_type.size()) == 0)) {
-                ok = false;
-                return;
+                break;
             }
             auto reg_buf = (send.is_prefetch_2d()
                             ? expr_t()
@@ -808,9 +789,9 @@ bool access_builder_t::try_build_2d(send_params_t &send_params) {
         }
 
         reg_layout_walker_->advance(send.access_size() / mem_type_.size());
-    });
+    }
 
-    return ok;
+    return true;
 }
 
 bool access_builder_t::fixup_send_2d_params(const type_t &send_type, bool vnni,
@@ -860,24 +841,24 @@ bool access_builder_t::fixup_send_2d_params(const type_t &send_type, bool vnni,
     return whp_ok();
 }
 
-bool access_builder_t::check_2d_mask(const tensor_t &tile,
-        bool use_virtual_surface, int w_dim_idx, int h_dim_idx,
+bool access_builder_t::check_2d_mask(const tile_t &tile, const coord_t &coord,
+        bool use_virtual_surface, size_t w_dim_idx, size_t h_dim_idx,
         expr_t &mask) const {
-    auto sub_view = mem_view_.create_sub_view(tile);
+    auto sub_view = mem_view_.create_sub_view(tile, coord);
     auto mask_tensor = sub_view.create_mask_tensor(ir_ctx_->cset());
     mask = mask_tensor.to_expr(1);
-    if (!mask.is_empty()) return true;
+    if (mask) return true;
 
     // Virtual surface implies no out-of-bound send checks.
     if (use_virtual_surface) return false;
 
     // Remove bound conditions that are covered by out-of-bound send checks.
     uint32_t tmask = 0xFFFFFFFF;
-    for (int i = 0; i < sub_view.nvdims(); i++) {
+    for (dim_idx_t i = 0; i < sub_view.nvdims(); i++) {
         if (!utils::one_of(i, w_dim_idx, h_dim_idx)) continue;
-        for (int j = 0; j < sub_view.ntdims(); j++) {
+        for (dim_idx_t j = 0; j < sub_view.ntdims(); j++) {
             auto &tdim = sub_view.tdim(j);
-            for (int k = 0; k < tdim.nvargs(); k++) {
+            for (dim_idx_t k = 0; k < tdim.nvargs(); k++) {
                 if (tdim.vidx(k) == i) {
                     // TODO: Check if tdim mask is a bound mask.
                     tmask &= ~(1U << i);
@@ -887,7 +868,7 @@ bool access_builder_t::check_2d_mask(const tensor_t &tile,
     }
     mask_tensor = sub_view.create_mask_tensor(ir_ctx_->cset(), tmask);
     mask = mask_tensor.to_expr(1);
-    if (!mask.is_empty()) return true;
+    if (mask) return true;
 
     return false;
 }
@@ -895,6 +876,8 @@ bool access_builder_t::check_2d_mask(const tensor_t &tile,
 bool access_builder_t::try_build(
         const layout_t &try_layout, memory_walker_t &mem_walker) {
     auto &try_layout_blocks = try_layout.blocks();
+    const int type_size = mem_type_.size();
+    const int type_packing = mem_type_.packing();
     int reg_stride
             = (try_layout_blocks.empty() ? 0
                                          : (int)try_layout_blocks[0].stride);
@@ -916,16 +899,17 @@ bool access_builder_t::try_build(
             int nmasks = s.nmasks();
             int payload_stride = s.payload_type_stride();
             int access_size = s.access_size();
-            int access_elems = access_size / mem_type_.size();
+            int access_elems = access_size * type_packing / type_size;
             bool is_last_chunk = mem_walker.remaining_size() <= access_size;
 
             if (reg_stride != 1 || payload_stride != slot_size) {
                 // Detected strided GRF layout or strided payload. In this
                 // case require full data type and stride match.
                 if (reg_stride != 0
-                        && payload_stride != reg_stride * mem_type_.size())
+                        && payload_stride * type_packing
+                                != reg_stride * type_size)
                     continue;
-                if (s.type.size() != mem_type_.size()) continue;
+                if (type_packing * s.type.size() != type_size) continue;
             }
             // Prefetches don't have payload so skip these conditions for
             // prefetch.
@@ -960,7 +944,8 @@ bool access_builder_t::try_build(
         send_stmt = try_promote_to_lsc(send_stmt);
         stmt_ = stmt_.append(send_stmt);
 
-        reg_layout_walker_->advance(send.access_size() / mem_type_.size());
+        reg_layout_walker_->advance(
+                send.access_size() * type_packing / type_size);
         mem_walker.advance(send.access_size());
     }
     reg_layout_ = try_layout;
@@ -978,8 +963,9 @@ std::vector<layout_t> access_builder_t::candidate_payload_layouts() const {
 
     // These payload layouts are to match payload for byte x {1,2} scattered
     // messages (they are dword-strided).
-    if (type_size == 2) ret.push_back(vlayout.make_strided(2));
-    if (type_size == 1) ret.push_back(vlayout.make_strided(4));
+    if (type_size == 2) ret.push_back(make_strided(vlayout, 2));
+    if (type_size == 1 && mem_type_.bitsize() == 8)
+        ret.push_back(make_strided(vlayout, 4));
 
     return ret;
 }
@@ -1004,7 +990,7 @@ stmt_t access_builder_t::create_send_stmt(
         } else if (!off_base.is_equal(off_base0)) {
             is_same_base = false;
         }
-        off_vec.push_back(off);
+        off_vec.push_back(std::move(off));
         off_const_vec.emplace_back(off_const - off_const0);
     }
     expr_t off;
@@ -1029,12 +1015,13 @@ stmt_t access_builder_t::create_send_stmt(
 
 static const int any_block = 0;
 
-send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &_type,
+send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &type,
         bool vnni, bool transpose, int w_tile, int h_tile,
         int w_blk = any_block, int h_blk = any_block) {
-    auto type = _type;
+    gpu_assert(!(vnni && transpose)) << "VNNI with transpose is not supported.";
 
-    ir_assert(!(vnni && transpose)) << "VNNI with transpose is not supported.";
+    // Disable sub-byte types for now.
+    if (type.packing() > 1) return send_2d_hint_t();
 
     // XXX: Convert transpose to VNNI when transpose is not
     // supported. This will require additional reorder but
@@ -1087,12 +1074,12 @@ send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &_type,
 
     if (vnni) {
         // TODO: Remove.
-        ir_assert(h_blk > 0);
+        gpu_assert(h_blk > 0);
         h_blk = find_block(h_tile, h_blk, h_max);
     }
     if (transpose && w_blk > 0) {
         // TODO: Remove.
-        ir_assert(w_blk > 0);
+        gpu_assert(w_blk > 0);
         w_blk = find_block(w_tile, w_blk, w_max);
     }
 
@@ -1106,8 +1093,8 @@ send_2d_hint_t get_send_2d_hint(send_op_t send_op, const type_t &_type,
     return hint;
 }
 
-bool send_2d_params_ok(const exec_config_t &exec_cfg, send_op_t send_op) {
-    if (exec_cfg.hw() < ngen::HW::XeHPC) return false;
+bool send_2d_params_ok(const kernel::options_t &options, send_op_t send_op) {
+    if (options.hw() < ngen::HW::XeHPC) return false;
     if (!utils::one_of(send_op, send_op_t::load, send_op_t::prefetch,
                 send_op_t::store))
         return false;
@@ -1120,34 +1107,34 @@ bool send_2d_vlayout_ok(const layout_t &vlayout) {
 
     const auto &b0 = blocks[0];
     const auto &b1 = blocks[1];
-    if (b0.dim_idx == b1.dim_idx) return false;
+    if (b0.idx == b1.idx) return false;
     if (b0.stride != stride_t(1)) return false;
     if (b1.stride.is_unknown()) return false;
     return true;
 }
 
-send_2d_hint_t get_send_2d_hint(const exec_config_t &exec_cfg,
+send_2d_hint_t get_send_2d_hint(const kernel::options_t &options,
         send_op_t send_op, const view_t &view, bool allow_2d,
         bool use_send_plan) {
     send_2d_hint_t hint;
-    if (!allow_2d || !send_2d_params_ok(exec_cfg, send_op)) return hint;
+    if (!allow_2d || !send_2d_params_ok(options, send_op)) return hint;
     auto vlayout = view.create_pseudo_vlayout();
     if (!send_2d_vlayout_ok(vlayout)) return hint;
     auto blocks = vlayout.blocks();
     auto &b0 = blocks[0];
     auto &b1 = blocks[1];
 
-    if (b0.block >= 128) return hint;
+    if (b0.size >= 128) return hint;
     return get_send_2d_hint(
-            send_op, view.type(), false, false, b0.block, b1.block);
+            send_op, view.type(), false, false, b0.size, b1.size);
 }
 
-send_2d_hint_t get_send_2d_hint(const exec_config_t &exec_cfg,
+send_2d_hint_t get_send_2d_hint(const kernel::options_t &options,
         send_op_t send_op, fma_kind_t fma_kind, abc_kind_t abc_kind,
         const view_t &view, const gemm_schedule_t &gemm_schedule, bool allow_2d,
         bool use_send_plan) {
     send_2d_hint_t hint;
-    if (!allow_2d || !send_2d_params_ok(exec_cfg, send_op)) return hint;
+    if (!allow_2d || !send_2d_params_ok(options, send_op)) return hint;
     auto vlayout = view.create_pseudo_vlayout();
     if (!send_2d_vlayout_ok(vlayout)) return hint;
     auto blocks = vlayout.blocks();
@@ -1162,25 +1149,25 @@ send_2d_hint_t get_send_2d_hint(const exec_config_t &exec_cfg,
         // src2, KxN: 16a16b -> 16b16a            (transpose)
         // src2, NxK: 16a16b -> 16a16b            ()
         bool is_dpas_src1 = (abc_kind == abc_kind_t::b);
-        int m_blk = exec_cfg.simd();
+        int m_blk = options.simd();
         int n_blk = any_block;
         int mn_blk = (is_dpas_src1 ? m_blk : n_blk);
         int k_blk = 32 / view.type().size();
         auto &bmnk_mapper = gemm_schedule.bmnk_mapper();
-        bool is_b0_k = (bmnk_mapper.bmnk_kind(abc_kind, b0.dim_idx)
-                == bmnk_kind_t::k);
+        bool is_b0_k
+                = (bmnk_mapper.bmnk_kind(abc_kind, b0.idx) == bmnk_kind_t::k);
         bool transpose = (is_dpas_src1 == is_b0_k);
         int b0_blk = is_b0_k ? k_blk : mn_blk;
         int b1_blk = !is_b0_k ? k_blk : mn_blk;
-        if (b0_blk != any_block && b0.block % b0_blk != 0) return hint;
-        if (b1_blk != any_block && b1.block % b1_blk != 0) return hint;
+        if (b0_blk != any_block && b0.size % b0_blk != 0) return hint;
+        if (b1_blk != any_block && b1.size % b1_blk != 0) return hint;
         bool vnni = is_dpas_src1 && !transpose;
-        hint = get_send_2d_hint(send_op, view.type(), vnni, transpose, b0.block,
-                b1.block, b0_blk, b1_blk);
+        hint = get_send_2d_hint(send_op, view.type(), vnni, transpose, b0.size,
+                b1.size, b0_blk, b1_blk);
     } else {
-        if (b0.block >= 128) return hint;
+        if (b0.size >= 128) return hint;
         hint = get_send_2d_hint(
-                send_op, view.type(), false, false, b0.block, b1.block);
+                send_op, view.type(), false, false, b0.size, b1.size);
     }
 
     // XXX: Special VNNI permute hint to use with Xa16b:bf16 layout which can't
@@ -1199,20 +1186,20 @@ send_2d_hint_t get_send_2d_hint(const exec_config_t &exec_cfg,
     if (use_send_plan && send_op == send_op_t::load && hint.vnni
             && !hint.transpose && view.type().size() == 2
             && utils::one_of(abc_kind, abc_kind_t::a, abc_kind_t::b)
-            && b0.block == 16 && (dim_t)b1.stride == 16
-            && utils::one_of(b1.block, 8, 16, 32)) {
+            && b0.size == 16 && (dim_t)b1.stride == 16
+            && utils::one_of(b1.size, 8, 16, 32)) {
         hint.vnni_permute_factor = 2;
     }
 
     return hint;
 }
 
-send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
-        send_address_t send_address, const view_t &view,
+send_params_t get_send_params(const kernel::options_t &options,
+        send_op_t send_op, send_address_t send_address, const view_t &view,
         send_cache_hint_t cache_hint, fma_kind_t fma_kind, abc_kind_t abc_kind,
         bool allow_2d) {
     send_params_t params;
-    params.hw = exec_cfg.hw();
+    params.hw = options.hw();
     params.mem_type = view.type();
     params.send_op = send_op;
     params.send_address = send_address;
@@ -1221,17 +1208,17 @@ send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
     params.prefer_dense
             = (fma_kind == fma_kind_t::dpas && abc_kind == abc_kind_t::a);
     params.hint_2d = get_send_2d_hint(
-            exec_cfg, send_op, view, allow_2d, params.use_send_plan);
+            options, send_op, view, allow_2d, params.use_send_plan);
     return params;
 }
 
-send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
-        send_address_t send_address, fma_kind_t fma_kind, abc_kind_t abc_kind,
-        const view_t &view, const gemm_schedule_t &gemm_schedule,
-        bool allow_2d) {
-    auto params = get_send_params(exec_cfg, send_op, send_address, view,
+send_params_t get_send_params(const kernel::options_t &options,
+        send_op_t send_op, send_address_t send_address, fma_kind_t fma_kind,
+        abc_kind_t abc_kind, const view_t &view,
+        const gemm_schedule_t &gemm_schedule, bool allow_2d) {
+    auto params = get_send_params(options, send_op, send_address, view,
             send_cache_hint_t::undef, fma_kind, abc_kind, false);
-    params.hint_2d = get_send_2d_hint(exec_cfg, send_op, fma_kind, abc_kind,
+    params.hint_2d = get_send_2d_hint(options, send_op, fma_kind, abc_kind,
             view, gemm_schedule, allow_2d, params.use_send_plan);
     return params;
 }
@@ -1241,3 +1228,7 @@ send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
 } // namespace gpu
 } // namespace impl
 } // namespace dnnl
+
+#ifdef ENABLE_LLVM_WCONVERSION
+#pragma clang diagnostic pop
+#endif
