@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -46,6 +46,7 @@ GPU_DEFINE_PARSE_ENUM(send_kind_t, send_kind_names)
 // Send operation kind.
 enum class send_op_t {
     undef,
+    atomic_add,
     atomic_fadd,
     atomic_cmpwr,
     load,
@@ -56,7 +57,19 @@ enum class send_op_t {
     store_2d,
 };
 
-std::ostream &operator<<(std::ostream &out, const send_op_t value);
+static auto send_op_names = nstl::to_array({
+        make_enum_name(send_op_t::undef, "undef"),
+        make_enum_name(send_op_t::atomic_add, "atomic_add"),
+        make_enum_name(send_op_t::atomic_fadd, "atomic_fadd"),
+        make_enum_name(send_op_t::atomic_cmpwr, "atomic_cmpwr"),
+        make_enum_name(send_op_t::load, "load"),
+        make_enum_name(send_op_t::load_2d, "load_2d"),
+        make_enum_name(send_op_t::prefetch, "prefetch"),
+        make_enum_name(send_op_t::prefetch_2d, "prefetch_2d"),
+        make_enum_name(send_op_t::store, "store"),
+        make_enum_name(send_op_t::store_2d, "store_2d"),
+});
+GPU_DEFINE_PARSE_ENUM(send_op_t, send_op_names)
 
 // Send address model.
 enum class send_address_t {
@@ -64,11 +77,6 @@ enum class send_address_t {
     slm,
 };
 
-enum class send_cache_hint_t {
-    undef,
-    load_once,
-};
-
 static auto send_cache_hint_names = nstl::to_array({
         make_enum_name(send_cache_hint_t::undef, "cache:undef"),
         make_enum_name(send_cache_hint_t::load_once, "cache:load_once"),
@@ -77,21 +85,21 @@ static auto send_cache_hint_names = nstl::to_array({
 GPU_DEFINE_PARSE_ENUM(send_cache_hint_t, send_cache_hint_names)
 
 struct block_2d_info_t {
-    bool is_empty() const { return surface_width == 0; }
+    bool is_empty() const { return surface_width.is_empty(); }
 
     bool operator==(const block_2d_info_t &other) const {
         if (is_empty() != other.is_empty()) return false;
         if (is_empty()) return true;
-        return (surface_width == other.surface_width)
-                && (surface_height == other.surface_height)
-                && (surface_pitch == other.surface_pitch)
+        return (surface_width.is_equal(other.surface_width))
+                && (surface_height.is_equal(other.surface_height))
+                && (surface_pitch.is_equal(other.surface_pitch))
                 && (width == other.width) && (height == other.height)
                 && (count == other.count) && (vnni == other.vnni)
                 && (transpose == other.transpose);
     }
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << count << "x";
         oss << height << "x";
         oss << width;
@@ -104,9 +112,9 @@ struct block_2d_info_t {
     }
 
     // Encoded in header.
-    int surface_width = 0;
-    int surface_height = 0;
-    int surface_pitch = 0;
+    expr_t surface_width;
+    expr_t surface_height;
+    expr_t surface_pitch;
     int width = 0;
     int height = 0;
     int count = 0;
@@ -116,10 +124,8 @@ struct block_2d_info_t {
 };
 
 // Function representing send messages.
-class send_t : public func_impl_t {
+class send_t : public func_impl_t, public object::info_t<send_t> {
 public:
-    IR_DECL_DERIVED_TYPE_ID(send_t, func_impl_t)
-
     static func_t make(const hw_t &hw, send_op_t op, send_address_t address,
             const type_t &type, int slots, bool zero_out,
             send_cache_hint_t cache_hint = send_cache_hint_t::undef) {
@@ -150,13 +156,14 @@ public:
     }
 
     static func_t make_2d(const hw_t &hw, send_op_t op, const type_t &type,
-            int surface_width, int surface_height, int surface_pitch, int width,
-            int height, int count, bool vnni, bool transpose, bool zero_out,
+            expr_t surface_width, expr_t surface_height, expr_t surface_pitch,
+            int width, int height, int count, bool vnni, bool transpose,
+            bool zero_out,
             send_cache_hint_t cache_hint = send_cache_hint_t::undef) {
         block_2d_info_t info;
-        info.surface_width = surface_width;
-        info.surface_height = surface_height;
-        info.surface_pitch = surface_pitch;
+        info.surface_width = std::move(surface_width);
+        info.surface_height = std::move(surface_height);
+        info.surface_pitch = std::move(surface_pitch);
         info.width = width;
         info.height = height;
         info.count = count;
@@ -169,12 +176,13 @@ public:
             int width, int height, int count, bool vnni, bool transpose,
             bool zero_out,
             send_cache_hint_t cache_hint = send_cache_hint_t::undef) {
-        return make_2d(hw, op, type, /*surface_width=*/0, /*surface_height=*/0,
-                /*surface_pitch=*/0, width, height, count, vnni, transpose,
+        return make_2d(hw, op, type, /*surface_width=*/ {},
+                /*surface_height=*/ {},
+                /*surface_pitch=*/ {}, width, height, count, vnni, transpose,
                 zero_out, cache_hint);
     }
 
-    bool is_equal(const object_impl_t &obj) const override {
+    bool is_equal(const impl_t &obj) const override {
         if (!obj.is<self_type>()) return false;
         auto &other = obj.as<self_type>();
 
@@ -187,8 +195,8 @@ public:
                 && (cache_hint == other.cache_hint);
     }
     std::string str() const override {
-        std::ostringstream oss;
-        oss << op;
+        ostringstream_t oss;
+        oss << to_string(op);
         oss << ".";
         oss << type.str();
         if (is_scattered()) oss << "x" << slots;
@@ -224,7 +232,10 @@ public:
         return call({mem_buf, mem_off, reg_buf, mask, x, y, pattern});
     }
 
-    bool is_atomic() const { return op == send_op_t::atomic_fadd; }
+    bool is_atomic() const {
+        return utils::one_of(op, send_op_t::atomic_add, send_op_t::atomic_fadd,
+                send_op_t::atomic_cmpwr);
+    }
     bool is_load() const { return op == send_op_t::load; }
     bool is_load_2d() const { return op == send_op_t::load_2d; }
     bool is_prefetch() const { return op == send_op_t::prefetch; }
@@ -238,8 +249,7 @@ public:
     bool is_slm() const { return address == send_address_t::slm; }
 
     bool is_block() const {
-        return utils::one_of(
-                type.kind(), type_kind_t::oword, type_kind_t::hword);
+        return utils::one_of(type.base(), type_t::oword(), type_t::hword());
     }
 
     bool is_scattered() const { return !is_block() && !is_2d(); }
@@ -254,7 +264,7 @@ public:
     }
 
     int payload_type_stride() const {
-        ir_assert(!is_2d());
+        gpu_assert(!is_2d());
         return std::max(4, type.size());
     }
 
@@ -279,7 +289,7 @@ public:
 
     int alignment() const {
         if (is_2d()) return 128;
-        if (is_block()) return type.scalar().size();
+        if (is_block()) return type.base().size();
         return 1;
     }
 
@@ -293,7 +303,7 @@ public:
 
         if (is_scattered()) return type.size();
 
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return 0;
     }
 
@@ -302,7 +312,7 @@ public:
         int masks = ir_utils::safe_divide(type.size() * slots, mask_size());
         if (hw < ngen::HW::XeHPC && is_block() && masks > 16) {
             // Round-robin masking, 16 bits are reused with dword granularity.
-            ir_assert(masks % 16 == 0);
+            gpu_assert(masks % 16 == 0);
             masks = 16;
         }
         return masks;
@@ -361,7 +371,7 @@ private:
     send_t(const hw_t &hw, send_op_t op, send_address_t address,
             const type_t &type, int slots, uint32_t slot_mask, bool is_lsc,
             bool zero_out, send_cache_hint_t cache_hint)
-        : func_impl_t(_type_info())
+        : func_impl_t(get_info())
         , hw(hw)
         , op(op)
         , address(address)
@@ -374,7 +384,7 @@ private:
 
     send_t(const hw_t &hw, send_op_t op, const type_t &type, bool zero_out,
             const block_2d_info_t &block_2d_info, send_cache_hint_t cache_hint)
-        : func_impl_t(_type_info())
+        : func_impl_t(get_info())
         , hw(hw)
         , op(op)
         , address(send_address_t::a64)
@@ -385,11 +395,11 @@ private:
         , fill_buf(zero_out)
         , block_2d_info(block_2d_info)
         , cache_hint(cache_hint) {
-        ir_assert(utils::one_of(op, send_op_t::load_2d, send_op_t::store_2d,
+        gpu_assert(utils::one_of(op, send_op_t::load_2d, send_op_t::store_2d,
                 send_op_t::prefetch_2d));
         if (is_store_2d()) {
-            ir_assert(!block_2d_info.vnni);
-            ir_assert(!block_2d_info.transpose);
+            gpu_assert(!block_2d_info.vnni);
+            gpu_assert(!block_2d_info.transpose);
         }
     }
 };
@@ -455,13 +465,13 @@ public:
     const layout_t &reg_layout() const { return reg_layout_; }
     int reg_buf_size() const {
         if (reg_buf_size_ == 0)
-            return into<int>(utils::rnd_up(reg_layout_.size(), grf_size()));
+            return into<int>(size_bytes(reg_layout_, grf_size()));
         return reg_buf_size_;
     }
     const stmt_t &stmt() const { return stmt_; }
 
     std::string str() const {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "Memory view:          " << mem_view_ << std::endl;
         oss << "Register layout:      " << reg_layout_ << std::endl;
         oss << "Register buffer:      " << reg_buf_ << std::endl;
@@ -479,8 +489,9 @@ private:
             bool transpose, bool use_xy, int &W, int &H, int &P, int &w, int &h,
             int &c, int &vnni_permute_factor);
 
-    bool check_2d_mask(const tensor_t &tile, bool use_virtual_surface,
-            dim_idx_t w_idx, dim_idx_t h_idx, expr_t &mask) const;
+    bool check_2d_mask(const tile_t &tile, const coord_t &coord,
+            bool use_virtual_surface, size_t w_idx, size_t h_idx,
+            expr_t &mask) const;
 
     std::vector<layout_t> candidate_payload_layouts() const;
     stmt_t create_send_stmt(
@@ -505,21 +516,21 @@ private:
     stmt_t stmt_;
 };
 
-send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
-        send_address_t send_address, const view_t &view,
+send_params_t get_send_params(const kernel::options_t &options,
+        send_op_t send_op, send_address_t send_address, const view_t &view,
         send_cache_hint_t cache_hint = send_cache_hint_t::undef,
         fma_kind_t fma_kind = fma_kind_t::undef,
         abc_kind_t abc_kind = abc_kind_t::undef, bool allow_2d = false);
 
-send_params_t get_send_params(const exec_config_t &exec_cfg, send_op_t send_op,
-        send_address_t send_address, fma_kind_t fma_kind, abc_kind_t abc_kind,
-        const view_t &view, const gemm_schedule_t &gemm_schedule,
-        bool allow_2d = true);
+send_params_t get_send_params(const kernel::options_t &options,
+        send_op_t send_op, send_address_t send_address, fma_kind_t fma_kind,
+        abc_kind_t abc_kind, const view_t &view,
+        const gemm_schedule_t &gemm_schedule, bool allow_2d = true);
 
-inline send_params_t get_send_params(const exec_config_t &exec_cfg,
+inline send_params_t get_send_params(const kernel::options_t &options,
         send_op_t send_op, send_address_t send_address, const view_t &view,
         bool allow_2d) {
-    return get_send_params(exec_cfg, send_op, send_address, view,
+    return get_send_params(options, send_op, send_address, view,
             send_cache_hint_t::undef, fma_kind_t::undef, abc_kind_t::undef,
             allow_2d);
 }
@@ -538,7 +549,7 @@ inline access_builder_t make_access_builder(ir_context_t &ir_ctx,
         send_cache_hint_t cache_hint = send_cache_hint_t::undef,
         bool zero_out = true) {
     auto send_params = get_send_params(
-            ir_ctx.exec_cfg(), send_op, send_address, mem_view, cache_hint);
+            ir_ctx.options(), send_op, send_address, mem_view, cache_hint);
     return make_access_builder(
             ir_ctx, mem_view, mem_buf, reg_buf, send_params, zero_out);
 }
