@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,13 +20,11 @@
 #include <string>
 #include <vector>
 
-#include "common/eltwise_pd.hpp"
 #include "gpu/intel/jit/ir/eltwise.hpp"
 #include "gpu/intel/jit/ir/gemm_schedule.hpp"
 #include "gpu/intel/jit/ir/ir.hpp"
 #include "gpu/intel/jit/ir/kernel_info.hpp"
 #include "gpu/intel/jit/ir/tensor.hpp"
-#include "gpu/intel/jit/utils/utils.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -46,7 +44,8 @@ public:
     bool is_common_src_zero_point = false;
     bool is_common_wei_zero_point = false;
     bool is_common_dst_zero_point = false;
-    bool needs_src_precalc = false;
+    bool needs_src_reorder_precalc = false;
+    bool needs_src_conv_precalc = false;
     int common_src_zero_point = 0;
     int common_wei_zero_point = 0;
     int common_dst_zero_point = 0;
@@ -62,28 +61,31 @@ public:
                           DNNL_ARG_WEIGHTS))
         , do_dst_compensation(pd
                   && !pd->attr()->zero_points_.has_default_values(DNNL_ARG_DST))
-        , is_runtime_src_zero_points(
-                  pd && !pd->attr()->zero_points_.defined(DNNL_ARG_SRC))
-        , is_runtime_wei_zero_points(
-                  pd && !pd->attr()->zero_points_.defined(DNNL_ARG_WEIGHTS))
-        , is_runtime_dst_zero_points(
-                  pd && !pd->attr()->zero_points_.defined(DNNL_ARG_DST))
+        , is_runtime_src_zero_points(pd
+                  && !pd->attr()->zero_points_.has_default_values(DNNL_ARG_SRC))
+        , is_runtime_wei_zero_points(pd
+                  && !pd->attr()->zero_points_.has_default_values(
+                          DNNL_ARG_WEIGHTS))
+        , is_runtime_dst_zero_points(pd
+                  && !pd->attr()->zero_points_.has_default_values(DNNL_ARG_DST))
         , is_common_src_zero_point(
-                  pd && pd->attr()->zero_points_.common(DNNL_ARG_SRC))
-        , is_common_wei_zero_point(
-                  pd && pd->attr()->zero_points_.common(DNNL_ARG_WEIGHTS))
+                  pd && pd->attr()->zero_points_.get_mask(DNNL_ARG_SRC) == 0)
+        , is_common_wei_zero_point(pd
+                  && pd->attr()->zero_points_.get_mask(DNNL_ARG_WEIGHTS) == 0)
         , is_common_dst_zero_point(
-                  pd && pd->attr()->zero_points_.common(DNNL_ARG_DST))
-        , needs_src_precalc(
-                  pd && do_src_compensation && is_src_precalc_compatible(pd))
+                  pd && pd->attr()->zero_points_.get_mask(DNNL_ARG_DST) == 0)
+        , needs_src_reorder_precalc(
+                  pd && do_src_compensation && can_use_src_reorder_precalc(pd))
+        , needs_src_conv_precalc(pd && do_src_compensation
+                  && !needs_src_reorder_precalc && can_use_src_conv_precalc(pd))
         , common_src_zero_point(0)
         , common_wei_zero_point(0)
         , common_dst_zero_point(0) {
         if (pd) {
             auto &zp = pd->attr()->zero_points_;
-            src_zp_type = zp.get_data_type(DNNL_ARG_SRC);
-            wei_zp_type = zp.get_data_type(DNNL_ARG_WEIGHTS);
-            dst_zp_type = zp.get_data_type(DNNL_ARG_DST);
+            src_zp_type = to_ir(zp.get_data_type(DNNL_ARG_SRC));
+            wei_zp_type = to_ir(zp.get_data_type(DNNL_ARG_WEIGHTS));
+            dst_zp_type = to_ir(zp.get_data_type(DNNL_ARG_DST));
         }
     }
 
@@ -101,17 +103,52 @@ public:
     }
 
 private:
-    bool is_src_precalc_compatible(const primitive_desc_t *pd) {
+    bool can_use_src_reorder_precalc(const primitive_desc_t *pd) const {
         if (pd->kind() != primitive_kind_t::dnnl_convolution) return false;
-        // In general, precomputed ZPs are slower than the regular ZPs up to a
-        // point where a nested convolution that does the precalc takes less
-        // time than the in-situ compensations; that usually happens around
-        // MB = 64, but the exact number is just a heuristic.
-        // TODO: a finer-grained estimate
-        return (pd->invariant_src_md()->dims[0] >= 64)
+        // Reorder-based precomputed ZPs are only available if the user did not
+        // specify the weights mem desc so the convolution can choose it freely
+        // and set a mem desc flag asking a reorder to precompute the values.
+        return (pd->invariant_wei_md()->format_kind == format_kind::any)
+                && pd->attr()->zero_points_.get_mask(DNNL_ARG_SRC) == 0
                 && pd->attr()->zero_points_.has_default_values(
                         DNNL_ARG_WEIGHTS);
     }
+    bool can_use_src_conv_precalc(const primitive_desc_t *pd) const {
+        if (pd->kind() != primitive_kind_t::dnnl_convolution) return false;
+        // In general, conv-based precomputed ZPs are slower than the regular
+        // ZPs up to a point where a nested convolution that does the precalc
+        // takes less time than the in-situ compensations; that usually happens
+        // around MB = 64 if IDHW > 16 or KDHW > 1, but the exact numbers are
+        // just heuristics.
+        // TODO: a finer-grained estimate
+        auto mb_threshold = gpu_utils::dev_getenv("DNNL_CBP_ZP_MB", 64);
+        auto idhw_threshold = gpu_utils::dev_getenv("DNNL_CBP_ZP_IDHW", 16);
+        auto kdhw_threshold = gpu_utils::dev_getenv("DNNL_CBP_ZP_KDHW", 1);
+        auto small_ik = (dhw_product(pd, abc_kind_t::a) <= idhw_threshold)
+                && (dhw_product(pd, abc_kind_t::b) <= kdhw_threshold);
+        return (pd->invariant_src_md()->dims[0] >= mb_threshold) && !small_ik
+                && pd->attr()->zero_points_.has_default_values(
+                        DNNL_ARG_WEIGHTS);
+    }
+    dim_t dhw_product(const primitive_desc_t *pd, abc_kind_t kind) const {
+        int swap = (pd->get_prop_kind() != prop_kind::backward_data)
+                ? (pd->get_prop_kind() != prop_kind::backward_weights) ? 2 : 1
+                : 0;
+        std::array<const memory_desc_t *, 3> tensors = {pd->invariant_src_md(),
+                pd->invariant_wei_md(), pd->invariant_dst_md()};
+        std::swap(tensors[swap], tensors[2]); // swap(x, x) is OK for POD
+        const memory_desc_t *md;
+        dim_t retn = 1;
+        switch (kind) {
+            case abc_kind_t::a: md = tensors[0]; break;
+            case abc_kind_t::b: md = tensors[1]; break;
+            case abc_kind_t::c: md = tensors[2]; break;
+            default: return retn;
+        }
+        for (int i = 2; i < md->ndims; i++)
+            retn *= std::max(md->dims[i], dim_t(1));
+        return retn;
+    }
 };
 
 class post_op_tensor_info_t {
@@ -120,14 +157,15 @@ public:
 
     post_op_tensor_info_t(bool is_input, bool is_output, const view_t &view,
             const expr_t &buf, uint32_t mask, const expr_t &op_var,
-            const expr_t &compute_expr)
+            const expr_t &compute_expr, const bool do_convert = true)
         : is_input_(is_input)
         , is_output_(is_output)
         , view_(view)
         , buf_(buf)
         , mask_(mask)
         , op_var_(op_var)
-        , compute_expr_(compute_expr) {
+        , compute_expr_(compute_expr)
+        , do_convert_(do_convert) {
         if (op_var_.is_empty())
             op_var_ = var_t::make(type_t::f32(), make_op_var_name(buf));
     }
@@ -148,14 +186,19 @@ public:
 
     const expr_t &compute_expr() const { return compute_expr_; }
 
-    bool needs_compute() const { return !compute_expr().is_empty(); }
+    bool needs_compute() const { return bool(compute_expr()); }
 
-    post_op_tensor_info_t create_sub_tensor(const tensor_t &tile) const {
+    bool do_convert() const { return do_convert_; }
+
+    post_op_tensor_info_t create_sub_tensor(
+            const tile_t &tile, const coord_t &coord) const {
         auto ret = *this;
-        ret.view_ = ret.view_.create_sub_view(tile);
+        ret.view_ = ret.view_.create_sub_view(tile, coord);
         return ret;
     }
 
+    void retype(const type_t &new_type) { view_ = view_.retype(new_type); }
+
     void require_masked_update() { needs_masked_update_ = true; }
 
 private:
@@ -166,12 +209,12 @@ private:
         auto *ptr = buf.as_ptr<ptr_t>();
         if (ptr) {
             auto prefix = make_op_var_name(ptr->base);
-            ir_assert(is_const(ptr->off));
-            int off = to_cpp<int>(ptr->off);
+            gpu_assert(is_const(ptr->off));
+            dim_t off = to_cpp<dim_t>(ptr->off);
             return prefix + "_" + std::to_string(off);
         }
 
-        ir_error_not_expected() << "Can't generate op var name: " << buf;
+        gpu_error_not_expected() << "Can't generate op var name: " << buf;
         return "unknown";
     }
     bool is_input_;
@@ -182,6 +225,7 @@ private:
     uint32_t mask_;
     expr_t op_var_;
     expr_t compute_expr_;
+    bool do_convert_ = true;
 };
 
 class post_op_view_mapper_t {
@@ -193,7 +237,7 @@ public:
     const view_t &cp_view() const { return cp_view_; };
 
     virtual view_t create_view(const type_t &type, uint32_t rhs_mask) const {
-        std::vector<dim_t> rhs_dims = cp_view_.vdims();
+        std::vector<dim_t> rhs_dims = cp_view_.vdims().values();
         uint32_t bound_check_mask = 0;
         for (int i = 0; i < int(rhs_dims.size()); i++) {
             if ((rhs_mask & (1 << i)) == 0) {
@@ -203,12 +247,12 @@ public:
                 bound_check_mask |= (1 << i);
             }
         }
-        return view_t(layout_t(type, 0, rhs_dims, /*do_normalize=*/false),
+        return view_t(layout_t(type, rhs_dims, 0, /*do_normalize=*/false),
                 cp_view_.vvars(), bound_check_mask);
     }
 
     virtual view_t create_view(const memory_desc_t &md) const {
-        return cp_view().retype(md.data_type);
+        return cp_view().retype(to_ir(md.data_type));
     }
 
     virtual view_t create_src_zp_view(uint32_t mask) const {
@@ -217,7 +261,7 @@ public:
 
     virtual view_t try_create_bias_view(uint32_t mask) const { return {}; }
 
-    virtual bool is_spurious_spatial(int dim_idx) const { return false; };
+    virtual bool is_spurious_spatial(const pvar_t &dim) const { return false; };
     virtual bool need_to_restore_zero_padding() const { return false; }
     virtual bool use_dst_in_sum_post_op() const { return true; }
     virtual bool can_use_scales() const { return true; }
@@ -259,6 +303,11 @@ public:
     bool uses(const expr_t &op_var) const {
         if (contains_object(lhs_, op_var)) return true;
         if (contains_object(rhs_, op_var)) return true;
+        if (eltwise_.is<eltwise_t>()) {
+            auto &eltwise_func = eltwise_.as<eltwise_t>();
+            if (eltwise_func.alg_kind == alg_kind::eltwise_stochastic_round)
+                if (contains_object(eltwise_func.seed, op_var)) return true;
+        }
         return false;
     }
 
@@ -282,7 +331,7 @@ inline op_kind_t alg_kind_to_op_kind(alg_kind_t alg) {
         case alg_kind::binary_lt: return op_kind_t::_lt;
         case alg_kind::binary_eq: return op_kind_t::_eq;
         case alg_kind::binary_ne: return op_kind_t::_ne;
-        default: ir_error_not_expected();
+        default: gpu_error_not_expected();
     }
     return op_kind_t::undef;
 }
@@ -322,35 +371,37 @@ private:
             const memory_desc_t &dst_md, const memory_desc_t &out_md,
             const zero_points_config_t &zp_cfg) const;
 
-    int cp_ndims() const { return cp_view().nvdims(); }
+    dim_idx_t cp_ndims() const { return cp_view().nvdims(); }
 
-    bool is_cp_dim_zero_padded(int idx) const {
+    bool is_cp_dim_zero_padded(dim_idx_t idx) const {
         return cp_view().is_masked_vdim(idx);
     }
 
     const expr_t &add_input_tensor(const view_t &view, const expr_t &buf,
+            const bool do_convert = true,
             const expr_t &compute_expr = expr_t()) {
         return add_tensor(/*is_input=*/true, /*is_output=*/false, view, buf,
-                expr_t(), compute_expr);
+                expr_t(), compute_expr, do_convert);
     }
 
     const expr_t &add_tensor(bool is_input, bool is_output, const view_t &view,
             const expr_t &buf, const expr_t &op_var,
-            const expr_t &compute_expr = expr_t()) {
-        ir_assert(cp_ndims() == view.nvdims());
+            const expr_t &compute_expr = expr_t(),
+            const bool do_convert = true) {
+        gpu_assert(cp_ndims() == view.nvdims());
         uint32_t mask = (buf.is_empty() && compute_expr.is_empty()
                         ? ~(1u << cp_ndims())
                         : compute_mask(view));
-        tensor_infos_.emplace_back(
-                is_input, is_output, view, buf, mask, op_var, compute_expr);
+        tensor_infos_.emplace_back(is_input, is_output, view, buf, mask, op_var,
+                compute_expr, do_convert);
         return tensor_infos_.back().op_var();
     }
 
     uint32_t compute_mask(const view_t &view) const {
-        ir_assert(cp_ndims() == view.nvdims());
+        gpu_assert(cp_ndims() == view.nvdims());
         uint32_t mask = 0;
-        for (int i = 0; i < cp_ndims(); i++) {
-            if (view.vdims()[i] != 1) mask |= (1 << i);
+        for (dim_idx_t i = 0; i < cp_ndims(); i++) {
+            if (view.vdims().get(i) != 1) mask |= (1 << i);
         }
         return mask;
     }
