@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -46,14 +46,14 @@ public:
         , path(path)
         , refs(refs)
         , cse_var(cse_var) {
-        ir_trace() << "cse_pass: add expression: " << expr << std::endl;
+        gpu_trace() << "cse_pass: add expression: " << expr;
     }
 
     void add_usage(const ir_path_t &other_path, bool do_increment = true) {
         if (do_increment) refs++;
         path.merge(other_path);
-        ir_trace() << "cse_pass: add usage: " << expr
-                   << ", total refs: " << refs << std::endl;
+        gpu_trace() << "cse_pass: add usage: " << expr
+                    << ", total refs: " << refs;
     }
 
     // Expression to eliminate via let.
@@ -95,14 +95,15 @@ public:
     }
 
     void recompute_cost() {
-        cost_ = expr_cost(cse_expr_->expr) * cse_expr_->refs;
+        cost_ = expr_cost(cse_expr_->expr, var2entry_) * cse_expr_->refs;
     }
 
-private:
-    int expr_cost(const expr_t &e) {
+    static int expr_cost(const expr_t &e,
+            const object_map_t<expr_t, cse_var_entry_t *> *var2entry) {
         if (is_var(e)) {
-            auto it = var2entry_->find(e);
-            if (it == var2entry_->end()) return 0;
+            if (var2entry == nullptr) return 0;
+            auto it = var2entry->find(e);
+            if (it == var2entry->end()) return 0;
             if (it->second->allocated()) return 0;
             // If variable is not allocated, its value
             // has to be recomputed every time.
@@ -111,17 +112,21 @@ private:
         if (is_const(e)) return 0;
         if (e.is<cast_t>()) return e.type().is_bool();
         if (auto *op = e.as_ptr<binary_op_t>()) {
-            return expr_cost(op->a) + expr_cost(op->b) + 1;
+            return expr_cost(op->a, var2entry) + expr_cost(op->b, var2entry)
+                    + 1;
+        }
+        if (auto *op = e.as_ptr<unary_op_t>()) {
+            return expr_cost(op->a, var2entry) + 1;
         }
-        if (auto *op = e.as_ptr<unary_op_t>()) { return expr_cost(op->a) + 1; }
         if (auto *s = e.as_ptr<shuffle_t>()) {
             if (s->is_broadcast()) return 0;
             return s->elems();
         }
-        ir_error_not_expected() << "Unhandled expression: " << e;
+        gpu_error_not_expected() << "Unhandled expression: " << e;
         return 0;
     }
 
+private:
     const cse_expr_t *cse_expr_ = nullptr;
     int cost_ = 0;
     bool allocated_ = true;
@@ -207,10 +212,10 @@ public:
                     });
             auto &e = **it;
 
-            ir_trace() << "cse_pass: skipping " << e.cse_expr()->expr
-                       << " with cost " << e.cost() << ", size " << e.size()
-                       << ", and cost per byte " << (double)e.cost() / e.size()
-                       << "\n";
+            gpu_trace() << "cse_pass: skipping " << e.cse_expr()->expr
+                        << " with cost " << e.cost() << ", size " << e.size()
+                        << ", and cost per byte "
+                        << (double)e.cost() / e.size();
 
             e.set_unallocated();
             grf_usage_ -= e.size();
@@ -239,17 +244,17 @@ public:
     bool has(const expr_t &e) const { return cse_exprs_.count(e) != 0; }
 
     cse_expr_t &find_cse_expr(const expr_t &e) {
-        ir_assert(has(e)) << e;
+        gpu_assert(has(e)) << e;
         return cse_exprs_.at(e);
     }
 
     const cse_expr_t &find_cse_expr(const expr_t &e) const {
-        ir_assert(has(e)) << e;
+        gpu_assert(has(e)) << e;
         return cse_exprs_.at(e);
     }
 
     bool has_var(const expr_t &e) const {
-        return !find_cse_expr(e).cse_var.is_empty();
+        return bool(find_cse_expr(e).cse_var);
     }
 
     int get_refs(const expr_t &e) const {
@@ -259,13 +264,13 @@ public:
 
     void register_expr(const expr_t &e, const ir_path_t &path) {
         auto ret = cse_exprs_.insert({e, cse_expr_t(e, e, path)});
-        ir_assert(ret.second) << e;
+        gpu_assert(ret.second) << e;
         MAYBE_UNUSED(ret);
     }
 
     void register_expr(const cse_expr_t &cse_expr) {
         auto ret = cse_exprs_.insert({cse_expr.expr, cse_expr});
-        ir_assert(ret.second);
+        gpu_assert(ret.second);
         MAYBE_UNUSED(ret);
     }
 
@@ -275,8 +280,8 @@ public:
             cse_expr.cse_var = ir_ctx_.create_tmp_var(e.type().is_bool()
                             ? bool_imm_t::get_packed_type(e.type().elems())
                             : e.type());
-            ir_trace() << "cse_pass: assigning var: " << e << " -> "
-                       << cse_expr.cse_var << std::endl;
+            gpu_trace() << "cse_pass: assigning var: " << e << " -> "
+                        << cse_expr.cse_var;
         }
         return cse_expr.cse_var;
     }
@@ -296,13 +301,13 @@ public:
 
     void update_expr(const expr_t &old_expr, const expr_t &new_expr) {
         auto it = cse_exprs_.find(old_expr);
-        ir_assert(it != cse_exprs_.end()) << old_expr;
+        gpu_assert(it != cse_exprs_.end()) << old_expr;
         auto &old_cse_expr = it->second;
         auto new_cse_expr = cse_expr_t(new_expr, old_cse_expr.orig_expr,
                 old_cse_expr.path, old_cse_expr.refs, old_cse_expr.cse_var);
         cse_exprs_.erase(it);
         auto ret = cse_exprs_.insert({new_expr, new_cse_expr});
-        ir_assert(ret.second);
+        gpu_assert(ret.second);
         MAYBE_UNUSED(ret);
     }
 
@@ -324,7 +329,14 @@ public:
                 || is_const(e))
             return false;
         auto &cse_expr = find_cse_expr(e);
-        if (cse_expr.refs <= (e.type().is_bool() ? 2 : 1)) return false;
+
+        if (cse_expr.refs <= 1) return false;
+        if (e.type().is_bool()) {
+            // Account for possible cost to move bool variable to and from flag
+            // register
+            auto cost = cse_var_entry_t::expr_cost(cse_expr.expr, nullptr);
+            if (cost + cse_expr.refs + 1 >= cost * cse_expr.refs) return false;
+        }
         if (skip_exprs_.count(cse_expr.orig_expr) != 0) return false;
         return true;
     }
@@ -378,6 +390,16 @@ private:
             return;
         }
 
+        // Support for expressions with mutable variables is unimplemented due
+        // hoisting logic not accounting for mutation.
+        auto vars = find_objects<var_t>(obj);
+        for (auto &v : vars) {
+            if (v.template as<var_t>().type.is_mutable()) {
+                ir_visitor_t::_visit(obj);
+                return;
+            }
+        }
+
         if (std::is_same<T, shuffle_t>::value) {
             auto &shuffle = reinterpret_cast<const shuffle_t &>(obj);
             if (shuffle.is_broadcast()) {
@@ -406,11 +428,11 @@ private:
     template <typename T>
     void visit_stmt(const T &obj) {
         if (std::is_same<T, for_t>::value) {
-            visit_for((const object_impl_t &)obj);
+            visit_for((const impl_t &)obj);
             return;
         }
         if (std::is_same<T, let_t>::value) {
-            visit_let((const object_impl_t &)obj);
+            visit_let((const impl_t &)obj);
             return;
         }
         root_path_.push(&obj);
@@ -418,7 +440,7 @@ private:
         root_path_.pop();
     }
 
-    void visit_for(const object_impl_t &_obj) {
+    void visit_for(const impl_t &_obj) {
         auto &obj = (const for_t &)_obj;
 
         visit(obj.var);
@@ -429,7 +451,7 @@ private:
         root_path_.pop();
     }
 
-    void visit_let(const object_impl_t &_obj) {
+    void visit_let(const impl_t &_obj) {
         auto &obj = (const let_t &)_obj;
 
         visit(obj.var);
@@ -450,7 +472,7 @@ class cse_verifier_t : public scope_visitor_t {
 public:
     cse_verifier_t(cse_context_t &ctx) : ctx_(ctx) {}
 
-    ~cse_verifier_t() override { ir_assert(to_check_.empty()); }
+    ~cse_verifier_t() override { gpu_assert(to_check_.empty()); }
 
     void _visit(const binary_op_t &obj) override { visit_expr(obj); }
     void _visit(const shuffle_t &obj) override { visit_expr(obj); }
@@ -497,7 +519,7 @@ private:
         auto it = to_check_.find(obj);
         if (it != to_check_.end()) {
             for (auto &e : it->second) {
-                ir_assert(is_expr_defined(e))
+                gpu_assert(is_expr_defined(e))
                         << "Expression contains undefined variables: " << e;
                 MAYBE_UNUSED(e);
             }
@@ -530,7 +552,7 @@ public:
         ctx_.for_each([&](const expr_t &e) {
             auto &cse_var = ctx_.get_var(e);
             auto ret = all_vars_.insert({cse_var, e});
-            ir_assert(ret.second);
+            gpu_assert(ret.second);
             MAYBE_UNUSED(ret);
         });
         ctx_.for_each([&](const expr_t &e) { generate_for_expr(e); });
@@ -672,9 +694,9 @@ stmt_t eliminate_common_subexprs_impl(const stmt_t &_stmt, cse_context_t &ctx,
     if (!has_skip) return stmt;
 
     int memory_usage = get_peak_regs(stmt, grf_size) * grf_size;
-    ir_trace() << "CSE exceeded GRF usage limit. Usage: " << memory_usage
-               << ", limit: " << memory_usage_limit
-               << ". Retry CSE and skip some expressions..." << std::endl;
+    gpu_trace() << "CSE exceeded GRF usage limit. Usage: " << memory_usage
+                << ", limit: " << memory_usage_limit
+                << ". Retry CSE and skip some expressions...";
     ctx.reset_cse_exprs();
     return stmt_t();
 }
@@ -703,7 +725,7 @@ public:
     int g2s_buf_size() const {
         int ret = 0;
         for (auto &kv : g2s_bufs_) {
-            ir_assert(kv.second != 0);
+            gpu_assert(kv.second != 0);
             ret += kv.second;
         }
         return ret;
@@ -721,7 +743,7 @@ public:
             return;
         }
         if (auto *func = obj.func.as_ptr<send_t>()) {
-            ir_assert(func->is_load()) << func;
+            gpu_assert(func->is_load()) << func;
             auto &buf = send_t::arg_reg_buf(obj);
             g2s_bufs_.emplace(get_base(buf), 0);
         }
@@ -743,7 +765,7 @@ private:
 stmt_t eliminate_common_subexprs(const stmt_t &stmt, ir_context_t &ir_ctx,
         int reserved_regs, int gmem_bufs) {
     int grf_size = ir_ctx.grf_size();
-    int available_regs = ir_ctx.exec_cfg().regs() - reserved_regs;
+    int available_regs = ir_ctx.options().regs() - reserved_regs;
     int memory_usage_limit = available_regs * grf_size;
     if (gmem_bufs > 1) {
         g2s_buf_visitor_t v;
