@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,23 +39,16 @@ using namespace ir_utils;
 
 // Generic pattern expression, used as a wild card during pattern matching. Can
 // match any expression.
-class pexpr_t : public expr_impl_t {
+class pexpr_t : public expr_iface_t<pexpr_t> {
 public:
-    IR_DECL_EXPR_TYPE_ID(pexpr_t)
-
     static expr_t make(int id) { return expr_t(new pexpr_t(id)); }
 
-    bool is_equal(const object_impl_t &obj) const override {
-        if (!obj.is<self_type>()) return false;
-        auto &other = obj.as<self_type>();
-
-        return id == other.id;
-    }
+    bool operator==(const pexpr_t &other) const { return id == other.id; }
 
     size_t get_hash() const override { return ir_utils::get_hash(id); }
 
     std::string str() const override {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "pexpr_t(" << id << ")";
         return oss.str();
     }
@@ -73,20 +66,21 @@ public:
         return z;
     }
 
-    IR_DECLARE_TRAVERSERS()
+    object_t _mutate(ir_mutator_t &mutator) const override;
+    void _visit(ir_visitor_t &visitor) const override {
+        gpu_error_not_expected();
+    }
 
     int id;
 
 private:
-    pexpr_t(int id) : expr_impl_t(_type_info(), type_t::undef()), id(id) {}
+    pexpr_t(int id) : expr_iface_t(type_t::undef()), id(id) {}
 };
 
 // Pattern expression for int_imm_t, used as a wild card during pattern
 // matching. Can match any int_imm_t with the given value.
-class pint_imm_t : public expr_impl_t {
+class pint_imm_t : public expr_iface_t<pint_imm_t> {
 public:
-    IR_DECL_EXPR_TYPE_ID(pint_imm_t)
-
     // Matches an integer constant with the given value.
     static expr_t make(int64_t value) {
         return expr_t(new pint_imm_t(-1, value));
@@ -102,24 +96,21 @@ public:
     }
 
     // Matches any integer constant.
-    static expr_t make_any(int64_t id) { return expr_t(new pint_imm_t(id, 0)); }
+    static expr_t make_any(int id) { return expr_t(new pint_imm_t(id, 0)); }
 
     bool matches(const int_imm_t &imm) const {
         if (id == -1) return value == imm.value;
         return true;
     }
 
-    bool is_equal(const object_impl_t &obj) const override {
-        if (!obj.is<self_type>()) return false;
-        auto &other = obj.as<self_type>();
-
+    bool operator==(const pint_imm_t &other) const {
         return (id == other.id) && (value == other.value);
     }
 
     size_t get_hash() const override { return ir_utils::get_hash(id, value); }
 
     std::string str() const override {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "pint_imm_t(" << value << ")";
         return oss.str();
     }
@@ -129,7 +120,7 @@ public:
 
 private:
     pint_imm_t(int id, int64_t value)
-        : expr_impl_t(_type_info(), type_t::undef()), id(id), value(value) {}
+        : expr_iface_t(type_t::undef()), id(id), value(value) {}
 };
 
 // Stores already matched pairs of <pattern expression, matched expression>.
@@ -140,9 +131,9 @@ public:
     }
 
     void set(const expr_t &ptrn, const expr_t &e) {
-        ir_assert(ptrn.is<pexpr_t>());
+        gpu_assert(ptrn.is<pexpr_t>());
         auto ret = expr_matched_.insert({ptrn, e});
-        ir_assert(ret.second);
+        gpu_assert(ret.second);
         MAYBE_UNUSED(ret);
     }
 
@@ -161,10 +152,21 @@ private:
     object_eq_map_t<expr_t, expr_t> expr_matched_;
 };
 
-class pexpr_substitute_t : public ir_mutator_t {
+class pexpr_mutator_t : public ir_mutator_t {
 public:
     using ir_mutator_t::_mutate;
 
+    virtual object_t _mutate(const pexpr_t &obj) { return obj; }
+};
+
+inline object_t pexpr_t::_mutate(ir_mutator_t &mutator) const {
+    return utils::downcast<pexpr_mutator_t *>(&mutator)->_mutate(*this);
+}
+
+class pexpr_substitute_t : public pexpr_mutator_t {
+public:
+    using pexpr_mutator_t::_mutate;
+
     pexpr_substitute_t(const match_context_t *ctx) : ctx_(ctx) {}
 
     object_t _mutate(const pexpr_t &obj) override {
@@ -201,7 +203,7 @@ bool match_binary(
     match_context_t ctx_copy = ctx;
     if (match(ptrn_op.a, expr_op.a, ctx_copy)
             && match(ptrn_op.b, expr_op.b, ctx_copy)) {
-        ctx = ctx_copy;
+        ctx = std::move(ctx_copy);
         return true;
     }
     return false;
@@ -220,7 +222,7 @@ bool match_iif(const expr_t &ptrn, const expr_t &expr, match_context_t &ctx) {
     if (match(ptrn_iif.cond, expr_iif.cond, ctx_copy)
             && match(ptrn_iif.true_expr, expr_iif.true_expr, ctx_copy)
             && match(ptrn_iif.false_expr, expr_iif.false_expr, ctx_copy)) {
-        ctx = ctx_copy;
+        ctx = std::move(ctx_copy);
         return true;
     }
 
@@ -314,7 +316,7 @@ expr_t simplify_rewrite_add(const expr_t &_e) {
     auto _0 = pint_imm_t::_0();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_add);
+    gpu_assert(obj.op_kind == op_kind_t::_add);
 
     auto e = _e;
 
@@ -330,7 +332,7 @@ expr_t simplify_rewrite_sub(const expr_t &_e) {
     auto _0 = pint_imm_t::_0();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_sub);
+    gpu_assert(obj.op_kind == op_kind_t::_sub);
 
     auto e = _e;
 
@@ -347,7 +349,7 @@ expr_t simplify_rewrite_mul(const expr_t &_e) {
     auto _1 = pint_imm_t::_1();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_mul);
+    gpu_assert(obj.op_kind == op_kind_t::_mul);
 
     auto e = _e;
 
@@ -366,7 +368,7 @@ expr_t simplify_rewrite_div(const expr_t &_e) {
     auto _1 = pint_imm_t::_1();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_div);
+    gpu_assert(obj.op_kind == op_kind_t::_div);
 
     auto e = _e;
 
@@ -383,7 +385,7 @@ expr_t simplify_rewrite_mod(const expr_t &_e) {
     auto _1 = pint_imm_t::_1();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_mod);
+    gpu_assert(obj.op_kind == op_kind_t::_mod);
 
     auto e = _e;
 
@@ -397,7 +399,7 @@ expr_t simplify_rewrite_and(const expr_t &_e) {
     auto x = pexpr_t::x();
 
     auto &obj = _e.as<binary_op_t>();
-    ir_assert(obj.op_kind == op_kind_t::_and);
+    gpu_assert(obj.op_kind == op_kind_t::_and);
 
     auto e = _e;
 
@@ -420,6 +422,33 @@ expr_t simplify_rewrite_and(const expr_t &_e) {
     return e;
 }
 
+expr_t simplify_rewrite_or(const expr_t &_e) {
+    auto x = pexpr_t::x();
+
+    auto &obj = _e.as<binary_op_t>();
+    gpu_assert(obj.op_kind == op_kind_t::_or);
+
+    auto e = _e;
+
+    // Boolean rules.
+    if (e.type().is_bool()) {
+        auto _true = (e.type().is_scalar()
+                        ? expr_t(true)
+                        : shuffle_t::make_broadcast(
+                                expr_t(true), e.type().elems()));
+        auto _false = (e.type().is_scalar()
+                        ? expr_t(false)
+                        : shuffle_t::make_broadcast(
+                                expr_t(false), e.type().elems()));
+        REWRITE_BINARY_NO_STATIC(_true | x, _true);
+        REWRITE_BINARY_NO_STATIC(x | _true, _true);
+        REWRITE_BINARY_NO_STATIC(_false | x, x);
+        REWRITE_BINARY_NO_STATIC(x | _false, x);
+    }
+
+    return e;
+}
+
 expr_t simplify_rewrite_iif(const expr_t &_e) {
     auto x = pexpr_t::x();
     auto y = pexpr_t::y();
@@ -468,6 +497,7 @@ public:
             case op_kind_t::_div: return simplify_rewrite_div(e);
             case op_kind_t::_mod: return simplify_rewrite_mod(e);
             case op_kind_t::_and: return simplify_rewrite_and(e);
+            case op_kind_t::_or: return simplify_rewrite_or(e);
             default: return e;
         }
     }
@@ -584,7 +614,7 @@ public:
                     new_op_kind = op_kind_t::_le;
                     div = (sign ? div + 1 : div);
                     break;
-                default: ir_error_not_expected();
+                default: gpu_error_not_expected();
             }
         }
 
@@ -605,7 +635,7 @@ public:
         if (!is_const(a_op.b)) return e;
 
         auto &c0 = a_op.b;
-        ir_assert(to_cpp<int64_t>(c0) > 0) << e;
+        gpu_assert(to_cpp<int64_t>(c0) > 0) << e;
 
         // Comparison against a constant is a continuous function, just check
         // boundary points.
@@ -637,9 +667,95 @@ public:
         return obj;
     }
 
+    object_t _mutate(const binary_op_t &obj) override {
+        auto a = ir_mutator_t::mutate(obj.a);
+        auto b = ir_mutator_t::mutate(obj.b);
+
+        if (obj.type.is_bool()) {
+            if (obj.op_kind == op_kind_t::_or) {
+                if (cset.can_prove(a) || cset.can_prove(b)) return expr_t(true);
+            } else if (obj.op_kind == op_kind_t::_and) {
+                auto not_a = try_not(a);
+                if (not_a && cset.can_prove(not_a)) return expr_t(false);
+                auto not_b = try_not(b);
+                if (not_b && cset.can_prove(not_b)) return expr_t(false);
+            }
+        }
+
+        if (a.is_same(obj.a) && b.is_same(obj.b)) return obj;
+        return const_fold_non_recursive(binary_op_t::make(obj.op_kind, a, b));
+    }
+
+    expr_t try_not(const expr_t &e) {
+        if (e.is<bool_imm_t>()) {
+            return bool_imm_t::make(!to_cpp<bool>(e));
+        } else if (e.is<binary_op_t>()) {
+            auto op_kind = e.as<binary_op_t>().op_kind;
+            auto a = e.as<binary_op_t>().a;
+            auto b = e.as<binary_op_t>().b;
+            switch (op_kind) {
+                case op_kind_t::_lt:
+                    return binary_op_t::make(op_kind_t::_ge, a, b);
+                case op_kind_t::_le:
+                    return binary_op_t::make(op_kind_t::_gt, a, b);
+                case op_kind_t::_gt:
+                    return binary_op_t::make(op_kind_t::_le, a, b);
+                case op_kind_t::_ge:
+                    return binary_op_t::make(op_kind_t::_lt, a, b);
+                case op_kind_t::_ne:
+                    return binary_op_t::make(op_kind_t::_eq, a, b);
+                case op_kind_t::_eq:
+                    return binary_op_t::make(op_kind_t::_ne, a, b);
+                default: return expr_t();
+            }
+        }
+        return expr_t();
+    }
+
     const constraint_set_t &cset;
 };
 
+// N-ary expression: (a[0] op a[1] op ... op a[n - 1]),
+// where <op> is either addition or multiplication.
+class nary_op_t : public expr_iface_t<nary_op_t> {
+public:
+    static expr_t make(op_kind_t op_kind, const std::vector<expr_t> &args) {
+        return expr_t(new nary_op_t(op_kind, args));
+    }
+
+    bool operator==(const nary_op_t &other) const {
+        return (op_kind == other.op_kind)
+                && ir_utils::is_equal(args, other.args);
+    }
+
+    size_t get_hash() const override {
+        return ir_utils::get_hash(op_kind, args);
+    }
+
+    std::string str() const override {
+        ostringstream_t oss;
+        oss << "(";
+        for (size_t i = 0; i < args.size(); i++) {
+            oss << (i != 0 ? " " + to_string(op_kind) + " " : "") << args[i];
+        }
+
+        oss << ")";
+        return oss.str();
+    }
+
+    object_t _mutate(ir_mutator_t &mutator) const override;
+    void _visit(ir_visitor_t &visitor) const override;
+
+    op_kind_t op_kind;
+    std::vector<expr_t> args;
+
+private:
+    nary_op_t(op_kind_t op_kind, const std::vector<expr_t> &args)
+        : expr_iface_t(nary_op_type(op_kind, args))
+        , op_kind(op_kind)
+        , args(args) {}
+};
+
 // Finds all constant operands on an N-ary operation, returns the folded
 // constant in `const_arg` and the remaining operands in `other_args`.
 void split_const_nary_op_arg(op_kind_t op_kind, const std::vector<expr_t> &args,
@@ -669,20 +785,20 @@ void fold_const_nary_op_args(op_kind_t op_kind, const std::vector<expr_t> &args,
     if (c.is_empty()) return;
     if (op_kind == op_kind_t::_mul && is_zero(c)) {
         new_args.clear();
-        new_args.push_back(c);
+        new_args.push_back(std::move(c));
         return;
     }
     if (op_kind == op_kind_t::_mul && is_one(c)) return;
     if (op_kind == op_kind_t::_add && is_zero(c)) return;
-    new_args.push_back(c);
+    new_args.push_back(std::move(c));
 }
 
 expr_t cvt_mul_to_nary_op(const expr_t &a, const expr_t &b) {
     auto *a_nary = a.as_ptr<nary_op_t>();
     auto *b_nary = b.as_ptr<nary_op_t>();
 
-    if (a_nary) ir_assert(a_nary->op_kind == op_kind_t::_mul);
-    if (b_nary) ir_assert(b_nary->op_kind == op_kind_t::_mul);
+    if (a_nary) gpu_assert(a_nary->op_kind == op_kind_t::_mul);
+    if (b_nary) gpu_assert(b_nary->op_kind == op_kind_t::_mul);
 
     auto a_args = cvt_expr_to_nary_op_args(a);
     auto b_args = cvt_expr_to_nary_op_args(b);
@@ -700,6 +816,10 @@ public:
     virtual void _visit(const nary_op_t &obj) { visit(obj.args); }
 };
 
+inline void nary_op_t::_visit(ir_visitor_t &visitor) const {
+    utils::downcast<nary_op_visitor_t *>(&visitor)->_visit(*this);
+}
+
 class nary_op_mutator_t : public ir_mutator_t {
 public:
     using ir_mutator_t::_mutate;
@@ -711,6 +831,10 @@ public:
     }
 };
 
+inline object_t nary_op_t::_mutate(ir_mutator_t &mutator) const {
+    return utils::downcast<nary_op_mutator_t *>(&mutator)->_mutate(*this);
+}
+
 class nary_op_transformer_t : public nary_op_mutator_t {
 public:
     using nary_op_mutator_t::_mutate;
@@ -730,7 +854,7 @@ public:
                     b *= -1;
                 }
                 b = mutate(b);
-                return make_nary_op(nary_op_kind, {a, b});
+                return make_nary_op(nary_op_kind, {std::move(a), std::move(b)});
             }
             default: return nary_op_mutator_t::_mutate(obj);
         }
@@ -747,7 +871,7 @@ public:
             if (nary && nary->op_kind == obj.op_kind) {
                 args.insert(args.end(), nary->args.begin(), nary->args.end());
             } else {
-                args.push_back(new_a);
+                args.emplace_back(new_a);
             }
         }
         return make_nary_op(obj.op_kind, args);
@@ -771,7 +895,7 @@ public:
             if (arg.is<cast_t>()) arg = args[i].as<cast_t>().expr;
             auto *nary = arg.as_ptr<nary_op_t>();
             if (nary && nary->op_kind != op_kind_t::_add) {
-                ir_error_not_expected();
+                gpu_error_not_expected();
             }
             auto i_args = cvt_expr_to_nary_op_args(arg);
             if (arg.type() != args[i].type()) {
@@ -780,7 +904,7 @@ public:
                 }
             }
             if (new_args.empty()) {
-                new_args = i_args;
+                new_args = std::move(i_args);
                 continue;
             }
             std::vector<expr_t> next_args;
@@ -788,7 +912,7 @@ public:
                 for (auto &b : i_args)
                     next_args.push_back(cvt_mul_to_nary_op(a, b));
 
-            new_args = next_args;
+            new_args = std::move(next_args);
         }
         return make_nary_op(op_kind_t::_add, new_args);
     }
@@ -828,6 +952,8 @@ public:
 
     void _visit(const unary_op_t &obj) override { visit_new_scope(obj); }
 
+    void _visit(const ternary_op_t &obj) override { visit_new_scope(obj); }
+
     void _visit(const nary_op_t &obj) override {
         if (parent_nary_) {
             if (!(parent_nary_->op_kind == op_kind_t::_add
@@ -868,10 +994,10 @@ bool is_nary_op_canonical(const expr_t &e) {
 
 class nary_op_back_transformer_t : public nary_op_mutator_t {
 public:
-    object_t _mutate(const nary_op_t &obj) {
+    object_t _mutate(const nary_op_t &obj) override {
         auto new_obj = nary_op_mutator_t::_mutate(obj);
         auto &nary = new_obj.as<nary_op_t>();
-        ir_assert(nary.args.size() > 0) << new_obj;
+        gpu_assert(!nary.args.empty()) << new_obj;
 
         if (nary.args.size() == 1) return nary.args[0];
 
@@ -886,7 +1012,7 @@ public:
                 ret *= nary.args[i];
             return std::move(ret);
         }
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return expr_t();
     }
 };
@@ -894,10 +1020,8 @@ public:
 // Stores factorization of an expression in the canonical (normalized) form:
 //     expr = (f(0), f(1), f(2), ... f(n))
 // f(0), ... f(n-1) are non-constant expressions, f(n) is a constant.
-class factored_expr_t : public expr_impl_t {
+class factored_expr_t : public expr_iface_t<factored_expr_t> {
 public:
-    IR_DECL_EXPR_TYPE_ID(factored_expr_t);
-
     static expr_t make(const expr_t &e) {
         return expr_t(new factored_expr_t(e));
     }
@@ -906,22 +1030,19 @@ public:
         return expr_t(new factored_expr_t(type, factors));
     }
 
-    bool is_equal(const object_impl_t &obj) const override {
-        if (!obj.is<self_type>()) return false;
-        auto &other = obj.as<self_type>();
-
+    bool operator==(const factored_expr_t &other) const {
         if (factors.size() != other.factors.size()) return false;
         if (!factors.back().is_equal(other.factors.back())) return false;
 
-        auto common = intersect(obj);
+        auto common = intersect(other);
         auto &f_common = common.as<factored_expr_t>();
         return f_common.factors.size() == factors.size();
     }
 
     // Constant factor is ignored during comparison.
-    bool is_equal_ignore_const(const object_impl_t &obj) const {
-        if (!obj.is<self_type>()) return false;
-        auto &other = obj.as<self_type>();
+    bool is_equal_ignore_const(const impl_t &obj) const {
+        if (!obj.is<factored_expr_t>()) return false;
+        auto &other = obj.as<factored_expr_t>();
 
         if (factors.size() != other.factors.size()) return false;
 
@@ -933,7 +1054,7 @@ public:
     size_t get_hash() const override { return ir_utils::get_hash(factors); }
 
     std::string str() const override {
-        std::ostringstream oss;
+        ostringstream_t oss;
         oss << "f(";
         for (size_t i = 0; i < factors.size(); i++) {
             oss << (i != 0 ? " x " : "") << factors[i];
@@ -1018,7 +1139,7 @@ public:
         auto f_common = intersect(other);
         auto diff_other = f_other.diff(f_common);
         // Other must be reducible.
-        ir_assert(diff_other.as<factored_expr_t>().is_one()) << diff_other;
+        gpu_assert(diff_other.as<factored_expr_t>().is_one()) << diff_other;
         return diff(f_common);
     }
 
@@ -1042,12 +1163,14 @@ public:
     std::vector<expr_t> factors;
 
 private:
-    factored_expr_t(const expr_t &e) : expr_impl_t(_type_info(), e.type()) {
+    factored_expr_t(const expr_t &e)
+        : expr_iface_t(
+                e.type().with_attr(e.type().attr() & ~type::attr_t::mut)) {
         init_factors(e);
     }
 
     factored_expr_t(const type_t &type, const std::vector<expr_t> &factors)
-        : expr_impl_t(_type_info(), type) {
+        : expr_iface_t(type) {
         init_normalize(factors);
     }
 
@@ -1065,7 +1188,7 @@ private:
             e_const = e_const * abs(e);
         }
         if (sign) e_const = -e_const;
-        factors.push_back(e_const);
+        factors.push_back(std::move(e_const));
     }
 
     void init_factors(const expr_t &e) {
@@ -1118,7 +1241,7 @@ private:
             factors = f_common.merge(rest).as<factored_expr_t>().factors;
             return;
         }
-        ir_error_not_expected();
+        gpu_error_not_expected();
     }
 
     expr_t intersect_impl(const expr_t &other, bool ignore_constants) const {
@@ -1159,7 +1282,7 @@ public:
     using nary_op_mutator_t::_mutate;
 
     object_t _mutate(const binary_op_t &obj) override {
-        if (obj.op_kind != op_kind_t::_div)
+        if (obj.op_kind != op_kind_t::_div || !obj.type.is_scalar())
             return nary_op_mutator_t::_mutate(obj);
 
         expr_t a = mutate(obj.a);
@@ -1193,8 +1316,8 @@ public:
         auto obj = nary_op_mutator_t::_mutate(_obj);
         auto *binary_op = obj.as_ptr<binary_op_t>();
         if (!binary_op) return obj;
-        if (!utils::one_of(
-                    binary_op->op_kind, op_kind_t::_div, op_kind_t::_mod))
+        if (!utils::one_of(binary_op->op_kind, op_kind_t::_div, op_kind_t::_mod)
+                || !_obj.type.is_scalar())
             return obj;
         if (!binary_op->type.is_int()) return obj;
 
@@ -1231,9 +1354,9 @@ public:
 
     expr_t mutate_with_add(const binary_op_t &obj) {
         expr_t ret = reduce_v1(obj);
-        if (!ret.is_empty()) return ret;
+        if (ret) return ret;
         ret = reduce_v2(obj);
-        return (!ret.is_empty()) ? ret : obj;
+        return ret ? std::move(ret) : obj;
     }
 
     // Applies the following rules:
@@ -1290,7 +1413,7 @@ public:
             return mutate(lhs_div) + mutate(rhs_div);
         }
 
-        ir_error_not_expected() << expr;
+        gpu_error_not_expected() << expr;
 
         return expr_t();
     }
@@ -1346,7 +1469,7 @@ public:
 
         // max_gcd is the GCD for some summand so at least one summand must be
         // reducible.
-        ir_assert(!lhs_args.empty());
+        gpu_assert(!lhs_args.empty());
 
         if (rhs_args.empty()) return expr_t();
 
@@ -1369,7 +1492,7 @@ public:
             return lhs_div;
         }
 
-        ir_error_not_expected() << expr;
+        gpu_error_not_expected() << expr;
 
         return expr_t();
     }
@@ -1428,7 +1551,7 @@ public:
         auto args = mutate(obj.args);
         for (auto &a : args) {
             auto *nary = a.as_ptr<nary_op_t>();
-            if (nary) ir_assert(nary->op_kind == op_kind_t::_mul) << a;
+            if (nary) gpu_assert(nary->op_kind == op_kind_t::_mul) << a;
         }
 
         // Fold same factors (find exact match, ignore constants).
@@ -1483,7 +1606,7 @@ public:
                         make_nary_op(op_kind_t::_add, {fi.expr(), fj.expr()}));
                 auto &fi_add_fj = e_fi_add_fj.as<factored_expr_t>();
                 args[i] = make_nary_op(op_kind_t::_mul, fi_add_fj.factors);
-                e_fi = e_fi_add_fj;
+                e_fi = std::move(e_fi_add_fj);
                 args[j] = to_expr(0, args[j].type());
             }
         }
@@ -1535,6 +1658,7 @@ expr_t simplify_with_nary(const expr_t &_e, const constraint_set_t &cset) {
     return e;
 }
 
+// NOLINTNEXTLINE(readability-identifier-naming)
 class _64_bit_add_optimizer_t : public nary_op_mutator_t {
 public:
     object_t _mutate(const nary_op_t &obj) override {
@@ -1562,31 +1686,32 @@ public:
     }
 
     object_t _mutate(const if_t &obj) override {
-        auto cond = simplify(obj.cond);
+        auto cond = simplify(obj.cond, cset_);
 
         if (all_of(cond, expr_t(true))) return mutate(obj.body);
         if (all_of(cond, expr_t(false))) return mutate(obj.else_body);
 
         auto body = obj.body;
-        if (!body.is_empty()) {
+        if (body) {
             auto cset_old = cset_;
             cset_.add_constraint(cond);
             body = ir_mutator_t::mutate(body);
-            cset_ = cset_old;
+            cset_ = std::move(cset_old);
         }
 
         auto else_body = obj.else_body;
-        if (!else_body.is_empty()) {
+        if (else_body) {
             auto cset_old = cset_;
             cset_.add_constraint(flip_condition(cond));
             else_body = ir_mutator_t::mutate(else_body);
-            cset_ = cset_old;
+            cset_ = std::move(cset_old);
         }
 
         return if_t::make(cond, body, else_body);
     }
 
     object_t _mutate(const let_t &obj) override {
+
         // Attempt to inline
         expr_t value;
         if (cset_.is_single_value(obj.var, value)) {
@@ -1595,11 +1720,13 @@ public:
             return mutate(body);
         }
 
+        if (obj.var.type().is_mutable()) return ir_mutator_t::_mutate(obj);
+
         // External variable.
         if (obj.value.is_empty()) return ir_mutator_t::_mutate(obj);
 
         // Substitute constants.
-        value = simplify(obj.value);
+        value = simplify(obj.value, cset_);
         if (is_const(value)) {
             // Constants are not necessarily the same type as the assigned
             // variable
@@ -1615,7 +1742,7 @@ public:
         auto cset_old = cset_;
         cset_.add_constraint(obj.var == value);
         auto body = mutate(obj.body);
-        cset_ = cset_old;
+        cset_ = std::move(cset_old);
 
         return let_t::make(obj.var, value, body);
     }
@@ -1627,13 +1754,13 @@ public:
         if (is_one(new_bound) && is_zero(new_init)) {
             auto body = substitute(obj.body, obj.var, expr_t(0));
             body = mutate(body);
-            new_obj = body;
+            new_obj = std::move(body);
         } else {
             auto cset_old = cset_;
             cset_.add_constraint(obj.var >= obj.init);
             cset_.add_constraint(obj.var < obj.bound);
             new_obj = ir_mutator_t::_mutate(obj);
-            cset_ = cset_old;
+            cset_ = std::move(cset_old);
         }
 
         return new_obj;
@@ -1655,8 +1782,17 @@ public:
         return stmt_t();
     }
 
+    object_t _mutate(const while_t &obj) override {
+        auto cond = simplify(obj.cond, cset_);
+        if (is_const(cond) && !to_cpp<bool>(cond)) return stmt_t();
+        auto body = mutate(obj.body);
+        if (body.is_empty()) return stmt_t();
+        if (obj.cond.is_same(cond) && obj.body.is_same(body)) return obj;
+        return while_t::make(cond, body);
+    }
+
 private:
-    static op_kind_t flip_cmp_op(op_kind_t op_kind) {
+    static op_kind_t flip_op(op_kind_t op_kind) {
         switch (op_kind) {
             case op_kind_t::_eq: return op_kind_t::_ne;
             case op_kind_t::_ge: return op_kind_t::_lt;
@@ -1664,20 +1800,28 @@ private:
             case op_kind_t::_le: return op_kind_t::_gt;
             case op_kind_t::_lt: return op_kind_t::_ge;
             case op_kind_t::_ne: return op_kind_t::_eq;
-            default: ir_error_not_expected();
+            case op_kind_t::_and: return op_kind_t::_or;
+            case op_kind_t::_or: return op_kind_t::_and;
+            default: gpu_error_not_expected();
         }
         return op_kind_t::undef;
     }
 
     static expr_t flip_condition(const expr_t &cond) {
-        ir_assert(cond.type().is_bool());
+        gpu_assert(cond.type().is_bool());
 
         auto *binary_op = cond.as_ptr<binary_op_t>();
         if (binary_op) {
             auto &a = binary_op->a;
             auto &b = binary_op->b;
             auto op_kind = binary_op->op_kind;
-            return binary_op_t::make(flip_cmp_op(op_kind), a, b);
+            if (is_cmp_op(op_kind))
+                return binary_op_t::make(flip_op(op_kind), a, b);
+            if (utils::one_of(op_kind, op_kind_t::_and, op_kind_t::_or)) {
+                auto a_neg = flip_condition(a);
+                auto b_neg = flip_condition(b);
+                return binary_op_t::make(flip_op(op_kind), a_neg, b_neg);
+            }
         }
 
         auto *shuffle = cond.as_ptr<shuffle_t>();
@@ -1686,7 +1830,7 @@ private:
                     flip_condition(shuffle->vec[0]), shuffle->elems());
         }
 
-        ir_error_not_expected();
+        gpu_error_not_expected();
         return expr_t();
     }
 
@@ -1717,7 +1861,7 @@ stmt_t simplify_stmt(const stmt_t &s, const constraint_set_t &cset) {
 }
 
 int64_t get_max_const_factor(const expr_t &_e, const constraint_set_t &cset) {
-    ir_assert(_e.type().is_int());
+    gpu_assert(_e.type().is_int());
     auto e = _e;
     // Some complex expressions need more than one simplify() call.
     int max_tries = 3;
@@ -1740,11 +1884,11 @@ struct op_traits_t {};
         static auto compute(T a, T b) -> decltype(a op b) { \
             return a op b; \
         } \
-        template <op_kind_t dummy_op = name, \
-                typename \
-                = typename std::enable_if<dummy_op == op_kind_t::_and>::type> \
+        template <op_kind_t dummy_op = (name), \
+                typename = typename std::enable_if<dummy_op == op_kind_t::_and \
+                        || dummy_op == op_kind_t::_or>::type> \
         static bool compute(bool a, bool b) { \
-            return a op b; \
+            return static_cast<bool>(a op b); \
         } \
     };
 
@@ -1760,6 +1904,10 @@ DECL_OP_TRAITS(op_kind_t::_lt, <)
 DECL_OP_TRAITS(op_kind_t::_le, <=)
 
 DECL_OP_TRAITS(op_kind_t::_and, &)
+DECL_OP_TRAITS(op_kind_t::_or, |)
+DECL_OP_TRAITS(op_kind_t::_xor, ^)
+DECL_OP_TRAITS(op_kind_t::_shl, <<)
+DECL_OP_TRAITS(op_kind_t::_shr, >>)
 
 template <>
 struct op_traits_t<op_kind_t::_min> {
@@ -1785,21 +1933,31 @@ struct op_traits_t<op_kind_t::_div> {
     template <typename T,
             typename = typename std::enable_if<is_int_t<T>::value>::type>
     static auto compute(T a, T b) -> decltype(a / b) {
-        ir_assert(b > 0);
-        int r = a % b;
-        int d = a / b;
+        gpu_assert(b > 0);
+        T r = a % b;
+        T d = a / b;
         if (r < 0) d--;
         return d;
     }
 };
 
+template <>
+struct op_traits_t<op_kind_t::_div_up> {
+    template <typename T,
+            typename = typename std::enable_if<is_int_t<T>::value>::type>
+    static auto compute(T a, T b) -> decltype(a / b) {
+        return op_traits_t<op_kind_t::_div>::compute(
+                static_cast<T>(a + b - 1), b);
+    }
+};
+
 template <>
 struct op_traits_t<op_kind_t::_mod> {
     template <typename T,
             typename = typename std::enable_if<is_int_t<T>::value>::type>
     static auto compute(T a, T b) -> decltype(a % b) {
-        ir_assert(b > 0);
-        int r = a % b;
+        gpu_assert(b > 0);
+        T r = a % b;
         if (r < 0) r += b;
         return r;
     }
@@ -1847,6 +2005,7 @@ public:
             CASE(op_kind_t::_sub)
             CASE(op_kind_t::_mul)
             CASE(op_kind_t::_div)
+            CASE(op_kind_t::_div_up)
             CASE(op_kind_t::_mod)
 
             CASE(op_kind_t::_eq)
@@ -1857,10 +2016,14 @@ public:
             CASE(op_kind_t::_le)
 
             CASE(op_kind_t::_and)
+            CASE(op_kind_t::_or)
+            CASE(op_kind_t::_xor)
+            CASE(op_kind_t::_shl)
+            CASE(op_kind_t::_shr)
             CASE(op_kind_t::_min)
             CASE(op_kind_t::_max)
 
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
 
 #undef CASE
         }
@@ -1892,10 +2055,11 @@ bool is_const_or_shuffle_const(const expr_t &e) {
 }
 
 expr_t const_fold_unary(op_kind_t op_kind, const expr_t &a) {
-    ir_assert(op_kind == op_kind_t::_minus);
+    gpu_assert(op_kind == op_kind_t::_minus);
     if (!a.type().is_scalar()) {
         int elems = a.type().elems();
         std::vector<expr_t> ret;
+        ret.reserve(elems);
         for (int i = 0; i < elems; i++) {
             ret.push_back(const_fold_unary(op_kind, a[i]));
         }
@@ -1914,7 +2078,7 @@ expr_t const_fold_unary(op_kind_t op_kind, const expr_t &a) {
 
 #undef CASE
 
-    ir_error_not_expected() << "Cannot handle type: " << a;
+    gpu_error_not_expected() << "Cannot handle type: " << a;
     return expr_t();
 }
 
@@ -1922,10 +2086,11 @@ expr_t const_fold_binary(const type_t &compute_type, op_kind_t op_kind,
         const expr_t &a, const expr_t &b) {
     if (!compute_type.is_scalar()) {
         int elems = compute_type.elems();
-        auto scalar_type = compute_type.scalar();
+        auto base_type = compute_type.base();
         std::vector<expr_t> ret;
+        ret.reserve(elems);
         for (int i = 0; i < elems; i++) {
-            ret.push_back(const_fold_binary(scalar_type, op_kind, a[i], b[i]));
+            ret.push_back(const_fold_binary(base_type, op_kind, a[i], b[i]));
         }
         return shuffle_t::make(ret);
     }
@@ -1933,7 +2098,7 @@ expr_t const_fold_binary(const type_t &compute_type, op_kind_t op_kind,
     if (compute_type.is_unsigned()) {
         auto a_s64 = to_cpp<int64_t>(a);
         auto b_s64 = to_cpp<int64_t>(b);
-        ir_assert(a_s64 >= 0 && b_s64 >= 0)
+        gpu_assert(a_s64 >= 0 && b_s64 >= 0)
                 << "Overflow detected: fix data types.";
         MAYBE_UNUSED(a_s64);
         MAYBE_UNUSED(b_s64);
@@ -1957,14 +2122,14 @@ expr_t const_fold_binary(const type_t &compute_type, op_kind_t op_kind,
 
 #undef CASE
 
-    ir_error_not_expected() << "Unknown type.";
+    gpu_error_not_expected() << "Unknown type.";
     return expr_t();
 }
 
 object_t simplify(const object_t &obj, const constraint_set_t &cset) {
     if (obj.is_expr()) return simplify_expr(obj, cset);
     if (obj.is_stmt()) return simplify_stmt(obj, cset);
-    ir_assert(obj.is_empty());
+    gpu_assert(obj.is_empty());
     return object_t();
 }
 
@@ -2075,7 +2240,7 @@ expr_t simplify_cmp_reduce_lhs_rhs(const expr_t &e) {
                 new_op_kind = op_kind_t::_le;
                 div = (sign ? div + 1 : div);
                 break;
-            default: ir_error_not_expected();
+            default: gpu_error_not_expected();
         }
     }
 
@@ -2095,12 +2260,12 @@ bool const_to_const_binary(const expr_t &e, op_kind_t op_kind,
     expr_t b1 = to_expr(1, b_type);
     expr_t a_eq = to_expr(0, a_type);
     expr_t b_eq = to_expr(0, b_type);
-    if (!a.is_empty()) {
+    if (a) {
         a0 = a1 = a;
         b0 = a - 1;
         b1 = a + 1;
         a_eq = b_eq = a;
-    } else if (!b.is_empty()) {
+    } else if (b) {
         b0 = b1 = b;
         a0 = b - 1;
         a1 = b + 1;
@@ -2108,6 +2273,7 @@ bool const_to_const_binary(const expr_t &e, op_kind_t op_kind,
     }
     switch (op_kind) {
         case op_kind_t::_and: a = b = e; return true;
+        case op_kind_t::_or: a = b = e; return true;
         case op_kind_t::_le:
         case op_kind_t::_lt:
             a = (is_true ? a0 : a1);
@@ -2134,7 +2300,7 @@ expr_t simplify_propagate_shuffle(const expr_t &e) {
     if (!e.type().is_bool()) return e;
 
     auto *shuffle = e.as_ptr<shuffle_t>();
-    if (!shuffle) return e;
+    if (!shuffle || shuffle->vec[0].type().is_simd()) return e;
 
     // Handle binary operation.
     {
@@ -2189,8 +2355,8 @@ expr_t simplify_propagate_shuffle(const expr_t &e) {
                     ok = false;
                     break;
                 }
-                a.push_back(op_a);
-                b.push_back(op_b);
+                a.push_back(std::move(op_a));
+                b.push_back(std::move(op_b));
             } else if (op_kind == op_kind_t::_and) {
                 // Replace with expression true <op_kind> elem to allow matching
                 // this op against future binary operation.
@@ -2268,7 +2434,7 @@ expr_t nary_op_canonicalize(const expr_t &_e) {
     e = nary_op_transformer_t().mutate(e);
     e = mul_nary_op_expander_t().mutate(e);
 
-    ir_assert(is_nary_op_canonical(e)) << e;
+    gpu_assert(is_nary_op_canonical(e)) << e;
     MAYBE_UNUSED(is_nary_op_canonical(e));
 
     return e;
@@ -2278,7 +2444,7 @@ expr_t make_nary_op(op_kind_t op_kind, const std::vector<expr_t> &args) {
     if (args.empty()) {
         if (op_kind == op_kind_t::_add) return 0;
         if (op_kind == op_kind_t::_mul) return 1;
-        ir_error_not_expected() << to_string(op_kind);
+        gpu_error_not_expected() << to_string(op_kind);
     }
     if (args.size() == 1) return args[0];
 
