@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@
 #define GPU_INTEL_UTILS_HPP
 
 #include <iostream>
-#include <sstream>
 
 #include "common/utils.hpp"
 #include "gpu/intel/compute/device_info.hpp"
@@ -33,6 +32,11 @@ namespace impl {
 namespace gpu {
 namespace intel {
 
+enum { OCL_BUFFER_ALIGNMENT = 128 };
+
+const char *get_kernel_header(const std::string &name);
+const char *get_kernel_source(const char *name);
+
 #define MAX_NDIMS 6
 #define MAX_POST_OPS_SUPPORTED 32
 
@@ -49,13 +53,6 @@ inline char as_tag(dim_idx_t idx, bool is_outer = false) {
 
 namespace gpu_utils {
 
-// Replacement implementation of std::enable_if_t from C++14, included here for
-// interoperability with C++11
-template <bool B, class T = void>
-using enable_if_t = typename std::enable_if<B, T>::type;
-template <typename T>
-using is_vector = std::is_same<T, typename std::vector<typename T::value_type>>;
-
 class error_stream_t {
 public:
     error_stream_t(const char *file, int line, const char *assert_msg)
@@ -99,7 +96,7 @@ private:
 
         const char *file;
         int line;
-        std::ostringstream out;
+        ostringstream_t out;
     };
 
     data_t *data_;
@@ -120,7 +117,7 @@ private:
 #define gpu_error_not_expected() gpu_assert(false) << "Not expected. "
 #define gpu_except_not_implemented(msg) \
     throw std::runtime_error(std::string(msg) + std::string(" at ") \
-            + std::string(__FILE_NAME__) + std::string(":") \
+            + std::string(__FILENAME__) + std::string(":") \
             + std::to_string(__LINE__))
 
 template <typename out_type, typename in_type,
@@ -224,7 +221,7 @@ inline std::vector<std::string> split(const std::string &s,
 
 inline std::string join(
         const std::string &delimiter, const std::vector<std::string> &parts) {
-    std::ostringstream oss;
+    ostringstream_t oss;
     bool is_first = true;
     for (auto &p : parts) {
         if (!is_first) oss << delimiter;
