@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,13 +34,10 @@ namespace graph {
 namespace dnnl_impl {
 
 struct dummy_kernel_t : public kernel_base_t {
-private:
-    std::shared_ptr<subgraph_t> subgraph_;
-
 public:
-    dummy_kernel_t() {}
+    dummy_kernel_t() = default;
 
-    ~dummy_kernel_t() override {}
+    ~dummy_kernel_t() override = default;
 
     status_t compile_impl(const dnnl_partition_impl_t *part,
             const engine_t *g_engine,
@@ -67,6 +64,7 @@ public:
 #endif
 
     DEF_KERNEL_METHOD_STR(dummy_kernel_t)
+    DNNL_DISALLOW_COPY_AND_ASSIGN(dummy_kernel_t)
 };
 
 kernel_ptr dummy_kernel_creator();
