@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2021-2024 Intel Corporation
+ * Copyright 2021-2025 Intel Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,7 +44,6 @@ namespace graph {
 namespace dnnl_impl {
 using op_t = op_t;
 using op_ptr = std::shared_ptr<op_t>;
-using value_ptr = std::shared_ptr<value_t>;
 using ltw = logical_tensor_wrapper_t;
 
 status_t set_given_inputs_outputs(std::shared_ptr<subgraph_t> &sg,
@@ -65,9 +64,8 @@ status_t set_given_inputs_outputs(std::shared_ptr<subgraph_t> &sg,
 
             // partition in/outs should not have default id. There must be some
             // errors in previous graph transformation stage
-            if (edge_id == std::numeric_limits<size_t>::max())
-                return status::invalid_graph;
-
+            VCHECK_UTILS(edge_id != std::numeric_limits<size_t>::max(),
+                    status::invalid_graph, "Invalid edge_id %zu", edge_id);
             bool found = false;
             for (const auto &given : givens) {
                 if (edge_id == given.id) {
@@ -87,7 +85,9 @@ status_t set_given_inputs_outputs(std::shared_ptr<subgraph_t> &sg,
                                 }
                             }
                         }
-                        if (!valid) return status::invalid_arguments;
+                        VCHECK_UTILS(valid, status::invalid_arguments,
+                                "Invalid given logical tensor for given.id %zu",
+                                given.id);
                     }
 
                     edge->set_logical_tensor(given);
@@ -96,17 +96,15 @@ status_t set_given_inputs_outputs(std::shared_ptr<subgraph_t> &sg,
                 }
             }
 
-            if (!found) return status::invalid_arguments;
+            VCHECK_UTILS(found, status::invalid_arguments,
+                    "Can't find given logical tensor for edge_id %zu", edge_id);
         }
         return status::success;
     };
 
-    status_t ret;
-    ret = func(graph_in_vals, inputs, true, true);
-    if (ret != status::success) return ret;
-
-    ret = func(graph_out_vals, outputs, true, false);
-    return ret;
+    CHECK(func(graph_in_vals, inputs, true, true));
+    CHECK(func(graph_out_vals, outputs, true, false));
+    return status::success;
 }
 
 status_t set_given_inputs_outputs(std::vector<op_ptr> &subgraph,
@@ -188,7 +186,8 @@ std::vector<value_t *> get_constant_block_output_values(
         return status::success;
     };
     status_t status = topo_order_visit(sg->get_output_ops(), func);
-    if (status != status::success) return {};
+    VCHECK_UTILS(status == status::success, {},
+            "Failed to get constant block output values");
     return ret;
 }
 
@@ -199,14 +198,11 @@ status_t infer_shape(std::shared_ptr<subgraph_t> &sg) {
     // internal dw_type attr to record the post-dw-conv info used to infer
     // correct shape. Here the dw_type attr is a temporary attr only used during
     // shape infer, and will be removed from the op before existing shape infer.
-    const auto &mgr = sg->fusion_info_mgr_;
     std::vector<op_ptr> conv_fused_post_s2_dw_conv;
     for (auto &op : sg->get_ops()) {
         fusion_info_t fusion_info;
-        if (op->has_attr(op_attr::fusion_info_key)
-                && op->get_attr<int64_t>(op_attr::fusion_info_key) != -1) {
-            int64_t key = op->get_attr<int64_t>(op_attr::fusion_info_key);
-            fusion_info = mgr.get_info(key);
+        if (op->has_attr(op_attr::fusion_info)) {
+            fusion_info = op->get_attr<fusion_info_t>(op_attr::fusion_info);
         }
 
         if (fusion_info.has_post_dw_conv()) {
@@ -221,8 +217,7 @@ status_t infer_shape(std::shared_ptr<subgraph_t> &sg) {
         }
     }
 
-    auto ret = sg->infer_shape();
-    if (ret != status::success) return ret;
+    CHECK(sg->infer_shape());
 
     // Fill the inferred shape and strides to subgraph's outputs
     for (size_t i = 0; i < sg->outs_.size(); i++) {
@@ -239,7 +234,7 @@ status_t infer_shape(std::shared_ptr<subgraph_t> &sg) {
         op->remove_attr(op_attr::dw_type);
     }
 
-    return ret;
+    return status::success;
 }
 
 const std::map<op_kind_t, dnnl::algorithm> &get_binary_alg_map() {
@@ -250,7 +245,9 @@ const std::map<op_kind_t, dnnl::algorithm> &get_binary_alg_map() {
                     {graph::op_kind::Minimum, dnnl::algorithm::binary_min},
                     {graph::op_kind::Maximum, dnnl::algorithm::binary_max},
                     {graph::op_kind::Subtract, dnnl::algorithm::binary_sub},
-                    {graph::op_kind::BiasAdd, dnnl::algorithm::binary_add}};
+                    {graph::op_kind::BiasAdd, dnnl::algorithm::binary_add},
+                    {graph::op_kind::GreaterEqual, dnnl::algorithm::binary_ge},
+                    {graph::op_kind::Select, dnnl::algorithm::binary_select}};
     return binary_alg_map;
 }
 
@@ -272,8 +269,11 @@ bool binary_doable(
 static bool post_binary_fusible_impl(const op_t *base_op,
         const std::vector<dim_t> &fused_shape,
         const std::vector<dim_t> &other_shape, engine_kind_t ekind) {
-    assertm(fused_shape.size() == other_shape.size(),
-            "must have same ndims, pls run binary_canonicalization pass first");
+    VCHECK_UTILS(fused_shape.size() == other_shape.size(), false,
+            "binary fusible ops must have same ndims, "
+            "fused_shape size is %zu, other_shape size is %zu."
+            "pls run binary_canonicalization pass first",
+            fused_shape.size(), other_shape.size());
     // full tensor and per tensor broadcasted
     if (fused_shape == other_shape
             || std::all_of(other_shape.begin(), other_shape.end(),
@@ -376,6 +376,16 @@ std::pair<bool, std::pair<size_t, int64_t>> shuffle_fusible(
 
 bool post_binary_fusible(
         const op_t *base_op, const op_t *bin_op, graph::engine_kind_t ekind) {
+
+// conv + binary post-op fusion is unsupported on NVIDIA GPU
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE \
+        && DNNL_GPU_VENDOR == DNNL_VENDOR_NVIDIA
+    if ((base_op->get_kind() == op_kind::dnnl_convolution)
+            && ekind == dnnl_gpu) {
+        return false;
+    }
+#endif
+
     auto fused_out = base_op->get_output_values()[0];
     auto consumers = fused_out->get_consumers();
     if (consumers.size() != 1) return false;
@@ -402,10 +412,42 @@ bool post_binary_fusible(
         if (ltw(fused_in).data_type() != ltw(bin_out).data_type()) return false;
     }
 
+    // Special check: subtract and divide binary post-op can only be fused
+    // when base_op is their first input's producer
+    if (static_cast<dnnl::algorithm>(
+                bin_op->get_attr<int64_t>(op_attr::alg_kind))
+                    == dnnl::algorithm::binary_sub
+            || static_cast<dnnl::algorithm>(
+                       bin_op->get_attr<int64_t>(op_attr::alg_kind))
+                    == dnnl::algorithm::binary_div) {
+        if (fused_in_off != 0) return false;
+    }
+
     return post_binary_fusible_impl(
             base_op, ltw(fused_in).vdims(), ltw(other_in).vdims(), ekind);
 }
 
+bool post_eltwise_fusible(
+        const op_t *base_op, const op_t *elt_op, graph::engine_kind_t ekind) {
+// binary + sqrt post-op fusion is unsupported on NVIDIA GPU
+#if DNNL_GPU_RUNTIME != DNNL_RUNTIME_NONE \
+        && DNNL_GPU_VENDOR == DNNL_VENDOR_NVIDIA
+    if (base_op->get_kind() == op_kind::dnnl_binary
+            && static_cast<dnnl::algorithm>(
+                       elt_op->get_attr<int64_t>(op_attr::alg_kind))
+                    == dnnl::algorithm::eltwise_sqrt
+            && ekind == dnnl_gpu) {
+        return false;
+    }
+    return true;
+#else
+    UNUSED(base_op);
+    UNUSED(elt_op);
+    UNUSED(ekind);
+    return true;
+#endif
+}
+
 bool post_depthwise_conv_fusible(
         const op_t *base_conv_op, const op_t *post_conv_op) {
     using spatial_dims_t = std::vector<int64_t>;
@@ -575,25 +617,21 @@ bool is_typecast(const op_t *op) {
     return is_typecast;
 }
 
-bool with_runtime_zps(const op_ptr &op, const fusion_info_mgr_t &mgr,
-        bool is_input, size_t indice) {
-    if (op->has_attr(op_attr::fusion_info_key)
-            && op->get_attr<int64_t>(op_attr::fusion_info_key) != -1) {
-        int64_t key = op->get_attr<int64_t>(op_attr::fusion_info_key);
-        const fusion_info_t &fusion_info = mgr.get_info(key);
-        return fusion_info.with_runtime_zero_points(is_input, indice);
+bool with_runtime_zps(const op_ptr &op, bool is_input, size_t index) {
+    if (op->has_attr(op_attr::fusion_info)) {
+        const fusion_info_t &fusion_info
+                = op->get_attr<fusion_info_t>(op_attr::fusion_info);
+        return fusion_info.with_runtime_zero_points(is_input, index);
     } else {
         return false;
     }
 }
 
-bool with_runtime_scales(const op_ptr &op, const fusion_info_mgr_t &mgr,
-        bool is_input, size_t indice) {
-    if (op->has_attr(op_attr::fusion_info_key)
-            && op->get_attr<int64_t>(op_attr::fusion_info_key) != -1) {
-        int64_t key = op->get_attr<int64_t>(op_attr::fusion_info_key);
-        const fusion_info_t &fusion_info = mgr.get_info(key);
-        return fusion_info.with_runtime_scales(is_input, indice);
+bool with_runtime_scales(const op_ptr &op, bool is_input, size_t index) {
+    if (op->has_attr(op_attr::fusion_info)) {
+        const fusion_info_t &fusion_info
+                = op->get_attr<fusion_info_t>(op_attr::fusion_info);
+        return fusion_info.with_runtime_scales(is_input, index);
     } else {
         return false;
     }
@@ -621,10 +659,10 @@ bool is_layout_reorder(const op_t *op) {
 }
 
 std::shared_ptr<op_t> clone_mul_scales(const std::shared_ptr<op_t> &scale_op) {
-    assertm(scale_op->num_inputs() <= 1,
-            "scale_op should have only one input value.");
-    assertm(!scale_op->has_attr(op_attr::with_runtime_scales),
-            "scale_op should be static");
+    VCHECK_UTILS(scale_op->num_inputs() <= 1
+                    && !scale_op->has_attr(op_attr::with_runtime_scales),
+            nullptr,
+            "scale_op should be static and have only one input value.");
     auto new_op = std::make_shared<op_t>(op_kind::dnnl_mul_scales);
     new_op->set_attr<std::vector<float>>(op_attr::scales,
             scale_op->get_attr<std::vector<float>>(op_attr::scales));
@@ -636,16 +674,31 @@ std::shared_ptr<op_t> clone_mul_scales(const std::shared_ptr<op_t> &scale_op) {
 }
 
 bool inverse_mul_scales(std::shared_ptr<op_t> &scale_op) {
-    assertm(scale_op->num_inputs() <= 1,
-            "scale_op should have only one input value.");
-    assertm(!scale_op->has_attr(op_attr::with_runtime_scales),
-            "scale_op should be static");
-    auto scales = scale_op->get_attr<std::vector<float>>(op_attr::scales);
-    scales = dnnl_impl::utils::fmap(scales, [](float s) { return 1.f / s; });
-    scale_op->set_attr(op_attr::scales, scales);
+    VCHECK_UTILS(scale_op->num_inputs() <= 1
+                    && !scale_op->has_attr(op_attr::with_runtime_scales),
+            false, "scale_op should be static and have only one input value.");
+    const auto scales = scale_op->get_attr<std::vector<float>>(op_attr::scales);
+    const auto scales_inv
+            = dnnl_impl::utils::fmap(scales, [](float s) { return 1.f / s; });
+    scale_op->set_attr(op_attr::scales, scales_inv);
     return true;
 }
 
+bool need_broadcast_for_inputs(
+        const std::shared_ptr<op_t> &op, size_t index1, size_t index2) {
+    auto in_vals = op->get_input_values();
+
+    const dims input1_dims
+            = logical_tensor_wrapper_t(in_vals[index1]->get_logical_tensor())
+                      .vdims();
+    const dims input2_dims
+            = logical_tensor_wrapper_t(in_vals[index2]->get_logical_tensor())
+                      .vdims();
+
+    if (input1_dims != input2_dims) { return true; }
+
+    return false;
+}
 } // namespace dnnl_impl
 } // namespace graph
 } // namespace impl
