@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -26,6 +26,7 @@
 #include "oneapi/dnnl/dnnl.h"
 
 #include "utils/fill.hpp"
+#include "utils/memory.hpp"
 #include "utils/parallel.hpp"
 
 #include "dnnl_common.hpp"
@@ -37,6 +38,7 @@ namespace bnorm {
 
 int fill_mean(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
         dnn_mem_t &mem_dt) {
+    if (fill_from_file(DNNL_ARG_MEAN, mem_dt, mem_fp)) return OK;
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         // Mean must be computed unless it is passed by user directly.
@@ -44,12 +46,16 @@ int fill_mean(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
 
         return fill_random_real(mem_dt, mem_fp, nullptr);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, [&](int64_t c) {
         float val = 0.f;
         if (cfg.check_alg_ != ALG_0 || (prb->flags & GLOB_STATS))
             val = 0.25f * (1 << (c % 7));
-        mem_fp.set_elem(c, val);
+        mem_fp.set_f32_elem(c, val);
     });
 
     if (mem_dt && prb->use_stats()) SAFE(mem_dt.reorder(mem_fp), WARN);
@@ -59,13 +65,18 @@ int fill_mean(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
 
 int fill_src(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
         dnn_mem_t &mem_dt, const dnn_mem_t &ref_mean, res_t *res) {
+    if (fill_from_file(DNNL_ARG_SRC, mem_dt, mem_fp)) return OK;
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         return fill_random_real(mem_dt, mem_fp, res);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, res, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, [&](int64_t c) {
-        const float m = ref_mean.get_elem(c);
+        const float m = ref_mean.get_f32_elem(c);
         for (int64_t mb = 0; mb < prb->mb; ++mb) {
             // l[0] must be even
             int64_t l_base = mb * prb->id * prb->ih * prb->iw + c * 239 * 2;
@@ -80,7 +91,7 @@ int fill_src(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                 // Shortcut for zero values.
                 if (cfg.check_alg_ == ALG_0
                         && !flip_coin(l / 2 * 257ULL, cfg.density_)) {
-                    mem_fp.set_elem(off + sp, 0);
+                    mem_fp.set_f32_elem(off + sp, 0);
                     continue;
                 }
 
@@ -96,7 +107,7 @@ int fill_src(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                                 == cfg.L_ - 1)) {
                     val = m;
                 }
-                mem_fp.set_elem(
+                mem_fp.set_f32_elem(
                         off + sp, round_to_nearest_representable(prb->dt, val));
             }
         }
@@ -110,6 +121,7 @@ int fill_src(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
 int fill_variance(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
         dnn_mem_t &mem_dt, const dnn_mem_t &ref_src,
         const dnn_mem_t &ref_mean) {
+    if (fill_from_file(DNNL_ARG_VARIANCE, mem_dt, mem_fp)) return OK;
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         // Variance must be computed unless it is passed by user directly.
@@ -120,13 +132,17 @@ int fill_variance(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                 attr_t::post_ops_t::kind_t::ADD, "variance");
         return fill_random_real(mem_dt, mem_fp, nullptr, fill_cfg);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, [&](int64_t c) {
         float val = 0.f;
         if (prb->flags & GLOB_STATS) {
             val = ((c % 7) << 1);
         } else {
-            const float m = ref_mean.get_elem(c);
+            const float m = ref_mean.get_f32_elem(c);
             for (int64_t mb = 0; mb < prb->mb; ++mb) {
                 int64_t off = data_off(prb, mb, c, 0, 0, 0);
 
@@ -134,13 +150,13 @@ int fill_variance(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                 for_(int64_t h = 0; h < prb->ih; ++h)
                 for (int64_t w = 0; w < prb->iw; ++w) {
                     const int64_t sp = d * prb->ih * prb->iw + h * prb->iw + w;
-                    const float s = ref_src.get_elem(sp + off);
+                    const float s = ref_src.get_f32_elem(sp + off);
                     val += (s - m) * (s - m);
                 }
             }
             val /= cfg.L_;
         }
-        mem_fp.set_elem(c, val);
+        mem_fp.set_f32_elem(c, val);
     });
 
     if (mem_dt && prb->use_stats()) SAFE(mem_dt.reorder(mem_fp), WARN);
@@ -153,11 +169,16 @@ int fill_src_add(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
         const dnn_mem_t &ref_mean) {
     const bool fill_src_add = prb->fuse_add_relu();
     if (!fill_src_add) return OK;
+    if (fill_from_file(DNNL_ARG_SRC_1, mem_dt, mem_fp)) return OK;
 
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         return fill_random_real(mem_dt, mem_fp, nullptr);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, prb->mb, prb->id, prb->ih, prb->iw,
             [&](int64_t c, int64_t mb, int64_t d, int64_t h, int64_t w) {
@@ -166,11 +187,11 @@ int fill_src_add(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                 const int64_t sp = d * prb->ih * prb->iw + h * prb->iw + w;
                 const int64_t offset = data_off(prb, mb, c, 0, 0, 0) + sp;
                 const int64_t l = l_base + sp;
-                const float s = ref_src.get_elem(offset);
-                const float m = ref_mean.get_elem(c);
+                const float s = ref_src.get_f32_elem(offset);
+                const float m = ref_mean.get_f32_elem(c);
 
                 if (!(prb->flags & GLOB_STATS) && s == 0) {
-                    mem_fp.set_elem(offset, 1.f);
+                    mem_fp.set_f32_elem(offset, 1.f);
                     return;
                 }
 
@@ -188,7 +209,7 @@ int fill_src_add(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
                     const int64_t sign_val = s < m ? -1 : 1;
                     val = mod2_val * sign_val;
                 }
-                mem_fp.set_elem(
+                mem_fp.set_f32_elem(
                         offset, round_to_nearest_representable(prb->dt, val));
             });
 
@@ -200,16 +221,21 @@ int fill_src_add(const prb_t *prb, const cfg_t &cfg, dnn_mem_t &mem_fp,
 int fill_scale(const prb_t *prb, dnn_mem_t &mem_fp, dnn_mem_t &mem_dt) {
     const bool use_sc = prb->use_sc();
     if (!use_sc) return OK;
+    if (fill_from_file(DNNL_ARG_SCALE, mem_dt, mem_fp)) return OK;
 
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         return fill_random_real(mem_dt, mem_fp, nullptr);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, [&](int64_t c) {
         float val = (1.f / 8) * (1 << (c % 7));
         if (prb->flags & GLOB_STATS) val *= 8.f;
-        mem_fp.set_elem(c, val);
+        mem_fp.set_f32_elem(c, val);
     });
 
     if (mem_dt) SAFE(mem_dt.reorder(mem_fp), WARN);
@@ -220,16 +246,21 @@ int fill_scale(const prb_t *prb, dnn_mem_t &mem_fp, dnn_mem_t &mem_dt) {
 int fill_shift(const prb_t *prb, dnn_mem_t &mem_fp, dnn_mem_t &mem_dt) {
     const bool use_sh = prb->use_sh();
     if (!use_sh) return OK;
+    if (fill_from_file(DNNL_ARG_SHIFT, mem_dt, mem_fp)) return OK;
 
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         return fill_random_real(mem_dt, mem_fp, nullptr);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     benchdnn_parallel_nd(prb->ic, [&](int64_t c) {
         float val = ((c % 3) - 1) * (1.f / 512 * (1 << (c % 7)));
         if (prb->flags & GLOB_STATS) val *= 512.f;
-        mem_fp.set_elem(c, val);
+        mem_fp.set_f32_elem(c, val);
     });
 
     if (mem_dt) SAFE(mem_dt.reorder(mem_fp), WARN);
@@ -282,6 +313,7 @@ int prepare_bwd(
 
     const auto nelems = mem_fp.nelems();
     if (nelems == 0) return OK;
+    if (fill_from_file(DNNL_ARG_DIFF_DST, mem_dt, mem_fp)) return OK;
 
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
@@ -296,6 +328,10 @@ int prepare_bwd(
 
         return OK;
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     // Idea behind filling: integer diff_dst values decrease norms unlike fp32
     // values in [-1.f, 1.f] range. To decrease norms more, make data pretty
@@ -326,7 +362,7 @@ int prepare_bwd(
             float value = flip_coin(igen_coin(msr), sparsity)
                     ? round_to_nearest_representable(prb->dt, igen_val(msr))
                     : 0;
-            mem_fp.set_elem(idx, value);
+            mem_fp.set_f32_elem(idx, value);
         }
     });
 
@@ -357,7 +393,7 @@ int check_fwd_ws(dnn_mem_map_t &mem_map, res_t *res) {
      * zero, and the respective ws_dt elements should be set accordingly */
     for (int64_t i = 0; i < nelems; i += 8) {
         for (int64_t j = 0; j < MIN2(8, nelems - i); ++j) {
-            const float data = dst_dt.get_elem(i + j);
+            const float data = dst_dt.get_f32_elem(i + j);
             const bool want = data > 0.f;
             const bool bit_set = ws_type == ws_byte ? *ws : !!(*ws & (1 << j));
 
@@ -392,7 +428,7 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
             force_f32_dt ? dnnl_f32 : prb->dt, prb->tag, prb->strides[0]);
 
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
-            create_dnnl_attr(prb->attr, attr_args_t()));
+            create_dnnl_attr(prb->attr, attr_args_t(), prb->ndims));
 
     auto flags = (dnnl_normalization_flags_t)prb->flags;
     if (dir & FLAG_FWD) {
@@ -427,6 +463,7 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
     skip_unimplemented_data_type({prb->dt}, prb->dir, res);
     skip_unimplemented_sum_po(
             prb->attr, res, dnnl_batch_normalization, prb->dt);
+    skip_unimplemented_binary_po(prb->attr, res);
     skip_unimplemented_prelu_po(prb->attr, res, dnnl_batch_normalization);
 
     // Non-zero alpha is not supported for training in general.
@@ -438,13 +475,13 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
         bool alpha_ok = IMPLICATION(alpha != 0.f, (prb->dir & FLAG_INF));
         if (!alpha_ok) {
             res->state = SKIPPED;
-            res->reason = CASE_NOT_SUPPORTED;
+            res->reason = skip_reason::case_not_supported;
         }
     }
     // BN+Add+ReLU fusion is not supported on CPU
     if (is_cpu() && prb->fuse_add_relu()) {
         res->state = SKIPPED;
-        res->reason = CASE_NOT_SUPPORTED;
+        res->reason = skip_reason::case_not_supported;
     }
     // int8 only supports forward s8 w/ global stats
     const bool u8_not_ok = prb->dt == dnnl_u8;
@@ -452,7 +489,7 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
             && ((prb->dir & FLAG_BWD) || (prb->flags & GLOB_STATS) == 0);
     if (s8_not_ok || u8_not_ok) {
         res->state = SKIPPED;
-        res->reason = CASE_NOT_SUPPORTED;
+        res->reason = skip_reason::case_not_supported;
     }
 }
 
@@ -533,9 +570,9 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
 
                 const auto &sh = ref_args.find(DNNL_ARG_SHIFT);
                 const auto &dst = ref_args.find(DNNL_ARG_DST);
-                const int64_t c = dst.get_scale_idx(
-                        args.idx, 1 << 1 /* channel_mask */);
-                const float beta = sh.get_elem(c);
+                const int64_t c
+                        = dst.get_idx(args.idx, 1 << 1 /* channel_mask */);
+                const float beta = sh.get_f32_elem(c);
                 // Using an empirically derived threshold, check if
                 // cancellation error in `|Y| = |a*X - (-b)|` is huge.
                 const float abs_exp = fabsf(args.exp);
@@ -583,7 +620,9 @@ std::vector<int> supported_exec_args(dir_t dir) {
 int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         dnnl_primitive_t prim, const prb_t *prb, res_t *res,
         dnnl_primitive_t prim_ref) {
-    if (has_bench_mode_modifier(mode_modifier_t::no_host_memory)) return OK;
+    if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) return OK;
+
+    if (!ref_mem_map.empty()) { erase_unused_args(ref_mem_map, mem_map); }
 
     // TODO: this function still allocates the full memory print needed to fill
     // the data and each argument can't be destroyed right away since filling
@@ -604,33 +643,21 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         // use switch below to define a memory desc for it.
         if (exec_arg != DNNL_ARG_SCRATCHPAD && exec_arg != DNNL_ARG_WORKSPACE) {
             ref_mem_map.emplace(exec_arg,
-                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine,
+                            /* prefill = */ false));
         }
 
         switch (exec_arg) {
             case DNNL_ARG_DST:
-                if (prb->dir & FLAG_BWD) {
-                    // Stash for backward which is used in reference code:
-                    //     src_hat[i] = (src[i] - mean) / sqrt(var + prb->eps)
-                    ref_mem_map.emplace(DNNL_ARG_DST_1,
-                            dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
-                }
-                break;
             case DNNL_ARG_DIFF_SRC: break; // Skip on backward.
-            case DNNL_ARG_DST_1: break; // Skip on backward.
             case DNNL_ARG_MEAN:
             case DNNL_ARG_VARIANCE:
                 if (prb->dir & FLAG_INF) {
                     const dnnl_dims_t dims1d = {prb->ic};
-                    ref_mem_map[exec_arg] = dnn_mem_t(
-                            1, dims1d, dnnl_f32, tag::abx, ref_engine);
+                    ref_mem_map[exec_arg] = dnn_mem_t(1, dims1d, dnnl_f32,
+                            tag::abx, ref_engine, /* prefill = */ false);
                 }
                 break;
-            case DNNL_ARG_WORKSPACE: {
-                ref_mem_map[exec_arg]
-                        = dnn_mem_t(mem.md_, dnnl_u8, tag::abx, ref_engine);
-                break;
-            }
             default: break;
         }
     }
@@ -648,10 +675,11 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
     }
 
     // Reference code uses different kind of workspace. Adjust to ref needs.
-    if (ref_mem_map.count(DNNL_ARG_WORKSPACE)) {
+    // Insert a reference workspace if it comes non-empty from the library side.
+    if (query_md_ndims(mem_map.at(DNNL_ARG_WORKSPACE).md_) > 0 && is_fwd_prim) {
         const auto &src_md = ref_mem_map[DNNL_ARG_SRC].md_;
-        ref_mem_map[DNNL_ARG_WORKSPACE]
-                = dnn_mem_t(src_md, dnnl_u8, tag::abx, ref_engine);
+        ref_mem_map[DNNL_ARG_WORKSPACE] = dnn_mem_t(
+                src_md, dnnl_u8, tag::abx, ref_engine, /* prefill = */ false);
     }
 
     return OK;
@@ -690,16 +718,23 @@ int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
     return OK;
 }
 
-int check_cacheit(
-        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+int checkit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res) {
-    SAFE(check_caches(v_prim[0], prb, res), WARN);
-    if (v_prim[1]) { SAFE(check_caches(v_prim[1], prb, res), WARN); }
+    if (has_bench_mode_bit(mode_bit_t::exec)) {
+        SAFE(check_total_size(res), WARN);
+        if (v_prim[1]) SAFE(check_total_size(res), WARN);
+    }
+    if (has_bench_mode_bit(mode_bit_t::corr)) {
+        SAFE(check_caches(v_prim[0], prb, res), WARN);
+        if (v_prim[1]) { SAFE(check_caches(v_prim[1], prb, res), WARN); }
+    }
     return OK;
 }
 
 int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res) {
+    set_zmalloc_max_expected_size(res->mem_size_args.zmalloc_expected_size);
+
     const auto &prim = prb->dir & FLAG_FWD ? v_prim[0] : v_prim[1];
 
     dnn_mem_map_t mem_map, ref_mem_map;
@@ -714,7 +749,7 @@ int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
     SAFE(execute_and_wait(v_prim[0], args, res), WARN);
 
     check_correctness(prb, get_kinds_to_check(prb, FLAG_FWD), args, ref_args,
-            setup_cmp, res);
+            setup_cmp, res, FLAG_FWD);
     SAFE(check_bitwise(prim, get_kinds_to_check(prb, FLAG_FWD), args, prb->attr,
                  prb->inplace, res),
             WARN);
@@ -735,7 +770,7 @@ int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         SAFE(execute_and_wait(v_prim[1], args, res), WARN);
 
         check_correctness(prb, get_kinds_to_check(prb, FLAG_BWD), args,
-                ref_args, setup_cmp, res);
+                ref_args, setup_cmp, res, FLAG_BWD);
         SAFE(check_bitwise(prim, get_kinds_to_check(prb, FLAG_BWD), args,
                      prb->attr, prb->inplace, res),
                 WARN);
