@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -135,7 +135,8 @@ dnnl_status_t brgemm_attr_init(
         PROCESS_KEY_VAL(hint_expected_A_size);
         PROCESS_KEY_VAL(hint_expected_B_size);
         PROCESS_KEY_VAL(hint_expected_C_size);
-        PROCESS_KEY_VAL(wary_tail_read);
+        PROCESS_KEY_VAL(wary_A_k_tail_read);
+        PROCESS_KEY_VAL(extendable_k);
         PROCESS_KEY_VAL(generate_skip_accumulation);
         // TODO: `bd_mask` can't be passed to the kernel at this moment, that's
         // why `bd_mask_level` has to stay `0` for now until it's enabled.
@@ -213,14 +214,20 @@ namespace_impl::brgemm_batch_kind_t str2batch_kind(const std::string &str) {
 }
 #endif
 
-int fill_data(data_kind_t kind, const prb_t *prb, const cfg_t &cfg,
-        dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
+int fill_data(data_kind_t kind, int exec_arg, const prb_t *prb,
+        const cfg_t &cfg, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *res) {
 
     const auto nelems = mem_dt.nelems();
     if (nelems == 0) return OK;
+    if (fill_from_file(exec_arg, mem_dt, mem_fp)) return OK;
 
     assert(mem_dt.nelems() == mem_fp.nelems());
 
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, res, get_perf_fill_cfg(mem_dt.dt()));
+    }
+
     cfg_t::density_args_t density_args;
     density_args.data_kind = kind;
     density_args.n_acc = prb->k;
@@ -251,15 +258,19 @@ int fill_data(data_kind_t kind, const prb_t *prb, const cfg_t &cfg,
             float val = 0;
             while (val <= 0)
                 val = gen(int_seed);
-            mem_fp.set_elem(
+            mem_fp.set_f32_elem(
                     0, round_to_nearest_representable(cfg.get_dt(kind), val));
             idx_start += 1;
         }
 
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
             bool is_one = density == 1.f ? true : b_dist(b_seed);
-            float val = is_one * gen(int_seed);
-            mem_fp.set_elem(
+            if (!is_one) {
+                mem_fp.set_f32_elem(idx, 0.f);
+                continue;
+            }
+            float val = gen(int_seed);
+            mem_fp.set_f32_elem(
                     idx, round_to_nearest_representable(cfg.get_dt(kind), val));
         }
     });
@@ -343,10 +354,11 @@ int init_kernel(kernel_args_t &kernel_args) {
     attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, prb->dst_dims.data());
     const auto &wei_scale = prb->attr.scales.get(DNNL_ARG_WEIGHTS);
     if (wei_scale.policy == policy_t::PER_OC) {
-        attr_args.prepare_scales(prb->attr, DNNL_ARG_WEIGHTS, 2);
+        attr_args.prepare_quant(
+                prb->attr, DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS, 2);
     }
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
-            create_dnnl_attr(prb->attr, attr_args));
+            create_dnnl_attr(prb->attr, attr_args, prb->ndims));
     dims_t dst_strides = {prb->get_ldd(), 1};
     auto dst_md = dnn_mem_t::init_md(
             prb->ndims, prb->dst_dims.data(), prb->dst_dt(), "", dst_strides);
@@ -364,6 +376,9 @@ int init_kernel(kernel_args_t &kernel_args) {
             WARN);
     if (res->state == SKIPPED) return OK;
 
+    SAFE(check_dnnl_status(brgemm_desc_finalize(&brgemm_desc), prb, res), WARN);
+    if (res->state == SKIPPED) return OK;
+
     kernel_args.generate_skip_accumulation_
             = brgemm_attr.generate_skip_accumulation;
 
@@ -388,7 +403,7 @@ int init_kernel(kernel_args_t &kernel_args) {
     attr_args_t attr_args;
     attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, prb->dst_dims.data());
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
-            create_dnnl_attr(prb->attr, attr_args));
+            create_dnnl_attr(prb->attr, attr_args, prb->ndims));
     auto dnnl_post_ops = query_post_ops(dnnl_attr);
 
     dnnl_status_t st = dnnl_success;
@@ -425,7 +440,9 @@ int init_kernel(kernel_args_t &kernel_args) {
     if (res->state == SKIPPED) return OK;
 
     dnnl_pack_type_t pack_type = dnnl_pack_type_undef;
-    DNN_SAFE(dnnl_brgemm_get_B_pack_type(brgemm, &pack_type), WARN);
+    DNN_SAFE(dnnl_brgemm_get_B_pack_type(
+                     &pack_type, prb->src_dt(), prb->wei_dt()),
+            WARN);
     kernel_args.need_pack_ = pack_type == dnnl_pack_type_pack32;
 
     DNN_SAFE(dnnl_brgemm_generate(brgemm), WARN);
@@ -479,6 +496,7 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
             prb->dir, res);
     skip_unimplemented_sum_po(
             prb->attr, res, dnnl_gemm, prb->src_dt(), prb->dst_dt());
+    skip_unimplemented_binary_po(prb->attr, res);
     skip_unimplemented_prelu_po(prb->attr, res, dnnl_gemm);
 
     // Unconditionally skip remaining unimplemented cases.
@@ -635,6 +653,7 @@ dnnl_status_t brgemm_kernel_execute_postops_wrapper(const_dnnl_brgemm_t brgemm,
         const bool use_dst_as_acc, const void *src_ptr,
         const void *wei_packed_ptr, const std::vector<dnnl_dim_t> &offsets,
         void *acc_ptr, void *dst_ptr, void *scratchpad_ptr,
+        const_dnnl_ukernel_attr_params_t attr_params,
         const dnnl_stream_t &stream,
         const std::vector<dnnl_exec_arg_t> &dnnl_args) {
 
@@ -644,7 +663,7 @@ dnnl_status_t brgemm_kernel_execute_postops_wrapper(const_dnnl_brgemm_t brgemm,
                 offsets.data(), dst_ptr, scratchpad_ptr);
     } else {
         st = dnnl_brgemm_execute_postops(brgemm, src_ptr, wei_packed_ptr,
-                offsets.data(), acc_ptr, dst_ptr, scratchpad_ptr, nullptr);
+                offsets.data(), acc_ptr, dst_ptr, scratchpad_ptr, attr_params);
     }
     return st;
 }
@@ -768,24 +787,30 @@ void init_memory_args(
 
     const auto &test_engine = get_test_engine();
 
-    mem_map.emplace(DNNL_ARG_SRC, dnn_mem_t(src_md, test_engine));
-    mem_map.emplace(DNNL_ARG_WEIGHTS, dnn_mem_t(wei_md, test_engine));
+    mem_map.emplace(
+            DNNL_ARG_SRC, dnn_mem_t(src_md, test_engine, /* prefill = */ true));
+    mem_map.emplace(DNNL_ARG_WEIGHTS,
+            dnn_mem_t(wei_md, test_engine, /* prefill = */ true));
 #if !defined(DNNL_EXPERIMENTAL_UKERNEL)
     if (prb->bia_dt != dnnl_data_type_undef) {
         // Need condition to extract bias pointer based on presence in map
         // condition.
-        mem_map.emplace(DNNL_ARG_BIAS, dnn_mem_t(bia_md, test_engine));
+        mem_map.emplace(DNNL_ARG_BIAS,
+                dnn_mem_t(bia_md, test_engine, /* prefill = */ true));
     }
 #else
-    mem_map.emplace(DNNL_ARG_WEIGHTS_1, dnn_mem_t(wei_packed_md, test_engine));
+    mem_map.emplace(DNNL_ARG_WEIGHTS_1,
+            dnn_mem_t(wei_packed_md, test_engine, /* prefill = */ true));
 #endif
-    mem_map.emplace(DNNL_ARG_DST_1, dnn_mem_t(acc_md, test_engine));
-    mem_map.emplace(DNNL_ARG_DST, dnn_mem_t(dst_md, test_engine));
+    mem_map.emplace(DNNL_ARG_DST_1,
+            dnn_mem_t(acc_md, test_engine, /* prefill = */ true));
+    mem_map.emplace(
+            DNNL_ARG_DST, dnn_mem_t(dst_md, test_engine, /* prefill = */ true));
     if (scratchpad_size > 0) {
         // Need condition to extract scratchpad pointer based on presence in map
         // condition.
-        mem_map.emplace(
-                DNNL_ARG_SCRATCHPAD, dnn_mem_t(scratchpad_md, test_engine));
+        mem_map.emplace(DNNL_ARG_SCRATCHPAD,
+                dnn_mem_t(scratchpad_md, test_engine, /* prefill = */ true));
     }
 
     // Binary post-op.
@@ -805,9 +830,9 @@ void init_memory_args(
         if (b.mask_input == mask_input_t::mask) {
             mask = b.mask;
         } else if (b.mask_input == mask_input_t::policy) {
-            mask = attr_t::policy2mask(po_arg, b.policy, dnnl_matmul, 2);
+            mask = attr_t::policy2mask(po_arg, b.policy, 2, dnnl_matmul);
         } else {
-            mask = attr_t::get_default_mask(b.policy);
+            mask = attr_t::get_default_mask(b.policy, ndims);
         }
 
         switch (mask) {
@@ -820,7 +845,8 @@ void init_memory_args(
 
         auto po_md
                 = dnn_mem_t::init_md(ndims, dims.data(), b.src1_dt, tag::abx);
-        mem_map.emplace(po_arg, dnn_mem_t(po_md, test_engine));
+        mem_map.emplace(
+                po_arg, dnn_mem_t(po_md, test_engine, /* prefill = */ true));
     }
 
     if (!prb->attr.scales.is_def()) {
@@ -847,7 +873,8 @@ void init_memory_args(
             const auto dt = sc.get(exec_arg).dt;
             auto scales_md
                     = dnn_mem_t::init_md(ndims, dims.data(), dt, tag::abx);
-            mem_map.emplace(exec_sc_arg, dnn_mem_t(scales_md, test_engine));
+            mem_map.emplace(exec_sc_arg,
+                    dnn_mem_t(scales_md, test_engine, /* prefill = */ true));
         }
     }
 
@@ -874,13 +901,17 @@ void init_memory_args(
             }
             const auto dt = zp.get(exec_arg).dt;
             auto zp_md = dnn_mem_t::init_md(ndims, dims.data(), dt, tag::abx);
-            mem_map.emplace(exec_zp_arg, dnn_mem_t(zp_md, test_engine));
+            mem_map.emplace(exec_zp_arg,
+                    dnn_mem_t(zp_md, test_engine, /* prefill = */ true));
         }
     }
 }
 
 int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         const prb_t *prb, const kernel_args_t &kernel_args, res_t *res) {
+
+    if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) return OK;
+
     const auto &ref_engine = get_cpu_engine();
 
     // Move cfg out of filling since its creation is not free.
@@ -902,31 +933,37 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         // use switch below to define a memory desc for it.
         if (exec_arg != DNNL_ARG_SCRATCHPAD) {
             ref_mem_map.emplace(exec_arg,
-                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine,
+                            /* prefill = */ false));
         }
 
         auto &ref_mem = ref_mem_map[exec_arg];
 
         switch (exec_arg) {
             case DNNL_ARG_SRC:
-                SAFE(fill_data(SRC, prb, cfg, mem, ref_mem, res), WARN);
+                SAFE(fill_data(SRC, exec_arg, prb, cfg, mem, ref_mem, res),
+                        WARN);
                 break;
             case DNNL_ARG_WEIGHTS:
-                SAFE(fill_data(WEI, prb, cfg, mem, ref_mem, res), WARN);
+                SAFE(fill_data(WEI, exec_arg, prb, cfg, mem, ref_mem, res),
+                        WARN);
                 break;
             case DNNL_ARG_BIAS:
-                SAFE(fill_data(BIA, prb, cfg, mem, ref_mem, res), WARN);
+                SAFE(fill_data(BIA, exec_arg, prb, cfg, mem, ref_mem, res),
+                        WARN);
                 break;
             case DNNL_ARG_DST: {
                 const auto &po = prb->attr.post_ops;
                 const int sum_idx = po.find(attr_t::post_ops_t::SUM);
                 if (sum_idx >= 0) {
-                    SAFE(fill_data(DST, prb, cfg, mem, ref_mem, res), WARN);
+                    SAFE(fill_data(DST, exec_arg, prb, cfg, mem, ref_mem, res),
+                            WARN);
                 }
             } break;
             case DNNL_ARG_DST_1: {
                 if (need_fill_acc) {
-                    SAFE(fill_data(DST, prb, cfg, mem, ref_mem, res), WARN);
+                    SAFE(fill_data(DST, exec_arg, prb, cfg, mem, ref_mem, res),
+                            WARN);
                 }
             } break;
             default:
@@ -954,7 +991,7 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
     dnnl_dims_t dims = {1};
     auto workspace_md = dnn_mem_t::init_md(1, dims, dnnl_u8, tag::abx);
     ref_mem_map.emplace(DNNL_ARG_WORKSPACE,
-            dnn_mem_t(workspace_md, ref_engine,
+            dnn_mem_t(workspace_md, ref_engine, /* prefill = */ false,
                     {false, (void *)&kernel_args.generate_skip_accumulation_}));
     ref_mem_map.at(DNNL_ARG_WORKSPACE).map();
 
@@ -963,6 +1000,34 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
 
 int scales_post_processing(dnn_mem_map_t &mem_map) {
 #if !defined(DNNL_EXPERIMENTAL_UKERNEL)
+    // dst scales are applied inversed in the brgemm kernel. However, for x64
+    // it's enough to have a single value, though we keep v16 just for the sake
+    // of a single code path with aarch64.
+    const bool has_dst_scale
+            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+
+    const auto replace_mem_to_v16 = [&](dnnl_data_type_t dt, int exec_arg,
+                                            float val) {
+        dims_t dims = {16};
+        auto new_md = dnn_mem_t::init_md(1, dims.data(), dt, tag::abx);
+        dnn_mem_t new_m(new_md, get_test_engine(), /* prefill = */ true);
+        if (!new_m.is_mapped()) new_m.map();
+        for (int64_t i = 0; i < new_m.nelems(); i++) {
+            new_m.set_elem(i, val);
+        }
+        mem_map[DNNL_ARG_ATTR_SCALES | exec_arg] = std::move(new_m);
+    };
+
+    if (has_dst_scale) {
+        const auto &dst_scales_m
+                = mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+        assert(dst_scales_m.nelems() == 1);
+        // Create a v16 dst scales memory and bcast inversed dst value there.
+        replace_mem_to_v16(dst_scales_m.dt(), DNNL_ARG_DST,
+                1.f / dst_scales_m.get_elem(0));
+    }
+
+#if defined(brg_aarch64)
     // Internal API has specific implementation details w.r.t. scales.
     // If any of source or weights scales present in the descriptor, then the
     // kernel expects to get a vector of 16 float values (v16) of "fused" scale
@@ -982,19 +1047,6 @@ int scales_post_processing(dnn_mem_map_t &mem_map) {
             = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
     const bool has_wei_scale
             = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
-    const bool has_dst_scale
-            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
-
-    const auto replace_mem_to_v16 = [&](dnnl_data_type_t dt, int exec_arg,
-                                            float val) {
-        dims_t dims = {16};
-        auto new_md = dnn_mem_t::init_md(1, dims.data(), dt, tag::abx);
-        dnn_mem_t new_m(new_md, get_test_engine());
-        for (int64_t i = 0; i < new_m.nelems(); i++) {
-            new_m.set_elem(i, val);
-        }
-        mem_map[DNNL_ARG_ATTR_SCALES | exec_arg] = std::move(new_m);
-    };
 
     if (has_wei_scale) {
         const auto &wei_scales_m
@@ -1024,15 +1076,7 @@ int scales_post_processing(dnn_mem_map_t &mem_map) {
                 src_scales_m.dt(), DNNL_ARG_WEIGHTS, src_scales_m.get_elem(0));
     }
 
-    if (has_dst_scale) {
-        const auto &dst_scales_m
-                = mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
-        assert(dst_scales_m.nelems() == 1);
-        // Create a v16 dst scales memory and bcast inversed dst value there.
-        replace_mem_to_v16(dst_scales_m.dt(), DNNL_ARG_DST,
-                1.f / dst_scales_m.get_elem(0));
-    }
-
+#endif
 #else // !defined(DNNL_EXPERIMENTAL_UKERNEL)
     // ukernel API takes split pointers for scales, no need to update them on
     // user level.
@@ -1087,7 +1131,19 @@ int release_hw_config(const kernel_args_t &kernel_args) {
         DNN_SAFE(namespace_impl::amx_tile_release(), WARN);
     }
 #endif
-#else // !defined(DNNL_EXPERIMENTAL_UKERNEL)
+#endif
+    return OK;
+}
+
+// `release_hw_config` and `brgemm_finalize` are doing the same thing -
+// releasing the hw resources they allocated. The difference is in the
+// implementation side - ukernel has lazy initialization and would reset the
+// state per `set_hw_config` call while internal API doesn't - it just sets
+// the new state unconditionally. Because of laziness, the test wants to ensure
+// resetting is correct and, thus, releasing is done once - at the end of the
+// suite, while for internal API it is done after each case.
+int brgemm_finalize() {
+#if defined(DNNL_EXPERIMENTAL_UKERNEL)
     DNN_SAFE(dnnl_brgemm_release_hw_context(), WARN);
 #endif
     return OK;
@@ -1125,6 +1181,13 @@ int doit(const prb_t *prb, res_t *res) {
     // "Reference" are used as usual.
     args_t args(mem_map), ref_args(ref_mem_map);
 
+    // The implementation memory must be mapped to setup point arguments for
+    // brgemm implementation call. This assumes that mapping is effectively a
+    // no-op on the target device.
+    for (auto &kv : mem_map) {
+        if (!kv.second.is_mapped()) kv.second.map();
+    }
+
     const char *src_ptr = (const char *)mem_map.at(DNNL_ARG_SRC);
     const char *wei_ptr = (const char *)mem_map.at(DNNL_ARG_WEIGHTS);
     char *acc_ptr = (char *)mem_map.at(DNNL_ARG_DST_1);
@@ -1136,6 +1199,16 @@ int doit(const prb_t *prb, res_t *res) {
     std::vector<const void *> binary_po_v;
     SAFE(binary_post_op_preprocessing(binary_po_v, mem_map), WARN);
 
+#if defined(brg_x64)
+    const void *src_scales_ptr
+            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC)
+            ? (const void *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC)
+            : nullptr;
+    const void *wei_scales_ptr
+            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
+            ? (const void *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
+            : nullptr;
+#endif
     const float *dst_scales_ptr
             = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST)
             ? (const float *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST)
@@ -1157,10 +1230,12 @@ int doit(const prb_t *prb, res_t *res) {
     }
 
     // For internal API, scales are combined. See `scales_post_processing`.
+#if defined(brg_aarch64)
     const float *scales_ptr
             = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
             ? (const float *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
             : nullptr;
+#endif
     const int32_t *dst_zp_ptr
             = mem_map.count(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST)
             ? (const int32_t *)mem_map.at(
@@ -1185,6 +1260,27 @@ int doit(const prb_t *prb, res_t *res) {
                             : 0);
     char *src_comp_ptr = const_cast<char *>(wei_ptr) + wei_offset_zp;
 
+#if defined(brg_x64)
+    namespace_impl::brgemm_post_ops_data_t post_ops_data(
+            /* bias */ bia_dt_ptr,
+            /* binary_post_ops_rhs */ binary_po_v.data(),
+            /* oc_logical_off */ 0, /* dst_row_logical_off */ 0,
+            // TODO: though the field is called `data_C_ptr_`, this is a
+            // misleading name since actually dst_ptr must be used there to
+            // have binary injector working for per_tensor policy.
+            /* data_C_ptr_ */ dst_ptr, /* first_mb_matrix_addr_off */ 0,
+            /* a_zp_compensations */ src_comp_ptr,
+            /* b_zp_compensations */ nullptr,
+            /* c_zp_values */ dst_zp_ptr,
+            /* skip_accumulation */
+            kernel_args.generate_skip_accumulation_,
+            /* zp_a_val */ zp_a_val,
+            /* do_only_comp */ false,
+            /* do_only_zp_a_val */ false,
+            /* src_scales */ src_scales_ptr,
+            /* wei_scales */ wei_scales_ptr,
+            /* dst_scales */ dst_scales_ptr);
+#elif defined(brg_aarch64)
     namespace_impl::brgemm_post_ops_data_t post_ops_data(
             /* bias */ bia_dt_ptr,
             /* scales */ scales_ptr,
@@ -1203,6 +1299,7 @@ int doit(const prb_t *prb, res_t *res) {
             /* do_only_comp */ false,
             /* do_only_zp_a_val */ false,
             /* dst_scales */ dst_scales_ptr);
+#endif
 
     // Note: hardware lacking native s8s8 support expects compensation buffer
     // passed through a scratchpad argument in postops execution call.
@@ -1243,14 +1340,6 @@ int doit(const prb_t *prb, res_t *res) {
                      attr_params, binary_po_v.data()),
             WARN);
 
-    const void *src_scales_ptr
-            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC)
-            ? (const void *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC)
-            : nullptr;
-    const void *wei_scales_ptr
-            = mem_map.count(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
-            ? (const void *)mem_map.at(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS)
-            : nullptr;
     DNN_SAFE(dnnl_ukernel_attr_params_set_A_scales(attr_params, src_scales_ptr),
             WARN);
     DNN_SAFE(dnnl_ukernel_attr_params_set_B_scales(attr_params, wei_scales_ptr),
@@ -1272,21 +1361,17 @@ int doit(const prb_t *prb, res_t *res) {
                 post_ops_data, scratchpad_ptr);
     }
 #else // !defined(DNNL_EXPERIMENTAL_UKERNEL)
-    if (prb->use_dst_as_acc()) {
-        DNN_SAFE(dnnl_brgemm_execute(brgemm, src_ptr, wei_packed_ptr,
-                         offsets.data(), dst_ptr, scratchpad_ptr),
-                WARN);
-    } else {
-        DNN_SAFE(dnnl_brgemm_execute_postops(brgemm, src_ptr, wei_packed_ptr,
-                         offsets.data(), acc_ptr, dst_ptr, scratchpad_ptr,
-                         attr_params),
-                WARN);
-    }
+    // `prb->use_dst_as_acc()=true` will make `dst_ptr=acc_ptr` and rest should
+    // be handled by API.
+    DNN_SAFE(dnnl_brgemm_execute_postops(brgemm, src_ptr, wei_packed_ptr,
+                     offsets.data(), acc_ptr, dst_ptr, scratchpad_ptr,
+                     attr_params),
+            WARN);
 #endif
     res->state = EXECUTED;
 
     if (has_bench_mode_bit(mode_bit_t::corr)) {
-        check_correctness(prb, {DST}, args, ref_args, setup_cmp, res);
+        check_correctness(prb, {DST}, args, ref_args, setup_cmp, res, prb->dir);
     }
 
     // Create a bind to match internals to run performance measurements.
@@ -1299,8 +1384,8 @@ int doit(const prb_t *prb, res_t *res) {
 #else // !defined(DNNL_EXPERIMENTAL_UKERNEL)
     perf_function_t perf_func = std::bind(brgemm_kernel_execute_postops_wrapper,
             kernel_args.brgemm_, prb->use_dst_as_acc(), src_ptr, wei_packed_ptr,
-            offsets, acc_ptr, dst_ptr, scratchpad_ptr, std::placeholders::_1,
-            std::placeholders::_2);
+            offsets, acc_ptr, dst_ptr, scratchpad_ptr, attr_params_ptr,
+            std::placeholders::_1, std::placeholders::_2);
 #endif
 
     measure_perf(prb->ctx_exe, res, perf_func, args);
@@ -1312,6 +1397,11 @@ int doit(const prb_t *prb, res_t *res) {
 
 #else
 
+// For builadability of non-x64 configuration.
+int brgemm_finalize() {
+    return OK;
+}
+
 int doit(const prb_t *prb, res_t *res) {
     return OK;
 }
