@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 #include <limits.h>
 #include <stdint.h>
 
+#include <algorithm>
 #include <cctype>
 #include <fstream>
 #include <functional>
@@ -28,25 +29,10 @@
 #include "oneapi/dnnl/dnnl.h"
 
 #include "common.hpp"
-#include "utils/parser.hpp"
 
+#include "utils/fill.hpp"
 #include "utils/parallel.hpp"
 
-// BENCHDNN_MEMORY_CHECK macro enables guarding mechanism for memory allocation:
-// memory block is allocated on a page boundary and the page after the block is
-// protected to catch possible invalid accesses.
-//
-// Note that the macro affects the correctness mode only.
-#ifdef __unix__
-#define BENCHDNN_MEMORY_CHECK
-#endif
-
-#ifdef BENCHDNN_MEMORY_CHECK
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/mman.h>
-#endif
-
 /* result structure */
 const char *state2str(res_state_t state) {
     if (state == UNTESTED) return "UNTESTED_FAILED"; // for easier fail search
@@ -62,23 +48,20 @@ const char *state2str(res_state_t state) {
     CASE(LISTED);
     CASE(EXECUTED);
     CASE(INITIALIZED);
+    CASE(DEFERRED);
 #undef CASE
     assert(!"unknown res state");
     return "STATE_UNDEF";
 }
 
-const char *skip_reason2str(skip_reason_t skip_reason) {
-#define CASE(x) \
-    if (skip_reason == (x)) return STRINGIFY(x)
-    CASE(CASE_NOT_SUPPORTED);
-    CASE(DATA_TYPE_NOT_SUPPORTED);
-    CASE(INVALID_CASE);
-    CASE(NOT_ENOUGH_RAM);
-    CASE(SKIP_IMPL_HIT);
-    CASE(SKIP_START);
-#undef CASE
-    return "SKIP_UNKNOWN";
-}
+namespace skip_reason {
+std::string case_not_supported("Case not supported");
+std::string data_type_not_supported("Data type not supported");
+std::string invalid_case("Invalid case");
+std::string not_enough_ram("Not enough RAM");
+std::string skip_impl_hit("Skip-impl option hit");
+std::string skip_start("Skip-start option hit");
+} // namespace skip_reason
 
 dir_t str2dir(const char *str) {
 #define CASE(x) \
@@ -97,76 +80,94 @@ dir_t str2dir(const char *str) {
 
 void parse_result(res_t &res, const char *pstr) {
     auto &bs = benchdnn_stat;
+
+    // Can be updated for `INITIALIZED`. TODO: remove this.
     const char *state = state2str(res.state);
+    bool is_failed = false;
+    bool print_me = true;
 
     switch (res.state) {
-        case UNTESTED:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            bs.failed++;
-            break;
+        case UNTESTED: is_failed = true; break;
         case EXECUTED:
-            if (bench_mode == bench_mode_t::exec)
-                BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.passed++;
+            if (bench_mode != bench_mode_t::exec) print_me = false;
             break;
-        case FAILED:
-            bs.failed++;
-            BENCHDNN_PRINT(0, "%d:%s (errors:%lu total:%lu) __REPRO: %s\n",
-                    bs.tests, state, (unsigned long)res.errors,
-                    (unsigned long)res.total, pstr);
-            break;
-        case SKIPPED:
-            BENCHDNN_PRINT(0, "%d:%s (%s) __REPRO: %s\n", bs.tests, state,
-                    skip_reason2str(res.reason), pstr);
-            bs.skipped++;
-            break;
+        case FAILED: is_failed = true; break;
+        case SKIPPED: bs.skipped++; break;
+        case DEFERRED:
         case UNIMPLEMENTED:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            is_failed = true;
             bs.unimplemented++;
-            bs.failed++;
             break;
         case INVALID_ARGUMENTS:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            is_failed = true;
             bs.invalid_arguments++;
-            bs.failed++;
-            break;
-        case MISTRUSTED:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            bs.mistrusted++;
-            break;
-        case PASSED:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            bs.passed++;
-            break;
-        case LISTED:
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            bs.listed++;
             break;
+        case MISTRUSTED: bs.mistrusted++; break;
+        case PASSED: bs.passed++; break;
+        case LISTED: bs.listed++; break;
         case INITIALIZED:
             // TODO: workaround for failed fill functions.
             if (bench_mode != bench_mode_t::init) {
+                is_failed = true;
                 state = "FAILED";
-                bs.failed++;
             } else {
                 bs.passed++;
             }
-
-            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             break;
-        default: assert(!"unknown state"); SAFE_V(FAIL);
+        default:
+            BENCHDNN_PRINT(0, "%s\n",
+                    "Error: unknown state encountered in \'parse_results()\'.");
+            SAFE_V(FAIL);
     }
 
+    std::string reason;
+    if (!res.reason.empty()) { reason = " (" + res.reason + ")"; }
+
+    std::string error_stat;
+    if (res.errors > 0) {
+        error_stat = " (errors:" + std::to_string(res.errors)
+                + " total:" + std::to_string(res.total) + ")";
+    }
+
+    using bt = timer::timer_t;
+
+    const auto &tct = res.timer_map.get_timer(timer::names::test_case_timer);
+    // Round to integer for nicer input.
+    // Use `sum` mode because it consists of two separate parts - creation and
+    // execution.
+    const int64_t tct_ms = static_cast<int64_t>(tct.ms(bt::mode_t::sum));
+    std::string tct_str = " (" + std::to_string(tct_ms) + " ms)";
+
+    // This is the common format of the repro line ([] - for optional entries):
+    // case_num:status[ (reason)][ (error_stats)] (time) __REPRO: prb_str
+    std::string full_repro = std::to_string(bs.tests) + ":" + std::string(state)
+            + reason + error_stat + tct_str + " __REPRO: " + pstr;
+    if (is_failed) {
+        bs.failed++;
+        bs.failed_cases.emplace(bs.tests, full_repro);
+        // In theory, this can pop up for unimplemented and invalid args, too.
+        // Shouldn't be a problem though.
+        if (!buffer_prefix.empty()) {
+            BENCHDNN_PRINT(0, "%s\n",
+                    "Note: importing data from a file led to a correctness "
+                    "issue. It is impossible to tell if that was a real oneDNN "
+                    "bug or just benchdnn refusing to acknowledge a minuscule "
+                    "difference as a match. Take this 'FAILED' with a grain of "
+                    "salt, and proceed with extra caution.");
+        }
+    }
+    if (print_me) { BENCHDNN_PRINT(0, "%s\n", full_repro.c_str()); }
+
+    // Update this after collecting stats.
     bs.tests++;
     assert(bs.tests
             == bs.passed + bs.skipped + bs.mistrusted + bs.failed + bs.listed);
 
-    using bt = timer::timer_t;
-    using namespace timer::names;
-
     if (has_bench_mode_bit(mode_bit_t::perf)) {
         const auto &t = res.timer_map.perf_timer();
         for (int mode = 0; mode < (int)bt::n_modes; ++mode)
-            bs.ms[perf_timer][mode] += t.ms((bt::mode_t)mode);
+            bs.ms[timer::names::perf_timer][mode] += t.ms((bt::mode_t)mode);
     }
 
     for (const auto &e : timer::get_global_service_timers()) {
@@ -178,108 +179,14 @@ void parse_result(res_t &res, const char *pstr) {
         // Only summary time is populated to the highest level report.
         bs.ms[t_name][bt::mode_t::sum] += t.sec(bt::mode_t::sum);
     }
+
+    // Append an impl name into the total stats.
+    // Skipped cases don't count.
+    if (!res.impl_name.empty()) bs.impl_names[res.impl_name]++;
 }
 
 /* misc */
 
-#ifdef BENCHDNN_MEMORY_CHECK
-static void *zmalloc_protect(size_t size) {
-    const size_t page_sz = getpagesize();
-
-    const size_t block_sz = size + 3 * sizeof(void *);
-    const size_t total_sz = rnd_up(block_sz, page_sz) + page_sz;
-
-    void *mem_ptr;
-    int rc = ::posix_memalign(&mem_ptr, page_sz, total_sz);
-    if (rc != 0) return nullptr;
-
-    uint8_t *ptr_start = (uint8_t *)mem_ptr;
-    uint8_t *ptr = ptr_start + total_sz - page_sz - size;
-
-    // Aligned on a page boundary
-    void *ptr_protect = ptr + size;
-
-    // Layout of the allocated region:
-    // ptr_start   <- start of the allocated region
-    // ptr[-16]    <- stores start address: ptr_start
-    // ptr[-8]     <- stores protected address: ptr_protect
-    // ptr         <- pointer to be returned from the function
-    // ptr_protect <- pointer to the block to protect
-
-    // Protect one page right after the block of size bytes
-    int err = mprotect(ptr_protect, page_sz, PROT_NONE);
-    if (err != 0) {
-        ::free(ptr_start);
-        return nullptr;
-    }
-
-    // Align down `ptr` on 8 bytes before storing addresses to make behavior
-    // defined.
-    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
-    void *ptr_aligned_8 = ptr - to_align;
-    // Save pointers for zfree_protect
-    ((void **)ptr_aligned_8)[-2] = ptr_start;
-    ((void **)ptr_aligned_8)[-1] = ptr_protect;
-
-    return ptr;
-}
-
-static void zfree_protect(void *ptr) {
-    // Get aligned ptr before obtaining addresses
-    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
-    void *ptr_aligned_8 = reinterpret_cast<uint8_t *>(ptr) - to_align;
-
-    // Restore read-write access for the protected region
-    void *ptr_protect = ((void **)ptr_aligned_8)[-1];
-    const size_t page_sz = getpagesize();
-    mprotect(ptr_protect, page_sz, PROT_READ | PROT_WRITE);
-
-    // Deallocate the whole region
-    void *ptr_start = ((void **)ptr_aligned_8)[-2];
-    ::free(ptr_start);
-}
-#endif
-
-void *zmalloc(size_t size, size_t align) {
-#ifdef BENCHDNN_MEMORY_CHECK
-    if (has_bench_mode_bit(mode_bit_t::corr)) { return zmalloc_protect(size); }
-#endif
-
-    void *ptr;
-#ifdef _WIN32
-    ptr = _aligned_malloc(size, align);
-    int rc = ((ptr) ? 0 : errno);
-#else
-    // posix_memalign requires alignment to be
-    // a power of 2 and a multiple of sizeof(void *)
-    if (align < sizeof(void *)) align = sizeof(void *);
-    assert(((align & (align - 1)) == 0) && "align must be a power of 2");
-
-    // TODO. Heuristics: Increasing the size to alignment increases
-    // the stability of performance results.
-    if (has_bench_mode_bit(mode_bit_t::perf) && (size < align)) size = align;
-    int rc = ::posix_memalign(&ptr, align, size);
-#endif /* _WIN32 */
-    return rc == 0 ? ptr : nullptr;
-}
-
-// zfree behavior is aligned with UNIX free().
-void zfree(void *ptr) {
-    if (!ptr) return;
-#ifdef BENCHDNN_MEMORY_CHECK
-    if (has_bench_mode_bit(mode_bit_t::corr)) {
-        zfree_protect(ptr);
-        return;
-    }
-#endif
-
-#ifdef _WIN32
-    _aligned_free(ptr);
-#else
-    return ::free(ptr);
-#endif /* _WIN32 */
-}
-
 bool str2bool(const char *str) {
     return !strcasecmp("true", str) || !strcasecmp("1", str);
 }
@@ -319,25 +226,10 @@ bool match_regex(const char *str, const char *pattern) {
 }
 #endif /* _WIN32 */
 
-bool maybe_skip(const std::string &impl_str) {
-    if (skip_impl.empty()) return false;
-
-    size_t start_pos = 0;
-    // Iterate over impls in skip list.
-    while (start_pos != std::string::npos) {
-        const auto skip_impl_item
-                = parser::get_substr(skip_impl, start_pos, ',');
-        if (skip_impl_item.empty()) continue;
-        if (impl_str.find(skip_impl_item) != std::string::npos) return true;
-    }
-
-    return false;
-}
-
 bool skip_start(res_t *res, int idx) {
     if (idx < test_start) {
         res->state = SKIPPED;
-        res->reason = SKIP_START;
+        res->reason = skip_reason::skip_start;
         return true;
     }
     return false;
@@ -439,6 +331,8 @@ std::string locate_file(const std::string &fname) {
                 BENCHDNN_PRINT(50, "file used: %s\n", fullname.c_str());
                 ifs.close();
                 return fullname;
+            } else {
+                BENCHDNN_PRINT(50, "File not found at: %s\n", fullname.c_str());
             }
             ifs.close();
         }
@@ -456,7 +350,7 @@ int batch(const char *fname, bench_f bench) {
     std::string str;
     bool continued_line = false;
     while (ifs >> str) {
-        if (str.length() == 0) continue;
+        if (str.empty()) continue;
 
         // shell style comments
         if (str.front() == '#') {
@@ -505,11 +399,21 @@ int64_t div_up(const int64_t a, const int64_t b) {
     return (a + b - 1) / b;
 }
 
+size_t div_up(const size_t a, const size_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return (a + b - 1) / b;
+}
+
 int64_t rnd_up(const int64_t a, const int64_t b) {
     SAFE_V(b != 0 ? OK : FAIL);
     return div_up(a, b) * b;
 }
 
+size_t rnd_up(const size_t a, const size_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return div_up(a, b) * b;
+}
+
 int64_t next_pow2(int64_t a) {
     assert(a > 0 && a <= ((int64_t)1 << 62));
     if (a > 1) a--;
@@ -654,3 +558,86 @@ void print_dhw(bool &print_d, bool &print_h, bool &print_w, int ndims,
     print_w = ndims == 3 || (ndims == 5 && (!cubic_shape || canonical))
             || (ndims == 4 && (!square_shape || canonical));
 }
+
+// Copied from utils::getenv.
+// An underlined unified implementation for getting env var value.
+int getenv(const char *name, char *buffer, int buffer_size) {
+    if (name == nullptr || buffer_size < 0
+            || (buffer == nullptr && buffer_size > 0))
+        return INT_MIN;
+
+    int result = 0;
+    int term_zero_idx = 0;
+    size_t value_length = 0;
+
+#ifdef _WIN32
+    value_length = GetEnvironmentVariable(name, buffer, buffer_size);
+#else
+    const char *value = ::getenv(name);
+    value_length = value == nullptr ? 0 : strlen(value);
+#endif
+
+    if (value_length > INT_MAX)
+        result = INT_MIN;
+    else {
+        int int_value_length = (int)value_length;
+        if (int_value_length >= buffer_size) {
+            result = -int_value_length;
+        } else {
+            term_zero_idx = int_value_length;
+            result = int_value_length;
+#ifndef _WIN32
+            if (value) strncpy(buffer, value, buffer_size - 1);
+#endif
+        }
+    }
+
+    if (buffer != nullptr) buffer[term_zero_idx] = '\0';
+    return result;
+}
+
+// Copied from utils::getenv_int_user.
+// Collects an integer value from an env var.
+int benchdnn_getenv_int(const char *name, int default_value) {
+    int value = default_value;
+    // # of digits in the longest 32-bit signed int + sign + terminating null
+    const int len = 12;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = atoi(value_str); }
+    return value;
+}
+
+// Copied from utils::getenv_string_user.
+// Collects a string lower case value from an env var.
+std::string benchdnn_getenv_string(const char *name) {
+    // Random number to fit possible string input.
+    std::string value;
+    const int len = 128;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = value_str; }
+    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+    return value;
+}
+
+std::string smart_bytes(double bytes) {
+    std::string s;
+    static constexpr int oneK = 1024;
+
+    if (bytes < oneK) {
+        s = std::to_string(static_cast<size_t>(bytes)) + " B";
+        return s;
+    }
+    auto KB = bytes / oneK;
+    if (KB < oneK) {
+        s = std::to_string(KB) + " KB";
+        return s;
+    }
+    auto MB = KB / oneK;
+    if (MB < oneK) {
+        s = std::to_string(MB) + " MB";
+        return s;
+    }
+    auto GB = MB / oneK;
+    s = std::to_string(GB) + " GB";
+    return s;
+}
