@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@
 
 #include <cinttypes>
 #include <functional>
+#include <map>
 #include <string>
 #include <vector>
 #include <unordered_map>
@@ -67,12 +68,12 @@ enum { CRIT = 1, WARN = 2 };
     do { \
         int status__ = (f); \
         if (status__ != OK) { \
-            if (s == CRIT || s == WARN) { \
+            if ((s) == CRIT || (s) == WARN) { \
                 BENCHDNN_PRINT(0, \
                         "Error: Function '%s' at (%s:%d) returned '%d'\n", \
                         __FUNCTION__, __FILE__, __LINE__, status__); \
                 fflush(0); \
-                if (s == CRIT) exit(1); \
+                if ((s) == CRIT) exit(1); \
             } \
             return status__; \
         } \
@@ -107,9 +108,14 @@ extern std::string driver_name;
         } \
     } while (0)
 
+//NOLINTBEGIN(bugprone-macro-parentheses)
+// dnnl_common.hpp:119:5: error: expected ';' at end of declaration list [clang-diagnostic-error]
+//  119 |     BENCHDNN_DISALLOW_COPY_AND_ASSIGN(stream_t);
+//      |     ^
 #define BENCHDNN_DISALLOW_COPY_AND_ASSIGN(T) \
     T(const T &) = delete; \
     T &operator=(const T &) = delete;
+//NOLINTEND(bugprone-macro-parentheses)
 
 /* perf */
 extern double max_ms_per_prb; // max time spend per prb in ms
@@ -136,6 +142,10 @@ struct stat_t {
     int invalid_arguments;
     int listed;
     std::unordered_map<std::string, double[timer::timer_t::mode_t::n_modes]> ms;
+    // Key is the number of the test, value is the repro string.
+    std::map<int, std::string> failed_cases;
+    // Key is an impl_name, value is the number of cases.
+    std::map<std::string, size_t> impl_names;
 };
 extern stat_t benchdnn_stat;
 
@@ -156,18 +166,13 @@ void parse_result(res_t &res, const char *pstr);
 /* misc */
 void init_fp_mode();
 
-void *zmalloc(size_t size, size_t align);
-void zfree(void *ptr);
-
 bool str2bool(const char *str);
 const char *bool2str(bool value);
 
-/* TODO: why two functions??? */
 bool match_regex(const char *str, const char *pattern);
-bool maybe_skip(const std::string &impl_str);
 bool skip_start(res_t *res, int idx = benchdnn_stat.tests);
 
-typedef int (*bench_f)(int argc, char **argv);
+using bench_f = int (*)(int, char **);
 std::string locate_file(const std::string &fname);
 int batch(const char *fname, bench_f bench);
 
@@ -175,7 +180,9 @@ int batch(const char *fname, bench_f bench);
 int flip_coin(ptrdiff_t seed, float probability);
 
 int64_t div_up(const int64_t a, const int64_t b);
+size_t div_up(const size_t a, const size_t b);
 int64_t rnd_up(const int64_t a, const int64_t b);
+size_t rnd_up(const size_t a, const size_t b);
 int64_t next_pow2(int64_t a);
 int mxcsr_cvt(float f);
 
@@ -203,4 +210,5 @@ void print_dhw(bool &print_d, bool &print_h, bool &print_w, int ndims,
 int benchdnn_getenv_int(const char *name, int default_value);
 std::string benchdnn_getenv_string(const char *name);
 
+std::string smart_bytes(double bytes);
 #endif
