@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -71,15 +71,16 @@ struct attr_t {
         PER_DIM_2, // ... dims[2] point.
         PER_DIM_3, // ... dims[3] point.
         PER_TENSOR, // ... point in the tensor.
+        HOST_SCALAR, // same as COMMON, but uses host-side scalar memory
         POLICY_TOTAL // guard
     };
 
     static policy_t str2policy(const std::string &str);
     static const char *policy2str(policy_t policy);
-    static int get_default_mask(policy_t policy);
-    static int policy2mask(int arg, policy_t policy,
+    static int get_default_mask(policy_t policy, int ndims);
+    static int policy2mask(int arg, policy_t policy, int ndims = -1,
             dnnl_primitive_kind_t prim_kind = dnnl_undefined_primitive,
-            int ndims = -1, bool has_groups = false);
+            bool has_groups = false);
 
     struct zero_points_t {
         struct entry_t {
@@ -92,9 +93,11 @@ struct attr_t {
 
             bool is_def() const {
                 return policy == COMMON && value == 0 && dt == dnnl_s32
-                        && groups.size() == 0;
+                        && groups.empty();
             }
 
+            bool is_host_scalar() const { return policy == HOST_SCALAR; }
+
             policy_t policy = COMMON;
             int value = 0;
             dnnl_data_type_t dt = dnnl_s32;
@@ -118,6 +121,13 @@ struct attr_t {
             return def;
         }
 
+        bool has_host_scalars() const {
+            for (const auto &e : points) {
+                if (e.second.is_host_scalar()) return true;
+            }
+            return false;
+        }
+
         void set(int arg, policy_t policy, int value) {
             set(arg, entry_t(policy, value));
         }
@@ -138,13 +148,67 @@ struct attr_t {
                 int ndims = -1, bool has_groups = false) const {
             const auto &e = get(arg);
             return attr_t::policy2mask(
-                    arg, e.policy, prim_kind, ndims, has_groups);
+                    arg, e.policy, ndims, prim_kind, has_groups);
         }
 
-        zero_points_t() : points() {} // needed for debug icc190 build;
         std::map<int, entry_t> points;
     };
 
+    struct precomputed_reductions_t {
+        struct entry_t {
+            entry_t(policy_t apolicy = COMMON, dnnl_data_type_t adt = dnnl_s32,
+                    const std::vector<dnnl_dim_t> &agroups = {})
+                : policy(apolicy), dt(adt), groups(agroups) {}
+
+            int from_str(const std::string &s);
+
+            bool is_def() const {
+                return policy == COMMON && dt == dnnl_s32 && groups.empty();
+            }
+
+            policy_t policy = COMMON;
+            dnnl_data_type_t dt = dnnl_s32;
+            std::vector<dnnl_dim_t> groups;
+        };
+
+        int from_str(const std::string &s);
+
+        bool is_def(int arg) const {
+            return entries.empty() || get(arg).is_def();
+        }
+        bool is_def() const {
+            if (entries.empty()) return true;
+
+            bool def = true;
+            for (const auto &e : entries) {
+                def = def && is_def(e.first);
+            }
+            return def;
+        }
+
+        void set(int arg, const entry_t &entry) {
+            if (!entry.is_def()) entries[arg] = entry;
+        }
+        void set(int arg, policy_t policy, dnnl_data_type_t data_type,
+                std::vector<dnnl_dim_t> &groups) {
+            set(arg, entry_t(policy, data_type, groups));
+        }
+        entry_t get(int arg) const {
+            const auto it = entries.find(arg);
+            return it == entries.end() ? entry_t() : it->second;
+        }
+
+        int get_mask(int arg,
+                dnnl_primitive_kind_t prim_kind = dnnl_undefined_primitive,
+                int ndims = -1, bool has_groups = false) const {
+            const auto &e = get(arg);
+            return attr_t::policy2mask(
+                    arg, e.policy, ndims, prim_kind, has_groups);
+        }
+
+        std::map<int, entry_t> entries;
+    };
+
     struct arg_scales_t {
         struct entry_t {
             entry_t(policy_t apolicy = COMMON, float ascale = 1.f,
@@ -156,16 +220,18 @@ struct attr_t {
 
             bool is_def() const {
                 return policy == COMMON && scale == 1.f && dt == dnnl_f32
-                        && groups.size() == 0;
+                        && groups.empty();
             }
 
+            bool is_host_scalar() const { return policy == HOST_SCALAR; }
+
             policy_t policy = COMMON;
             float scale = 1.f;
             dnnl_data_type_t dt = dnnl_f32;
             std::vector<dnnl_dim_t> groups;
         };
 
-        void set(int arg, entry_t scale) { scales[arg] = scale; }
+        void set(int arg, const entry_t &scale) { scales[arg] = scale; }
 
         entry_t get(int arg) const {
             const auto &s = scales.find(arg);
@@ -177,7 +243,7 @@ struct attr_t {
                 int ndims = -1, bool has_groups = false) const {
             const auto &e = get(arg);
             return attr_t::policy2mask(
-                    arg, e.policy, prim_kind, ndims, has_groups);
+                    arg, e.policy, ndims, prim_kind, has_groups);
         }
 
         bool is_def(int arg) const {
@@ -192,10 +258,14 @@ struct attr_t {
             }
             return def;
         }
+        bool has_host_scalars() const {
+            for (const auto &e : scales) {
+                if (e.second.is_host_scalar()) return true;
+            }
+            return false;
+        }
         int from_str(const std::string &s);
 
-        arg_scales_t() : scales() {} // needed for debug icc190 build;
-
         std::map<int, entry_t> scales;
     };
 
@@ -274,6 +344,7 @@ struct attr_t {
             MIN,
             MUL,
             NE,
+            SELECT,
             SUB,
             BINARY_END, // a guard to check kind is binary
             // prelu
@@ -326,6 +397,14 @@ struct attr_t {
                 int64_t mask = -1;
                 mask_input_t mask_input = mask_input_t::none;
                 std::string tag = tag::any;
+
+                // For the src2 tensor when the algorithm takes ternary inputs
+                dnnl_data_type_t src2_dt = dnnl_data_type_undef;
+                policy_t src2_policy = policy_t::COMMON;
+                int64_t src2_mask = -1;
+                mask_input_t src2_mask_input = mask_input_t::none;
+                std::string src2_tag = tag::any;
+
             } binary;
             struct {
                 policy_t policy = policy_t::COMMON;
@@ -335,10 +414,11 @@ struct attr_t {
             bool is_convolution_kind() const;
             bool is_eltwise_kind() const;
             bool is_binary_kind() const;
+            bool is_binary_kind_with_ternary_op() const;
             bool is_prelu_kind() const;
         };
 
-        post_ops_t() : entry() {}
+        post_ops_t() = default;
 
         int len() const { return (int)entry.size(); }
         bool is_def() const { return len() == 0; }
@@ -353,9 +433,9 @@ struct attr_t {
         // on the ndims. Currently this includes only lnorm with binary post-op
         // with policy PER_OC.
         // Some primitives might have a special handling for a policy provided.
-        // For such primitives prim_kind must be set so get_po_masks generates
+        // For such primitives prim_kind must be set so get_po_masks(prb->ndims) generates
         // a correct mask. Currently this behavior depends on policy2mask().
-        std::vector<std::pair<int, int>> get_po_masks(int ndims = -1,
+        std::vector<std::pair<int, int>> get_po_masks(int ndims,
                 dnnl_primitive_kind_t prim_kind
                 = dnnl_undefined_primitive) const;
 
@@ -411,6 +491,9 @@ struct attr_t {
 
     void insert(const arg_scales_t &as) { this->scales = as; }
     void insert(const zero_points_t &zp) { this->zero_points = zp; }
+    void insert(const precomputed_reductions_t &pr) {
+        this->precomputed_reductions = pr;
+    }
     void insert(const post_ops_t &po) { this->post_ops = po; }
     void insert(dnnl_scratchpad_mode_t sm) { this->scratchpad_mode = sm; }
     void insert(const fpmath_mode_t &fpm) { this->fpmath_mode = fpm; }
@@ -432,6 +515,7 @@ struct attr_t {
 
     arg_scales_t scales;
     zero_points_t zero_points;
+    precomputed_reductions_t precomputed_reductions;
     post_ops_t post_ops;
     dnnl_scratchpad_mode_t scratchpad_mode;
     fpmath_mode_t fpmath_mode;
@@ -440,7 +524,7 @@ struct attr_t {
     dropout_t dropout;
     rounding_mode_t rounding_mode;
 
-    bool is_def(bool skip_fpmath = false) const;
+    bool is_def(bool skip_fpmath = false, bool skip_acc_mode = false) const;
 };
 
 struct isa_hints_t {
@@ -459,7 +543,7 @@ struct isa_hints_t {
     cpu_hints_t hints_;
     isa_hints_t(cpu_hints_t hints) : hints_(hints) {}
 
-    cpu_hints_t get() { return hints_; }
+    cpu_hints_t get() const { return hints_; }
 
     static std::string hints2str(const isa_hints_t &isa_hints) {
         switch (isa_hints.hints_) {
@@ -484,7 +568,6 @@ struct isa_hints_t {
 
 using policy_t = attr_t::policy_t;
 
-#ifdef DNNL_EXPERIMENTAL_SPARSE
 struct sparse_options_t {
     static constexpr dnnl_sparse_encoding_t def_encoding
             = dnnl_sparse_encoding_undef;
@@ -503,6 +586,17 @@ struct sparse_options_t {
         if (options_.count(arg) == 0) return dnnl_sparse_encoding_undef;
         return options_.at(arg).first;
     }
+    dnnl_sparse_encoding_t get_encoding(data_kind_t kind) const {
+        // Note: the commented code doesn't work as `arg` returned is a
+        // backward exec_arg. See the function comment.
+        // const auto arg = data_kind2exec_arg(kind);
+        // return get_encoding(arg);
+        switch (kind) {
+            case SRC: return get_encoding(DNNL_ARG_SRC);
+            case WEI: return get_encoding(DNNL_ARG_WEIGHTS);
+            default: return def_encoding;
+        }
+    }
 
     float get_sparsity(int arg) const {
         if (options_.count(arg) == 0) return 0.0f;
@@ -528,6 +622,7 @@ struct sparse_options_t {
 
     std::vector<int> get_args() const {
         std::vector<int> args;
+        args.reserve(options_.size());
         for (const auto &opt : options_) {
             args.push_back(opt.first);
         }
@@ -542,11 +637,11 @@ private:
 
 std::ostream &operator<<(
         std::ostream &s, const sparse_options_t &sparse_options);
-#endif
-
 std::ostream &operator<<(std::ostream &s, const policy_t &policy);
 std::ostream &operator<<(
         std::ostream &s, const attr_t::zero_points_t &zero_points);
+std::ostream &operator<<(std::ostream &s,
+        const attr_t::precomputed_reductions_t &precomputed_reductions);
 std::ostream &operator<<(std::ostream &s, const attr_t::arg_scales_t &scales);
 std::ostream &operator<<(std::ostream &s, const attr_t::post_ops_t::kind_t &k);
 std::ostream &operator<<(std::ostream &s, const attr_t::post_ops_t &post_ops);
@@ -567,12 +662,8 @@ struct attr_args_t {
 
     attr_args_t() = default;
 
-    void prepare_scales(const attr_t &attr, int arg, int mask = -1) {
-        entries.insert(std::make_pair(DNNL_ARG_ATTR_SCALES | arg, mask));
-    };
-
-    void prepare_zero_points(const attr_t &attr, int arg, int mask = -1) {
-        entries.insert(std::make_pair(DNNL_ARG_ATTR_ZERO_POINTS | arg, mask));
+    void prepare_quant(const attr_t &attr, int arg, int mask = -1) {
+        entries.insert(std::make_pair(arg, mask));
     };
 
     int prepare_post_ops_mds(const attr_t &attr, int ndims,
@@ -631,9 +722,9 @@ std::string trim_tag_by_mask(const std::string &tag, int mask);
 std::string normalize_tag(const std::string &tag, int ndims = -1);
 
 dnnl_primitive_attr_t create_dnnl_attr(
-        const attr_t &attr, const attr_args_t &attr_args);
+        const attr_t &attr, const attr_args_t &attr_args, int ndims);
 
-dnnl_engine_kind_t str2engine_kind(const char *str);
+dnnl_engine_kind_t str2engine_kind(const std::string &s);
 dnnl_scratchpad_mode_t str2scratchpad_mode(const char *str);
 dnnl_fpmath_mode_t str2fpmath_mode(const char *str);
 dnnl_accumulation_mode_t str2accumulation_mode(const char *str);
@@ -649,7 +740,8 @@ float compute_eltwise_fwd(
         attr_t::post_ops_t::kind_t kind, float src, float alpha, float beta);
 float compute_eltwise_bwd(attr_t::post_ops_t::kind_t kind, float d_dst,
         float src, float alpha, float beta);
-float compute_binary(attr_t::post_ops_t::kind_t kind, float src0, float src1);
+float compute_binary(
+        attr_t::post_ops_t::kind_t kind, float src0, float src1, bool src2);
 void maybe_dropout(const attr_t &attr, float &val, int64_t offset,
         const dnn_mem_t &dropout);
 void maybe_round(const attr_t &attr, int arg, float &val, int64_t offset,
@@ -661,9 +753,5 @@ inline void maybe_post_ops(
     maybe_post_ops(attr, val, sum_val, std::vector<float>());
 }
 
-// When using fast-ref option, reference expects everything to be in f32
-// data type and also no additional memories coming from runtime attributes.
-// That's why we update all data types to f32 and remove all runtime arguments
-// to makes them constant when possible.
-void update_cpu_ref_attrs(attr_t &attr, dnnl_data_type_t new_dt = dnnl_f32);
+void update_cpu_ref_attrs(attr_t &attr, dnnl_data_type_t dst_dt);
 #endif
