@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@
 #include "dnnl_memory.hpp"
 #include "utils/cold_cache.hpp"
 #include "utils/dnnl_query.hpp"
+#include "utils/memory.hpp"
 #include "utils/parallel.hpp"
 
 extern "C" dnnl_status_t dnnl_memory_desc_create_with_string_tag(
@@ -49,48 +50,79 @@ extern "C" dnnl_status_t dnnl_memory_desc_set_data_type(
         dnnl_memory_desc_t memory_desc, dnnl_data_type_t data_type);
 
 dnn_mem_t::dnn_mem_t(const_dnnl_memory_desc_t md, dnnl_engine_t engine,
-        const handle_info_t &handle_info) {
+        bool prefill, const handle_info_t &handle_info) {
     if (query_md_ndims(md) > 0) {
         auto status = dnnl_memory_desc_clone(&md_, md);
         (void)status;
         assert(status == dnnl_success);
-        active_ = (initialize(engine, handle_info) == OK);
+        active_ = (initialize(engine, prefill, handle_info) == OK);
     }
 }
 
+dnn_mem_t::dnn_mem_t(const_dnnl_memory_desc_t md, void *value) {
+    dnnl_format_kind_t format_kind = query_md_format_kind(md);
+    if (format_kind != dnnl_format_kind_host_scalar) { return; }
+
+    auto status = dnnl_memory_desc_clone(&md_, md);
+    (void)status;
+    assert(status == dnnl_success);
+
+    status = dnnl_memory_create_host_scalar(&m_, md_, value);
+    assert(status == dnnl_success);
+
+    // Map the memory for compatibility,
+    // allowing data to be accessed just like a regular memory object
+    map();
+
+    active_ = (status == dnnl_success);
+}
+
 dnn_mem_t::dnn_mem_t(const_dnnl_memory_desc_t md, dnnl_data_type_t dt,
-        const std::string &tag, dnnl_engine_t engine) {
+        const std::string &tag, dnnl_engine_t engine, bool prefill) {
     const int ndims = query_md_ndims(md);
     if (ndims > 0) {
         auto md_wrapper = dnn_mem_t::init_md(ndims, query_md_dims(md), dt, tag);
         md_ = md_wrapper.release();
-        active_ = (initialize(engine) == OK);
+        active_ = (initialize(engine, prefill) == OK);
+    }
+}
+
+dnn_mem_t::dnn_mem_t(const_dnnl_memory_desc_t md, dnnl_data_type_t dt,
+        const dnnl_dims_t strides, dnnl_engine_t engine, bool prefill) {
+    const int ndims = query_md_ndims(md);
+    if (ndims > 0) {
+        auto status = dnnl_memory_desc_create_with_strides(
+                &md_, ndims, query_md_dims(md), dt, strides);
+        (void)status;
+        assert(status == dnnl_success);
+        active_ = (initialize(engine, prefill) == OK);
     }
 }
 
 dnn_mem_t::dnn_mem_t(int ndims, const dnnl_dims_t dims, dnnl_data_type_t dt,
-        const std::string &tag, dnnl_engine_t engine) {
+        const std::string &tag, dnnl_engine_t engine, bool prefill) {
     if (ndims > 0) {
         auto md_wrapper = dnn_mem_t::init_md(ndims, dims, dt, tag);
         md_ = md_wrapper.release();
-        active_ = (initialize(engine) == OK);
+        active_ = (initialize(engine, prefill) == OK);
     }
 }
 
 dnn_mem_t::dnn_mem_t(int ndims, const dnnl_dims_t dims, dnnl_data_type_t dt,
-        const dnnl_dims_t strides, dnnl_engine_t engine) {
+        const dnnl_dims_t strides, dnnl_engine_t engine, bool prefill) {
     if (ndims > 0) {
         auto status = dnnl_memory_desc_create_with_strides(
                 &md_, ndims, dims, dt, strides);
         (void)status;
         assert(status == dnnl_success);
-        active_ = (initialize(engine) == OK);
+        active_ = (initialize(engine, prefill) == OK);
     }
 }
 
 dnn_mem_t::dnn_mem_t(const dnn_mem_t &rhs, dnnl_data_type_t dt,
         const std::string &tag, dnnl_engine_t engine)
-    : dnn_mem_t(rhs.md_, dt, tag, engine) {
+    : dnn_mem_t(rhs.md_, dt, tag, engine, /* prefill = */ true) {
+    // Prefill is `true` unconditionally because of reorder involved.
     if (active_) {
         int status = reorder(rhs);
         if (status != OK) {
@@ -137,16 +169,47 @@ int execute_reorder(const dnn_mem_t &src, dnn_mem_t &dst,
     }
 #endif
 
-    if (!r_pd_) {
-        DNN_SAFE(dnnl_reorder_primitive_desc_create(&r_pd_, src.md_,
-                         src.engine(), dst.md_, dst.engine(), attr),
-                WARN);
+    while (!r_pd_) {
+        // Fallback to GPU reorder.
+        auto status = dnnl_reorder_primitive_desc_create(
+                &r_pd_, src.md_, src.engine(), dst.md_, dst.engine(), attr);
+        if (status == dnnl_success) break;
+        if (dnnl_memory_desc_equal(src.md_, dst.md_)) {
+            // If fail to create reorder pd, use plain data copy for identical
+            // mds.
+            BENCHDNN_PRINT(2, "%s\n", "[REORDER] Fallback to plain copy.");
+            const int64_t chunk_size = 64;
+            const int64_t n_chunks = div_up(src.nelems(), chunk_size);
+            benchdnn_parallel_nd(n_chunks, [&](int64_t idx_chunk) {
+                int64_t idx_start = idx_chunk * chunk_size;
+                int64_t idx_end = MIN2(idx_start + chunk_size, src.nelems());
+                for (int64_t idx = idx_start; idx < idx_end; ++idx) {
+                    float e = src.get_elem(idx);
+                    dst.set_elem(idx, e);
+                }
+            });
+            return OK;
+        }
+        return FAIL;
     }
+
     auto r_pd = make_benchdnn_dnnl_wrapper(r_pd_);
     const auto &scratchpad_md = query_md(r_pd, DNNL_ARG_SCRATCHPAD);
     const auto &scratchpad_engine
             = dst.engine_kind() == dnnl_gpu ? dst.engine() : src.engine();
-    dnn_mem_t scratchpad(scratchpad_md, scratchpad_engine);
+
+    // Scratchpad memory will be mapped at the call below (if `attr` utilizes
+    // user-mode scratchpad) or inside `execute_and_wait` (for the library-mode)
+    // in the scenario when cpu2gpu or pure gpu reorder will be called (e.g.,
+    // for f64). This might trigger an OOM issue if the amount of previously
+    // allocated memory has already been close to the upper limit, and the large
+    // amount for scratchpad is requested on top of that amount.
+    //
+    // It might be worked around but it will require extra external management
+    // to prevent it. It's decided to increase the amount of "safety pillow"
+    // memory in `check_total_size` instead to save on engineering efforts.
+    dnn_mem_t scratchpad(
+            scratchpad_md, scratchpad_engine, /* prefill = */ true);
 
     DNN_SAFE(dnnl_primitive_create(&prim_, r_pd), CRIT);
     auto prim = make_benchdnn_dnnl_wrapper(prim_);
@@ -169,10 +232,15 @@ int dnn_mem_t::reorder(const dnn_mem_t &rhs, const_dnnl_primitive_attr_t attr,
     // Do nothing, return a good status. Keep here to avoid guarding externally.
     if (query_md_ndims(rhs.md_) == 0) return OK;
 
-    // Assumption is `no_host_memory` assigned values at construction, and no
+    // Assumption is `no_ref_memory` assigned values at construction, and no
     // actual reorder needed. This check is to avoid extra code outside of
     // reorder interface.
-    if (has_bench_mode_modifier(mode_modifier_t::no_host_memory)) return OK;
+    // Note: pass sparse reorder due to the same reason as described under
+    // `FILL_SPARSE_METADATA`.
+    const bool mem_has_indirect_access = is_sparse_md();
+    if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)
+            && !mem_has_indirect_access)
+        return OK;
 
     const bool do_swap_dt = swap_dt != dnnl_data_type_undef;
     dnnl_data_type_t orig_dt = this->dt();
@@ -182,8 +250,12 @@ int dnn_mem_t::reorder(const dnn_mem_t &rhs, const_dnnl_primitive_attr_t attr,
     return status;
 }
 
-size_t dnn_mem_t::size() const {
-    return dnnl_memory_desc_get_size(md_);
+size_t dnn_mem_t::size(int index) const {
+    return dnnl_memory_desc_get_size_v2(md_, index);
+}
+
+bool dnn_mem_t::is_sparse_md() const {
+    return query_md_sparse_encoding(md_) != dnnl_sparse_encoding_undef;
 }
 
 size_t dnn_mem_t::sizeof_dt() const {
@@ -216,17 +288,27 @@ float dnn_mem_t::get_elem(int64_t idx, int buffer_index) const {
             elem = static_cast<dnnl::impl::float8_e4m3_t *>(data)[idx];
             break;
         case dnnl_s4: {
-            auto half = idx % 2 ? dnnl::impl::int4_extract_t::high_half
-                                : dnnl::impl::int4_extract_t::low_half;
-            elem = static_cast<float>(dnnl::impl::int4_t::extract(
-                    static_cast<uint8_t *>(data)[idx / 2], half));
+            dnnl::impl::nibble2_t nibble_pair(
+                    reinterpret_cast<uint8_t *>(data)[idx / 2]);
+            elem = dnnl::impl::int4_t(nibble_pair.get(idx % 2));
             break;
         }
         case dnnl_u4: {
-            auto half = idx % 2 ? dnnl::impl::int4_extract_t::high_half
-                                : dnnl::impl::int4_extract_t::low_half;
-            elem = static_cast<float>(dnnl::impl::uint4_t::extract(
-                    static_cast<uint8_t *>(data)[idx / 2], half));
+            dnnl::impl::nibble2_t nibble_pair(
+                    reinterpret_cast<uint8_t *>(data)[idx / 2]);
+            elem = dnnl::impl::uint4_t(nibble_pair.get(idx % 2));
+            break;
+        }
+        case dnnl_f4_e2m1: {
+            dnnl::impl::nibble2_t nibble_pair(
+                    reinterpret_cast<uint8_t *>(data)[idx / 2]);
+            elem = dnnl::impl::float4_e2m1_t(nibble_pair.get(idx % 2));
+            break;
+        }
+        case dnnl_f4_e3m0: {
+            dnnl::impl::nibble2_t nibble_pair(
+                    reinterpret_cast<uint8_t *>(data)[idx / 2]);
+            elem = dnnl::impl::float4_e3m0_t(nibble_pair.get(idx % 2));
             break;
         }
         default: assert(!"bad data type");
@@ -253,25 +335,97 @@ void dnn_mem_t::set_elem(int64_t idx, float value, int buffer_index) const {
             ((dnnl::impl::float8_e4m3_t *)data)[idx] = value;
             break;
         case dnnl_s4: {
-            using type = dnnl::impl::int4_t;
-            auto half = idx % 2 ? dnnl::impl::int4_extract_t::high_half
-                                : dnnl::impl::int4_extract_t::low_half;
-            uint8_t dst_val = ((uint8_t *)data)[idx / 2];
-            ((type *)data)[idx / 2] = type(value).insert(dst_val, half);
+            auto dst_val = ((dnnl::impl::nibble2_t *)data)[idx / 2];
+            dst_val.set(dnnl::impl::int4_t(value).raw_bits_, idx % 2);
+            ((dnnl::impl::nibble2_t *)data)[idx / 2] = dst_val;
             break;
         }
         case dnnl_u4: {
-            using type = dnnl::impl::uint4_t;
-            auto half = idx % 2 ? dnnl::impl::int4_extract_t::high_half
-                                : dnnl::impl::int4_extract_t::low_half;
-            uint8_t dst_val = ((uint8_t *)data)[idx / 2];
-            ((type *)data)[idx / 2] = type(value).insert(dst_val, half);
+            auto dst_val = ((dnnl::impl::nibble2_t *)data)[idx / 2];
+            dst_val.set(dnnl::impl::uint4_t(value).raw_bits_, idx % 2);
+            ((dnnl::impl::nibble2_t *)data)[idx / 2] = dst_val;
+            break;
+        }
+        case dnnl_f4_e2m1: {
+            auto dst_val = ((dnnl::impl::nibble2_t *)data)[idx / 2];
+            dst_val.set(dnnl::impl::float4_e2m1_t(value).raw_bits_, idx % 2);
+            ((dnnl::impl::nibble2_t *)data)[idx / 2] = dst_val;
+            break;
+        }
+        case dnnl_f4_e3m0: {
+            auto dst_val = ((dnnl::impl::nibble2_t *)data)[idx / 2];
+            dst_val.set(dnnl::impl::float4_e3m0_t(value).raw_bits_, idx % 2);
+            ((dnnl::impl::nibble2_t *)data)[idx / 2] = dst_val;
             break;
         }
         default: assert(!"bad data type");
     }
 }
 
+// Returns an updated logical index based on input `logical_index` and
+// `dims_mask`.
+// `logical_idx` is a generally composed index where dims[ndims - 1] is most
+// dense and dims[0] is least dense dimensions, as tensor in `abx` format.
+// `dims_mask` represents dimensions to keep or remove. A value is composed of
+// number of bits equal to `ndims` and value of `1` indicates the dimension to
+// present in final calculation. E.g., mask=0 means a single value, and mask=2,
+// or 1 << 1, means to keep dims[1] only.
+// `ndims` allows to reduce the number of dimensions from innermost direction.
+// It is used to find an index in batched dimensions. E.g., if ndims() returns
+// `4`, but `ndims` argument is passed as `2`, it will count only first two
+// dimensions instead of all 4.
+// `groups` is an extension of scales when their dimensions are different.
+// In this case, it's required to adjust dimensions values according to group
+// values to properly compute stride in a smaller tensor. The definition is
+// aligned with the API and expects groups of size 2 or empty.
+// When `ndims()/ndims` are bigger than 2, groups are applied only to two most
+// dense dimensions, which is aligned with 2D matmul definition. E.g., dims=2x6,
+// mask=3, ndims=2 and groups=1x3; in this case indices 0,1,2 will return 0 as
+// those indices represent the first group. 3,4,5 will return 1. Changing the
+// example like dims=6x2, mask=3, ndims=2 and groups=3x1 will change ouput as
+// well due to groups are dense not over the last dimension. The match will look
+// like this: 0->0, 1->1, 2->0, 3->1, ..., 6->2, ..., 11->4.
+//
+// Helps to find an index of smaller tensor in bigger one, e.g., scales. Scales
+// are usually represented by a 1D array and have a mask indicating dims to be
+// applied for. It coincides with `dims_mask`.
+// Another usage is to identify an index for operations that support broadcast
+// over different dimensions, e.g., matmul batch dimensions, or binary
+// primitive.
+//
+// Example: there's a tensor 3x4 and scales with mask=2, which means to apply
+// scales over the dimension of 4. Passing indices from 0 to 3 will return
+// values from 0 to 3 correspondently. However, passing `logical_idx` of 4 will
+// return 0, as 4 is a logical representation of point 1x0.
+int64_t dnn_mem_t::get_idx(int64_t logical_idx, int dims_mask, const int ndims,
+        const dims_t &groups) const {
+    if (dims_mask == 0) return 0;
+
+    const auto &dims = this->dims();
+    int64_t stride = 1;
+    int64_t offset = 0;
+
+    assert(groups.empty() || groups.size() == 2);
+    assert(groups.size() <= static_cast<size_t>(ndims));
+    dims_t groups_ext(ndims, 1);
+    if (!groups.empty()) {
+        groups_ext[ndims - 2] = groups[0];
+        groups_ext[ndims - 1] = groups[1];
+    }
+
+    for (int i = 0; i < ndims; ++i) {
+        int d = ndims - 1 - i;
+        auto pos = logical_idx % dims[d];
+        logical_idx /= dims[d];
+        if (dims_mask & (1 << d)) {
+            offset += (pos / groups_ext[d]) * stride;
+            stride *= (dims[d] / groups_ext[d]);
+        }
+    }
+
+    return offset;
+}
+
 // Creates a memory object from the underlying buffer of an existing memory
 // object `mem`. The size of `mem` must not be less than the size of `md`.
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL || defined(DNNL_WITH_SYCL)
@@ -284,50 +438,45 @@ static int init_memory(
     bool is_sycl = is_sycl_engine(engine);
     bool is_opencl = is_opencl_engine(engine);
 
+    if (ret == nullptr) return FAIL;
     *ret = nullptr;
 
     const int nhandles = query_md_num_handles(md);
     std::vector<void *> handles(nhandles);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
     for (int i = 0; i < nhandles; i++)
         DNN_SAFE(dnnl_memory_get_data_handle_v2(mem, &handles[i], i), CRIT);
-#else
-    DNN_SAFE(dnnl_memory_get_data_handle(mem, &handles[0]), CRIT);
-#endif
 
     if (is_opencl) {
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
         dnnl_ocl_interop_memory_kind_t mem_kind;
         DNN_SAFE(dnnl_ocl_interop_memory_get_memory_kind(mem, &mem_kind), CRIT);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         DNN_SAFE(dnnl_ocl_interop_memory_create_v2(ret, md, engine, mem_kind,
                          (int)handles.size(), handles.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_ocl_interop_memory_create(
-                         ret, md, engine, mem_kind, handles[0]),
-                CRIT);
-#endif
 #endif
     } else if (is_sycl) {
 #ifdef DNNL_WITH_SYCL
         dnnl_sycl_interop_memory_kind_t mem_kind;
         DNN_SAFE(
                 dnnl_sycl_interop_memory_get_memory_kind(mem, &mem_kind), CRIT);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         DNN_SAFE(dnnl_sycl_interop_memory_create_v2(ret, md, engine, mem_kind,
                          (int)handles.size(), handles.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_sycl_interop_memory_create(
-                         ret, md, engine, mem_kind, handles[0]),
-                CRIT);
-#endif
 #endif
     }
 
     // Memory must be initialized at this point in some of the branches above.
-    if (!*ret) assert(!"not expected");
+    if (!*ret) { assert(!"not expected"); }
+
+    // Register device memory
+    for (int i = 0; i < nhandles; i++) {
+        size_t sz = dnnl_memory_desc_get_size_v2(md, i);
+        if (*ret) {
+            DNN_SAFE(
+                    dnnl_memory_get_data_handle_v2(*ret, &handles[i], i), CRIT);
+        }
+        memory_registry_t::get_instance().add_device(handles[i], sz);
+    }
 
     return OK;
 }
@@ -342,21 +491,17 @@ void dnn_mem_t::map() const {
     const int nhandles = query_md_num_handles(md_);
     mapped_ptrs_.resize(nhandles);
     for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         auto st = dnnl_memory_map_data_v2(mem, &mapped_ptrs_[i], i);
-#else
-        auto st = dnnl_memory_map_data(mem, &mapped_ptrs_[i]);
-#endif
         if (st != dnnl_success) {
-            for (int j = 0; j < i; j++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
+            for (int j = 0; j < i; j++)
                 DNN_SAFE_V(dnnl_memory_unmap_data_v2(mem, mapped_ptrs_[i], i));
-#else
-                DNN_SAFE_V(dnnl_memory_unmap_data(mem, mapped_ptrs_[i]));
-#endif
-            }
             DNN_SAFE_V(st);
         }
+        // Register a mapped memory entry if a different pointer from `data_`
+        // was returned.
+        if (IMPLICATION(!data_.empty(), data_[i] != mapped_ptrs_[i]))
+            memory_registry_t::get_instance().add_mapped(
+                    mapped_ptrs_[i], size(i));
     }
 }
 
@@ -368,21 +513,24 @@ void dnn_mem_t::unmap() const {
     auto mem = m_padded_ ? m_padded_ : m_;
     const int nhandles = query_md_num_handles(md_);
     for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
+        // Unregister a mapped memory entry if a different pointer from `data_`
+        // was returned when mapped.
+        if (IMPLICATION(!data_.empty(), data_[i] != mapped_ptrs_[i]))
+            memory_registry_t::get_instance().remove_mapped(
+                    mapped_ptrs_[i], size(i));
+
         DNN_SAFE_V(dnnl_memory_unmap_data_v2(mem, mapped_ptrs_[i], i));
-#else
-        DNN_SAFE_V(dnnl_memory_unmap_data(mem, mapped_ptrs_[i]));
-#endif
         mapped_ptrs_[i] = nullptr;
     }
 }
 
-void dnn_mem_t::memset(int value, size_t size) const {
+void dnn_mem_t::memset(int value, size_t size, int buffer_index) const {
     bool is_opencl = is_opencl_engine(engine_);
     bool is_sycl = is_sycl_engine(engine_);
     auto mem = m_padded_ ? m_padded_ : m_;
     void *mem_handle;
-    DNN_SAFE_V(dnnl_memory_get_data_handle(mem, &mem_handle));
+    DNN_SAFE_V(dnnl_memory_get_data_handle_v2(mem, &mem_handle, buffer_index));
+
     if (is_opencl) {
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
         stream_t stream(engine_);
@@ -419,12 +567,7 @@ void dnn_mem_t::memset(int value, size_t size) const {
                 auto &buf = *static_cast<::sycl::buffer<uint8_t, 1> *>(
                         mem_handle);
                 queue.submit([&](::sycl::handler &cgh) {
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-                    constexpr auto target_device
-                            = ::sycl::target::global_buffer;
-#else
                     constexpr auto target_device = ::sycl::target::device;
-#endif
                     ::sycl::accessor<uint8_t, 1, ::sycl::access::mode::write,
                             target_device>
                             acc(buf, cgh);
@@ -454,7 +597,8 @@ void dnn_mem_t::memset(int value, size_t size) const {
 
 dnn_mem_t dnn_mem_t::create_from_host_ptr(
         const dnnl_memory_desc_t &md, dnnl_engine_t engine, void *host_ptr) {
-    return dnn_mem_t(md, engine, {true, host_ptr});
+    // Pre-allocated handle_info won't use prefill no matter what.
+    return dnn_mem_t(md, engine, /* prefill = */ false, {true, host_ptr});
 }
 
 size_t dnn_mem_t::pad_memory_size(
@@ -464,18 +608,17 @@ size_t dnn_mem_t::pad_memory_size(
             || engine_kind == dnnl_cpu)
         return sz;
 
-    const int pad_size = 4096;
-    if (was_padded) *was_padded = true;
-    return sz + pad_size;
+    const size_t page_size = 4096;
+    auto padded_sz = rnd_up(sz, page_size);
+    if (was_padded) *was_padded = padded_sz != sz;
+    return padded_sz;
 }
 
 dnnl_memory_desc_t dnn_mem_t::pad_memory_desc(const_dnnl_memory_desc_t md,
         dnnl_engine_kind_t engine_kind, bool *was_padded) {
     if (was_padded) *was_padded = false;
-#ifdef DNNL_EXPERIMENTAL_SPARSE
     // TODO: add padded memory descriptor support for sparse memory.
     if (query_md_format_kind(md) == dnnl_format_kind_sparse) return nullptr;
-#endif
     size_t old_sz = dnnl_memory_desc_get_size(md);
     if (old_sz == 0 || !has_bench_mode_bit(mode_bit_t::corr)
             || engine_kind == dnnl_cpu)
@@ -498,7 +641,7 @@ benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_md(int ndims,
     const bool use_strides = !strides_.empty();
     // Ignore tag_ in case strides_ are explicitly provided
     if (use_strides) {
-        std::vector<dnnl_dim_t> strides(strides_);
+        const std::vector<dnnl_dim_t> &strides(strides_);
         DNN_SAFE_V(dnnl_memory_desc_create_with_strides(
                 &md, ndims, dims, data_type, strides.data()));
         return md;
@@ -520,7 +663,6 @@ benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_md(int ndims,
     return md;
 }
 
-#ifdef DNNL_EXPERIMENTAL_SPARSE
 benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_csr_md(int ndims,
         const dnnl_dims_t dims, dnnl_data_type_t data_type, dnnl_dim_t nnz,
         dnnl_data_type_t indices_dt, dnnl_data_type_t pointers_dt) {
@@ -547,24 +689,32 @@ benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_sparse_packed_md(
             &md, ndims, dims, data_type, nnz));
     return md;
 }
-#endif
+
+benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_host_scalar_md(
+        dnnl_data_type_t data_type) {
+    dnnl_memory_desc_t md {};
+    DNN_SAFE_V(dnnl_memory_desc_create_host_scalar(&md, data_type));
+    return md;
+}
 
 int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
 #ifdef DNNL_WITH_SYCL
     if (handle_info.is_host_ptr) {
         // Ignore memory_kind with host pointers and force USM.
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         const int nhandles = query_md_num_handles(md_);
         std::vector<void *> handles(nhandles, handle_info.ptr);
         DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_, md_, engine_,
                          dnnl_sycl_interop_usm, (int)handles.size(),
                          handles.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_sycl_interop_memory_create(&m_, md_, engine_,
-                         dnnl_sycl_interop_usm, handle_info.ptr),
-                CRIT);
-#endif
+
+        // Register device memory, happens on CPU SYCL path.
+        for (int i = 0; i < nhandles; i++) {
+            size_t sz = dnnl_memory_desc_get_size_v2(md_, i);
+            DNN_SAFE(dnnl_memory_get_data_handle_v2(m_, &handles[i], i), CRIT);
+            memory_registry_t::get_instance().add_device(handles[i], sz);
+        }
+
         return OK;
     }
 
@@ -578,18 +728,12 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
                     = (memory_kind == memory_kind_ext_t::usm
                                     ? dnnl_sycl_interop_usm
                                     : dnnl_sycl_interop_buffer);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             const int nhandles = query_md_num_handles(md_);
             std::vector<void *> handles(nhandles, handle_info.ptr);
             DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_padded_, md_padded,
                              engine_, mem_kind, (int)handles.size(),
                              handles.data()),
                     CRIT);
-#else
-            DNN_SAFE(dnnl_sycl_interop_memory_create(&m_padded_, md_padded,
-                             engine_, mem_kind, handle_info.ptr),
-                    CRIT);
-#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -604,11 +748,7 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
 
             const int nhandles = query_md_num_handles(md_);
             for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
                 size_t sz = dnnl_memory_desc_get_size_v2(md_padded, i);
-#else
-                size_t sz = dnnl_memory_desc_get_size(md_padded);
-#endif
                 if (memory_kind == memory_kind_ext_t::usm_device) {
                     data_.push_back(::sycl::malloc_device(sz, dev, ctx));
                 } else {
@@ -620,16 +760,10 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
                     DNN_SAFE(dnnl_out_of_memory, CRIT);
                 }
             }
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_padded_, md_padded,
                              engine_, dnnl_sycl_interop_usm, (int)data_.size(),
                              data_.data()),
                     CRIT);
-#else
-            DNN_SAFE(dnnl_sycl_interop_memory_create(&m_padded_, md_padded,
-                             engine_, dnnl_sycl_interop_usm, data_[0]),
-                    CRIT);
-#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -648,18 +782,12 @@ int dnn_mem_t::initialize_memory_create_opencl(
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
     if (handle_info.is_host_ptr) {
         // Ignore memory_kind with host pointers and force USM.
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         const int nhandles = query_md_num_handles(md_);
         std::vector<void *> handles(nhandles, handle_info.ptr);
         DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_, md_, engine_,
                          dnnl_ocl_interop_usm, (int)handles.size(),
                          handles.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_ocl_interop_memory_create(&m_, md_, engine_,
-                         dnnl_ocl_interop_usm, handle_info.ptr),
-                CRIT);
-#endif
         return OK;
     }
 
@@ -675,18 +803,12 @@ int dnn_mem_t::initialize_memory_create_opencl(
                     = (memory_kind == memory_kind_ext_t::usm
                                     ? dnnl_ocl_interop_usm
                                     : dnnl_ocl_interop_buffer);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             const int nhandles = query_md_num_handles(md_);
             std::vector<void *> handles(nhandles, handle_info.ptr);
             DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_padded_, md_padded,
                              engine_, mem_kind, (int)handles.size(),
                              handles.data()),
                     CRIT);
-#else
-            DNN_SAFE(dnnl_ocl_interop_memory_create(&m_padded_, md_padded,
-                             engine_, mem_kind, handle_info.ptr),
-                    CRIT);
-#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -696,11 +818,7 @@ int dnn_mem_t::initialize_memory_create_opencl(
 
             const int nhandles = query_md_num_handles(md_);
             for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
                 size_t sz = dnnl_memory_desc_get_size_v2(md_padded, i);
-#else
-                size_t sz = dnnl_memory_desc_get_size(md_padded);
-#endif
                 if (memory_kind == memory_kind_ext_t::usm_device) {
                     data_.push_back(dnnl::impl::xpu::ocl::usm::malloc_device(
                             engine_, sz));
@@ -715,16 +833,10 @@ int dnn_mem_t::initialize_memory_create_opencl(
                     DNN_SAFE(dnnl_out_of_memory, CRIT);
                 }
             }
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_padded_, md_padded,
                              engine_, dnnl_ocl_interop_usm, (int)data_.size(),
                              data_.data()),
                     CRIT);
-#else
-            DNN_SAFE(dnnl_ocl_interop_memory_create(&m_padded_, md_padded,
-                             engine_, dnnl_ocl_interop_usm, data_[0]),
-                    CRIT);
-#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -754,11 +866,7 @@ int dnn_mem_t::initialize_memory_create(const handle_info_t &handle_info) {
 
         const int nhandles = query_md_num_handles(md_);
         for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             size_t sz = dnnl_memory_desc_get_size_v2(md_, i);
-#else
-            size_t sz = dnnl_memory_desc_get_size(md_);
-#endif
             data_.push_back(zmalloc(sz, alignment));
         }
         if (std::any_of(
@@ -767,13 +875,9 @@ int dnn_mem_t::initialize_memory_create(const handle_info_t &handle_info) {
                 zfree(p);
             DNN_SAFE(dnnl_out_of_memory, CRIT);
         }
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         DNN_SAFE(dnnl_memory_create_v2(
                          &m_, md_, engine_, (int)data_.size(), data_.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_memory_create(&m_, md_, engine_, data_[0]), CRIT);
-#endif
 
     } else if (is_sycl) {
         SAFE(initialize_memory_create_sycl(handle_info), CRIT);
@@ -783,19 +887,15 @@ int dnn_mem_t::initialize_memory_create(const handle_info_t &handle_info) {
         is_data_owner_ = false;
         const int nhandles = query_md_num_handles(md_);
         std::vector<void *> handles(nhandles, handle_info.ptr);
-#ifdef DNNL_EXPERIMENTAL_SPARSE
         DNN_SAFE(dnnl_memory_create_v2(&m_, md_, engine_, (int)handles.size(),
                          handles.data()),
                 CRIT);
-#else
-        DNN_SAFE(dnnl_memory_create(&m_, md_, engine_, handles[0]), CRIT);
-#endif
     }
     return OK;
 }
 
 int dnn_mem_t::initialize(
-        dnnl_engine_t engine, const handle_info_t &handle_info) {
+        dnnl_engine_t engine, bool prefill, const handle_info_t &handle_info) {
     is_mapped_ = false;
     engine_ = engine;
     engine_kind_ = query_engine_kind(engine_);
@@ -803,31 +903,41 @@ int dnn_mem_t::initialize(
     SAFE(initialize_memory_create(handle_info), CRIT);
 
     if (handle_info.is_allocate()) {
-        if (!has_bench_mode_modifier(mode_modifier_t::no_host_memory)) map();
+        // Memory objects consisting of several buffers can rely on indirect
+        // data access through metadata (e.g., sparse memory objects).
+        // Filling metadata buffers with random values can lead to accessing an
+        // address location not controlled by the process. Thus, such metadata
+        // buffers must be always properly filled according to the driver rules.
+        // Filling buffers requires them to be mapped.
+        // To save code on updating every case separately, update the logic in
+        // this common place.
+        // ANCHOR: FILL_SPARSE_METADATA.
+        const bool mem_has_indirect_access = is_sparse_md();
+        if (!has_bench_mode_modifier(mode_modifier_t::no_ref_memory)
+                || mem_has_indirect_access)
+            map();
 
         const int nhandles = query_md_num_handles(md_);
         for (int i = 0; i < nhandles; i++) {
-#ifdef DNNL_EXPERIMENTAL_SPARSE
             size_t sz = dnnl_memory_desc_get_size_v2(md_, i);
-#else
-            size_t sz = dnnl_memory_desc_get_size(md_);
-#endif
             if (is_canary_protected_) sz = pad_memory_size(sz, engine_kind_);
+
             // Do not fill a memory if its size is zero. Moreover, memset
             // expects defined pointer, nullptr is not allowed.
-            if (sz != 0) {
-                // Avoid costy data reorders for cold cache mode when
-                // initializing cold cache buffers.
-                // TODO: consider enabling broadly for perf mode.
-                if (has_bench_mode_modifier(mode_modifier_t::no_host_memory)
-                        || cold_cache_mode != default_cold_cache_mode) {
-                    // Fill memory directly with 0x3F3F3F3F (0.747059f) number.
-                    this->memset(dnnl_mem_default_perf_test_value, sz);
-                } else {
-                    // Fill memory with a magic number (NAN for fp data types)
-                    // to catch possible uninitialized access.
-                    ::memset(mapped_ptrs_[i], dnnl_mem_default_value, sz);
-                }
+            if (sz == 0 || !prefill) continue;
+
+            // Avoid costy data reorders for cold cache mode when
+            // initializing cold cache buffers.
+            // TODO: consider enabling broadly for perf mode.
+            if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)
+                    || cold_cache_input.cold_cache_mode_
+                            != default_cold_cache_input().cold_cache_mode_) {
+                // Fill memory directly with 0x3F3F3F3F (0.747059f) number.
+                this->memset(dnnl_mem_default_perf_test_value, sz, i);
+            } else {
+                // Fill memory with a magic number (NAN for fp data types)
+                // to catch possible uninitialized access.
+                ::memset(mapped_ptrs_[i], dnnl_mem_default_value, sz);
             }
         }
     }
@@ -870,7 +980,21 @@ static int cleanup_opencl(
 
 int dnn_mem_t::cleanup() {
     if (!active_) return OK;
-    if (!has_bench_mode_modifier(mode_modifier_t::no_host_memory)) unmap();
+    if (!has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) unmap();
+
+    // Unregister device memory.
+    // Done it here to account memory allocated inside the library for
+    // correspondent memory objects.
+    if (is_sycl_engine(engine_) || is_opencl_engine(engine_)) {
+        const int nhandles = query_md_num_handles(md_);
+        std::vector<void *> handles(nhandles);
+        for (int i = 0; i < nhandles; i++) {
+            size_t sz = dnnl_memory_desc_get_size_v2(md_, i);
+            DNN_SAFE(dnnl_memory_get_data_handle_v2(m_, &handles[i], i), CRIT);
+            memory_registry_t::get_instance().remove_device(handles[i], sz);
+        }
+    }
+
     DNN_SAFE(dnnl_memory_desc_destroy(md_), CRIT);
     DNN_SAFE(dnnl_memory_destroy(m_), CRIT);
     if (is_data_owner_) {
@@ -974,6 +1098,14 @@ static int check_zero_padding_impl(
     int errors = 0;
     std::atomic<int> ok(true);
 
+    for (int dim_m_idx = 0; dim_m_idx < ndims; ++dim_m_idx) {
+        if (dims[dim_m_idx] != pdims[dim_m_idx]) {
+            BENCHDNN_PRINT(
+                    8, "[CHECK_ZERO_PAD]: Arg:%d\n", exec_arg2data_kind(arg));
+            break;
+        }
+    }
+
     for (int dim_m_idx = 0; dim_m_idx < ndims; ++dim_m_idx) {
         if (dims[dim_m_idx] == pdims[dim_m_idx]) continue;
 
@@ -1046,6 +1178,8 @@ int check_zero_padding(
             CASE(dnnl_u8, uint8_t);
             CASE(dnnl_s4, dnnl::impl::int4_t);
             CASE(dnnl_u4, dnnl::impl::uint4_t);
+            CASE(dnnl_f4_e2m1, dnnl::impl::float4_e2m1_t);
+            CASE(dnnl_f4_e3m0, dnnl::impl::float4_e3m0_t);
         default: assert(!"bad data_type");
     };
 #undef CASE
@@ -1110,6 +1244,14 @@ dnnl_dim_t md_off_v(
     return phys_offset;
 }
 
+bool has_sparse_md(const dnn_mem_map_t &dnn_mem_map) {
+    for (const auto &e : dnn_mem_map) {
+        const auto &m = e.second;
+        if (m.is_sparse_md()) return true;
+    }
+    return false;
+}
+
 dnnl_memory_desc_t clone_md(const_dnnl_memory_desc_t md) {
     dnnl_memory_desc_t cloned_md;
     auto status = dnnl_memory_desc_clone(&cloned_md, md);
