@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@ namespace graph {
 using namespace dnnl::graph;
 using namespace dnnl::impl::graph;
 
-struct deserialized_attr {
+struct deserialized_attr_t {
     std::string type_;
     std::string str_value_;
     bool bool_value_;
@@ -43,7 +43,7 @@ struct deserialized_attr {
     void load(utils::json::json_reader_t *reader);
 };
 
-struct deserialized_lt {
+struct deserialized_lt_t {
     size_t id_;
     std::string data_type_;
     logical_tensor::dims shape_;
@@ -60,22 +60,27 @@ struct deserialized_lt {
     void load(utils::json::json_reader_t *reader);
     // Outputs the information about lt from operator<< into a string.
     std::string get_string() const;
-};
-std::ostream &operator<<(std::ostream &s, const deserialized_lt &dlt);
 
-struct deserialized_op {
+    // check if the property type of the lt is host_scalar
+    bool is_host_scalar() const;
+};
+std::ostream &operator<<(std::ostream &s, const deserialized_lt_t &dlt);
+
+struct deserialized_op_t {
     size_t id_;
     std::string name_;
     std::string kind_;
     std::string fpmath_mode_;
     std::string fpmath_mode_apply_to_int_;
 
-    std::unordered_map<std::string, deserialized_attr> attrs_;
-    std::vector<deserialized_lt> in_lts_;
-    std::vector<deserialized_lt> out_lts_;
+    std::unordered_map<std::string, deserialized_attr_t> attrs_;
+    std::vector<deserialized_lt_t> in_lts_;
+    std::vector<deserialized_lt_t> out_lts_;
 
     op create() const;
 
+    dnnl_driver_t opkind2driver() const;
+
     void load(utils::json::json_reader_t *reader);
     // Outputs the information about op from operator<< into a string.
     std::string get_string() const;
@@ -98,21 +103,21 @@ struct deserialized_op {
 
     logical_tensor::dims get_NCX_shape(size_t idx, bool input) const;
 
-    // Returns `true` if `deserialized_op` wasn't created.
+    // Returns `true` if `deserialized_op_t` wasn't created.
     bool empty() const { return kind_.empty(); }
 };
-std::ostream &operator<<(std::ostream &s, const deserialized_op &dop);
+std::ostream &operator<<(std::ostream &s, const deserialized_op_t &dop);
 
-using op_ref_t = std::reference_wrapper<const deserialized_op>;
+using op_ref_t = std::reference_wrapper<const deserialized_op_t>;
 using op_ref_list_t = std::list<op_ref_t>;
 
-struct deserialized_graph {
+struct deserialized_graph_t {
     void load(const std::string &pass_config_json);
 
     dnnl::graph::graph to_graph(const graph_fpmath_mode_t &fpmath_mode) const;
     const std::vector<size_t> &get_input_ports() const { return input_ports_; };
 
-    std::vector<deserialized_op> ops_;
+    std::vector<deserialized_op_t> ops_;
     // record all tensors id and its dims
     std::map<size_t, logical_tensor::dims> graph_tensors_;
     // reorder logical tensor id to memory tag.
@@ -121,18 +126,30 @@ struct deserialized_graph {
     std::map<size_t, std::string> lt_2_mtag_;
     std::vector<size_t> graph_inputs_with_mb_;
 
-    // Returns an op based on its ID.
-    const deserialized_op &get_op(size_t id) const;
-    // Returns an op based on its output logical tensor ID.
-    const deserialized_op &get_op_by_out_lt(size_t out_lt_id) const;
-    // Returns an op based on its input logical tensor ID.
-    const deserialized_op &get_op_by_in_lt(size_t in_lt_id) const;
+    // returns an op based on its ID.
+    const deserialized_op_t &get_op(size_t id) const;
+    // returns an op based on its output logical tensor ID.
+    const deserialized_op_t &get_op_by_out_lt(size_t out_lt_id) const;
+    // returns an op based on its input logical tensor ID.
+    const deserialized_op_t &get_op_by_in_lt(size_t in_lt_id) const;
+    // returns the child ops that consumes the first output of given op.
+    const std::vector<deserialized_op_t> &get_child_ops(
+            const deserialized_op_t &op) const;
+    // Traverses downward through the graph starting from the first consumer of the
+    // given operation's first output tensor. The traversal continues until either:
+    // 1. A disallowed operation is encountered, or
+    // 2. An operation of the specified target kind is reached.
+    //
+    // Note: If start_op already matches target_kind, it will be returned directly.
+    const deserialized_op_t &find_next_until(const deserialized_op_t &start_op,
+            const std::string &target_kind,
+            const std::unordered_set<std::string> &allowed_skips) const;
 
-    // Outputs the information about graph from operator<< into a string.
+    // outputs the information about graph from operator<< into a string.
     std::string get_string() const;
 
-    // Return the fpmath mode attribute
-    const std::pair<std::string, std::string> get_fpmath_mode() const {
+    // return the fpmath mode attribute
+    std::pair<std::string, std::string> get_fpmath_mode() const {
         return std::make_pair(fpmath_mode_, fpmath_mode_apply_to_int_);
     }
 
@@ -141,6 +158,10 @@ struct deserialized_graph {
         fpmath_mode_apply_to_int_ = bool2str(fpmath_mode.apply_to_int_);
     }
 
+    graph_recognized_pattern_t get_recognized_pattern() const {
+        return recognized_pattern_;
+    }
+
 private:
     std::string engine_kind_;
     std::string version_;
@@ -149,8 +170,8 @@ private:
     std::vector<size_t> input_ports_;
     std::vector<size_t> output_ports_;
 
-    std::map<size_t, std::vector<deserialized_op>> in_lt_2_ops_;
-    std::map<size_t, deserialized_op> out_lt_2_op_;
+    std::map<size_t, std::vector<deserialized_op_t>> in_lt_2_ops_;
+    std::map<size_t, deserialized_op_t> out_lt_2_op_;
     std::vector<std::string> binary_ops_ {"Add", "BiasAdd", "Divide", "Maximum",
             "Minimum", "Multiply", "Substract"};
     // need change dst_shape or weight_shape attribute value
@@ -167,10 +188,25 @@ private:
             "MaxPoolBackward", "MishBackward", "ReLUBackward",
             "SigmoidBackward", "SoftMaxBackward", "SoftPlusBackward",
             "SqrtBackward", "TanhBackward"};
+    // indicate whether the graph belongs to the recognized patterns.
+    graph_recognized_pattern_t recognized_pattern_
+            = graph_recognized_pattern_t::ordinary;
+    bool is_pattern_detected_ = false;
 
-    bool check_tensor_with_mb(size_t tensor_id) const;
+    // check whether the tensor supports mb rewrite.
+    bool check_tensor_with_mb(size_t tensor_id,
+            std::unordered_map<size_t, bool> &mb_rewrite_ret) const;
+
+    // check whether the graph is a recoginized pattern, such as SDPA. the
+    // function is intended to be called once during the deserialized graph
+    // loading.
+    void detect_recognized_patterns();
+    // check whether the graph is a scaled dot-product implementation.
+    bool detect_sdpa_fwd_impl() const;
+    // check whether the graph is a sdpa backpropagation implementation.
+    bool detect_sdpa_bwd_impl() const;
 };
-std::ostream &operator<<(std::ostream &s, const deserialized_graph &dg);
+std::ostream &operator<<(std::ostream &s, const deserialized_graph_t &dg);
 
 } // namespace graph
 
