@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2023-2024 Intel Corporation
+* Copyright 2023-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ void check_memory_fit(
 
 } // namespace
 
-ref_partition_t::ref_partition_t(const deserialized_graph &dg,
+ref_partition_t::ref_partition_t(const deserialized_graph_t &dg,
         const dnnl::graph::partition &par,
         const std::vector<dnnl::graph::logical_tensor> &ins,
         const std::vector<dnnl::graph::logical_tensor> &outs)
@@ -70,8 +70,12 @@ ref_partition_t::ref_partition_t(const deserialized_graph &dg,
     }
 };
 
-int ref_partition_t::init_ref(const std::vector<size_t> &graph_in_ports,
-        partition_mem_map_t &partition_mem_map, res_t *res) {
+int ref_partition_t::init_ref(
+        const std::vector<size_t> &graph_in_ports, res_t *res) {
+
+    // Not create reference primitives and filling data with pre-designed
+    // strategies if reference memories are not required.
+    if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) return OK;
 
     for (const auto &par_op_ref : partition_ops_ref_) {
         // res should be independent from op to op
@@ -87,7 +91,7 @@ int ref_partition_t::init_ref(const std::vector<size_t> &graph_in_ports,
         // Check whether the op has any output logical tensor that is the
         // output of the partition. If so, the driver need to allocate memory
         // for correctness check.
-        const auto check_mem_sizes_args = res->mem_size_args;
+        const auto &check_mem_sizes_args = res->mem_size_args;
         const auto is_output = is_output_op(par_op_ref.get());
         SAFE_V(check_partition_total_size(
                 check_mem_sizes_args, is_output, res));
@@ -125,9 +129,10 @@ int ref_partition_t::init_ref(const std::vector<size_t> &graph_in_ports,
         // reference would diverge from the values passed to the Graph API.
         SAFE(ref_prim->displace_scales(), WARN);
 
-        // Initialze the rest ops if current status is UNTESTED or EXECUTED
+        // Initialze the rest ops if current status is UNTESTED or EXECUTED or DEFERRED.
         // otherwise there is no need to init memory for the rest ops.
-        if (res->state != UNTESTED && res->state != EXECUTED) {
+        if (res->state != UNTESTED && res->state != EXECUTED
+                && res->state != DEFERRED) {
             // But for perf mode, when the tensors in the current op is not
             // the graph in/out, continue, otherwise return.
             if (has_bench_mode_bit(mode_bit_t::perf)) {
@@ -145,35 +150,49 @@ int ref_partition_t::init_ref(const std::vector<size_t> &graph_in_ports,
         }
     }
 
-    // displace data if needed
-    for (const auto &entry : lt_id_2_mems_) {
-        SAFE_V(data_displacer.displace_input_data(
-                entry.first, const_cast<dnn_mem_t &>(entry.second), res));
-    }
+    return OK;
+}
+
+int ref_partition_t::init_graph_mem(
+        partition_mem_map_t &partition_mem_map, res_t *res) {
 
     // init graph input/oputput memory from lt_id_2_mems_
     for (const auto &id : partition_in_ids_) {
-        if (lt_id_2_mems_.find(id) == lt_id_2_mems_.end()) {
+        if (!has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) {
+            if (lt_id_2_mems_.find(id) == lt_id_2_mems_.end()) {
+                BENCHDNN_PRINT(0, "Fail: cannot find memory for %zu\n", id);
+                res->state = FAILED;
+                return FAIL;
+            }
+            partition_mem_map.emplace(id,
+                    dnn_graph_mem_t(lt_id_2_mems_.at(id), lt_id_2_lt_.at(id),
+                            /*is_op_input=*/true));
+        } else
+            partition_mem_map.emplace(id,
+                    dnn_graph_mem_t({}, lt_id_2_lt_.at(id),
+                            /*is_op_input=*/true));
+    }
+
+    for (const auto &id : partition_out_ids_) {
+
+        if (fake_lt_ids_.find(id) != fake_lt_ids_.end()
+                || has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) {
+            partition_mem_map.emplace(id,
+                    dnn_graph_mem_t({}, lt_id_2_lt_.at(id),
+                            /*is_op_input=*/false, /*use_graph_layout=*/true));
+        } else if (lt_id_2_mems_.find(id) != lt_id_2_mems_.end()) {
+            // For output memories of graph, they need to be in compliance with
+            // the reference memories regarding the shapes and memory tags, as
+            // the memories of both paths will be reordered to abx for
+            // comparison.
+            partition_mem_map.emplace(id,
+                    dnn_graph_mem_t(lt_id_2_mems_.at(id), lt_id_2_lt_.at(id),
+                            /*is_op_input=*/false));
+        } else {
             BENCHDNN_PRINT(0, "Fail: cannot find memory for %zu\n", id);
             res->state = FAILED;
             return FAIL;
         }
-        partition_mem_map.emplace(id,
-                dnn_graph_mem_t(
-                        lt_id_2_mems_.at(id), lt_id_2_lt_.at(id), true));
-    }
-    for (const auto &id : partition_out_ids_) {
-        if (fake_lt_ids_.find(id) != fake_lt_ids_.end()) {
-            partition_mem_map.emplace(
-                    id, dnn_graph_mem_t({}, lt_id_2_lt_.at(id), false, true));
-        } else if (lt_id_2_mems_.find(id) == lt_id_2_mems_.end()) {
-            BENCHDNN_PRINT(0, "Fail: cannot find memory for %zu\n", id);
-            res->state = FAILED;
-            return FAIL;
-        } else
-            partition_mem_map.emplace(id,
-                    dnn_graph_mem_t(
-                            lt_id_2_mems_.at(id), lt_id_2_lt_.at(id), false));
     }
 
     return OK;
@@ -182,7 +201,18 @@ int ref_partition_t::init_ref(const std::vector<size_t> &graph_in_ports,
 void ref_partition_t::exec_ops(res_t *res) {
     for (const auto &par_op_ref : partition_ops_ref_) {
         const auto &op = par_op_ref.get();
+        // displace data if needed, before executing the ref_prim
+        for (size_t i = 0; i < op.in_lts_.size(); i++) {
+            size_t lt_id = op.in_lts_[i].id_;
+            SAFE_V(data_displacer.displace_input_data(
+                    lt_id, lt_id_2_mems_, res));
+        }
+
         auto ref_prim = ref_prims_.at(op.id_);
+        // check if the condition input of Select op is from the parent op.
+        bool select_op_cond_has_parent
+                = ref_prim->get_kind() == dnnl::graph::op::kind::Select
+                && get_parent_op(op.in_lts_[0].id_);
 
         // link args && replace the memory before execution
         bool use_dst = ::graph::eltwise::get_flag_use_dst_for_bwd_compute(
@@ -191,6 +221,16 @@ void ref_partition_t::exec_ops(res_t *res) {
             const auto &lt = op.in_lts_[i];
             int arg = get_prim_arg_name_from_graph_op_input_offset(
                     ref_prim->get_kind(), i, use_dst);
+            if (select_op_cond_has_parent && i == 0) {
+                // Since select primitive implementation only support s8 data
+                // type for the condition input, need to transfer the f32
+                // results from the previous op to s8.
+                dnn_mem_t &dst_i
+                        = const_cast<dnn_mem_t &>(ref_prim->get_arg(arg));
+                const auto &src_i = lt_id_2_mems_.at(lt.id_);
+                SAFE_V(dst_i.reorder(src_i));
+                continue;
+            }
             ref_prim->replace_arg(arg, lt_id_2_mems_.at(lt.id_));
         }
         for (size_t i = 0; i < op.out_lts_.size(); i++) {
@@ -202,33 +242,71 @@ void ref_partition_t::exec_ops(res_t *res) {
             ref_prim->replace_arg(arg, lt_id_2_mems_.at(lt.id_));
         }
 
-        // There are unfusable operations (such as Softmax) inside a partition
-        // that are executed with user-requested data type. To have correctness
+        // There are unfusable operations inside complex fusion partitions
+        // (such as Softmax in SDPA or chains of MatMuls in MLP) that are
+        // executed with user-requested data type. To have correctness
         // validation working as expected, the data for such operations should
         // be adjusted accordingly in case of low precision data types. E.g.,
         // if pattern is bfloat16 only, the output of a matmul op is bfloat16.
         // Having a float reference implies that is should use "same" bfloat16
-        // data, otherwise, the output from bfloat16 softmax inside the graph
+        // data, otherwise, the output from bfloat16 softmax inside the library
         // and float softmax inside the reference will mismatch, which happens
         // due to the property of softmax, and exponent part in particular.
-        const bool unfusable_transcendental_op
-                = ref_prim->get_kind() == dnnl::graph::op::kind::SoftMax;
-
-        // However, this practice must be limited to the cases when it's
-        // mandatory. The requirement for input adjustment is having a parent
-        // op, since there's an assumption the current op is unfusable.
         //
-        // Note: Compiler backend doesn't down-convert to the lower precision
-        // data type when pass data to a transcendental op. However, this
-        // conversion can't be disabled without additional library API which
-        // would provide an info if Compiler or DNNL backend was used, since
-        // fall back to DNNL from Compiler backend fails.
-        if (unfusable_transcendental_op && has_parent_op(op)) {
+        // However, this practice of data conversion to a lower precision and
+        // back must be limited to the cases when it's necessary.
+        //
+        // For SDPA, it is limited for a Softmax with a parent op presented, as
+        // it's assumed Softmax is unfusable.
+        const bool is_softmax_in_sdpa_pattern
+                = ref_prim->get_kind() == dnnl::graph::op::kind::SoftMax
+                && dg_->get_recognized_pattern()
+                        == graph_recognized_pattern_t::sdpa_fwd;
+
+        // For SDPA training backward, it is limited for MatMuls used to compute
+        // dQ, dK, dV (which has no child ops or child ops = ReduceSum for GQA).
+        const deserialized_op_t *child_op = nullptr;
+        const bool is_matmul_in_sdpa_bwd_pattern
+                = ref_prim->get_kind() == dnnl::graph::op::kind::MatMul
+                && dg_->get_recognized_pattern()
+                        == graph_recognized_pattern_t::sdpa_bwd
+                && (!has_child_op(op, &child_op)
+                        || child_op->kind_ == "ReduceSum");
+
+        // For gated-MLP, it is complicated - the Swish op is decomposed into
+        // Sigmoid and Multiply which has inputs from MatMul0 and Sigmoid. Its
+        // output is passed to another Multiply which is the target for the
+        // reorder, both input and output (since its input is down-converted
+        // by MatMul0, and its output would be a down-converted output of
+        // MatMul1). The variable below carefully checks which Multiply it is
+        // there - Swish's one or not.
+        const bool is_child_multiply
+                = ref_prim->get_kind() == dnnl::graph::op::kind::Multiply
+                && has_parent_op(op, /* check_all_in_lts */ true);
+        bool is_multiply_in_gated_mlp_pattern = false;
+        if (is_child_multiply && op.in_lts_.size() == 2) {
+            const auto &parent0 = get_parent_op(op.in_lts_[0].id_)->kind_;
+            const auto &parent1 = get_parent_op(op.in_lts_[1].id_)->kind_;
+            is_multiply_in_gated_mlp_pattern
+                    = (parent0 == "MatMul" && parent1 == "Multiply")
+                    || (parent0 == "Multiply" && parent1 == "MatMul");
+        }
+
+        if (is_softmax_in_sdpa_pattern || is_matmul_in_sdpa_bwd_pattern
+                || is_multiply_in_gated_mlp_pattern) {
             for (size_t i = 0; i < op.in_lts_.size(); i++) {
                 const auto dt = ref_prim->get_lt_dt(op.in_lts_[i].id_);
                 // There's no need to reorder data for f32 tensors.
                 if (dt == dnnl_f32 || dt == dnnl_data_type_undef) continue;
 
+                // MLP pattern requires reorder only for an input coming from
+                // MatMul0 directly, not from Swish.
+                if (is_multiply_in_gated_mlp_pattern) {
+                    const auto parent_op = get_parent_op(op.in_lts_[i].id_);
+                    if (!parent_op) continue;
+                    if (parent_op->kind_ != "MatMul") continue;
+                }
+
                 int arg = get_prim_arg_name_from_graph_op_input_offset(
                         ref_prim->get_kind(), i, use_dst);
                 dnn_mem_t &src_i
@@ -238,7 +316,7 @@ void ref_partition_t::exec_ops(res_t *res) {
             }
         }
 
-        ref_prim->execute_prim(res);
+        SAFE_V(ref_prim->execute_prim(res));
 
         // For an output, because of various graph compositions, there's a more
         // detailed guide when data adjustment should happen. It's covered by
@@ -246,11 +324,13 @@ void ref_partition_t::exec_ops(res_t *res) {
         //
         // A data type to where transform the data will also be provided by the
         // same function since there are corner cases.
-        dnnl_data_type_t dt = dnnl_data_type_undef;
-        if (unfusable_transcendental_op && need_unfusable_output_crop(op, dt)) {
+        if (is_softmax_in_sdpa_pattern || is_matmul_in_sdpa_bwd_pattern
+                || is_multiply_in_gated_mlp_pattern) {
             for (size_t i = 0; i < op.out_lts_.size(); i++) {
-                // There's no need to reorder data for undefined or f32 tensors.
-                if (dt == dnnl_data_type_undef || dt == dnnl_f32) continue;
+                dnnl_data_type_t dt = dnnl_data_type_undef;
+                if (!need_unfusable_output_crop(op, i, dt)) continue;
+                // There's no need to reorder data for f32 tensors.
+                if (dt == dnnl_f32) continue;
 
                 int arg = get_prim_arg_name_from_graph_op_output_offset(
                         ref_prim->get_kind(), i);
@@ -271,15 +351,15 @@ int ref_partition_t::check_partition_correctness(
 
     for (const auto &op : partition_ops_ref_) {
         size_t op_id = op.get().id_;
-        const auto op_kind = op.get().kind_;
-        const auto ref_prim = ref_prims_.at(op_id);
+        const auto &op_kind = op.get().kind_;
+        const auto &ref_prim = ref_prims_.at(op_id);
 
         // if there is eltwise post-ops or binary div post-ops (GPU test), need
         // to relax compare critria.
         // Currently, both cases use set_has_eltwise_post_op flag in benchdnn
         // compare function.
         // The flag name is not very accurate, add this note to avoid confusion
-        const auto op_driver = opkind2driver(ref_prim->get_kind());
+        const auto op_driver = op.get().opkind2driver();
         has_eltwise = has_eltwise
                 || (op_driver == dnnl_driver_t::eltwise
                         || ((opstr2kind(op_kind)
@@ -319,9 +399,26 @@ int ref_partition_t::check_partition_correctness(
         // reset the state
         res->state = EXECUTED;
 
+        // TODO(zhitao): need to check whether the operation that produces the
+        // output args is the children of the operations that affect
+        // output_has_nans, such as:
+        //
+        //             |
+        //       _____MatMul_______
+        //      |                  |
+        //      |                  |
+        //     SQRT              ReLU
+        //      |                  |
+        // The graph driver allows nans from the branch of Sqrt, but for the
+        // other branch, the driver should not tolerate that.
         ref_prim->check_correctness(
                 output_args, has_eltwise, output_has_nans, res);
         if (res->state == FAILED) {
+            if (is_nvidia_gpu()) {
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return OK;
+            }
             BENCHDNN_PRINT(
                     2, "Op failed: {(%zu) %s}\n", op_id, op_kind.c_str());
             return FAIL;
@@ -340,26 +437,31 @@ int ref_partition_t::check_partition_correctness(
     return OK;
 }
 
-bool ref_partition_t::has_parent_op(const deserialized_op &op) const {
+bool ref_partition_t::has_parent_op(
+        const deserialized_op_t &op, bool check_all_in_lts) const {
     if (partition_ops_ref_.size() < 2) return false;
 
     for (const auto &in_lt : op.in_lts_) {
-        // Check if parent op exist for an `op`.
-        const auto &parent_op = dg_->get_op_by_out_lt(in_lt.id_);
-        if (parent_op.empty()) continue;
-
-        // If it does, check its ID presents in a partition.
-        for (const auto &op_ref : partition_ops_ref_) {
-            const auto &cur_op = op_ref.get();
-            if (parent_op.id_ == cur_op.id_) return true;
+        const auto *parent_op = get_parent_op(in_lt.id_);
+        if (!parent_op) {
+            if (check_all_in_lts) return false;
+            continue;
+        } else {
+            if (check_all_in_lts) continue;
+            return true;
         }
     }
 
-    return false;
+    // The logic for `check_all_in_lts=true` is exclusive along the
+    // verification. If it made till the end, all lts had a parent. The logic
+    // for `check_all_in_lts=false` would return during the verification, and if
+    // reached the end, it means no parent was met.
+    return check_all_in_lts;
 }
 
-bool ref_partition_t::has_child_op(
-        const deserialized_op &op, const deserialized_op **child_op_ptr) const {
+// TODO: add get_child and remove the second arg.
+bool ref_partition_t::has_child_op(const deserialized_op_t &op,
+        const deserialized_op_t **child_op_ptr) const {
     if (partition_ops_ref_.size() < 2) return false;
 
     for (const auto &out_lt : op.out_lts_) {
@@ -380,23 +482,39 @@ bool ref_partition_t::has_child_op(
     return false;
 }
 
+const deserialized_op_t *ref_partition_t::get_parent_op(size_t in_lt_id) const {
+    if (partition_ops_ref_.size() < 2) return nullptr;
+
+    // Check if a parent op exists for an `op`.
+    const auto &parent_op = dg_->get_op_by_out_lt(in_lt_id);
+    if (parent_op.empty()) return nullptr;
+
+    // If it does, check its ID presents in a partition.
+    for (const auto &op_ref : partition_ops_ref_) {
+        const auto &cur_op = op_ref.get();
+        if (parent_op.id_ == cur_op.id_) { return &parent_op; }
+    }
+
+    return nullptr;
+}
+
 // This function decides when unfusable transcendental op output should be
 // reordered to lower data type and back to f32 for a reference path.
-bool ref_partition_t::need_unfusable_output_crop(
-        const deserialized_op &op, dnnl_data_type_t &dt) const {
-    const deserialized_op *child_op = nullptr;
+bool ref_partition_t::need_unfusable_output_crop(const deserialized_op_t &op,
+        size_t output_offset, dnnl_data_type_t &dt) const {
+    const deserialized_op_t *child_op = nullptr;
     // First of all, the output should have a child op...
     if (!has_child_op(op, &child_op)) return false;
     // If the child op is not a TypeCast, it's safe to crop.
     if (child_op->kind_ != "TypeCast") {
         // Target dt in this case is the output dt of input `op`.
-        dt = convert_dt(op.out_lts_[0].get_data_type());
+        dt = convert_dt(op.out_lts_[output_offset].get_data_type());
         return true;
     }
     // When it is a TypeCast (it always changes `cur_dt` <-> f32, both ways are
     // possible), there are options:
     // * If it's the last one, no crop, as f32 will happen on the other end.
-    const deserialized_op *next_child_op = nullptr;
+    const deserialized_op_t *next_child_op = nullptr;
     if (!has_child_op(*child_op, &next_child_op)) return false;
     // * If there's a child Quantize, no crop either, since output would
     //   perform a reorder with a proper scale value to match the other end.
@@ -404,19 +522,19 @@ bool ref_partition_t::need_unfusable_output_crop(
     // * However, a second TypeCast would negate an effect of the previous...
     if (next_child_op->kind_ == "TypeCast") {
         // Target dt in this case is the output dt of the last TypeCast.
-        dt = convert_dt(next_child_op->out_lts_[0].get_data_type());
+        dt = convert_dt(next_child_op->out_lts_[output_offset].get_data_type());
         return true;
     }
 
     // Rest potential outcomes are default to make a crop. The target dt in
     // this case is the output dt of the child op.
-    dt = convert_dt(child_op->out_lts_[0].get_data_type());
+    dt = convert_dt(child_op->out_lts_[output_offset].get_data_type());
     return true;
 }
 
-bool ref_partition_t::is_output_op(const deserialized_op &op) const {
+bool ref_partition_t::is_output_op(const deserialized_op_t &op) const {
     return std::any_of(op.out_lts_.begin(), op.out_lts_.end(),
-            [this](const deserialized_lt &lt) {
+            [this](const deserialized_lt_t &lt) {
                 return std::find(partition_out_ids_.begin(),
                                partition_out_ids_.end(), lt.id_)
                         != partition_out_ids_.end();
@@ -425,7 +543,7 @@ bool ref_partition_t::is_output_op(const deserialized_op &op) const {
 
 // check the partition memory footprint of the graph path
 int ref_partition_t::check_partition_total_size(
-        const deserialized_op &op, res_t *res) {
+        const deserialized_op_t &op, res_t *res) {
 
     // Prepare the memory limit for benchdnn graph
     static size_t benchdnn_cpu_limit = get_benchdnn_cpu_limit();
@@ -485,7 +603,9 @@ int ref_partition_t::check_partition_total_size(
     // after reference path data filling(`C` mode only)
     // 3. Memory to be allocated for comparing results(`C` mode only)
     // 4. Memory to be allocated for mapping device memory(GPU backend only)
-    size_t new_cpu_req = check_mem_size_args.total_size_cpu;
+    size_t new_cpu_req = check_mem_size_args.total_size_ref
+            + check_mem_size_args.total_size_compare
+            + check_mem_size_args.total_size_mapped;
     size_t new_gpu_req = check_mem_size_args.total_size_device;
 
     // STEP 1: Memory allocation stage for the reference path
@@ -530,17 +650,17 @@ int ref_partition_t::check_partition_total_size(
 // Return the logical tensor ids of the given op which is the input/output of
 // the partition.
 std::vector<size_t> ref_partition_t::get_in_out_lt_ids(
-        const deserialized_op &op) const {
+        const deserialized_op_t &op) const {
     std::vector<size_t> in_out_lt_ids;
     std::for_each(op.in_lts_.begin(), op.in_lts_.end(),
-            [&in_out_lt_ids, this](const deserialized_lt &lt) {
+            [&in_out_lt_ids, this](const deserialized_lt_t &lt) {
                 if (std::find(partition_in_ids_.begin(),
                             partition_in_ids_.end(), lt.id_)
                         != partition_in_ids_.end())
                     in_out_lt_ids.emplace_back(lt.id_);
             });
     std::for_each(op.out_lts_.begin(), op.out_lts_.end(),
-            [&in_out_lt_ids, this](const deserialized_lt &lt) {
+            [&in_out_lt_ids, this](const deserialized_lt_t &lt) {
                 if (std::find(partition_out_ids_.begin(),
                             partition_out_ids_.end(), lt.id_)
                         != partition_out_ids_.end())
