@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022-2024 Intel Corporation
+* Copyright 2022-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -49,7 +49,7 @@
 
 namespace graph {
 
-struct deserialized_lt;
+struct deserialized_lt_t;
 
 struct bdnn_state_t {
     res_state_t state;
@@ -76,38 +76,55 @@ enum class dnnl_driver_t {
     others
 };
 
+enum class graph_recognized_pattern_t {
+    ordinary,
+    sdpa_fwd,
+    sdpa_bwd,
+};
+
 extern bdnn_state_t convert_state(const dnnl_status_t &s);
 
 // Flags that controls the behavior for handling exceptions. The logic
 // relies on the fact that the values not intersect with each other.
 enum { CRIT = 0x001, WARN = 0x002, NEED_CLEANUP = 0x004 };
 
+// For now, there are some feature gaps between Nvidia GPU and Intel GPU, as
+// fitting those gaps is a long-term task, many cases will be unsupported on
+// Nvidia GPU, here we directly skip those cases on Nvidia GPU.
 #define DNN_GRAPH_SAFE(f, s, ss) \
     do { \
         try { \
-            f; \
+            (f); \
         } catch (const dnnl::error &e) { \
-            if ((s & CRIT) || (s & WARN)) { \
+            if (((s)&CRIT) || ((s)&WARN)) { \
                 bdnn_state_t bs = convert_state(e.status); \
-                ss->state = bs.state; \
-                if (ss->state == res_state_t::SKIPPED) { \
-                    ss->reason = bs.reason; \
+                (ss)->state = bs.state; \
+                if ((ss)->state == res_state_t::SKIPPED) { \
+                    (ss)->reason = bs.reason; \
+                } else if ((((ss)->state == res_state_t::UNIMPLEMENTED) \
+                                   || ((ss)->state \
+                                           == res_state_t::INVALID_ARGUMENTS)) \
+                        && is_nvidia_gpu()) { \
+                    (ss)->state = SKIPPED; \
+                    (ss)->reason = skip_reason::case_not_supported; \
+                    BENCHDNN_PRINT(2, \
+                            "SKIP: Function '%s' at (%s:%d) returned '%s'\n", \
+                            __FUNCTION__, __FILE__, __LINE__, e.what()); \
                 } else { \
                     BENCHDNN_PRINT(0, \
                             "Error: Function '%s' at (%s:%d) returned '%s'\n", \
                             __FUNCTION__, __FILE__, __LINE__, e.what()); \
                 } \
                 fflush(0); \
-                if (s & CRIT) exit(2); \
+                if ((s)&CRIT) exit(2); \
             } \
-            if (!(s & NEED_CLEANUP)) return FAIL; \
+            if (!((s)&NEED_CLEANUP)) return FAIL; \
         } \
     } while (0)
 
-typedef std::function<void(dnnl::stream &,
+using perf_function_t = std::function<void(dnnl::stream &,
         const std::vector<dnnl::graph::tensor> &inputs,
-        const std::vector<dnnl::graph::tensor> &outputs)>
-        perf_function_t;
+        const std::vector<dnnl::graph::tensor> &outputs)>;
 
 void compiled_partition_executor(dnnl::graph::compiled_partition &cp,
         dnnl::stream &stream, const std::vector<dnnl::graph::tensor> &inputs,
@@ -126,14 +143,16 @@ int measure_perf(timer::timer_t &t,
 
 dnnl::graph::op::kind opstr2kind(const std::string &kind);
 dnnl::graph::op::attr attrstr2kind(const std::string &attr_name);
+const std::string &attrstr2type(const std::string &attr_name);
 
 std::string get_default_tag(size_t length);
 std::string strides2memory_tag(const size_t ndims,
         const dnnl::graph::logical_tensor::dims &strides,
         bool use_x_tag = true);
-
 dnnl::graph::logical_tensor::dims memory_tag2strides(
         const dnnl::graph::logical_tensor::dims &shape, const std::string &tag);
+bool is_contiguous_memory(const dnnl::graph::logical_tensor::dims &strides,
+        const dnnl::graph::logical_tensor::dims &shape, const std::string &tag);
 
 inline bool is_plain(dnnl_format_tag_t fmt_tag) {
     return fmt_tag >= dnnl_a && fmt_tag <= dnnl_abcdefghijlk;
@@ -142,8 +161,6 @@ inline bool is_plain(dnnl_format_tag_t fmt_tag) {
 dnnl::graph::op::kind opstr2kind(const std::string &kind);
 dnnl::graph::op::attr attrstr2kind(const std::string &attr_name);
 
-dnnl_driver_t opkind2driver(const dnnl::graph::op::kind &kind);
-
 // permute md based on permutation
 void permute_md(dnn_mem_t &mem, std::vector<int64_t> permutation);
 
@@ -195,7 +212,7 @@ private:
 // engine used for graph lib, graph lib engine needs allocator to allocate
 // memory for constant cache, scratchpad.
 struct cpp_engine_t {
-    cpp_engine_t();
+    cpp_engine_t(bool use_host);
     dnnl::engine::kind get_kind() const { return engine_.get_kind(); }
     operator dnnl::engine &() { return engine_; }
     operator const dnnl::engine &() const { return engine_; }
@@ -208,11 +225,19 @@ private:
 // engine used for graph lib, graph lib engine needs allocator to allocate
 // memory for constant cache, scratchpad.
 inline const cpp_engine_t &get_graph_engine() {
-    static const cpp_engine_t instance;
+    static const cpp_engine_t instance(/*use_host*/ false);
     return instance;
 }
 
-bool is_gc_backend();
+inline const cpp_engine_t &get_graph_host_engine() {
+    // return `get_graph_engine` for `is_cpu` to avoid different engine instances.
+    const dnnl::engine &g_eng
+            = get_graph_engine().operator const dnnl::engine &();
+    if (is_cpu(g_eng.get())) { return get_graph_engine(); }
+
+    static const cpp_engine_t instance(/*use_host*/ true);
+    return instance;
+}
 
 dnnl_data_type_t convert_dt(const dnnl::graph::logical_tensor::data_type dt);
 
@@ -220,5 +245,25 @@ inline double GB(double bytes) {
     return bytes / powf(2, 30);
 }
 
+struct graph_fpmath_mode_t {
+    graph_fpmath_mode_t() = default;
+    graph_fpmath_mode_t(const std::string &mode, bool apply_to_int,
+            bool override_json_value)
+        : mode_(mode)
+        , apply_to_int_(apply_to_int)
+        , override_json_value_(override_json_value) {}
+
+    bool operator==(const graph_fpmath_mode_t &rhs) const {
+        return mode_ == rhs.mode_ && apply_to_int_ == rhs.apply_to_int_
+                && override_json_value_ == rhs.override_json_value_;
+    }
+
+    std::string mode_ = "strict";
+    bool apply_to_int_ = false;
+    // Since fpmath_mode doesn't provide an "undef" value that would indicate
+    // it was not set externally to the json case, need to maintain this flag.
+    bool override_json_value_ = false;
+};
+
 } // namespace graph
 #endif
