@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2024 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,12 +14,21 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <algorithm>
+
 #include "utils/parallel.hpp"
 
 #include "matmul/matmul.hpp"
 
 namespace matmul {
 
+int64_t wei_ab_off_f(const prb_t *prb, int64_t mb, int64_t k, int64_t n) {
+    return (mb * prb->k + k) * prb->n + n;
+}
+int64_t wei_ba_off_f(const prb_t *prb, int64_t mb, int64_t k, int64_t n) {
+    return (mb * prb->n + n) * prb->k + k;
+}
+
 void compute_ref_matmul(const prb_t *prb, const args_t &args) {
     const dnn_mem_t &src_m = args.find(DNNL_ARG_SRC);
     const dnn_mem_t &wei_m = args.find(DNNL_ARG_WEIGHTS);
@@ -39,65 +48,67 @@ void compute_ref_matmul(const prb_t *prb, const args_t &args) {
             = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
     const dnn_mem_t &dropout = args.find(DNNL_ARG_ATTR_DROPOUT_MASK);
 
-    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
-    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
-    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
-    assert(IMPLICATION(has_dst_scale, dst_scales.nelems() == 1));
-    float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
-    const int src_scale_mask = prb->attr.scales.get_mask(
-            DNNL_ARG_SRC, dnnl_matmul, src_m.ndims());
-    const int wei_scale_mask = prb->attr.scales.get_mask(
-            DNNL_ARG_WEIGHTS, dnnl_matmul, wei_m.ndims());
-
-    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
-    const bool has_wei_zp
-            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
-    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
-
-    const int src_zp_mask = attr_t::get_default_mask(
-            prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
-    const int wei_zp_mask = prb->attr.zero_points.get_mask(
-            DNNL_ARG_WEIGHTS, dnnl_matmul, wei_m.ndims());
-    const int dst_zp_mask = attr_t::get_default_mask(
-            prb->attr.zero_points.get(DNNL_ARG_DST).policy);
-
     const int64_t M = prb->m;
     const int64_t N = prb->n;
     const int64_t K = prb->k;
     const int64_t MB = prb->mb;
     const int batch_ndims = dst_m.ndims() - 2;
 
-    const bool src_scale_per_k = src_scale_mask & (1 << (src_m.ndims() - 1));
-    const bool src_scale_per_m = src_scale_mask & (1 << (src_m.ndims() - 2));
-    const auto src_scale_groups = prb->attr.scales.get(DNNL_ARG_SRC).groups;
-    const int64_t src_scale_group_k
-            = !src_scale_groups.empty() ? src_scale_groups[1] : 1;
-    const int64_t src_scale_stride_k = src_scale_per_k ? 1 : 0;
-    const int64_t src_scale_stride_m = src_scale_per_m
-            ? src_scale_per_k ? (K / src_scale_group_k) : 1
-            : 0;
+    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
 
-    const bool wei_scale_per_n = wei_scale_mask & (1 << (wei_m.ndims() - 1));
-    const bool wei_scale_per_k = wei_scale_mask & (1 << (wei_m.ndims() - 2));
-    const auto wei_scale_groups = prb->attr.scales.get(DNNL_ARG_WEIGHTS).groups;
-    const int64_t wei_scale_group_k
-            = !wei_scale_groups.empty() ? wei_scale_groups[0] : 1;
-    const int64_t wei_scale_stride_n = wei_scale_per_n ? 1 : 0;
-    const int64_t wei_scale_stride_k
-            = wei_scale_per_k ? wei_scale_per_n ? N : 1 : 0;
+    const int src_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_SRC, dnnl_matmul, src_m.ndims());
+    const int wei_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_matmul, wei_m.ndims());
+    const int dst_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_DST, dnnl_matmul, dst_m.ndims());
 
-    const bool wei_zp_per_n = wei_zp_mask & (1 << (wei_m.ndims() - 1));
-    const bool wei_zp_per_k = wei_zp_mask & (1 << (wei_m.ndims() - 2));
-    const int64_t wei_zp_stride_n = wei_zp_per_n ? 1 : 0;
-    const int64_t wei_zp_stride_k = wei_zp_per_k ? wei_zp_per_n ? N : 1 : 0;
-    const auto wei_zp_groups
+    const bool has_src_single_scale = has_src_scale && src_scale_mask == 0;
+    const bool has_wei_single_scale = has_wei_scale && wei_scale_mask == 0;
+
+    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_zp
+            = !prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
+
+    const int src_zp_mask = prb->attr.zero_points.get_mask(
+            DNNL_ARG_SRC, dnnl_matmul, src_m.ndims());
+    const int wei_zp_mask = prb->attr.zero_points.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_matmul, wei_m.ndims());
+    const int dst_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_DST).policy, prb->ndims);
+
+    const bool has_src_single_zp = has_src_zp && src_zp_mask == 0;
+    const bool has_wei_single_zp = has_wei_zp && wei_zp_mask == 0;
+
+    const auto &src_scale_groups = prb->attr.scales.get(DNNL_ARG_SRC).groups;
+    const auto &wei_scale_groups
+            = prb->attr.scales.get(DNNL_ARG_WEIGHTS).groups;
+    const auto &src_zp_groups = prb->attr.zero_points.get(DNNL_ARG_SRC).groups;
+    const auto &wei_zp_groups
             = prb->attr.zero_points.get(DNNL_ARG_WEIGHTS).groups;
-    const int64_t wei_zp_group_k
-            = !wei_zp_groups.empty() ? wei_zp_groups[0] : 1;
 
-    const bool wei_decompression = prb->weights_decompression();
-    const bool apply_scales_in_ker
-            = wei_decompression || wei_scale_per_k || src_scale_per_k;
+    const int64_t src_scale_group = !src_scale_groups.empty()
+            ? src_scale_groups[1]
+            : ((src_scale_mask >> (src_m.ndims() - 1)) % 2) > 0 ? 1
+                                                                : K;
+    const int64_t wei_scale_group = !wei_scale_groups.empty()
+            ? wei_scale_groups[0]
+            : ((wei_scale_mask >> (wei_m.ndims() - 2)) % 2) > 0 ? 1
+                                                                : K;
+    const int64_t src_zp_group = !src_zp_groups.empty()      ? src_zp_groups[1]
+            : ((src_zp_mask >> (src_m.ndims() - 1)) % 2) > 0 ? 1
+                                                             : K;
+    const int64_t wei_zp_group = !wei_zp_groups.empty()      ? wei_zp_groups[0]
+            : ((wei_zp_mask >> (wei_m.ndims() - 2)) % 2) > 0 ? 1
+                                                             : K;
+
+    const auto smallest_k_group = gcd<int64_t>(
+            {src_scale_group, wei_scale_group, src_zp_group, wei_zp_group});
+
+    const auto n_k_groups = K / smallest_k_group;
 
     // Fast return if any dim is zero. Common logic doesn't apply because of
     // broadcast semantics.
@@ -105,91 +116,97 @@ void compute_ref_matmul(const prb_t *prb, const args_t &args) {
         if (prb->src_dims()[d] == 0 || prb->weights_dims()[d] == 0) return;
     }
 
-    dnn_mem_t dst_tmp(dst_m, dnnl_f32, tag::abx, dst_m.engine());
-
     const auto src_broadcast_mask = prb->src_broadcast_mask();
     const auto wei_broadcast_mask = prb->weights_broadcast_mask();
-
-    benchdnn_parallel_nd(MB, M, N, [&](int64_t mb, int64_t m, int64_t n) {
-        auto src = (const float *)src_m;
-        auto wei = (const float *)wei_m;
-
-        float dst = 0;
-        const int64_t src_mb
-                = dst_m.get_scale_idx(mb, src_broadcast_mask, batch_ndims);
-        const int64_t wei_mb
-                = dst_m.get_scale_idx(mb, wei_broadcast_mask, batch_ndims);
-
-        for (int64_t k = 0; k < K; ++k) {
-            int src_zp = has_src_zp ? src_zps.get_elem(src_zp_mask > 0 ? k : 0)
-                                    : 0;
-            int wei_zp = has_wei_zp
-                    ? wei_zps.get_elem(wei_zp_stride_k * (k / wei_zp_group_k)
-                            + wei_zp_stride_n * n)
-                    : 0;
-            auto s = src[src_off_f(prb, src_mb, m, k)] - src_zp;
-            auto w = wei[wei_off_f(prb, wei_mb, k, n)] - wei_zp;
-            // Compression scaling happens before the matmul, unlike regular
-            // quantization, to preserve the accuracy.
-            // Also, regular quantized matmul can have per group K-dim scales
-            // which require handling inside the kernel.
-            if (apply_scales_in_ker) {
-                if (has_src_scale) {
-                    float src_scale = src_scales.get_elem(
-                            src_scale_stride_k * (k / src_scale_group_k)
-                            + src_scale_stride_m * m);
-                    s *= src_scale;
-                }
-                if (has_wei_scale) {
-                    float wei_scale = wei_scales.get_elem(
-                            wei_scale_stride_k * (k / wei_scale_group_k)
-                            + wei_scale_stride_n * n);
-                    w *= wei_scale;
-                }
-            }
-            dst += s * w;
-        }
-        ((float *)dst_tmp)[dst_off_f(prb, mb, m, n)] = dst;
-    });
-
-    auto v_po_masks = prb->attr.post_ops.get_po_masks();
     const auto bias_broadcast_mask = prb->bias_broadcast_mask();
-    benchdnn_parallel_nd(MB, M, N, [&](int64_t mb, int64_t m, int64_t n) {
-        size_t dst_off = dst_off_f(prb, mb, m, n);
-        float &dst = ((float *)dst_m)[dst_off];
+    auto v_po_masks = prb->attr.post_ops.get_po_masks(prb->ndims);
 
-        float wei_scale = 1.f;
-        float src_scale = 1.f;
-        if (!apply_scales_in_ker) {
-            assert(IMPLICATION(has_src_scale, src_scales.nelems() == 1));
-            if (has_src_scale) { src_scale = src_scales.get_elem(0); }
-            if (has_wei_scale) {
-                wei_scale = wei_scales.get_elem(wei_scale_mask > 0 ? n : 0);
+    benchdnn_parallel_nd(MB, M, N, [&](int64_t mb, int64_t m, int64_t n) {
+        float dst = 0;
+        int64_t src_mb = 0;
+        int64_t wei_mb = 0;
+        if (MB > 1) {
+            src_mb = dst_m.get_idx(mb, src_broadcast_mask, batch_ndims);
+            wei_mb = dst_m.get_idx(mb, wei_broadcast_mask, batch_ndims);
+        }
+
+        int src_zp = has_src_single_zp ? src_zps.get_elem(0) : 0;
+        int wei_zp = has_wei_single_zp ? wei_zps.get_elem(0) : 0;
+        float src_scale
+                = has_src_single_scale ? src_scales.get_f32_elem(0) : 1.f;
+        float wei_scale
+                = has_wei_single_scale ? wei_scales.get_f32_elem(0) : 1.f;
+
+        for (int64_t gK = 0; gK < n_k_groups; gK++) {
+            const auto src_gK_off
+                    = src_off_f(prb, src_mb, m, gK * smallest_k_group);
+            // Note: scales/zero-points are still always in `tag::abx` format.
+            const auto wei_gK_off
+                    = wei_ab_off_f(prb, wei_mb, gK * smallest_k_group, n);
+
+            if (has_src_zp && !has_src_single_zp) {
+                const auto src_zp_idx = src_m.get_idx(
+                        src_gK_off, src_zp_mask, src_m.ndims(), src_zp_groups);
+                src_zp = src_zps.get_elem(src_zp_idx);
+            }
+            if (has_wei_zp && !has_wei_single_zp) {
+                const auto wei_zp_idx = wei_m.get_idx(
+                        wei_gK_off, wei_zp_mask, wei_m.ndims(), wei_zp_groups);
+                wei_zp = wei_zps.get_elem(wei_zp_idx);
+            }
+
+            if (has_src_scale && !has_src_single_scale) {
+                const auto src_scale_idx = src_m.get_idx(src_gK_off,
+                        src_scale_mask, src_m.ndims(), src_scale_groups);
+                src_scale = src_scales.get_f32_elem(src_scale_idx);
+            }
+            if (has_wei_scale && !has_wei_single_scale) {
+                const auto wei_scale_idx = wei_m.get_idx(wei_gK_off,
+                        wei_scale_mask, wei_m.ndims(), wei_scale_groups);
+                wei_scale = wei_scales.get_f32_elem(wei_scale_idx);
+            }
+
+            for (int64_t k = 0; k < smallest_k_group; ++k) {
+                const auto src_off
+                        = src_off_f(prb, src_mb, m, gK * smallest_k_group + k);
+                const auto wei_off = wei_ba_off_f(
+                        prb, wei_mb, gK * smallest_k_group + k, n);
+
+                auto s = src_scale * (src_m.get_f32_elem(src_off) - src_zp);
+                auto w = wei_scale * (wei_m.get_f32_elem(wei_off) - wei_zp);
+
+                dst += s * w;
             }
         }
 
-        float tmp = ((float *)dst_tmp)[dst_off] * src_scale * wei_scale;
-
+        const auto dst_off = dst_off_f(prb, mb, m, n);
         if (prb->bia_dt != dnnl_data_type_undef) {
-            int64_t bia_off = dst_m.get_scale_idx(dst_off, bias_broadcast_mask);
-            float *bia_ptr = (float *)bia_m;
-            tmp += bia_ptr[bia_off];
+            const auto bia_idx = dst_m.get_idx(dst_off, bias_broadcast_mask);
+            dst += bia_m.get_f32_elem(bia_idx);
         }
 
         const auto v_po_vals
                 = prepare_po_vals(dst_m, args, v_po_masks, dst_off);
+        maybe_dropout(prb->attr, dst, dst_off, dropout);
+        const auto sum_val = dst_m.get_f32_elem(dst_off);
+        maybe_post_ops(prb->attr, dst, sum_val, v_po_vals);
 
-        maybe_dropout(prb->attr, tmp, dst_off, dropout);
-        maybe_post_ops(prb->attr, tmp, dst, v_po_vals);
-
-        int dst_zp = has_dst_zp ? dst_zps.get_elem(dst_zp_mask > 0 ? n : 0) : 0;
-        dst = tmp * dst_scale + dst_zp;
-        maybe_round(prb->attr, DNNL_ARG_DST, dst, dst_off, prb->dst_dt());
+        int dst_zp = 0;
+        if (has_dst_zp) {
+            const auto dst_zp_idx = dst_m.get_idx(dst_off, dst_zp_mask);
+            dst_zp = dst_zps.get_elem(dst_zp_idx);
+        }
+        float dst_scale = 1.f;
+        if (has_dst_scale) {
+            dst_scale
+                    = 1.f / dst_scales.get_f32_elem(dst_scale_mask > 0 ? n : 0);
+        }
+        float dst_val = dst_scale * dst + dst_zp;
+        maybe_round(prb->attr, DNNL_ARG_DST, dst_val, dst_off, prb->dst_dt());
+        dst_m.set_f32_elem(dst_off, dst_val);
     });
 }
 
-#ifdef DNNL_EXPERIMENTAL_SPARSE
-
 void cvt_coo_indices_to_csr_pointers(const int32_t *indices, int32_t *pointers,
         const int nnz, const int nrows) {
     for (int i = 0; i < nnz; ++i) {
@@ -228,7 +245,7 @@ void compute_ref_sparse_matmul(const prb_t *prb, const args_t &args) {
     // Batch is not supported.
     const int64_t mb = 0;
     benchdnn_parallel_nd(M, N, [&](int64_t m, int64_t n) {
-        dst_m.set_elem(dst_off_f(prb, mb, m, n), 0.0f);
+        dst_m.set_f32_elem(dst_off_f(prb, mb, m, n), 0.0f);
     });
 
     if (is_wei_sparse) {
@@ -255,11 +272,11 @@ void compute_ref_sparse_matmul(const prb_t *prb, const args_t &args) {
                     const int64_t src_idx = src_off_f(prb, mb, m, k);
                     const int64_t dst_idx
                             = dst_off_f(prb, mb, m, wei_indices[n]);
-                    const float src_val = src_m.get_elem(src_idx);
+                    const float src_val = src_m.get_f32_elem(src_idx);
                     const float wei_val = wei_m.get_elem(n, 0);
-                    float dst_val = dst_m.get_elem(dst_idx);
+                    float dst_val = dst_m.get_f32_elem(dst_idx);
                     dst_val += src_val * wei_val;
-                    dst_m.set_elem(dst_idx, dst_val);
+                    dst_m.set_f32_elem(dst_idx, dst_val);
                 }
             }
         });
@@ -281,30 +298,28 @@ void compute_ref_sparse_matmul(const prb_t *prb, const args_t &args) {
             const int64_t row_end = src_pointers[m + 1];
             for (int64_t n = 0; n < N; n++) {
                 const int64_t dst_idx = dst_off_f(prb, mb, m, n);
-                float dst_val = dst_m.get_elem(dst_idx);
+                float dst_val = dst_m.get_f32_elem(dst_idx);
 
                 for (int64_t k = row_start; k < row_end; k++) {
                     const int64_t wei_idx
-                            = wei_off_f(prb, mb, src_indices[k], n);
+                            = wei_ba_off_f(prb, mb, src_indices[k], n);
                     const float src_val = src_m.get_elem(k, 0);
-                    const float wei_val = wei_m.get_elem(wei_idx);
+                    const float wei_val = wei_m.get_f32_elem(wei_idx);
                     dst_val += src_val * wei_val;
                 }
-                dst_m.set_elem(dst_idx, dst_val);
+                dst_m.set_f32_elem(dst_idx, dst_val);
             }
         });
     }
 }
-#endif
 
-void compute_ref(
-        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+void compute_ref(const prb_t *prb, dir_t dir, const args_t &args,
+        dnnl_primitive_t prim_ref) {
     if (prim_ref) {
         SAFE_V(execute_and_wait(prim_ref, args));
         return;
     }
 
-#ifdef DNNL_EXPERIMENTAL_SPARSE
     const auto src_encoding = prb->sparse_options.get_encoding(DNNL_ARG_SRC);
     const auto wei_encoding
             = prb->sparse_options.get_encoding(DNNL_ARG_WEIGHTS);
@@ -315,9 +330,6 @@ void compute_ref(
     } else {
         compute_ref_matmul(prb, args);
     }
-#else
-    compute_ref_matmul(prb, args);
-#endif
 }
 
 } // namespace matmul
