@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@
 #include "common/memory_desc.hpp"
 
 #include "utils/fill.hpp"
+#include "utils/memory.hpp"
 #include "utils/parallel.hpp"
 
 #include "dnn_types.hpp"
@@ -35,15 +36,20 @@
 
 namespace reorder {
 
-int fill_mem(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
-        dnn_mem_t &mem_fp) {
+int fill_mem(int exec_arg, const prb_t *prb, data_kind_t kind,
+        dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
     const auto nelems = mem_fp.nelems();
     if (nelems == 0) return OK;
+    if (fill_from_file(exec_arg, mem_dt, mem_fp)) return OK;
 
     // Refer to modes documentation for filling principles.
     if (has_bench_mode_bit(mode_bit_t::bitwise)) {
         return fill_random_real(mem_dt, mem_fp, nullptr);
     }
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        return fill_random_real(
+                mem_dt, mem_fp, nullptr, get_perf_fill_cfg(mem_dt.dt()));
+    }
 
     const auto conf = prb->get_conf(kind);
 
@@ -67,7 +73,7 @@ int fill_mem(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
         const int64_t table_idx = kind == SRC
                 ? (i % table_size)
                 : ((i * (table_size + 1) / table_size) % table_size);
-        mem_fp.set_elem(
+        mem_fp.set_f32_elem(
                 i, round_to_nearest_representable(conf->dt, gen[table_idx]));
         if (zero_out_wa) mem_dt.set_elem(i, 0);
     });
@@ -83,7 +89,7 @@ dnn_mem_t setup_compensation_memory(const prb_t *prb, flag_bit_t flag) {
         dims_t dims = prb->get_compensation_dims(flag);
         int ndims = static_cast<int>(dims.size());
         auto md = dnn_mem_t::init_md(ndims, dims.data(), dnnl_s32, tag::abx);
-        m = dnn_mem_t(md, get_cpu_engine());
+        m = dnn_mem_t(md, get_cpu_engine(), /* prefill = */ false);
     }
     return m;
 };
@@ -114,7 +120,9 @@ int compare_compensation(const prb_t *prb, dnn_mem_map_t &mem_map,
         // straight comparison of values in native plain layout.
         auto comp_md = dnn_mem_t::init_md(mem_ref.ndims(), mem_ref.dims(),
                 mem_ref.dt(), trim_tag_by_mask(prb->dtag, comp_mask));
-        dnn_mem_t comp_m(comp_md, mem_ref.engine(), {false, comp_handle});
+        // Predefined handle_info won't use prefilling no matter what.
+        dnn_mem_t comp_m(comp_md, mem_ref.engine(), /* prefill = */ false,
+                {false, comp_handle});
 
         compare::compare_t cmp;
         cmp.set_zero_trust_percent(100.f); // No sense in zero trust test.
@@ -187,7 +195,7 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
     }
 
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
-            create_dnnl_attr(prb->attr, attr_args_t()));
+            create_dnnl_attr(prb->attr, attr_args_t(), prb->ndims));
 
     init_pd_args.is_iterator_supported = false;
     TIME_C_PD(DNN_SAFE_STATUS(dnnl_reorder_primitive_desc_create(
@@ -202,110 +210,169 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
     const auto ddt = prb->ddt;
     skip_unimplemented_data_type({sdt, ddt}, prb->dir, res);
     skip_unimplemented_sum_po(prb->attr, res, dnnl_reorder, sdt);
+    skip_unimplemented_binary_po(prb->attr, res);
     skip_unimplemented_prelu_po(prb->attr, res, dnnl_reorder);
 
-    bool scales_ok = true;
-#if !defined(DNNL_X64) || DNNL_X64 == 0
-    {
-        // reference reorder supports only a subset of scale policies
-        const std::vector<policy_t> supported_policy = {policy_t::COMMON,
-                policy_t::PER_DIM_0, policy_t::PER_DIM_1, policy_t::PER_DIM_01};
-
-        for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
-            scales_ok = std::any_of(supported_policy.cbegin(),
-                    supported_policy.cend(), [&](const policy_t policy) {
-                        return prb->attr.scales.get(arg).policy == policy;
-                    });
-        }
-    }
-#endif
-    if (!scales_ok) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+    const bool s32_src_ok = IMPLICATION(sdt == dnnl_s32,
+            ddt != dnnl_f8_e5m2 && ddt != dnnl_f8_e4m3 && ddt != dnnl_bf16
+                    && ddt != dnnl_f16);
+    const bool s32_dst_ok = IMPLICATION(ddt == dnnl_s32,
+            sdt != dnnl_f8_e5m2 && sdt != dnnl_f8_e4m3 && sdt != dnnl_bf16
+                    && sdt != dnnl_f16);
+    if (!s32_src_ok || !s32_dst_ok) {
+        BENCHDNN_PRINT(2,
+                "[SKIP][%s:%d]: Mixed (xf8,xf16)<-->s32 support is limited.\n",
+                __FILE__, __LINE__);
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 
-    if (prb->is_reorder_with_compensation(FLAG_ANY)) {
-        // Compensation is supported for s8 dst data type.
-        const bool dt_ok = ddt == dnnl_s8;
-        // Compensation can be paired with dst scale only.
-        const bool attr_ok
-                = prb->attr.zero_points.is_def() && prb->attr.post_ops.is_def();
-        // Compensation does not support runtime dims.
-        const bool rt_ok = prb->runtime_dim_mask == 0;
-
-        // Compensation and scales mask should coincide
-        const auto comp_mask = prb->get_compensation_mask(FLAG_ANY);
-        bool masks_ok = true;
-        for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
-            const auto &e = prb->attr.scales.get(arg);
-            if (!e.is_def()) {
-                int e_mask = attr_t::get_default_mask(e.policy);
-                masks_ok = masks_ok && e_mask == comp_mask;
-            }
-        }
-
-        if (!dt_ok || !attr_ok || !rt_ok || !masks_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-            return;
-        }
-
-#if !defined(DNNL_X64) || DNNL_X64 == 0
-        // Simple reorder doesn't provide decent coverage for compensated cases.
-        // Shut them down unconditionally by default.
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-        return;
-#endif
-    }
-
-    // Destination scale is not supported for runtime dimensions since the
-    // implementation logic inverts dst scales and requires scratchpad for
-    // `mask > 0` cases which is impossible to estimate with rt dims.
-    const auto &dst_scales = prb->attr.scales.get(DNNL_ARG_DST);
-    if (!dst_scales.is_def() && attr_t::get_default_mask(dst_scales.policy) > 0
-            && prb->runtime_dim_mask != 0) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-        return;
-    }
-
-    // Compensation is supported through jit reorder only, but jit reorder
-    // doesn't support different masks for source and destination scales.
-    const auto &src_scales = prb->attr.scales.get(DNNL_ARG_SRC);
-    if (!src_scales.is_def() && !dst_scales.is_def()) {
-        if (attr_t::get_default_mask(src_scales.policy)
-                        != attr_t::get_default_mask(dst_scales.policy)
-                && prb->is_reorder_with_compensation(FLAG_ANY)) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-            return;
-        }
-    }
-
     if (is_cpu()) {
-        // Int4 reorder support is limited on CPU.
+        bool scales_ok = true;
+#if !defined(DNNL_X64) || DNNL_X64 == 0
+        {
+            // reference reorder supports only a subset of scale policies
+            const std::vector<policy_t> supported_policy
+                    = {policy_t::COMMON, policy_t::PER_DIM_0,
+                            policy_t::PER_DIM_1, policy_t::PER_DIM_01};
+
+            for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
+                scales_ok = std::any_of(supported_policy.cbegin(),
+                        supported_policy.cend(), [&](const policy_t policy) {
+                            return prb->attr.scales.get(arg).policy == policy;
+                        });
+            }
+        }
+#endif
+        if (!scales_ok) {
+            BENCHDNN_PRINT(2,
+                    "[SKIP][%s:%d]: Generic CPU doesn't support specified "
+                    "scale mask.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+        }
+
+        if (prb->is_reorder_with_compensation(FLAG_ANY)) {
+            const bool dt_ok = ddt == dnnl_s8;
+            if (!dt_ok) {
+                BENCHDNN_PRINT(2,
+                        "[SKIP][%s:%d]: Compensation is supported only for s8 "
+                        "dst data type.\n",
+                        __FILE__, __LINE__);
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return;
+            }
+
+            const bool attr_ok = prb->attr.zero_points.is_def()
+                    && prb->attr.post_ops.is_def();
+            if (!attr_ok) {
+                BENCHDNN_PRINT(2,
+                        "[SKIP][%s:%d]: Compensation is supported with scale "
+                        "attribute only.\n",
+                        __FILE__, __LINE__);
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return;
+            }
+
+            const bool rt_ok = prb->runtime_dim_mask == 0;
+            if (!rt_ok) {
+                BENCHDNN_PRINT(2,
+                        "[SKIP][%s:%d]: Compensation is not supported for "
+                        "runtime dimensions.\n",
+                        __FILE__, __LINE__);
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return;
+            }
+
+            const auto comp_mask = prb->get_compensation_mask(FLAG_ANY);
+            bool masks_ok = true;
+            for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_DST}) {
+                const auto &e = prb->attr.scales.get(arg);
+                if (!e.is_def()) {
+                    int e_mask = attr_t::get_default_mask(e.policy, prb->ndims);
+                    masks_ok = masks_ok && e_mask == comp_mask;
+                }
+            }
+            if (!masks_ok) {
+                BENCHDNN_PRINT(2,
+                        "[SKIP][%s:%d]: Compensation mask doesn't coincide "
+                        "with scaling mask.\n",
+                        __FILE__, __LINE__);
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return;
+            }
+
+#if !defined(DNNL_X64) || DNNL_X64 == 0
+            // Simple reorder doesn't provide decent coverage for compensated
+            // cases. Shut them down unconditionally by default.
+            BENCHDNN_PRINT(2,
+                    "[SKIP][%s:%d]: Generic CPU doesn't support compensation "
+                    "cases uniformly.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+#endif
+        }
+
+        const auto &dst_scales = prb->attr.scales.get(DNNL_ARG_DST);
+        if (!dst_scales.is_def()
+                && attr_t::get_default_mask(dst_scales.policy, prb->ndims) > 0
+                && prb->runtime_dim_mask != 0) {
+            // Destination scale is not supported for runtime dimensions since
+            // the implementation logic inverts dst scales and requires
+            // scratchpad for `mask > 0` cases which is impossible to estimate
+            // with runtime dims.
+            BENCHDNN_PRINT(2,
+                    "[SKIP][%s:%d]: Destination scale is not supported for "
+                    "runtime dimensions.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            return;
+        }
+
+        const auto &src_scales = prb->attr.scales.get(DNNL_ARG_SRC);
+        if (!src_scales.is_def() && !dst_scales.is_def()) {
+            if (attr_t::get_default_mask(src_scales.policy, prb->ndims)
+                            != attr_t::get_default_mask(
+                                    dst_scales.policy, prb->ndims)
+                    && prb->is_reorder_with_compensation(FLAG_ANY)) {
+                BENCHDNN_PRINT(2,
+                        "[SKIP][%s:%d]: Compensation cases when both scales "
+                        "specified but with different masks isn't supported.\n",
+                        __FILE__, __LINE__);
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+                return;
+            }
+        }
+
         if (sdt == dnnl_s4 || ddt == dnnl_s4 || sdt == dnnl_u4
                 || ddt == dnnl_u4) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            BENCHDNN_PRINT(2, "[SKIP][%s:%d]: Int4 support is limited.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
 
-        // CPU reorder doesn't support (xf8,xf16)<-->s32 combinations.
-        const bool s32_src_ok = IMPLICATION(sdt == dnnl_s32,
-                ddt != dnnl_f8_e5m2 && ddt != dnnl_f8_e4m3 && ddt != dnnl_bf16
-                        && ddt != dnnl_f16);
-        const bool s32_dst_ok = IMPLICATION(ddt == dnnl_s32,
-                sdt != dnnl_f8_e5m2 && sdt != dnnl_f8_e4m3 && sdt != dnnl_bf16
-                        && sdt != dnnl_f16);
-        if (!s32_src_ok || !s32_dst_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-            return;
-        }
-
-        // CPU f16 reorders only support f16<->f32 combinations
         const bool f16_src_ok = IMPLICATION(
                 sdt == dnnl_f16, ddt == dnnl_f16 || ddt == dnnl_f32);
         const bool f16_dst_ok = IMPLICATION(
                 ddt == dnnl_f16, sdt == dnnl_f16 || sdt == dnnl_f32);
         if (!f16_src_ok || !f16_dst_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            BENCHDNN_PRINT(2, "[SKIP][%s:%d]: f16 support is limited.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
 
@@ -317,54 +384,101 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
                 = IMPLICATION(sdt == dnnl_f8_e5m2 || sdt == dnnl_f8_e4m3,
                         ddt == dnnl_f16 || ddt == dnnl_f32);
         if (!xf8_src_ok || !xf8_dst_ok) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            BENCHDNN_PRINT(2, "[SKIP][%s:%d]: f8 support is limited.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
 
     if (is_gpu()) {
-        // GPU does not support run-time dims.
-        // Reorders w/ compensation are not supported by design: zp_comp is done
-        // in kernels directly, but s8s8 instructions are available in HW.
-        if (prb->runtime_dim_mask != 0
-                || prb->is_reorder_with_compensation(FLAG_ANY)) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        if (prb->runtime_dim_mask != 0) {
+            BENCHDNN_PRINT(2,
+                    "[SKIP][%s:%d]: GPU doesn't support runtime dimensions.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
 
-        // GPU doesn't support f8_e5m2/f8_e4m3.
-        const bool is_xf8 = prb->sdt == dnnl_f8_e5m2 || prb->sdt == dnnl_f8_e4m3
-                || prb->ddt == dnnl_f8_e5m2 || prb->ddt == dnnl_f8_e4m3;
-        if (is_xf8) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        if (prb->is_reorder_with_compensation(FLAG_ANY)) {
+            // Reorders w/ compensation are not supported by design: zp_comp is
+            // done in kernels directly, but s8s8 instructions are available in
+            // HW.
+            BENCHDNN_PRINT(2,
+                    "[SKIP][%s:%d]: GPU doesn't support cases with "
+                    "compensation.\n",
+                    __FILE__, __LINE__);
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
+        auto is_blocked_format = [](const std::string &format_tag) {
+            // if the user provided tag has a capital letter,
+            // that indicates it's a blocked format, otherwise it's plain
+
+            for (const auto &c : format_tag) {
+                if (std::isupper(c)) { return true; }
+            }
+            return false;
+        };
+        // Skip blocked format tags and 4 bit formats for Nvidia/AMD/Generic SYCL backends
+        if (is_generic_gpu()) {
+            const bool is_4bit_format
+                    = is_subbyte_type(prb->sdt) || is_subbyte_type(prb->ddt);
+
+            // sycl reorder implementation does not support grouped zero points / scales
+            bool zero_point_has_groups = !prb->attr.scales.is_def();
+            bool scales_has_groups = !prb->attr.zero_points.is_def();
+
+            if (is_blocked_format(prb->stag) || is_blocked_format(prb->dtag)
+                    || is_4bit_format || scales_has_groups
+                    || zero_point_has_groups) {
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
+            }
+        }
     }
 }
 
 void skip_invalid_prb(const prb_t *prb, res_t *res) {
-    // No sense in cross engine reorders when one of devices is switched off.
 #if DNNL_CPU_RUNTIME == DNNL_RUNTIME_NONE \
         || DNNL_GPU_RUNTIME == DNNL_RUNTIME_NONE
     auto cross_engine = prb->cross_engine;
-    if (cross_engine == CPU2GPU || cross_engine == GPU2CPU)
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+    if (cross_engine == CPU2GPU || cross_engine == GPU2CPU) {
+        BENCHDNN_PRINT(2,
+                "[INVALID][%s:%d]: Cross-engine case isn't supported when just "
+                "one runtime is enabled.\n",
+                __FILE__, __LINE__);
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
+        return;
+    }
 #endif
 
-    // Zero-points can't be used with sum post-op.
     if (!prb->attr.zero_points.is_def(DNNL_ARG_DST)
             && prb->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM) != -1) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        BENCHDNN_PRINT(2,
+                "[INVALID][%s:%d]: Zero-points can't be used with sum "
+                "post-op.\n",
+                __FILE__, __LINE__);
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 
-    // only integral data types can have zero points
     const bool is_src_zp_ok = is_integral_dt(prb->sdt)
             || prb->attr.zero_points.is_def(DNNL_ARG_SRC);
     const bool is_dst_zp_ok = is_integral_dt(prb->ddt)
             || prb->attr.zero_points.is_def(DNNL_ARG_DST);
     if (!(is_src_zp_ok && is_dst_zp_ok)) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        BENCHDNN_PRINT(2,
+                "[INVALID][%s:%d]: Non-integral data types don't support "
+                "zero-points\n",
+                __FILE__, __LINE__);
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 }
@@ -404,7 +518,7 @@ std::vector<int> supported_exec_args(dir_t dir) {
 int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         dnnl_primitive_t prim, const prb_t *prb, res_t *res,
         dnnl_primitive_t prim_ref) {
-    if (has_bench_mode_modifier(mode_modifier_t::no_host_memory)) return OK;
+    if (has_bench_mode_modifier(mode_modifier_t::no_ref_memory)) return OK;
 
     const auto &ref_engine = get_cpu_engine();
 
@@ -421,13 +535,14 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
         // use switch below to define a memory desc for it.
         if (exec_arg != DNNL_ARG_SCRATCHPAD) {
             ref_mem_map.emplace(exec_arg,
-                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+                    dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine,
+                            /* prefill = */ false));
         }
         auto &ref_mem = ref_mem_map[exec_arg];
 
         switch (exec_arg) {
             case DNNL_ARG_FROM: {
-                SAFE(fill_mem(prb, SRC, mem, ref_mem), WARN);
+                SAFE(fill_mem(exec_arg, prb, SRC, mem, ref_mem), WARN);
                 // Additional inputs to compare compensation buffers.
                 ref_mem_map.emplace(DNNL_ARG_SRC_1,
                         setup_compensation_memory(prb, FLAG_S8S8_COMP));
@@ -439,7 +554,7 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
                 const int sum_idx = po.find(attr_t::post_ops_t::SUM);
                 // MIOpen doesn't work properly when tensors are filled with 0xFF.
                 if (sum_idx >= 0 || is_amd_gpu()) {
-                    SAFE(fill_mem(prb, DST, mem, ref_mem), WARN);
+                    SAFE(fill_mem(exec_arg, prb, DST, mem, ref_mem), WARN);
 
                     // Bitwise mode for sum requires a copy due to data for
                     // post-op will be overwritten and it must be refreshed.
@@ -468,14 +583,21 @@ int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
     return OK;
 }
 
-int check_cacheit(
-        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+int checkit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res) {
-    return check_caches(v_prim[0], prb, res);
+    if (has_bench_mode_bit(mode_bit_t::exec)) {
+        SAFE(check_total_size(res), WARN);
+    }
+    if (has_bench_mode_bit(mode_bit_t::corr)) {
+        SAFE(check_caches(v_prim[0], prb, res), WARN);
+    }
+    return OK;
 }
 
 int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res) {
+    set_zmalloc_max_expected_size(res->mem_size_args.zmalloc_expected_size);
+
     const auto &prim = v_prim[0];
 
     dnn_mem_map_t mem_map, ref_mem_map;
@@ -496,7 +618,7 @@ int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         dst_dt.md_->extra = empty_extra;
 
         // Validate main reorder part.
-        check_correctness(prb, {DST}, args, ref_args, setup_cmp, res);
+        check_correctness(prb, {DST}, args, ref_args, setup_cmp, res, prb->dir);
 
         // Restore extra for compensation comparison and performance mode.
         dst_dt.md_->extra = orig_dst_extra;
