@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2024 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -28,6 +28,7 @@
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_debug.hpp"
+#include "utils/memory.hpp"
 #include "utils/perf_report.hpp"
 #include "utils/settings.hpp"
 
@@ -226,12 +227,7 @@ struct dt_conf_t {
 };
 
 struct settings_t : public base_settings_t {
-    settings_t() = default;
-
-    // ctor to save certain fields from resetting
-    settings_t(const char *perf_template) : settings_t() {
-        this->perf_template = perf_template;
-    }
+    using base_settings_t::base_settings_t;
 
     desc_t desc {};
 
@@ -326,18 +322,18 @@ struct prb_t : public desc_t {
         , attr(attr)
         , ctx_init(ctx_init)
         , ctx_exe(ctx_exe)
-        , impl_filter(impl_filter) {
+        , impl_filter(impl_filter)
+        , wei_nscales(0)
+        , wei_scales_mask(0x0)
+        , wei_proj_nscales(0)
+        , wei_proj_scales_mask(0x0) {
 
         if (n_layer) this->n_layer = n_layer;
         if (n_iter) this->n_iter = n_iter;
         if (mb) this->mb = mb;
         count_ops();
 
-        // Broadcast data types if needed
-        if (tag.size() == 1) {
-            const auto val = tag[0]; // Need a copy here.
-            this->tag.assign(3, val);
-        }
+        broadcast_vector(this->tag, 3);
 
         wei_scales = nullptr;
         wei_proj_scales = nullptr;
@@ -411,9 +407,9 @@ struct prb_t : public desc_t {
                 ? 2
                 : 1;
     }
-    int64_t n_states() const { return alg == VANILLA_LSTM ? 2 : 1; }
+    int64_t n_states() const { return 1 + is_lstm(); }
     int64_t n_gates() const {
-        return alg == VANILLA_LSTM
+        return is_lstm()
                 ? 4
                 : (alg == VANILLA_GRU || alg == LBR_GRU || alg == VANILLA_AUGRU
                                         || alg == LBR_AUGRU
@@ -432,15 +428,83 @@ struct prb_t : public desc_t {
         return 0;
     }
 
+    int ndims(rnn_data_kind_t kind) const {
+        switch (kind) {
+            case SRC_LAYER:
+            case DST_LAYER:
+            case AUGRU_ATTENTION:
+            case DIFF_SRC_LAYER:
+            case DIFF_DST_LAYER:
+            case DIFF_AUGRU_ATTENTION: return 3;
+            case SRC_ITER:
+            case SRC_ITER_C:
+            case WEIGHTS_PEEPHOLE:
+            case WEIGHTS_PROJECTION:
+            case BIAS:
+            case DST_ITER:
+            case DST_ITER_C:
+            case DIFF_SRC_ITER:
+            case DIFF_SRC_ITER_C:
+            case DIFF_WEIGHTS_PEEPHOLE:
+            case DIFF_WEIGHTS_PROJECTION:
+            case DIFF_BIAS:
+            case DIFF_DST_ITER:
+            case DIFF_DST_ITER_C: return 4;
+            case WEIGHTS_LAYER:
+            case WEIGHTS_ITER:
+            case DIFF_WEIGHTS_LAYER:
+            case DIFF_WEIGHTS_ITER: return 5;
+            default: assert(!"unknown data kind");
+        }
+        return 0;
+    }
+
     bool is_int8() const {
         return cfg[SRC_LAYER].dt == dnnl_u8 || cfg[SRC_LAYER].dt == dnnl_s8;
     }
     bool is_u8() const { return cfg[SRC_LAYER].dt == dnnl_u8; }
     bool is_s8() const { return cfg[SRC_LAYER].dt == dnnl_s8; }
+    bool is_lstm() const { return alg == VANILLA_LSTM; }
     bool is_lstm_peephole() const { return with_peephole; }
     bool is_lstm_projection() const { return with_projection; }
     bool is_augru() const { return alg == VANILLA_AUGRU || alg == LBR_AUGRU; }
 
+    // Returns the memory size in bytes needed for workspaces used for reference
+    // computations.
+    size_t get_ws_size(dir_t dir) const {
+        if (dir & FLAG_FWD) {
+            auto ws_src_layer = AOC<float>(
+                    nullptr, n_layer + 2, n_dir(), n_iter + 2, mb, wc);
+            auto ws_src_iter_c = AOC<float>(
+                    nullptr, n_layer + 2, n_dir(), n_iter + 2, mb, wc);
+            auto ws_gates = AOC<float>(
+                    nullptr, n_layer, n_dir(), n_iter, mb, n_gates(), dhc);
+            auto ws_ht = AOC<float>(nullptr, n_layer, n_dir(), n_iter, mb, wc);
+
+            int64_t size = ws_src_layer.nelems()
+                    + is_lstm() * ws_src_iter_c.nelems() + ws_gates.nelems()
+                    + is_lstm_projection() * ws_ht.nelems();
+
+            return sizeof(float) * size;
+        } else if (dir & FLAG_BWD) {
+            auto ws_diff_src_layer = AOC<float>(
+                    nullptr, n_layer + 2, n_dir(), n_iter + 2, mb, wc);
+            auto ws_diff_src_iter = AOC<float>(
+                    nullptr, n_layer + 2, n_dir(), n_iter + 2, mb, wc);
+            auto ws_diff_src_iter_c = AOC<float>(
+                    nullptr, n_layer + 2, n_dir(), n_iter + 2, mb, wc);
+
+            int64_t size = ws_diff_src_layer.nelems()
+                    + ws_diff_src_iter.nelems()
+                    + is_lstm() * ws_diff_src_iter_c.nelems();
+
+            return sizeof(float) * size;
+        } else {
+            assert(!"unexpected dir");
+            return 0;
+        }
+    }
+
     // Used to construct memory desc when dimensions are runtime since such mds
     // can't be used directly from query and memory objects can't be constructed.
     benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> get_md(int arg) const {
@@ -535,7 +599,7 @@ private:
     const prb_t *p_;
 };
 
-void prepare_ws_fwd(const prb_t &prb, std::vector<float> &ws_fwd_buffer,
+void prepare_ws_fwd(const prb_t &prb, float *ws_fwd_buffer,
         AOC<float> &ws_src_layer, AOC<float> &ws_src_iter,
         AOC<float> &ws_src_iter_c, AOC<float> &ws_gates, AOC<float> &ws_ht);
 
@@ -554,15 +618,14 @@ int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
 
 void skip_unimplemented_prb(const prb_t *prb, res_t *res);
 void skip_invalid_prb(const prb_t *prb, res_t *res);
-void compute_ref(const prb_t *prb, const args_t &args,
+void compute_ref(const prb_t *prb, dir_t dir, const args_t &args,
         dnnl_primitive_t prim_ref = nullptr);
 void compute_ref_fwd(const prb_t &prb, const args_t &args);
 void compute_ref_bwd(const prb_t &prb, const args_t &args);
 
 int createit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t &prb, res_t *res);
-int check_cacheit(
-        std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
+int checkit(std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t *prb, res_t *res);
 int doit(const std::vector<benchdnn_dnnl_wrapper_t<dnnl_primitive_t>> &v_prim,
         const prb_t &prb, res_t *res);
