@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -26,13 +26,10 @@
 namespace graph = dnnl::impl::graph;
 namespace utils = dnnl::graph::tests::unit::utils;
 
-TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32_CPU) {
+TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // u8 to s8 shift by using a reroder with 128 zps is not supported on gpu
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
-
     std::string qtype = "per_tensor";
     // prepare fp32 data
     std::vector<int64_t> src_shape = {1, 4, 16, 8};
@@ -94,17 +91,17 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32_CPU) {
     g.add_op(&matmul_op);
     g.finalize();
 
-    test_tensor src_u8_ts(src_u8, engine);
+    test_tensor_t src_u8_ts(src_u8, engine);
     src_u8_ts.fill<uint8_t>(20, 10);
-    test_tensor weight_u8_ts(weight_u8, engine);
+    test_tensor_t weight_u8_ts(weight_u8, engine);
     weight_u8_ts.fill<uint8_t>(20, 10);
     // -------------------------case 1----------------------------------
-    test_tensor dst_f32_ts(dst_f32, engine);
+    test_tensor_t dst_f32_ts(dst_f32, engine);
     ASSERT_EQ(run_graph(g, {src_u8_ts, weight_u8_ts}, {dst_f32_ts}, *engine,
                       *strm),
             graph::status::success);
     // -------------------------case 2----------------------------------
-    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops_cpu");
+    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops");
     apass->run(g);
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
@@ -120,7 +117,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32_CPU) {
 
     p.compile(&cp, lt_ins, lt_outs, engine);
 
-    test_tensor dst_f32_case2_ts(dst_f32, engine);
+    test_tensor_t dst_f32_case2_ts(dst_f32, engine);
     cp.execute(strm, {src_u8_ts.get(), weight_u8_ts.get()},
             {dst_f32_case2_ts.get()});
     strm->wait();
@@ -134,13 +131,10 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32_CPU) {
     }
 }
 
-TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32NonContiguous_CPU) {
+TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32NonContiguous) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // u8 to s8 shift by using a reroder with 128 zps is not supported on gpu
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
-
     std::string qtype = "per_tensor";
     // prepare fp32 data
     std::vector<int64_t> src_shape = {1, 1, 50, 50};
@@ -226,16 +220,16 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32NonContiguous_CPU) {
     g.add_op(&matmul_op);
     g.finalize();
 
-    test_tensor src_u8_ts(src_u8, engine, src_data);
-    test_tensor weight_f32_ts(weight_f32, engine, weight_data);
+    test_tensor_t src_u8_ts(src_u8, engine, src_data);
+    test_tensor_t weight_f32_ts(weight_f32, engine, weight_data);
     // -------------------------case 1----------------------------------
     std::vector<float> case1_out_data(product(dst_shape));
-    test_tensor dst_f32_ts(dst_f32, engine, case1_out_data);
+    test_tensor_t dst_f32_ts(dst_f32, engine, case1_out_data);
     ASSERT_EQ(run_graph(g, {src_u8_ts, weight_f32_ts}, {dst_f32_ts}, *engine,
                       *strm),
             graph::status::success);
     // -------------------------case 2----------------------------------
-    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops_cpu");
+    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops");
     apass->run(g);
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
@@ -252,7 +246,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32NonContiguous_CPU) {
     p.compile(&cp, lt_ins, lt_outs, engine);
 
     std::vector<float> case2_out_data(product(dst_shape));
-    test_tensor dst_f32_case2_ts(dst_f32, engine, case2_out_data);
+    test_tensor_t dst_f32_case2_ts(dst_f32, engine, case2_out_data);
     cp.execute(strm, {weight_f32_ts.get(), src_u8_ts.get()},
             {dst_f32_case2_ts.get()});
     strm->wait();
@@ -265,13 +259,10 @@ TEST(test_bmm_execute_subgraph_int8, BmmU8u8f32NonContiguous_CPU) {
     }
 }
 
-TEST(test_bmm_execute_subgraph_int8, BmmDivU8u8f32_CPU) {
+TEST(test_bmm_execute_subgraph_int8, BmmDivU8u8f32) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // u8 to s8 shift by using a reroder with 128 zps is not supported on gpu
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
-
     std::string qtype = "per_tensor";
     // prepare fp32 data
     std::vector<int64_t> src_shape = {1, 4, 16, 8};
@@ -354,10 +345,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivU8u8f32_CPU) {
     g.add_op(&binary_op);
     g.finalize();
 
-    graph::pass::pass_base_ptr apass
-            = get_pass(engine->kind() == graph::engine_kind::gpu
-                            ? "x8s8x_matmul_post_ops_gpu"
-                            : "x8x8x_matmul_post_ops_cpu");
+    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops");
     apass->run(g);
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
@@ -374,23 +362,20 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivU8u8f32_CPU) {
 
     p.compile(&cp, lt_ins, lt_outs, engine);
 
-    test_tensor src_u8_ts(src_u8, engine, src_data);
-    test_tensor weight_u8_ts(weight_u8, engine, weight_data);
-    test_tensor div_src1_ts(div_src1, engine, div_src1_data);
+    test_tensor_t src_u8_ts(src_u8, engine, src_data);
+    test_tensor_t weight_u8_ts(weight_u8, engine, weight_data);
+    test_tensor_t div_src1_ts(div_src1, engine, div_src1_data);
     std::vector<float> case2_out_data(product(dst_shape));
-    test_tensor dst_f32_case2_ts(div_f32, engine, case2_out_data);
+    test_tensor_t dst_f32_case2_ts(div_f32, engine, case2_out_data);
     cp.execute(strm, {src_u8_ts.get(), weight_u8_ts.get(), div_src1_ts.get()},
             {dst_f32_case2_ts.get()});
     strm->wait();
 }
 
-TEST(test_bmm_execute_subgraph_int8, BmmDivAddU8u8f32_CPU) {
+TEST(test_bmm_execute_subgraph_int8, BmmDivAddU8u8f32) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // u8 to s8 shift by using a reroder with 128 zps is not supported on gpu
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
-
     std::string qtype = "per_tensor";
     // prepare fp32 data
     std::vector<int64_t> src_shape = {1, 4, 16, 8};
@@ -489,7 +474,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivAddU8u8f32_CPU) {
     g.add_op(&binary_op2);
     g.finalize();
 
-    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops_cpu");
+    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops");
     apass->run(g);
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
@@ -506,11 +491,11 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivAddU8u8f32_CPU) {
 
     p.compile(&cp, lt_ins, lt_outs, engine);
 
-    test_tensor src_u8_ts(src_u8, engine, src_data);
-    test_tensor weight_u8_ts(weight_u8, engine, weight_data);
-    test_tensor div_src1_ts(div_src1, engine, div_src1_data);
-    test_tensor add_src1_ts(add_src1, engine, add_src1_data);
-    test_tensor dst_f32_case2_ts(add_f32, engine);
+    test_tensor_t src_u8_ts(src_u8, engine, src_data);
+    test_tensor_t weight_u8_ts(weight_u8, engine, weight_data);
+    test_tensor_t div_src1_ts(div_src1, engine, div_src1_data);
+    test_tensor_t add_src1_ts(add_src1, engine, add_src1_data);
+    test_tensor_t dst_f32_case2_ts(add_f32, engine);
     cp.execute(strm,
             {src_u8_ts.get(), weight_u8_ts.get(), div_src1_ts.get(),
                     add_src1_ts.get()},
@@ -522,9 +507,6 @@ TEST(test_bmm_execute_subgraph_int8, BmmX8x8bf16_CPU) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // gpu doesn't support mixed int8-bf16 matmul with runtime zero points
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
-
     std::vector<std::string> dtypes = {"uint8", "int8"};
 
     std::vector<int64_t> src_shape = {1, 4, 16, 8};
@@ -544,8 +526,6 @@ TEST(test_bmm_execute_subgraph_int8, BmmX8x8bf16_CPU) {
             float src_range = (src_dtype == "uint8") ? 255.f : 127.f;
             float weight_range = (weight_dtype == "uint8") ? 255.f : 127.f;
 
-            // u8 2 s8 shift by using reorder with -128 zps is not supported on
-            // GPU
             if (weight_dtype == "uint8"
                     && engine->kind() == graph::engine_kind::gpu)
                 continue;
@@ -635,9 +615,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmX8x8bf16_CPU) {
             g.finalize();
 
             graph::pass::pass_base_ptr apass
-                    = get_pass(engine->kind() == graph::engine_kind::gpu
-                                    ? "x8s8x_tc_matmul_post_ops_gpu"
-                                    : "x8x8x_tc_matmul_post_ops_cpu");
+                    = get_pass("x8x8x_tc_matmul_post_ops");
             apass->run(g);
             ASSERT_EQ(g.get_num_partitions(), 1U);
             auto part = g.get_partitions()[0];
@@ -653,9 +631,9 @@ TEST(test_bmm_execute_subgraph_int8, BmmX8x8bf16_CPU) {
 
             p.compile(&cp, lt_ins, lt_outs, engine);
 
-            test_tensor src_ts(src, engine, src_data);
-            test_tensor weight_ts(weight, engine, weight_data);
-            test_tensor dst_ts(dst_bf16, engine);
+            test_tensor_t src_ts(src, engine, src_data);
+            test_tensor_t weight_ts(weight, engine, weight_data);
+            test_tensor_t dst_ts(dst_bf16, engine);
             cp.execute(strm, {src_ts.get(), weight_ts.get()}, {dst_ts.get()});
             strm->wait();
         }
@@ -669,6 +647,11 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivX8x8bf16_CPU) {
     // gpu doesn't support mixed int8-bf16 matmul with runtime zero points
     SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
 
+    SKIP_IF(unsupported_data_type(static_cast<dnnl::memory::data_type>(
+                                          graph::data_type::bf16),
+                    graph::dnnl_impl::make_dnnl_engine(*engine)),
+            "Skip bf16 examples for systems that do not support avx512_core.");
+
     std::vector<std::string> dtypes = {"uint8", "int8"};
 
     std::vector<int64_t> src_shape = {1, 4, 16, 8};
@@ -688,12 +671,6 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivX8x8bf16_CPU) {
             float src_range = (src_dtype == "uint8") ? 255.f : 127.f;
             float weight_range = (weight_dtype == "uint8") ? 255.f : 127.f;
 
-            // u8 2 s8 shift by using reorder with -128 zps is not supported on
-            // GPU
-            if (weight_dtype == "uint8"
-                    && engine->kind() == graph::engine_kind::gpu)
-                continue;
-
             // random generate src, weight data
             // random seed = 7
             std::default_random_engine generator(7);
@@ -792,9 +769,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivX8x8bf16_CPU) {
             ASSERT_EQ(g.finalize(), graph::status::success);
 
             graph::pass::pass_base_ptr apass
-                    = get_pass(engine->kind() == graph::engine_kind::gpu
-                                    ? "x8s8x_tc_matmul_post_ops_gpu"
-                                    : "x8x8x_tc_matmul_post_ops_cpu");
+                    = get_pass("x8x8x_tc_matmul_post_ops");
             apass->run(g);
             ASSERT_EQ(g.get_num_partitions(), 1U);
             auto part = g.get_partitions()[0];
@@ -812,10 +787,10 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivX8x8bf16_CPU) {
             p.compile(&cp, lt_ins, lt_outs, engine);
 
             std::vector<bfloat16_t> div_src1_data(1);
-            test_tensor src_ts(src, engine, src_data);
-            test_tensor weight_ts(weight, engine, weight_data);
-            test_tensor div_src1_ts(div_src1, engine, div_src1_data);
-            test_tensor dst_ts(div_bf16, engine);
+            test_tensor_t src_ts(src, engine, src_data);
+            test_tensor_t weight_ts(weight, engine, weight_data);
+            test_tensor_t div_src1_ts(div_src1, engine, div_src1_data);
+            test_tensor_t dst_ts(div_bf16, engine);
             cp.execute(strm, {src_ts.get(), weight_ts.get(), div_src1_ts.get()},
                     {dst_ts.get()});
             strm->wait();
@@ -827,13 +802,10 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivBlockedX8x8bf16_CPU) {
     graph::engine_t *engine = get_engine();
     graph::stream_t *strm = get_stream();
 
-    // The below case will be skipped on GPU:
-    // benchdnn --matmul --reset --allow-enum-tags-only=0 --engine=gpu
-    // --cfg=u8s8bf16  --stag=acbd --wtag=adbc --dtag=abcd
-    // --attr-post-ops=binary_div:bf16:common --attr-oscale=common:3.087849e-05
-    // --attr-zero-points=src:common:110+wei:common:114 --attr-scratchpad=user
-    // 1x4x16x8:1x4x8x16:1x4x16x16
-    SKIP_IF(engine->kind() == graph::engine_kind::gpu, "skip on gpu");
+    SKIP_IF(unsupported_data_type(static_cast<dnnl::memory::data_type>(
+                                          graph::data_type::bf16),
+                    graph::dnnl_impl::make_dnnl_engine(*engine)),
+            "Skip bf16 examples for systems that do not support avx512_core.");
 
     std::vector<std::string> dtypes = {"uint8", "int8"};
 
@@ -856,12 +828,6 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivBlockedX8x8bf16_CPU) {
             float src_range = (src_dtype == "uint8") ? 255.f : 127.f;
             float weight_range = (weight_dtype == "uint8") ? 255.f : 127.f;
 
-            // u8 2 s8 shift by using reorder with -128 zps is not supported on
-            // GPU
-            if (weight_dtype == "uint8"
-                    && engine->kind() == graph::engine_kind::gpu)
-                continue;
-
             // random generate src, weight data
             // random seed = 7
             std::default_random_engine generator(7);
@@ -960,7 +926,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivBlockedX8x8bf16_CPU) {
             g.finalize();
 
             graph::pass::pass_base_ptr apass
-                    = get_pass("x8x8x_tc_matmul_post_ops_cpu");
+                    = get_pass("x8x8x_tc_matmul_post_ops");
             apass->run(g);
             ASSERT_EQ(g.get_num_partitions(), 1U);
             auto part = g.get_partitions()[0];
@@ -978,10 +944,10 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivBlockedX8x8bf16_CPU) {
             p.compile(&cp, lt_ins, lt_outs, engine);
 
             std::vector<bfloat16_t> div_src1_data(1);
-            test_tensor src_ts(src, engine, src_data);
-            test_tensor weight_ts(weight, engine, weight_data);
-            test_tensor div_src1_ts(div_src1, engine, div_src1_data);
-            test_tensor dst_ts(div_bf16, engine);
+            test_tensor_t src_ts(src, engine, src_data);
+            test_tensor_t weight_ts(weight, engine, weight_data);
+            test_tensor_t div_src1_ts(div_src1, engine, div_src1_data);
+            test_tensor_t dst_ts(div_bf16, engine);
             cp.execute(strm, {src_ts.get(), weight_ts.get(), div_src1_ts.get()},
                     {dst_ts.get()});
             strm->wait();
@@ -1022,12 +988,6 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivAddX8x8bf16_CPU) {
             float src_range = (src_dtype == "uint8") ? 255.f : 127.f;
             float weight_range = (weight_dtype == "uint8") ? 255.f : 127.f;
 
-            // u8 2 s8 shift by using reorder with -128 zps is not supported on
-            // GPU
-            if (weight_dtype == "uint8"
-                    && engine->kind() == graph::engine_kind::gpu)
-                continue;
-
             // random generate src, weight data
             // random seed = 7
             std::default_random_engine generator(7);
@@ -1139,9 +1099,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivAddX8x8bf16_CPU) {
             g.finalize();
 
             graph::pass::pass_base_ptr apass
-                    = get_pass(engine->kind() == graph::engine_kind::gpu
-                                    ? "x8s8x_tc_matmul_post_ops_gpu"
-                                    : "x8x8x_tc_matmul_post_ops_cpu");
+                    = get_pass("x8x8x_tc_matmul_post_ops");
             apass->run(g);
             ASSERT_EQ(g.get_num_partitions(), 1U);
             auto part = g.get_partitions()[0];
@@ -1158,11 +1116,11 @@ TEST(test_bmm_execute_subgraph_int8, BmmDivAddX8x8bf16_CPU) {
 
             p.compile(&cp, lt_ins, lt_outs, engine);
 
-            test_tensor src_ts(src, engine, src_data);
-            test_tensor weight_ts(weight, engine, weight_data);
-            test_tensor div_src1_ts(div_src1, engine);
-            test_tensor add_src1_ts(add_src1, engine);
-            test_tensor dst_ts(add_bf16, engine);
+            test_tensor_t src_ts(src, engine, src_data);
+            test_tensor_t weight_ts(weight, engine, weight_data);
+            test_tensor_t div_src1_ts(div_src1, engine);
+            test_tensor_t add_src1_ts(add_src1, engine);
+            test_tensor_t dst_ts(add_bf16, engine);
             cp.execute(strm,
                     {src_ts.get(), weight_ts.get(), div_src1_ts.get(),
                             add_src1_ts.get()},
@@ -1275,10 +1233,7 @@ TEST(test_bmm_execute_subgraph_int8, BmmMulAddTransposeBU8s8f32) {
     g.add_op(&binary_op2);
     g.finalize();
 
-    graph::pass::pass_base_ptr apass
-            = get_pass(engine->kind() == graph::engine_kind::gpu
-                            ? "x8s8x_matmul_post_ops_gpu"
-                            : "x8x8x_matmul_post_ops_cpu");
+    graph::pass::pass_base_ptr apass = get_pass("x8x8x_matmul_post_ops");
     apass->run(g);
     ASSERT_EQ(g.get_num_partitions(), 1U);
     auto part = g.get_partitions()[0];
@@ -1295,12 +1250,12 @@ TEST(test_bmm_execute_subgraph_int8, BmmMulAddTransposeBU8s8f32) {
 
     p.compile(&cp, lt_ins, lt_outs, engine);
 
-    test_tensor src_u8_ts(src_u8, engine, src_data);
-    test_tensor weight_s8_ts(weight_s8, engine, weight_data);
-    test_tensor mul_src1_ts(mul_src1, engine, mul_src1_data);
-    test_tensor add_src1_ts(add_src1, engine, add_src1_data);
+    test_tensor_t src_u8_ts(src_u8, engine, src_data);
+    test_tensor_t weight_s8_ts(weight_s8, engine, weight_data);
+    test_tensor_t mul_src1_ts(mul_src1, engine, mul_src1_data);
+    test_tensor_t add_src1_ts(add_src1, engine, add_src1_data);
     std::vector<float> case2_out_data(product(dst_shape));
-    test_tensor dst_f32_case2_ts(add_f32, engine, case2_out_data);
+    test_tensor_t dst_f32_case2_ts(add_f32, engine, case2_out_data);
     cp.execute(strm,
             {src_u8_ts.get(), weight_s8_ts.get(), mul_src1_ts.get(),
                     add_src1_ts.get()},
