@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2023 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -118,11 +118,11 @@ void compute_ref_conv_bwd_weights(const test_convolution_sizes_t &c,
 
 template <typename data_t_src, typename data_t_diff_dst,
         typename data_t_diff_weights, typename data_t_diff_bias>
-class convolution_backward_weights_test
+class convolution_backward_weights_test_t
     : public ::testing::TestWithParam<test_convolution_params_t> {
 protected:
-    virtual void SetUp() {
-        auto p = ::testing::TestWithParam<
+    void SetUp() override {
+        const auto &p = ::testing::TestWithParam<
                 test_convolution_params_t>::GetParam();
 
         SKIP_IF_CUDA(
@@ -138,11 +138,11 @@ protected:
                                         memory::format_tag::owi,
                                         memory::format_tag::ohwi,
                                         memory::format_tag::odhwi)))
-                        && data_traits<data_t_src>::data_type
+                        && data_traits_t<data_t_src>::data_type
                                 == memory::data_type::f32
-                        && data_traits<data_t_diff_dst>::data_type
+                        && data_traits_t<data_t_diff_dst>::data_type
                                 == memory::data_type::f32
-                        && data_traits<data_t_diff_weights>::data_type
+                        && data_traits_t<data_t_diff_weights>::data_type
                                 == memory::data_type::f32
                         && check_cuda_alg_format(p.formats.dst_format,
                                 p.formats.weights_format, p.aalgorithm)),
@@ -160,16 +160,38 @@ protected:
                                             memory::format_tag::owi,
                                             memory::format_tag::ohwi,
                                             memory::format_tag::odhwi)))
-                            && data_traits<data_t_src>::data_type
+                            && data_traits_t<data_t_src>::data_type
                                     == memory::data_type::f32
-                            && data_traits<data_t_diff_dst>::data_type
+                            && data_traits_t<data_t_diff_dst>::data_type
                                     == memory::data_type::f32
-                            && data_traits<data_t_diff_weights>::data_type
+                            && data_traits_t<data_t_diff_weights>::data_type
                                     == memory::data_type::f32
                             && check_hip_alg_format(p.formats.dst_format,
                                     p.formats.weights_format, p.aalgorithm)),
                 "Format is not supported.");
 
+        SKIP_IF_GENERIC(
+                !(generic_check_format_tags(p.formats.src_format)
+                        && generic_check_format_tags(p.formats.dst_format)
+                        && (generic_check_format_tags(p.formats.weights_format)
+                                || (impl::utils::one_of(
+                                        p.formats.weights_format,
+                                        memory::format_tag::goiw,
+                                        memory::format_tag::goihw,
+                                        memory::format_tag::goidhw,
+                                        memory::format_tag::oiw,
+                                        memory::format_tag::oihw,
+                                        memory::format_tag::oidhw,
+                                        memory::format_tag::bacd,
+                                        memory::format_tag::bcda,
+                                        memory::format_tag::acbde,
+                                        memory::format_tag::iohw,
+                                        memory::format_tag::hwigo)))
+                        && check_generic_dt<data_t_src>()
+                        && check_generic_dt<data_t_diff_dst>()
+                        && check_generic_dt<data_t_diff_weights>()),
+                "Format is not supported.");
+
         catch_expected_failures(
                 [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
@@ -214,20 +236,36 @@ protected:
         return res;
     }
 
+    bool generic_check_format_tags(memory::format_tag tag) {
+        return impl::utils::one_of(tag, memory::format_tag::ab,
+                memory::format_tag::abc, memory::format_tag::abcd,
+                memory::format_tag::abcde, memory::format_tag::abcdef,
+                memory::format_tag::acb, memory::format_tag::acdb,
+                memory::format_tag::acdeb, memory::format_tag::any);
+    }
+
+    template <typename dt>
+    bool check_generic_dt() {
+        return impl::utils::one_of(data_traits_t<dt>::data_type,
+                memory::data_type::f32, memory::data_type::bf16,
+                memory::data_type::f16, memory::data_type::s32,
+                memory::data_type::s8, memory::data_type::u8);
+    }
+
     void Test() {
-        auto p = ::testing::TestWithParam<
+        const auto &p = ::testing::TestWithParam<
                 test_convolution_params_t>::GetParam();
 
         ASSERT_EQ(p.aalgorithm, algorithm::convolution_direct);
         auto eng = get_test_engine();
         auto strm = stream(eng);
-        memory::data_type data_type_src = data_traits<data_t_src>::data_type;
+        memory::data_type data_type_src = data_traits_t<data_t_src>::data_type;
         memory::data_type data_type_diff_dst
-                = data_traits<data_t_diff_dst>::data_type;
+                = data_traits_t<data_t_diff_dst>::data_type;
         memory::data_type data_type_diff_weights
-                = data_traits<data_t_diff_weights>::data_type;
+                = data_traits_t<data_t_diff_weights>::data_type;
         memory::data_type data_type_diff_bias
-                = data_traits<data_t_diff_bias>::data_type;
+                = data_traits_t<data_t_diff_bias>::data_type;
 
         test_convolution_sizes_t cd = p.sizes;
 
