@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2024 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 * Copyright 2020-2021 FUJITSU LIMITED
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -160,21 +160,15 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScratchpadArg) {
 
 TEST_F(attr_test_t, TestZeroPoints) {
     dnnl::primitive_attr attr;
-
     const std::vector<int> supported_args = {DNNL_ARG_SRC, DNNL_ARG_DST};
-    const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
-            DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
 
     for (auto arg : supported_args) {
         // single non-default zero_point for supported arg
         attr.set_zero_points_mask(arg, 0);
         // multiple zero_points for supported arg
         attr.set_zero_points_mask(arg, 1 << 0);
-    }
-
-    for (auto arg : unsupported_args) {
-        // single **default** zero_point for **unsupported** arg
-        EXPECT_ANY_THROW(attr.set_zero_points_mask(arg, 0));
+        // host zero_point for supported arg
+        attr.set_host_zero_point(arg);
     }
 }
 
@@ -192,21 +186,30 @@ TEST_F(attr_test_t, TestZeroPointsWithGroups) {
         attr.set_zero_points(arg, 0, {}, data_type::s8);
         // multiple zero_points for supported arg with data type specified
         attr.set_zero_points(arg, 1 << 0, {4}, data_type::s8);
+        // single host zero_point for supported arg with data type specified
+        attr.set_zero_points(arg, 0, {}, data_type::s8, true);
     }
 
     for (auto arg : unsupported_args) {
-        // multiple zero_points for supported arg with data type specified
+        // multiple zero_points for unsupported arg with data type specified
         EXPECT_ANY_THROW(attr.set_zero_points(arg, 1 << 0, {4}));
-        // single zero_point for **unsupported** arg with data type specified
+        // single zero_point for unsupported arg with data type specified
         EXPECT_ANY_THROW(attr.set_zero_points(arg, 0, {}, data_type::s8));
+        // single host zero_point for unsupported arg with data type specified
+        EXPECT_ANY_THROW(attr.set_zero_points(arg, 0, {}, data_type::s8, true));
     }
+
+    // multiple zero_points that are not supported for host zero_point
+    int any_support_arg = DNNL_ARG_SRC;
+    EXPECT_ANY_THROW(attr.set_zero_points(
+            any_support_arg, 1 << 0, {4}, data_type::s8, true));
 }
 
 TEST_F(attr_test_t, TestZeroPointsDataTypes) {
     dnnl::primitive_attr attr;
 
-    const std::vector<int> supported_args = {DNNL_ARG_WEIGHTS};
-    const std::vector<int> unsupported_args = {DNNL_ARG_SRC};
+    const std::vector<int> supported_args = {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS};
+    const std::vector<int> unsupported_args = {DNNL_ARG_DST};
 
     const std::vector<data_type> supported_dts = {data_type::s32, data_type::s8,
             data_type::u8, data_type::s4, data_type::u4};
@@ -219,16 +222,18 @@ TEST_F(attr_test_t, TestZeroPointsDataTypes) {
         }
         for (auto dt : unsupported_dts) {
             EXPECT_ANY_THROW(attr.set_zero_points(arg, 0, {}, dt));
+            EXPECT_ANY_THROW(attr.set_zero_points(arg, 0, {}, dt, true));
+            EXPECT_ANY_THROW(attr.set_host_zero_point(arg, dt));
         }
     }
 
     for (auto arg : unsupported_args) {
         for (auto dt : supported_dts) {
             if (dt == data_type::s32) {
-                // s32 is a default zero point data type supported by all eligible arguments
+                // s32 is a default zero point data type supported by all
+                // eligible arguments.
                 attr.set_zero_points(arg, 0, {}, dt);
             } else {
-                // Only weights support non-default zero point data type
                 EXPECT_ANY_THROW(attr.set_zero_points(arg, 0, {}, dt));
             }
         }
@@ -240,34 +245,54 @@ TEST_F(attr_test_t, TestZeroPointsDataTypes) {
 
 TEST_F(attr_test_t, TestZeroPointsExpectFailure) {
     dnnl::primitive_attr attr;
+    const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
+            DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
 
-    const int unsupported_arg = DNNL_ARG_MEAN;
+    for (auto arg : unsupported_args) {
+        // single non-default zero_point for unsupported arg
+        EXPECT_ANY_THROW(attr.set_zero_points_mask(arg, 0));
+        // multiple zero points for unsupported args
+        EXPECT_ANY_THROW(attr.set_zero_points_mask(arg, 1 << 1));
+        // host zero point
+        EXPECT_ANY_THROW(attr.set_host_zero_point(arg));
+    }
+}
 
-    // single non-default zero_point for unsupported arg
-    EXPECT_ANY_THROW(attr.set_zero_points_mask(unsupported_arg, 0));
+TEST_F(attr_test_t, TestPrecomputedReductionsWithGroups) {
+    dnnl::primitive_attr attr;
 
-    // multiple zero points for unsupported args
-    EXPECT_ANY_THROW(attr.set_zero_points_mask(unsupported_arg, 1 << 1));
+    const std::vector<int> supported_args = {DNNL_ARG_SRC};
+    const std::vector<int> unsupported_args = {DNNL_ARG_WEIGHTS, DNNL_ARG_DST};
+
+    for (auto arg : supported_args) {
+        // single precomputed_reductions (not supported, but valid).
+        attr.set_precomputed_reductions(arg, 0, {});
+        // multiple precomputed_reductions for supported arg.
+        attr.set_precomputed_reductions(arg, 1 << 0, {4});
+        // multiple precomputed_reductions for supported arg with data type
+        // specified. Anything but s32 is unsupported so far.
+        EXPECT_ANY_THROW(attr.set_precomputed_reductions(
+                arg, 1 << 0, {4}, data_type::s8));
+    }
+
+    for (auto arg : unsupported_args) {
+        // multiple zero_points for supported arg with data type specified
+        EXPECT_ANY_THROW(attr.set_precomputed_reductions(arg, 1 << 0, {4}));
+    }
 }
 
 HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScales) {
     dnnl::primitive_attr attr;
-
     const std::vector<int> supported_args = {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1,
             DNNL_ARG_MULTIPLE_SRC, DNNL_ARG_MULTIPLE_SRC + 42};
-    const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
-            DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
 
     for (auto arg : supported_args) {
         // single non-default scales for supported arg
         attr.set_scales_mask(arg, 0);
         // multiple scales
         attr.set_scales_mask(arg, 1 << 1);
-    }
-
-    for (auto arg : unsupported_args) {
-        // single scales for unsupported args
-        EXPECT_ANY_THROW(attr.set_scales_mask(arg, 0));
+        // host scale
+        attr.set_host_scale(arg);
     }
 }
 
@@ -281,17 +306,42 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestScalesWithGroups) {
     for (auto arg : supported_args) {
         // single non-default scales for supported arg
         attr.set_scales(arg, 0, {});
-        // multiple scales with groups
+        // multiple scales with a single group dim
         attr.set_scales(arg, 1 << 0, {4});
+        // multiple scales with multiple group dims
+        attr.set_scales(arg, 1 << 0, {4, 1});
         // scales with groups and a data type
-        attr.set_scales(arg, 1 << 0, {4}, data_type::f32);
+        attr.set_scales(arg, 1 << 0, {4, 1}, data_type::f32);
+        // single host scale for supported arg with data type specified
+        attr.set_scales(arg, 0, {}, data_type::f32, true);
     }
 
     for (auto arg : unsupported_args) {
         // multiple scales with groups for unsupported args
-        EXPECT_ANY_THROW(attr.set_scales(arg, 1 << 0, {4}));
+        EXPECT_ANY_THROW(attr.set_scales(arg, 1 << 0, {4, 1}));
         // multiple scales with non-default data type for unsupported args
         EXPECT_ANY_THROW(attr.set_scales(arg, 1 << 0, {}, data_type::bf16));
+        // single host scale for unsupported arg with data type specified
+        EXPECT_ANY_THROW(attr.set_scales(arg, 0, {}, data_type::bf16, true));
+    }
+
+    // multiple scales with groups that are not supported for host scale
+    int any_support_arg = DNNL_ARG_SRC;
+    EXPECT_ANY_THROW(attr.set_scales(
+            any_support_arg, 1 << 0, {4, 1}, data_type::s8, true));
+}
+
+TEST_F(attr_test_t, TestScalesExpectFailure) {
+    dnnl::primitive_attr attr;
+    const std::vector<int> unsupported_args = {DNNL_ARG_BIAS, DNNL_ARG_DST_2,
+            DNNL_ARG_MEAN, DNNL_ARG_WORKSPACE, DNNL_ARG_SCRATCHPAD};
+
+    for (auto arg : unsupported_args) {
+        // single scales for unsupported args
+        EXPECT_ANY_THROW(attr.set_scales_mask(arg, 0));
+        EXPECT_ANY_THROW(attr.set_scales_mask(arg, 1 << 1));
+        // host scale
+        EXPECT_ANY_THROW(attr.set_host_scale(arg));
     }
 }
 
@@ -370,15 +420,6 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestRNNProjWeightsQuantization) {
     ASSERT_EQ(scales[2], 4.f);
 }
 
-TEST_F(attr_test_t, TestScalesExpectFailure) {
-    dnnl::primitive_attr attr;
-    const int unsupported_arg = DNNL_ARG_MEAN;
-
-    // non-default scales for unsupported arg
-    EXPECT_ANY_THROW(attr.set_scales_mask(unsupported_arg, 0));
-    EXPECT_ANY_THROW(attr.set_scales_mask(unsupported_arg, 1 << 1));
-}
-
 HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestPostOps) {
     dnnl::primitive_attr attr;
     dnnl::post_ops ops;
@@ -791,7 +832,7 @@ HANDLE_EXCEPTIONS_FOR_TEST_F(attr_test_t, TestGetCppObjects) {
     // of using a dangling pointer from destroyed object via
     // `pd.get_primitive_attr().get_post_ops()` construction as attributes will
     // be destroyed once post-ops are saved on stack.
-    // See https://github.com/oneapi-src/oneDNN/issues/1337 for details.
+    // See https://github.com/uxlfoundation/oneDNN/issues/1337 for details.
     dnnl::primitive_attr attr;
     dnnl::post_ops ops;
     memory::desc po_src1_md({1, 1, 1, 1}, data_type::f32, tag::abcd);
